#ifndef _android_apk_H_
#define _android_apk_H_

#include <android/log.h>
#include <string>

#include "OgreFileSystemLayer.h"
#include "Android/OgreAPKFileSystemArchive.h"
#include "Android/OgreAPKZipArchive.h"

#include <android/storage_manager.h>
#include "android_strings.h"

extern JNIEnv* gAndroidActivityEnv;
extern jobject  gActivityInstance;
extern jmethodID gCloseRailsApplication;
extern jmethodID gIsChaptersUnlocked;
extern jmethodID gBuyUnlockChapters;
extern jmethodID gShowRateDialogIfNeeds;
extern jmethodID gShowRegisterDialogIfNeeds;
extern jmethodID gResetRegisterAndRateDialogPrefs;
extern jmethodID gOgreHasBeenStarted;
extern jmethodID gNeedsUseLowResTextures;
extern jmethodID gNeedsUseLowResOverlays;

#ifdef OBB_EXPANSOIN_FILE
extern jmethodID gDownloadOBBPackage;
extern jmethodID gValidateMainOBB;
extern jmethodID gGetMountingOBBState;
extern jmethodID gGetMountedOBBPath;
extern jmethodID gGetUnmountedOBBPath;

static Ogre::DataStreamPtr openOBBFile(const Ogre::String& path)
{
	FILE * pFile = NULL;
	pFile = fopen(path.c_str(), "rb");
	
	Ogre::DataStreamPtr istream;
	
	if (pFile)
	{
		istream = Ogre::DataStreamPtr(new Ogre::FileHandleDataStream(path.c_str(), pFile, Ogre::DataStream::READ));
		//fclose (pFile);
	}
	
	return istream;
}

static void jDownloadOBBPackage()
{
	if (!gDownloadOBBPackage)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "downloadOBBPackage");
		return;
	}
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, gDownloadOBBPackage);
}

static bool jValidateMainOBB()
{
	if (!gValidateMainOBB)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "validateMainOBB");
		return false;
	}

	bool res = (int)gAndroidActivityEnv->CallObjectMethod(gActivityInstance, gValidateMainOBB);

	return res;
}

static int jGetMountingOBBState()
{
	if (!gGetMountingOBBState)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "getMountingOBBState");
		return -1;
	}

	int res = (int)gAndroidActivityEnv->CallObjectMethod(gActivityInstance, gGetMountingOBBState);

	return res;
}

static std::string jGetUnmountedOBBPath()
{
	if (!gGetUnmountedOBBPath)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "getUnmountedOBBPath");
		return "";
	}
	
	jstring jstrRes = (jstring)gAndroidActivityEnv->CallObjectMethod(gActivityInstance, gGetUnmountedOBBPath);
	
	const char* nativeString = gAndroidActivityEnv->GetStringUTFChars(jstrRes, 0);
	std::string str(nativeString);
	gAndroidActivityEnv->ReleaseStringUTFChars(jstrRes, nativeString);
	gAndroidActivityEnv->DeleteLocalRef(jstrRes);
	
	return str;
}

static std::string jGetMountedOBBPath()
{
	if (!gGetMountedOBBPath)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "getMountedOBBPath");
		return "";
	}
	
	jstring jstrRes = (jstring)gAndroidActivityEnv->CallObjectMethod(gActivityInstance, gGetMountedOBBPath);
	
	const char* nativeString = gAndroidActivityEnv->GetStringUTFChars(jstrRes, 0);
	std::string str(nativeString);
	gAndroidActivityEnv->ReleaseStringUTFChars(jstrRes, nativeString);
	gAndroidActivityEnv->DeleteLocalRef(jstrRes);
	
	return str;
}

#endif ///////////// OBB_EXPANSOIN_FILE ///////////////

static Ogre::DataStreamPtr openAPKFile(const Ogre::String& fileName, AAssetManager* assetMgr)
{
    Ogre::DataStreamPtr stream;
    AAsset* asset = AAssetManager_open(assetMgr, fileName.c_str(), AASSET_MODE_BUFFER);
    if(asset)
    {
        off_t length = AAsset_getLength(asset);
        void* membuf = OGRE_MALLOC(length, Ogre::MEMCATEGORY_GENERAL);
        memcpy(membuf, AAsset_getBuffer(asset), length);
        AAsset_close(asset);
        
        stream = Ogre::DataStreamPtr(new Ogre::MemoryDataStream(membuf, length, true, true));
    }
    return stream;
}

static void jShowRateDialogIfNeeds(int levelIndex)
{
	if (!gShowRateDialogIfNeeds)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "showRateDialogIfNeeds");
		return;
	}
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, gShowRateDialogIfNeeds, levelIndex);
}

static void jOgreHasBeenStarted()
{
	if (!gOgreHasBeenStarted)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "ogreHasBeenStarted");
		return;
	}
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, gOgreHasBeenStarted);
}

static void jShowRegisterDialogIfNeeds()
{
	if (!gShowRegisterDialogIfNeeds)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "showRegisterDialogIfNeeds");
		return;
	}
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, gShowRegisterDialogIfNeeds);
}

static void jResetRegisterAndRateDialogPrefs()
{
	if (!gResetRegisterAndRateDialogPrefs)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "resetRegisterAndRateDialogPrefs");
		return;
	}
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, gResetRegisterAndRateDialogPrefs);
}

static void jCloseRailsApplication()
{
	if (!gCloseRailsApplication)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "closeRailsApplication");
		return;
	}
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, gCloseRailsApplication);
}

static bool jNeedsUseLowResTextures()
{
	static int needsUseLowResTextures_ = -1;
	if (!gNeedsUseLowResTextures)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "needsUseLowResTextures");
		return false;
	}
	
	if (needsUseLowResTextures_ < 0)
		needsUseLowResTextures_ = (int)gAndroidActivityEnv->CallObjectMethod(gActivityInstance, gNeedsUseLowResTextures);
	
	bool res = (bool)needsUseLowResTextures_;

	return res;
}

static bool jNeedsUseLowResOverlays()
{
	static int needsUseLowResOverlays_ = -1;
	if (!gNeedsUseLowResOverlays)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "needsUseLowResOverlays");
		return false;
	}
	
	if (needsUseLowResOverlays_ < 0)
		needsUseLowResOverlays_ = (int)gAndroidActivityEnv->CallObjectMethod(gActivityInstance, gNeedsUseLowResOverlays);
	
	bool res = (bool)needsUseLowResOverlays_;

	return res;
}

static bool jIsChaptersUnlocked()
{
	if (!gIsChaptersUnlocked)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "isChaptersUnlocked");
		return false;
	}

	bool res = (int)gAndroidActivityEnv->CallObjectMethod(gActivityInstance, gIsChaptersUnlocked);

	return res;
}

static void jBuyUnlockChapters()
{
	if (!gBuyUnlockChapters)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "unlockChaptersButtonClicked");
		return;
	}
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, gBuyUnlockChapters);
}

#endif