#ifndef _android_strings_H_
#define _android_strings_H_

#include <jni.h>
#include <android_native_app_glue.h>
#include <android/log.h>
#include <string>

extern JNIEnv* gAndroidActivityEnv;
extern jobject  gActivityInstance;
extern jmethodID gGetString;
extern jmethodID gGetFString1;
extern jmethodID gGetFString2;
extern jmethodID gGetVersionName;
extern jmethodID gGetVersionCode;
extern jmethodID gIsJapanLocale;
extern jmethodID gGetLocStringsCodePointRanges;

static int s_isJapanLocale = -1;

static int jIsJapanLocale()
{
	if (s_isJapanLocale < 0)
	{
		if (!gIsJapanLocale)
		{
			__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "isJapanLocale");
			return false;
		}
	
		s_isJapanLocale = (bool)gAndroidActivityEnv->CallObjectMethod(gActivityInstance, gIsJapanLocale);
	}
	
	return (bool)s_isJapanLocale;
}

static std::string jGetLocStringsCodePointRanges()
{
	if (!gGetLocStringsCodePointRanges)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "getLocStringsCodePointRanges");
		return "";
	}
	
	jstring jstrRes = (jstring)gAndroidActivityEnv->CallObjectMethod(gActivityInstance, gGetLocStringsCodePointRanges);
	
	const char* nativeString = gAndroidActivityEnv->GetStringUTFChars(jstrRes, 0);
	std::string str(nativeString);
	gAndroidActivityEnv->ReleaseStringUTFChars(jstrRes, nativeString);
	gAndroidActivityEnv->DeleteLocalRef(jstrRes);
	
	return str;
}

static int jGetVersionCode()
{
	if (!gGetVersionCode)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "getVersionCode");
		return -1;
	}
	
	int res = (int)gAndroidActivityEnv->CallObjectMethod(gActivityInstance, gGetVersionCode);
	
	return res;
}

static std::string jGetVersionName()
{
	if (!gGetVersionName)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "getVersionName");
		return "";
	}
	
	jstring jstrRes = (jstring)gAndroidActivityEnv->CallObjectMethod(gActivityInstance, gGetVersionName);
	
	const char* nativeString = gAndroidActivityEnv->GetStringUTFChars(jstrRes, 0);
	std::string str(nativeString);
	gAndroidActivityEnv->ReleaseStringUTFChars(jstrRes, nativeString);
	gAndroidActivityEnv->DeleteLocalRef(jstrRes);
	
	return str;
}

static std::string jGetLocalizedString(const char* key)
{

	//Log_Printf("url=%s\n",url);

	if (!gGetString)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "getLocalizedString");
		return "";
	}
	jstring jkeyStr = gAndroidActivityEnv->NewStringUTF(key);
	
	jstring jstrRes = (jstring)gAndroidActivityEnv->CallObjectMethod(gActivityInstance, gGetString, jkeyStr);
	
	const char* nativeString = gAndroidActivityEnv->GetStringUTFChars(jstrRes, 0);
	std::string str(nativeString);
	gAndroidActivityEnv->ReleaseStringUTFChars(jstrRes, nativeString);
	gAndroidActivityEnv->DeleteLocalRef(jstrRes);
	gAndroidActivityEnv->DeleteLocalRef(jkeyStr);
	
	if (str.length() == 0)
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No localised string defined for key \"%s\"!", key);
	
	return str;
}

static std::string jGetLocalizedFString(const char* key, int val)
{

	//Log_Printf("url=%s\n",url);

	if (!gGetFString1)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "getLocalizedFString");
		return "";
	}
	jstring jkeyStr = gAndroidActivityEnv->NewStringUTF(key);
	
	jstring jstrRes = (jstring)gAndroidActivityEnv->CallObjectMethod(gActivityInstance, gGetFString1, jkeyStr, val);
	
	const char* nativeString = gAndroidActivityEnv->GetStringUTFChars(jstrRes, 0);
	std::string str(nativeString);
	gAndroidActivityEnv->ReleaseStringUTFChars(jstrRes, nativeString);
	gAndroidActivityEnv->DeleteLocalRef(jstrRes);
	gAndroidActivityEnv->DeleteLocalRef(jkeyStr);
	
	if (str.length() == 0)
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No localised string defined for key \"%s\"!", key);
	
	return str;
}

static std::string jGetLocalizedFString(const char* key, int val1, int val2)
{

	//Log_Printf("url=%s\n",url);

	if (!gGetFString2)
	{
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No Java VM method defined. Cannot invocate %s method!", "getLocalizedFString");
		return "";
	}
	jstring jkeyStr = gAndroidActivityEnv->NewStringUTF(key);
	
	jstring jstrRes = (jstring)gAndroidActivityEnv->CallObjectMethod(gActivityInstance, gGetFString2, jkeyStr, val1, val2);
	
	const char* nativeString = gAndroidActivityEnv->GetStringUTFChars(jstrRes, 0);
	std::string str(nativeString);
	gAndroidActivityEnv->ReleaseStringUTFChars(jstrRes, nativeString);
	gAndroidActivityEnv->DeleteLocalRef(jstrRes);
	gAndroidActivityEnv->DeleteLocalRef(jkeyStr);
	
	if (str.length() == 0)
		__android_log_print(ANDROID_LOG_INFO, "Rails", "No localised string defined for key \"%s\"!", key);
	
	return str;
}

#endif