/*
 * Copyright (C) 2013 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <jni.h>

#include <sys/atomics.h>
//#include "engine.hpp"
#include "java_bgnactivity.hpp"
//#include "jni_util.hpp"

#include "com_belightsoft_rails_RailsLauncher.h"

#include "CPrefs.h"

#include <android/log.h>

#include "NSDictionary.h"
#include "OgreString.h"

extern JNIEnv* gAndroidActivityEnv;
extern jobject gActivityInstance;
extern jclass gActivityClass;

extern Ogre::String gUserDefaultsPath;

#define SIGNIN_FAILED 0
#define SIGNIN_IN_PROGRESS 1
#define SIGNIN_SUCCEEDED 2
static int _signInStatus = SIGNIN_FAILED;
static int _cloud_load_success = 0;

#define DEBUG_TAG "Rails:Native"
#define LOGD(...) ((void)__android_log_print(ANDROID_LOG_DEBUG, DEBUG_TAG, __VA_ARGS__))
#define LOGI(...) ((void)__android_log_print(ANDROID_LOG_INFO, DEBUG_TAG, __VA_ARGS__))
#define LOGW(...) ((void)__android_log_print(ANDROID_LOG_WARN, DEBUG_TAG, __VA_ARGS__))
#define LOGE(...) ((void)__android_log_print(ANDROID_LOG_ERROR, DEBUG_TAG, __VA_ARGS__))
#define ABORT_GAME { LOGE("*** GAME ABORTING."); *((char*)0) = 'a'; }
#define MY_ASSERT(cond) { if (!(cond)) { LOGE("ASSERTION FAILED: %s", #cond); ABORT_GAME; } }

JNIEXPORT void JNICALL Java_com_belightsoft_rails_RailsLauncher_native_1ReportSignInState(JNIEnv *env, jclass thiz, jboolean signedIn, jboolean inProgress)
{
    int newState = inProgress ? SIGNIN_IN_PROGRESS : signedIn ? SIGNIN_SUCCEEDED : SIGNIN_FAILED;
    __atomic_swap(newState, &_signInStatus);
}

JNIEXPORT void JNICALL Java_com_belightsoft_rails_RailsLauncher_native_1ReportCloudLoadResult(JNIEnv *env, jclass thiz, jboolean success, jstring jServerData)
{
	if (jServerData != NULL)
	{
		const char* nativeServerData = env->GetStringUTFChars(jServerData, 0);
		int serverDataLength = env->GetStringLength( jServerData );
		
		NSDictionaryPtr serverDict = NSDictionaryPtr(new NSDictionary());
		serverDict->loadRootSubDictFromData(nativeServerData, serverDataLength);
		env->ReleaseStringUTFChars(jServerData, nativeServerData);
		env->DeleteLocalRef(jServerData);
		
		NSDictionaryPtr localDict = NSDictionaryPtr(new NSDictionary());
		localDict->loadRootSubDictFromFile(gUserDefaultsPath.c_str());
		
		NSDictionaryPtr resDict = CPrefs::getPrefsMergeResult(localDict, serverDict);
		
		resDict->saveRootSubDictToFile(gUserDefaultsPath.c_str());
		
		__atomic_swap(success ? 1 : 0, &_cloud_load_success);
	}
}

JNIEXPORT jstring JNICALL Java_com_belightsoft_rails_RailsLauncher_native_1ResolveCloudStateConflict(JNIEnv *env, jclass thiz, jstring jLocalData, jstring jServerData)
{
	if (jLocalData == NULL && jServerData != NULL)
		return jServerData;
	else if (jLocalData != NULL && jServerData == NULL)
		return jLocalData;
	else if (jLocalData == NULL && jServerData == NULL)
		return NULL;
	
	int serverDataLength = env->GetStringLength( jServerData );
	int localDataLength = env->GetStringLength( jLocalData );
	
	const char* nativeServerData = env->GetStringUTFChars(jServerData, 0);
	const char* nativeLocalData = env->GetStringUTFChars(jLocalData, 0);
	
	NSDictionaryPtr serverDict = NSDictionaryPtr(new NSDictionary());
	NSDictionaryPtr localDict = NSDictionaryPtr(new NSDictionary());
	serverDict->loadRootSubDictFromData(nativeServerData, serverDataLength);
	localDict->loadRootSubDictFromData(nativeLocalData, localDataLength);
	NSDictionaryPtr resDict = CPrefs::getPrefsMergeResult(localDict, serverDict);
	char* data = resDict->getDictAsData();
	jstring jResData = env->NewStringUTF(data);
	delete[] data;
	
	env->ReleaseStringUTFChars(jLocalData, nativeLocalData);
	env->ReleaseStringUTFChars(jServerData, nativeServerData);
	env->DeleteLocalRef(jLocalData);
	env->DeleteLocalRef(jServerData);
	
	return jResData;
}

bool BGNActivity_IsCloudLoadSuccess()
{
	return _cloud_load_success == 1;
}

void BGNActivity_ResetCloudLoadState()
{
	__atomic_swap(0, &_cloud_load_success);
}

bool BGNActivity_IsSignedIn()
{
	return _signInStatus == SIGNIN_SUCCEEDED;
}

bool BGNActivity_IsInProgress()
{
	return _signInStatus == SIGNIN_IN_PROGRESS;
}

void BGNActivity_StartSignIn()
{
#ifdef GOOGLE_PLUS_ENABLED
	jmethodID m_postStartSignIn = gAndroidActivityEnv->GetMethodID(gActivityClass, "postStartSignIn", "()V");
	MY_ASSERT(m_postStartSignIn != NULL);
	_signInStatus = SIGNIN_IN_PROGRESS;
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, m_postStartSignIn);
#endif
}

void BGNActivity_StartSignOut()
{
#ifdef GOOGLE_PLUS_ENABLED
	jmethodID m_postStartSignOut = gAndroidActivityEnv->GetMethodID(gActivityClass, "postStartSignOut", "()V");
	MY_ASSERT(m_postStartSignOut != NULL);
	_signInStatus = SIGNIN_FAILED;
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, m_postStartSignOut);
#endif
}

void BGNActivity_ShowAchievements()
{
#ifdef GOOGLE_PLUS_ENABLED
	jmethodID m_postShowAchievements = gAndroidActivityEnv->GetMethodID(gActivityClass, "postShowAchievements", "()V");
	MY_ASSERT(m_postShowAchievements);
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, m_postShowAchievements);
#endif
}

void BGNActivity_ShowLeaderboards()
{
#ifdef GOOGLE_PLUS_ENABLED
	jmethodID m_postShowLeaderboards = gAndroidActivityEnv->GetMethodID(gActivityClass, "postShowLeaderboards", "()V");
	MY_ASSERT(m_postShowLeaderboards);
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, m_postShowLeaderboards);
#endif
}

void BGNActivity_EraseLiderboards()
{
#ifdef GOOGLE_PLUS_ENABLED
	jmethodID m_postEraseLiderboards = gAndroidActivityEnv->GetMethodID(gActivityClass, "postEraseLiderboards", "()V");
	MY_ASSERT(m_postEraseLiderboards);
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, m_postEraseLiderboards);
#endif
}

void BGNActivity_EraseState()
{
#ifdef GOOGLE_PLUS_ENABLED
	jmethodID m_postEraseState = gAndroidActivityEnv->GetMethodID(gActivityClass, "postEraseState", "()V");
	MY_ASSERT(m_postEraseState);
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, m_postEraseState);
#endif
}

void BGNActivity_ShowLeaderboard(const char *lb_id)
{
#ifdef GOOGLE_PLUS_ENABLED
	jmethodID m_postShowLeaderboard = gAndroidActivityEnv->GetMethodID(gActivityClass, "postShowLeaderboard", "(Ljava/lang/String;)V");
	MY_ASSERT(m_postShowLeaderboard);
	jstring j_lb_id = gAndroidActivityEnv->NewStringUTF(lb_id);
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, m_postShowLeaderboard, j_lb_id);
	gAndroidActivityEnv->DeleteLocalRef(j_lb_id);
#endif
}

void BGNActivity_UnlockAchievement(const char *ach_id)
{
#ifdef GOOGLE_PLUS_ENABLED
	jmethodID m_postUnlockAchievement = gAndroidActivityEnv->GetMethodID(gActivityClass, "postUnlockAchievement", "(Ljava/lang/String;)V");
	MY_ASSERT(m_postUnlockAchievement);
	jstring j_ach_id = gAndroidActivityEnv->NewStringUTF(ach_id);
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, m_postUnlockAchievement, j_ach_id);
	gAndroidActivityEnv->DeleteLocalRef(j_ach_id);
#endif
}

void BGNActivity_IncrementAchievement(const char *ach_id, int steps)
{
#ifdef GOOGLE_PLUS_ENABLED
	jmethodID m_postIncrementAchievement = gAndroidActivityEnv->GetMethodID(gActivityClass, "postIncrementAchievement", "(Ljava/lang/String;I)V");
	MY_ASSERT(m_postIncrementAchievement);
	jstring j_ach_id = gAndroidActivityEnv->NewStringUTF(ach_id);
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, m_postIncrementAchievement, j_ach_id, (jint) steps);
	gAndroidActivityEnv->DeleteLocalRef(j_ach_id);
#endif
}

void BGNActivity_EraseAchievements()
{
#ifdef GOOGLE_PLUS_ENABLED
	jmethodID m_postEraseAchievements = gAndroidActivityEnv->GetMethodID(gActivityClass, "postEraseAchievements", "()V");
	MY_ASSERT(m_postEraseAchievements);
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, m_postEraseAchievements);
#endif
}

void BGNActivity_SubmitScore(const char *lb_id, long score)
{
#ifdef GOOGLE_PLUS_ENABLED
	jmethodID m_postSubmitScore = gAndroidActivityEnv->GetMethodID(gActivityClass, "postSubmitScore", "(Ljava/lang/String;J)V");
	MY_ASSERT(m_postSubmitScore);
	jstring j_lb_id = gAndroidActivityEnv->NewStringUTF(lb_id);
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, m_postSubmitScore, j_lb_id, (jlong) score);
	gAndroidActivityEnv->DeleteLocalRef(j_lb_id);
#endif
}

void BGNActivity_SaveState()
{
#ifdef GOOGLE_PLUS_ENABLED
    jmethodID m_postSaveState = gAndroidActivityEnv->GetMethodID(gActivityClass, "postSaveState", "()V");
    MY_ASSERT(m_postSaveState);
    gAndroidActivityEnv->CallVoidMethod(gActivityInstance, m_postSaveState);
#endif
}

void BGNActivity_ShowToastMessage(const char* msg)
{
	jmethodID m_postShowToastMessage = gAndroidActivityEnv->GetMethodID(gActivityClass,"postShowToastMessage", "(Ljava/lang/String;)V");
	MY_ASSERT(m_postShowToastMessage);
	jstring j_msg = gAndroidActivityEnv->NewStringUTF(msg);
	gAndroidActivityEnv->CallVoidMethod(gActivityInstance, m_postShowToastMessage, j_msg);
	gAndroidActivityEnv->DeleteLocalRef(j_msg);
}

/*
void BGNActivity_GetDeviceMotionRange(int deviceId, int source, float *outMinX, float *outMaxX, float *outMinY, float *outMaxY)
{
    jmethodID m_getMinX = gAndroidActivityEnv->GetMethodID(gActivityClass, "getDeviceMotionRangeMinX", "(II)F");
    jmethodID m_getMaxX = gAndroidActivityEnv->GetMethodID(gActivityClass, "getDeviceMotionRangeMaxX", "(II)F");
    jmethodID m_getMinY = gAndroidActivityEnv->GetMethodID(gActivityClass, "getDeviceMotionRangeMinY", "(II)F");
    jmethodID m_getMaxY = gAndroidActivityEnv->GetMethodID(gActivityClass, "getDeviceMotionRangeMaxY", "(II)F");
    MY_ASSERT(m_getMinX);
    MY_ASSERT(m_getMaxX);
    MY_ASSERT(m_getMinY);
    MY_ASSERT(m_getMaxY);
    *outMinX = gAndroidActivityEnv->CallFloatMethod(gActivityInstance, m_getMinX, deviceId, source);
    *outMaxX = gAndroidActivityEnv->CallFloatMethod(gActivityInstance, m_getMaxX, deviceId, source);
    *outMinY = gAndroidActivityEnv->CallFloatMethod(gActivityInstance, m_getMinY, deviceId, source);
    *outMaxY = gAndroidActivityEnv->CallFloatMethod(gActivityInstance, m_getMaxY, deviceId, source);

    LOGD("Motion range for device %d, source %d is X:%f-%f, Y:%f-%f", deviceId, source,
            *outMinX, *outMaxX, *outMinY, *outMaxY);
}
*/
// int BGNActivity_GetApiLevel() {
//     jmethodID m_getApiLevel = gAndroidActivityEnv->GetMethodID(gActivityClass,
//             "getApiLevel", "()I");
//     MY_ASSERT(m_getApiLevel);
//     return gAndroidActivityEnv->CallIntMethod(gActivityInstance, m_getApiLevel);
// }



