/*
 -----------------------------------------------------------------------------
 This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
 Copyright (c) 2000-2013 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
#include "OgrePlatform.h"

//#define OGRE_STATIC_GLES2
//#define OGRE_STATIC_ParticleFX

#include "RailsContext.h"

#include "OgreAndroidBridge.h"
#include <android/log.h>
#include <android/asset_manager.h>

#include <jni.h>

JavaVM  *gJVM;

RailsContext* OgreAndroidBridge::mRailsContext = NULL;
AndroidInputInjector* OgreAndroidBridge::mInputInjector = NULL;
AndroidMultiTouch* OgreAndroidBridge::mTouch = NULL;
AndroidKeyboard* OgreAndroidBridge::mKeyboard = NULL;
Ogre::RenderWindow* OgreAndroidBridge::mRenderWnd = NULL;
Ogre::Root* OgreAndroidBridge::mRoot = NULL;
bool OgreAndroidBridge::mInit = false;

#ifdef OGRE_STATIC_LIB
Ogre::StaticPluginLoader* OgreAndroidBridge::mStaticPluginLoader = NULL;
#endif

AAssetManager* gAssetMgr = NULL;

JNIEnv* gAndroidActivityEnv = NULL;
jobject  gActivityInstance = NULL;
jclass gActivityClass = NULL;

jmethodID gGoToWebsite = NULL;
jmethodID gGetString = NULL;
jmethodID gGetFString1 = NULL;
jmethodID gGetFString2 = NULL;

jmethodID gGetInternalDataPath = NULL;

jmethodID gShowTrainopediaPopup = NULL;
jmethodID gHideTrainopediaPopup = NULL;

jmethodID gPlaySound = NULL;
jmethodID gStopMusic = NULL;
jmethodID gPlayIntroMusic = NULL;
jmethodID gPlayLevelCompleteMusic = NULL;
jmethodID gPlayMainTheme = NULL;

jmethodID gGetVersionName = NULL;
jmethodID gGetVersionCode = NULL;

#ifdef OBB_EXPANSOIN_FILE
jmethodID gValidateMainOBB = NULL;
jmethodID gGetMountingOBBState = NULL;
jmethodID gGetMountedOBBPath = NULL;
jmethodID gGetUnmountedOBBPath = NULL;
jmethodID gDownloadOBBPackage = NULL;
#endif

jmethodID gShowProgress = NULL;
jmethodID gHideProgress = NULL;

jmethodID gCloseRailsApplication = NULL;

jmethodID gIsChaptersUnlocked = NULL;
jmethodID gBuyUnlockChapters = NULL;
jmethodID gIsJapanLocale = NULL;

jmethodID gGetLocStringsCodePointRanges = NULL;

jmethodID gShowRateDialogIfNeeds = NULL;
jmethodID gShowRegisterDialogIfNeeds = NULL;
jmethodID gResetRegisterAndRateDialogPrefs = NULL;

jmethodID gOgreHasBeenStarted = NULL;

jmethodID gNeedsUseLowResTextures = NULL;
jmethodID gNeedsUseLowResOverlays = NULL;

Ogre::String gInternalDataPath;
Ogre::String gUserDefaultsPath;
Ogre::String gUserSnapshotPath;

//struct android_app *__gandroidapp__;

void registerEnvironmentAndActivity(ANativeActivity* activity);

void android_main(struct android_app* state)
{
	//__gandroidapp__ = state;
    // Make sure glue isn't stripped.
    app_dummy();
	
	registerEnvironmentAndActivity(state->activity);
	
	//gInternalDataPath = String("sdcard/Download");
	if (state->activity->internalDataPath) // bugfix for Android 2.3.x
		gInternalDataPath = String(state->activity->internalDataPath);
	else
	{
		jstring jstrRes = (jstring)gAndroidActivityEnv->CallObjectMethod(gActivityInstance, gGetInternalDataPath);
	
		const char* nativeString = gAndroidActivityEnv->GetStringUTFChars(jstrRes, 0);
		gInternalDataPath = String(nativeString);
		gAndroidActivityEnv->ReleaseStringUTFChars(jstrRes, nativeString);
		gAndroidActivityEnv->DeleteLocalRef(jstrRes);
	}
	
	gUserDefaultsPath = gInternalDataPath + "/com.belightsoft.rails.android.plist";
	gUserSnapshotPath = gInternalDataPath + "/com.belightsoft.rails.android.snapshot";
	
	gAssetMgr = state->activity->assetManager;
	
    OgreAndroidBridge::init(state);
    OgreAndroidBridge::go(state);
}

void getMethodID(jmethodID& methodID, const char* jMethodName, const char* jMethodSignature)
{
	methodID = gAndroidActivityEnv->GetMethodID(gActivityClass, jMethodName, jMethodSignature);
	if (!methodID)
	{
		__android_log_print(ANDROID_LOG_INFO, "RailsNative", "Unable to find method %s in class.", jMethodName);
		return;
	}
}

void registerEnvironmentAndActivity(ANativeActivity* activity)
{
	gJVM = activity->vm;
	JNIEnv *env;

	gJVM->AttachCurrentThread(&env , NULL);

	gAndroidActivityEnv = env;

	gActivityClass = env->GetObjectClass(activity->clazz);
	if (!gActivityClass)
	{
		__android_log_print(ANDROID_LOG_INFO, "RailsNative", "Unable to get class of main activity.");
		return;
	}

	gActivityInstance = activity->clazz;
	
	//---------------
	
	getMethodID(gGetInternalDataPath, "getInternalDataPath", "()Ljava/lang/String;");
	
	getMethodID(gCloseRailsApplication, "closeRailsApplication", "()V");
		
	getMethodID(gGoToWebsite, "goToWebsite", "(Ljava/lang/String;)V");
	
	getMethodID(gGetString, "getLocString", "(Ljava/lang/String;)Ljava/lang/String;");
	getMethodID(gGetFString1, "getLocFString", "(Ljava/lang/String;I)Ljava/lang/String;");
	getMethodID(gGetFString2, "getLocFString", "(Ljava/lang/String;II)Ljava/lang/String;");
	
	getMethodID(gShowTrainopediaPopup, "showTrainopediaPopup", "()V");
	getMethodID(gHideTrainopediaPopup, "hideTrainopediaPopup", "()V");
	
	getMethodID(gPlaySound, "playSound", "(Ljava/lang/String;)V");
	getMethodID(gStopMusic, "stopMusic", "()V");
	getMethodID(gPlayIntroMusic, "playIntroMusic", "()V");
	getMethodID(gPlayLevelCompleteMusic, "playLevelCompleteMusic", "()V");
	getMethodID(gPlayMainTheme, "playMainTheme", "()V");
	
	getMethodID(gGetVersionName, "versionName", "()Ljava/lang/String;");
	getMethodID(gGetVersionCode, "versionCode", "()I");
	
#ifdef OBB_EXPANSOIN_FILE
	getMethodID(gValidateMainOBB, "validateMainOBB", "()Z");
	getMethodID(gGetMountingOBBState, "getMountingOBBState", "()I");
	getMethodID(gGetMountedOBBPath, "getMountedOBBPath", "()Ljava/lang/String;");
	getMethodID(gGetUnmountedOBBPath, "getUnmountedOBBPath", "()Ljava/lang/String;");
	getMethodID(gDownloadOBBPackage, "downloadOBBPackage", "()V");
#endif
	
	getMethodID(gShowProgress, "showProgress", "()V");
	getMethodID(gHideProgress, "hideProgress", "()V");
	
	getMethodID(gIsChaptersUnlocked, "isChaptersUnlocked", "()Z");
	getMethodID(gBuyUnlockChapters, "unlockChaptersButtonClicked", "()V");
	getMethodID(gIsJapanLocale, "isJapanLocale", "()Z");
	
	getMethodID(gGetLocStringsCodePointRanges, "getLocStringsCodePointRanges", "()Ljava/lang/String;");
	
	getMethodID(gShowRateDialogIfNeeds, "showRateDialogIfNeeds", "(I)V");
	getMethodID(gShowRegisterDialogIfNeeds, "showRegisterDialogIfNeeds", "()V");
	getMethodID(gResetRegisterAndRateDialogPrefs, "resetRegisterAndRateDialogPrefs", "()V");
	
	getMethodID(gOgreHasBeenStarted, "ogreHasBeenStarted", "()V");
	
	getMethodID(gNeedsUseLowResTextures, "needsUseLowResTextures", "()Z");
	getMethodID(gNeedsUseLowResOverlays, "needsUseLowResOverlays", "()Z");
}
