#include "NSDictionary.h"

#include <iostream>
#include <algorithm>

#include "NSDictionary_Extra.h"

#include <android/asset_manager.h>
#include <android/log.h>

#define DICT_TAG "RAILS/Dict"
/*int is_digit(int c)
 {
 return c >= '1' && c <= '9';
 }*/

int ndigits(const char *s)
{
    int i, nd = 0;
    int n = strlen(s);
    
    for (i = 0; i < n; i++) {
        if (isdigit(s[i]))
            nd++;
    }
    return nd;
}

int alphaNumericallyLessThan(const char *s, const char *t)
{
    int sd, td;
    int i, j;
    
    sd = ndigits(s);
    td = ndigits(t);
    
    /* presence of digits */
    if (!sd && !td)
        return strcasecmp(s, t);
    else if (!sd)
        return 1;
    else if (!td)
        return -1;
    
    /* value of digits */
    for (i = 0, j = 0; i < sd && j < td; i++, j++) {
        while (! isdigit(*s))
            s++;
        while (! isdigit(*t))
            t++;
        
        if (*s != *t)
            return *s - *t;
        s++;
        t++;
    }
    
    /* number of digits */
    if (i < sd)
        return 1;
    else if (j < td)
        return -1;
    
    /* value of string after last digit */
    return strcasecmp(s, t);
}


//Constructor
NSDictionary::NSDictionary()
{
    initBlankDict();
}

void NSDictionary::initBlankDict()
{
	dictTree.clear();
	
    //Initializers
    dictTree.push_back(xml_node());
    
    //Create default format
    xml_node plistNode = doc.append_child("plist");
    plistNode.append_attribute("version") = "1.0";
    plistNode.append_child("dict");
    
    //Set root dictTree node
    dictTree.back() = doc.child("plist").child("dict");
}

//Private methods
string NSDictionary::cleanStringWhiteSpace(const string& _str)
{
    string cleanStr = _str;
    cleanStr.erase(remove_if(cleanStr.begin(), cleanStr.end(),  static_cast<int(*)(int)>( isspace )), cleanStr.end());
    return cleanStr;
}

void NSDictionary::split(const string& src, const char* token, vector<string>& vect)
{
    int nend=0;
    int nbegin=0;
    while(nend != -1)
    {
        nend = src.find(token, nbegin);
        if(nend == -1)
            vect.push_back(src.substr(nbegin, src.length()-nbegin));
        else
            vect.push_back(src.substr(nbegin, nend-nbegin));
        nbegin = nend + strlen(token);
    }
}

bool NSDictionary::splitWithForm(const string& content, vector<string>& strs)
{
    // string is empty
    if(content.length() == 0){ return false; }
    
    int nPosLeft  = content.find('{');
    int nPosRight = content.find('}');
    
    // don't have '{' and '}'
    if(nPosLeft == (int)string::npos || nPosRight == (int)string::npos){ return false; }
    
    // '}' is before '{'
    if(nPosLeft > nPosRight){ return false; }
    
    string pointStr = content.substr(nPosLeft + 1, nPosRight - nPosLeft - 1);
    
    // nothing between '{' and '}'
    if(pointStr.length() == 0){ return false; }
    
    int nPos1 = pointStr.find('{');
    int nPos2 = pointStr.find('}');
    
    // contain '{' or '}'
    if(nPos1 != (int)string::npos || nPos2 != (int)string::npos){ return false; }
    
    split(pointStr, ",", strs);
    if (strs.size() != 2 || strs[0].length() == 0 || strs[1].length() == 0)
    {
        return false;
    }
    
    return true;
}

bool NSDictionary::rectFromString(const string& pszContent, CXRect& rect)
{
    //Clean white spaces
    string content = cleanStringWhiteSpace(pszContent);
    
    // find the first '{' and the third '}'
    int nPosLeft  = content.find('{');
    int nPosRight = content.find('}');
    for (int i = 1; i < 3; ++i)
    {
        if (nPosRight == (int)string::npos)
        {
            break;
        }
        nPosRight = content.find('}', nPosRight + 1);
    }
    if(nPosLeft == (int)string::npos || nPosRight == (int)string::npos){ return false; }
    
    content = content.substr(nPosLeft + 1, nPosRight - nPosLeft - 1);
    int nPointEnd = content.find('}');
    if(nPointEnd == (int)string::npos){ return false; }
    nPointEnd = content.find(',', nPointEnd);
    if(nPointEnd == (int)string::npos){ return false; }
    
    // get the point string and size string
    string pointStr = content.substr(0, nPointEnd);
    string sizeStr  = content.substr(nPointEnd + 1, content.length() - nPointEnd);
    
    // split the string with ','
    vector<string> pointInfo;
    if(!splitWithForm(pointStr, pointInfo)){ return false; }
    vector<string> sizeInfo;
    if(!splitWithForm(sizeStr, sizeInfo)){ return false; }
    
    float x = (float) atof(pointInfo[0].c_str());
    float y = (float) atof(pointInfo[1].c_str());
    float width  = (float) atof(sizeInfo[0].c_str());
    float height = (float) atof(sizeInfo[1].c_str());
    
    rect = CXMakeRect(x, y, width, height);
    return true;
}

bool NSDictionary::pointFromString(const string& pszContent, CXPoint& vec2)
{
    vector<string> strs;
    if(!splitWithForm(cleanStringWhiteSpace(pszContent), strs))
    {
        return false;
    }
    
    float x = (float) atof(strs[0].c_str());
    float y = (float) atof(strs[1].c_str());
    
    vec2 = CXPoint(x, y);
    return true;
}

const char* NSDictionary::getFileData(const char* pszFileName, const char* pszMode, off_t& pSize, AAssetManager* assetMgr)
{
	unsigned char* pData = 0;
    AAsset* asset = AAssetManager_open(assetMgr, pszFileName, AASSET_MODE_UNKNOWN);
	
	if (asset)
	{
	    off_t length = AAsset_getLength(asset);
	    pSize = length;
	    pData = new unsigned char[pSize];
	    AAsset_read(asset, pData, pSize);
	    AAsset_close(asset);
	    return (const char*)pData;
	}
	else
	{
		__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, "ERROR: Assets doesn\'t contain file with name: %s", pszFileName);
	}
	
	return NULL;	
}

//Methods
bool NSDictionary::loadRootSubDictFromFile(const char* fileName)
{
    //Clear existing
    dictTree.clear();
    dictTree.push_back(xml_node());
	
	
	//Open file
	//__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, ">>>>>>> try to open file: %s", fileName);
	xml_parse_result result = doc.load_file(fileName);
    
    if(!result)
    {
		//__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, ">>>>>>> ERROR while opening file");
		//__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, ">>>>>>> try to open file: %s", "/data/data/com.belightsoft.rails/files/com.belightsoft.rails.snapshot");
        result = doc.load_file("/data/data/com.belightsoft.rails/files/com.belightsoft.rails.snapshot");
	    if(!result)
	    {
			//__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, ">>>>>>> ERROR while opening file");
			//__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, ">>>>>>> try to open file: %s", "/data/data/com.belightsoft.rails/com.belightsoft.rails.snapshot");
	        result = doc.load_file("/data/data/com.belightsoft.rails/com.belightsoft.rails.snapshot");
	    }
    }
	
	if (result)
	{
    	//Set root dictTree node
    	dictTree.back() = doc.child("plist").child("dict");
    	return true;
	}
	else
	{
		initBlankDict();
		
		return false;
	}
}


bool NSDictionary::loadRootSubDictFromFile(const char* fileName, AAssetManager* assetMgr)
{
    //Clear existing
    dictTree.clear();
    dictTree.push_back(xml_node());
	
    off_t bufferSize = 0;
    const char* pBuffer = getFileData(fileName, "r", bufferSize, assetMgr);
	if (pBuffer)
	{
	    if(!loadRootSubDictFromData(pBuffer, bufferSize))
	    {
			__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, "ERROR: while parsing data from: %s", fileName);
	    	return false;
	    }
	    return true;
	}
	else
	{
		initBlankDict();
		__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, "ERROR: Can\'t open file with name: %s", fileName);
		return false;
	}
}

bool NSDictionary::loadRootSubDictFromData(const char* data, off_t dataSize)
{
	if (data)
	{
	    xml_parse_result result = doc.load_buffer(data,dataSize);
	
	    if(!result)
	    {
			__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, "ERROR: while parsing data.");
	    	return false;
	    }
	
	    //Set root dictTree node
	    dictTree.back() = doc.child("plist").child("dict");
	    return true;
	}
	else
	{
		initBlankDict();
		__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, "ERROR: Can\'t load dict - data is null.");
		return false;
	}
}

char* NSDictionary::getDictAsData()
{
	char_t* heap_buf = doc.xml_document_as_buffer();
	
	return heap_buf; //!!!! don't forget to release memory !!!!
}

void NSDictionary::dump()
{
	__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, "-----------BEGIN------------NSDictionary::dump-----------BEGIN------------");
	char_t* heap_buf = doc.xml_document_as_buffer();
	//char_t* heap_buf = doc.node_to_buffer_heap(dictTree.back());
	
	int heap_len = strlen(heap_buf);
	int max = 500;
	if (heap_len > max)
	{
		int start = 0;
		int length = max;
		
	// 	int cycles = heap_len / max;
	// 	for (int i = 0; i < cycles; i++)
	// 	{
	// 		__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, std::string(heap_buf + start, length).c_str());
	// 		start += length;
	// 	}
	// 	
	// 	if (start < heap_len)
	// 	{
	// 		length = heap_len - start;
	// 		__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, std::string(heap_buf + start, length).c_str());
	// 	}
		
		
		for (int i = max; i > 0; i--)
		{
			if (heap_buf[start + i] == '\n')
			{
				length = i;
				if (length > 0 && (start + length) <= heap_len)
				{
					__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, std::string(heap_buf + start, length).c_str());
					start += length + 1;
					i = (start + max < heap_len)? max : heap_len - start;
				}
				
				if (heap_len - start <= max)
				{
					length = heap_len - start;
					__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, std::string(heap_buf + start, length).c_str());
					break;
				}
			}
		}
	}
	else
		__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, heap_buf);
	
	delete[] heap_buf;
	__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, "------------END-------------NSDictionary::dump------------END-------------");
}

bool NSDictionary::saveRootSubDictToFile(const char* fileName)
{
	//activity.getApplicationContext().getFilesDir().getAbsolutePath()
	// ANativeActivity* nativeActivity = state->activity;                              
	//     const char* internalPath = nativeActivity->internalDataPath;
	//     std::string dataPath(internalPath);                               
    // internalDataPath points directly to the files/ directory                                  
    //std::string filePath = dataPath + std::string("/") + filePath;

    // sometimes if this is the first time we run the app 
    // then we need to create the internal storage "files" directory
    // struct stat sb;
    // int32_t res = stat(dataPath.c_str(), &sb);
    // if (0 == res && sb.st_mode & S_IFDIR)
    // {
    //     LOGD("'files/' dir already in app's internal data storage.");
    // }
    // else if (ENOENT == errno)
    // {
    //     res = mkdir(dataPath.c_str(), 0770);
    // }
	
	bool result = false;
	//__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, ">>>>>>> try to save into file: %s", fileName);
	result = doc.save_file(fileName, PUGIXML_TEXT("\t"), format_default, encoding_utf8);
	if (!result)
	{
		//__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, ">>>>>>> Save operation is failed.");
		//__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, ">>>>>>> try to save into file: %s", "/data/data/com.belightsoft.rails/com.belightsoft.rails.snapshot");
		result = doc.save_file("/data/data/com.belightsoft.rails/com.belightsoft.rails.snapshot", PUGIXML_TEXT("\t"), format_default, encoding_utf8);
		if (!result)
		{
			//__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, ">>>>>>> Save operation is failed.");
			//__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, ">>>>>>> try to save into file: %s", "com.belightsoft.rails.snapshot");
			result = doc.save_file("com.belightsoft.rails.snapshot", PUGIXML_TEXT("\t"), format_default, encoding_utf8);
		}
	}
	
    if(!result)
    {
		__android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, "ERROR: Save operation is failed.");
        ////if(DS_DEBUG_MODE){ printf("%s \n",string("DS_ENGINE:> :ERROR: FAILED TO SAVE XML FILE: ").append(fileName).c_str()); }
    }
    
    return result;
}

bool NSDictionary::stepIntoSubDictWithKey(const char* key)
{
    for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
    {
        if(node.child_value() == string(key) && node.next_sibling() == node.next_sibling("dict"))
        {
            dictTree.push_back(node.next_sibling());
            return true;
        }
    }
    
    ////if(DS_DEBUG_MODE){ printf("%s \n",string("DS_ENGINE:> :ERROR: FAILED TO STEP INTO XML DICTIONARY WITH KEY: ").append(key).c_str()); }
    return false;
}

void NSDictionary::stepOutOfSubDict()
{
    if(dictTree.size() > 1)
    {
        dictTree.pop_back();
    }
}

void NSDictionary::stepBackToRootSubDict()
{
    if(dictTree.size() > 1)
    {
        dictTree.clear();
        dictTree.push_back(xml_node());
        dictTree.back() = doc.child("plist").child("dict");
    }
}

uint32_t NSDictionary::getNumKeys()
{
    uint32_t numKeys = 0;
    
    for(xml_node node = dictTree.back().first_child(); node; node = node.next_sibling().next_sibling())
    {
        numKeys++;
    }
    
    return numKeys;
}

string NSDictionary::getKey(uint32_t index)
{
    uint32_t count = 0;
    
    for(xml_node node = dictTree.back().first_child(); node; node = node.next_sibling().next_sibling())
    {
        if(count == index){ return node.child_value(); }
        count++;
    }
    
    return string();
}

vector<string> NSDictionary::getAllKeys()
{
    vector<string> strings;
    
    for(xml_node node = dictTree.back().first_child(); node; node = node.next_sibling().next_sibling())
    {
        strings.push_back(node.child_value());
    }
    
    return strings;
}

uint32_t NSDictionary::getIndexOfKey(const char* key)
{
    uint32_t count = 0;
    
    for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
    {
        if(node.child_value() == string(key))
        {
            return count;
        }
        count++;
    }
    
    return 0;
}

uint32_t NSDictionary::getIndexOfKeyWithClosestAlphaNumericalMatch(const char* key)
{
    uint32_t count = 0;
    
    for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
    {
        if(alphaNumericallyLessThan(key, node.child_value()))
        {
            return count;
        }
        count++;
    }
    
    return 0;
}

void NSDictionary::removeKey(uint32_t index)
{
    uint32_t count = 0;
    
    for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
    {
        if(count == index)
        {
            //Remove the node
            dictTree.back().remove_child(node.next_sibling());
            dictTree.back().remove_child(node);
            break;
        }
        count++;
    }
}

void NSDictionary::removeKey(const char* key)
{
    //Check to see if the value already exists withing the current sub dict
    for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
    {
        if(node.child_value() == string(key))
        {
            //Already exists, remove the node
            dictTree.back().remove_child(node.next_sibling());
            dictTree.back().remove_child(node);
            break;
        }
    }
}

void NSDictionary::removeAllKeys()
{
    while(xml_node node = dictTree.back().child("key"))
    {
        dictTree.back().remove_child(node.next_sibling());
        dictTree.back().remove_child(node);
    }
}

bool NSDictionary::hasObjectForKey(const char* key)
{
    for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
    {
        if(node.child_value() == string(key))
        {
            return true;
        }
    }
	
	return false;
}

int NSDictionary::getIntegerForKey(const char* key, int def)
{
    for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
    {
        if(node.child_value() == string(key) && (node.next_sibling() == node.next_sibling("integer") || node.next_sibling() == node.next_sibling("real")))
        {
            return strtol(node.next_sibling().child_value(), NULL, 10);
        }
    }
    
    //if(DS_DEBUG_MODE){ printf("%s \n",string("DS_ENGINE:> :ERROR: FAILED TO READ INTEGER VALUE FOR KEY: ").append(key).c_str()); }
    return def;
}

bool NSDictionary::getBoolForKey(const char* key, bool def)
{
    for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
    {
        if(node.child_value() == string(key) && ((node.next_sibling() == node.next_sibling("true") || node.next_sibling() == node.next_sibling("1")) || (node.next_sibling() == node.next_sibling("false") || node.next_sibling() == node.next_sibling("0"))))
        {
            if(node.next_sibling() == node.next_sibling("true") || node.next_sibling() == node.next_sibling("1"))
                return true;
            else
				return false;
        }
    }
    
    //if(DS_DEBUG_MODE){ printf("%s \n",string("DS_ENGINE:> :ERROR: FAILED TO READ BOOL VALUE FOR KEY: ").append(key).c_str()); }
    return def;
}

float NSDictionary::getFloatForKey(const char* key, float def)
{
    for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
    {
        if(node.child_value() == string(key) && (node.next_sibling() == node.next_sibling("integer") || node.next_sibling() == node.next_sibling("real")))
        {
            return strtod(node.next_sibling().child_value(), NULL);
        }
    }
    
    //if(DS_DEBUG_MODE){ printf("%s \n",string("DS_ENGINE:> :ERROR: FAILED TO READ FLOAT VALUE FOR KEY: ").append(key).c_str()); }
    return def;
}

string NSDictionary::getStringForKey(const char* key, const char* def)
{
    for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
    {
        if(node.child_value() == string(key) && node.next_sibling() == node.next_sibling("string"))
        {
            return node.next_sibling().child_value();
        }
    }
    
    //if(DS_DEBUG_MODE){ printf("%s \n",string("DS_ENGINE:> :ERROR: FAILED TO READ STRING VALUE FOR KEY: ").append(key).c_str()); }
    return def;
}

CXPoint NSDictionary::getPointForKey(const char* key, CXPoint def)
{
    for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
    {
        if(node.child_value() == string(key) && node.next_sibling() == node.next_sibling("string"))
        {
            CXPoint vec2;
            
            //return CXPointFromString(node.next_sibling().child_value());
            
            if(pointFromString(node.next_sibling().child_value(), vec2))
            {
                return vec2;
            }
            
            else
            {
                //if(DS_DEBUG_MODE){ printf("%s \n",string("DS_ENGINE:> :ERROR: MALFORMED VEC2 VALUE FOR KEY: ").append(key).c_str()); }
                return def;
            }
        }
    }
    
    //if(DS_DEBUG_MODE){ printf("%s \n",string("DS_ENGINE:> :ERROR: FAILED TO READ VEC2 VALUE FOR KEY: ").append(key).c_str()); }
    return def;
}

CXRect NSDictionary::getRectForKey(const char* key, CXRect def)
{
    for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
    {
        if(node.child_value() == string(key) && node.next_sibling() == node.next_sibling("string"))
        {
            CXRect rect;
            
            if(rectFromString(node.next_sibling().child_value(), rect))
            {
                return rect;
            }
            
            else
            {
                //if(DS_DEBUG_MODE){ printf("%s \n",string("DS_ENGINE:> :ERROR: MALFORMED RECT VALUE FOR KEY: ").append(key).c_str()); }
                return def;
            }
        }
    }
    
    //if(DS_DEBUG_MODE){ printf("%s \n",string("DS_ENGINE:> :ERROR: FAILED TO READ RECT VALUE FOR KEY: ").append(key).c_str()); }
    return def;
}

vector<int> NSDictionary::getIntArrayForKey(const char* key)
{
    for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
    {
        if(node.child_value() == string(key) && node.next_sibling() == node.next_sibling("array"))
        {
            vector<int> value;
            
            for(xml_node arrayNode = node.next_sibling().first_child(); arrayNode; arrayNode = arrayNode.next_sibling())
            {
				value.push_back(strtol(arrayNode.child_value(), NULL, 10));
            }
            
            return value;
        }
    }
    
    //if(DS_DEBUG_MODE){ printf("%s \n",string("DS_ENGINE:> :ERROR: FAILED TO READ STRING VALUE FOR ARRAY KEY: ").append(key).c_str()); }
    return vector<int>();
}

vector<string> NSDictionary::getStringArrayForKey(const char* key)
{
    for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
    {
        if(node.child_value() == string(key) && node.next_sibling() == node.next_sibling("array"))
        {
            vector<string> value;
            
            for(xml_node arrayNode = node.next_sibling().first_child(); arrayNode; arrayNode = arrayNode.next_sibling())
            {
                value.push_back(arrayNode.child_value());
            }
            
            return value;
        }
    }
    
    //if(DS_DEBUG_MODE){ printf("%s \n",string("DS_ENGINE:> :ERROR: FAILED TO READ STRING VALUE FOR ARRAY KEY: ").append(key).c_str()); }
    return vector<string>();
}

vector<CXPoint> NSDictionary::getPointArrayForKey(const char* key)
{
    vector<string> strings = getStringArrayForKey(key);
    vector<CXPoint> vectorVec2;
    
    for(int i=0; i<strings.size(); i++)
    {
        vectorVec2.push_back(CXZeroPoint);
        if(!pointFromString(strings[i], vectorVec2.back()))
        {
            //if(DS_DEBUG_MODE){ printf("%s \n",string("DS_ENGINE:> :ERROR: MALFORMED VEC2 VALUE FOR ARRAY KEY: ").append(key).c_str()); }
            vectorVec2.clear();
            break;
        }
    }
    
    return vectorVec2;
}

vector<CXRect> NSDictionary::getRectArrayForKey(const char* key)
{
    vector<string> strings = getStringArrayForKey(key);
    vector<CXRect> vectorRect;
    
    for(int i=0; i<strings.size(); i++)
    {
        vectorRect.push_back(CXZeroRect);
        if(!rectFromString(strings[i], vectorRect.back()))
        {
            //if(DS_DEBUG_MODE){ printf("%s \n",string("DS_ENGINE:> :ERROR: MALFORMED RECT VALUE FOR ARRAY KEY: ").append(key).c_str()); }
            vectorRect.clear();
            break;
        }
    }
    
    return vectorRect;
}

void NSDictionary::setDictForKey(const char* key, NSDictionaryPtr dict)
{
	//Check to see if the value already exists withing the current sub dict
	removeKey(key);

	//Create a key value pair as a child of the current sub dict
	xml_node keyNode;
	xml_node dictNode;
	bool appendKey = true;

	if(appendKey){ keyNode = dictTree.back().append_child("key"); }
	keyNode.append_child(node_pcdata).set_value(key);
	dictTree.back().insert_copy_after(dict->dictTree.back(), keyNode);

	return;
}

void NSDictionary::setSubDictForKey(const char* key){ setSubDictForKey(key, false); }
void NSDictionary::setSubDictForKey(const char* key, bool alphaNumericallySorted)
{
    //Check to see if the sub dict already exists withing the current sub dict
    removeKey(key);
    
    //Create a key value pair as a child of the current sub dict
    xml_node keyNode;
    bool appendKey = true;
    if(alphaNumericallySorted)
    {
        for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
        {
            if(!alphaNumericallyLessThan(node.child_value(), key))
            {
                keyNode = dictTree.back().insert_child_before("key", node);
                appendKey = false;
                break;
            }
        }
    }
    
    if(appendKey){ keyNode = dictTree.back().append_child("key"); }
    keyNode.append_child(node_pcdata).set_value(key);
    dictTree.back().insert_child_after("dict", keyNode);
}

void NSDictionary::setIntegerForKey(const char* key, int value){ setIntegerForKey(key, value, false); }
void NSDictionary::setIntegerForKey(const char* key, int value, bool alphaNumericallySorted)
{
    //Create a string from the input value
    stringstream stream;
    stream << value;
    
    //Check to see if the value already exists withing the current sub dict
    removeKey(key);
    
    //Create a key value pair as a child of the current sub dict
    xml_node keyNode;
    bool appendKey = true;
    if(alphaNumericallySorted)
    {
        for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
        {
            if(!alphaNumericallyLessThan(node.child_value(), key))
            {
                keyNode = dictTree.back().insert_child_before("key", node);
                appendKey = false;
                break;
            }
        }
    }
    
    if(appendKey){ keyNode = dictTree.back().append_child("key"); }
    keyNode.append_child(node_pcdata).set_value(key);
    dictTree.back().insert_child_after("integer", keyNode).append_child(node_pcdata).set_value(stream.str().c_str());
}

void NSDictionary::setBoolForKey(const char* key, bool value){ setBoolForKey(key, value, false); }
void NSDictionary::setBoolForKey(const char* key, bool value, bool alphaNumericallySorted)
{
    //Check to see if the value already exists withing the current sub dict
    removeKey(key);
    
    //Create a key value pair as a child of the current sub dict
    xml_node keyNode;
    bool appendKey = true;
    if(alphaNumericallySorted)
    {
        for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
        {
            if(!alphaNumericallyLessThan(node.child_value(), key))
            {
                keyNode = dictTree.back().insert_child_before("key", node);
                appendKey = false;
                break;
            }
        }
    }
    
    if(appendKey){ keyNode = dictTree.back().append_child("key"); }
    keyNode.append_child(node_pcdata).set_value(key);
    if(value){ dictTree.back().insert_child_after("true", keyNode); }
    else     { dictTree.back().insert_child_after("false", keyNode); }
}

void NSDictionary::setFloatForKey(const char* key, float value){ setFloatForKey(key, value, false); }
void NSDictionary::setFloatForKey(const char* key, float value, bool alphaNumericallySorted)
{
    //Create a string from the input value
    stringstream stream;
    stream << value;
    
    //Check to see if the value already exists withing the current sub dict
    removeKey(key);
    
    //Create a key value pair as a child of the current sub dict
    xml_node keyNode;
    bool appendKey = true;
    if(alphaNumericallySorted)
    {
        for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
        {
            if(!alphaNumericallyLessThan(node.child_value(), key))
            {
                keyNode = dictTree.back().insert_child_before("key", node);
                appendKey = false;
                break;
            }
        }
    }
    
    if(appendKey){ keyNode = dictTree.back().append_child("key"); }
    keyNode.append_child(node_pcdata).set_value(key);
    dictTree.back().insert_child_after("real", keyNode).append_child(node_pcdata).set_value(stream.str().c_str());
}

void NSDictionary::setStringForKey(const char* key, const string& value){ setStringForKey(key, value, false); }
void NSDictionary::setStringForKey(const char* key, const string& value, bool alphaNumericallySorted)
{
    //Check to see if the value already exists withing the current sub dict
    removeKey(key);
    
    //Create a key value pair as a child of the current sub dict
    xml_node keyNode;
    bool appendKey = true;
    if(alphaNumericallySorted)
    {
        for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
        {
            if(!alphaNumericallyLessThan(node.child_value(), key))
            {
                keyNode = dictTree.back().insert_child_before("key", node);
                appendKey = false;
                break;
            }
        }
    }
    
    if(appendKey){ keyNode = dictTree.back().append_child("key"); }
    keyNode.append_child(node_pcdata).set_value(key);
    dictTree.back().insert_child_after("string", keyNode).append_child(node_pcdata).set_value(value.c_str());
}

void NSDictionary::setPointForKey(const char* key, const CXPoint& value){ setPointForKey(key, value, false); }
void NSDictionary::setPointForKey(const char* key, const CXPoint& value, bool alphaNumericallySorted)
{
    //Create a string from the input value
    stringstream stream;
    stream << "{";
    stream << value.x;
    stream << ",";
    stream << value.y;
    stream << "}";
    
    //Check to see if the value already exists withing the current sub dict
    removeKey(key);
    
    //Create a key value pair as a child of the current sub dict
    xml_node keyNode;
    bool appendKey = true;
    if(alphaNumericallySorted)
    {
        for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
        {
            if(!alphaNumericallyLessThan(node.child_value(), key))
            {
                keyNode = dictTree.back().insert_child_before("key", node);
                appendKey = false;
                break;
            }
        }
    }
    
    if(appendKey){ keyNode = dictTree.back().append_child("key"); }
    keyNode.append_child(node_pcdata).set_value(key);
    dictTree.back().insert_child_after("string", keyNode).append_child(node_pcdata).set_value(stream.str().c_str());
}

void NSDictionary::setRectForKey(const char* key, const CXRect& value){ setRectForKey(key, value, false); }
void NSDictionary::setRectForKey(const char* key, const CXRect& value, bool alphaNumericallySorted)
{
    //Create a string from the input value
    stringstream stream;
    stream << "{{";
    stream << value.origin.x;
    stream << ",";
    stream << value.origin.y;
    stream << "},{";
    stream << value.size.width;
    stream << ",";
    stream << value.size.height;
    stream << "}}";
    
    //Check to see if the value already exists withing the current sub dict
    removeKey(key);
    
    //Create a key value pair as a child of the current sub dict
    xml_node keyNode;
    bool appendKey = true;
    if(alphaNumericallySorted)
    {
        for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
        {
            if(!alphaNumericallyLessThan(node.child_value(), key))
            {
                keyNode = dictTree.back().insert_child_before("key", node);
                appendKey = false;
                break;
            }
        }
    }
    
    if(appendKey){ keyNode = dictTree.back().append_child("key"); }
    keyNode.append_child(node_pcdata).set_value(key);
    dictTree.back().insert_child_after("string", keyNode).append_child(node_pcdata).set_value(stream.str().c_str());
}

vector<NSDictionaryPtr> NSDictionary::getDictArrayForKey(const char* key)
{
    for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
    {
        if(node.child_value() == string(key) && node.next_sibling() == node.next_sibling("array"))
        {
            vector<NSDictionaryPtr> value;
            
            for(xml_node arrayNode = node.next_sibling().first_child(); arrayNode; arrayNode = arrayNode.next_sibling())
            {
				NSDictionaryPtr dict(new NSDictionary());
				dict->dictTree.push_back(arrayNode);
                value.push_back(dict);
            }
            
            return value;
        }
    }
    
    //if(DS_DEBUG_MODE){ printf("%s \n",string("DS_ENGINE:> :ERROR: FAILED TO READ STRING VALUE FOR ARRAY KEY: ").append(key).c_str()); }
    return vector<NSDictionaryPtr>();
}

NSDictionaryPtr NSDictionary::getDictForKey(const char* key)
{
	//NSDictionaryPtr dict(new NSDictionary());
	NSDictionary* dict = new NSDictionary();
	
    for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
    {
        if(node.child_value() == string(key) && node.next_sibling() == node.next_sibling("dict"))
        {
			dict->doc.child("plist").remove_child("dict");
			dict->doc.child("plist").append_copy(node.next_sibling());
			
	        dict->dictTree.clear();
	        dict->dictTree.push_back(xml_node());
	        dict->dictTree.back() = dict->doc.child("plist").child("dict");
			
			return NSDictionaryPtr(dict);
        }
    }
	
	return NSDictionaryPtr(dict);
}

void NSDictionary::setDictArrayForKey(const char* key, const vector<NSDictionaryPtr>& value)
{
    //Check to see if the value already exists withing the current sub dict
    removeKey(key);
    
    //Create a key value pair as a child of the current sub dict
    xml_node keyNode;
    xml_node arrayNode;
    bool appendKey = true;
    
    if(appendKey){ keyNode = dictTree.back().append_child("key"); }
    keyNode.append_child(node_pcdata).set_value(key);
    arrayNode = dictTree.back().insert_child_after("array", keyNode);
    
    //Fill the array
    for(int i=0; i<value.size(); i++)
    {
		NSDictionaryPtr dict = value[i];
		// char_t* heap_buf = dict->doc.xml_document_as_buffer();
		// arrayNode.append_child("string").append_child(node_pcdata).set_value(heap_buf);
		// delete[] heap_buf;
		//arrayNode.append_child("dict").append_copy(dict->dictTree.back());
		arrayNode.append_copy(dict->dictTree.back());
    }
	
	// char* heap_buf = doc.node_to_buffer_heap(arrayNode);
	// //__android_log_print(ANDROID_LOG_VERBOSE, "APPNAME", "The value of 1 + 1 is %d", 1+1);
	// __android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, "------------------------BEGIN------------------------");
	// __android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, heap_buf);
	// __android_log_print(ANDROID_LOG_VERBOSE, DICT_TAG, "-------------------------END-------------------------");
	// delete[] heap_buf;
}

void NSDictionary::setIntArrayForKey(const char* key, const std::vector<int>& value){ setIntArrayForKey(key, value, false); }
void NSDictionary::setIntArrayForKey(const char* key, const std::vector<int>& value, bool alphaNumericallySorted)
{
    //Check to see if the value already exists withing the current sub dict
    removeKey(key);
    
    //Create a key value pair as a child of the current sub dict
    xml_node keyNode;
    xml_node arrayNode;
    bool appendKey = true;
    if(alphaNumericallySorted)
    {
        for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
        {
            if(!alphaNumericallyLessThan(node.child_value(), key))
            {
                keyNode = dictTree.back().insert_child_before("key", node);
                appendKey = false;
                break;
            }
        }
    }
    
    if(appendKey){ keyNode = dictTree.back().append_child("key"); }
    keyNode.append_child(node_pcdata).set_value(key);
    arrayNode = dictTree.back().insert_child_after("array", keyNode);
    
    //Fill the array
    for(int i=0; i<value.size(); i++)
    {
	    stringstream stream;
	    stream << value[i];
        arrayNode.append_child("integer").append_child(node_pcdata).set_value(stream.str().c_str());
    }
}

void NSDictionary::setStringArrayForKey(const char* key, const vector<string>& value){ setStringArrayForKey(key, value, false); }
void NSDictionary::setStringArrayForKey(const char* key, const vector<string>& value, bool alphaNumericallySorted)
{
    //Check to see if the value already exists withing the current sub dict
    removeKey(key);
    
    //Create a key value pair as a child of the current sub dict
    xml_node keyNode;
    xml_node arrayNode;
    bool appendKey = true;
    if(alphaNumericallySorted)
    {
        for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
        {
            if(!alphaNumericallyLessThan(node.child_value(), key))
            {
                keyNode = dictTree.back().insert_child_before("key", node);
                appendKey = false;
                break;
            }
        }
    }
    
    if(appendKey){ keyNode = dictTree.back().append_child("key"); }
    keyNode.append_child(node_pcdata).set_value(key);
    arrayNode = dictTree.back().insert_child_after("array", keyNode);
    
    //Fill the array
    for(int i=0; i<value.size(); i++)
    {
        arrayNode.append_child("string").append_child(node_pcdata).set_value(value[i].c_str());
    }
}

void NSDictionary::setPointArrayForKey(const char* key, const vector<CXPoint>& value){ setPointArrayForKey(key, value, false); }
void NSDictionary::setPointArrayForKey(const char* key, const vector<CXPoint>& value, bool alphaNumericallySorted)
{
    //Check to see if the value already exists withing the current sub dict
    removeKey(key);
    
    //Create a key value pair as a child of the current sub dict
    xml_node keyNode;
    xml_node arrayNode;
    bool appendKey = true;
    if(alphaNumericallySorted)
    {
        for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
        {
            if(!alphaNumericallyLessThan(node.child_value(), key))
            {
                keyNode = dictTree.back().insert_child_before("key", node);
                appendKey = false;
                break;
            }
        }
    }
    
    if(appendKey){ keyNode = dictTree.back().append_child("key"); }
    keyNode.append_child(node_pcdata).set_value(key);
    arrayNode = dictTree.back().insert_child_after("array", keyNode);
    
    //Fill the array
    for(int i=0; i<value.size(); i++)
    {
        //Create a string from the input value
        stringstream stream;
        stream << "{";
        stream << value[i].x;
        stream << ",";
        stream << value[i].y;
        stream << "}";
        
        arrayNode.append_child("string").append_child(node_pcdata).set_value(stream.str().c_str());
    }
}

void NSDictionary::setRectArrayForKey(const char* key, const vector<CXRect>& value){ setRectArrayForKey(key, value, false); }
void NSDictionary::setRectArrayForKey(const char* key, const vector<CXRect>& value, bool alphaNumericallySorted)
{
    //Check to see if the value already exists withing the current sub dict
    removeKey(key);
    
    //Create a key value pair as a child of the current sub dict
    xml_node keyNode;
    xml_node arrayNode;
    bool appendKey = true;
    if(alphaNumericallySorted)
    {
        for(xml_node node = dictTree.back().child("key"); node; node = node.next_sibling("key"))
        {
            if(!alphaNumericallyLessThan(node.child_value(), key))
            {
                keyNode = dictTree.back().insert_child_before("key", node);
                appendKey = false;
                break;
            }
        }
    }
    
    if(appendKey){ keyNode = dictTree.back().append_child("key"); }
    keyNode.append_child(node_pcdata).set_value(key);
    arrayNode = dictTree.back().insert_child_after("array", keyNode);
    
    //Fill the array
    for(int i=0; i<value.size(); i++)
    {
        //Create a string from the input value
        stringstream stream;
        stream << "{{";
        stream << value[i].origin.x;
        stream << ",";
        stream << value[i].origin.y;
        stream << "},{";
        stream << value[i].size.width;
        stream << ",";
        stream << value[i].size.height;
        stream << "}}";
        
        arrayNode.append_child("string").append_child(node_pcdata).set_value(stream.str().c_str());
    }
}