#ifndef NSDictionary_h
#define NSDictionary_h

#include "pugixml.hpp"
#include <vector>
//#include "cocos2d.h"
#include "CXMacDataTypes.h"

#include <tr1/memory>
#include <tr1/shared_ptr.h>

class NSDictionary;
typedef std::tr1::shared_ptr< NSDictionary > NSDictionaryPtr;

class AAssetManager;

using namespace pugi;
//using namespace std;

class NSDictionary
{
private:
	//Data
	xml_document doc;
	std::vector<xml_node> dictTree;

	//Constructor blockers
	NSDictionary(const NSDictionary& dictionary);                //Copy constructor - blocker
	NSDictionary& operator= (const NSDictionary& dictionary);    //Assignment - blocker

	//Methods
	std::string cleanStringWhiteSpace(const std::string& _str);
	void split(const std::string& src, const char* token, std::vector<std::string>& vect);
	bool splitWithForm(const std::string& content, std::vector<std::string>& strs);
	bool rectFromString(const std::string& pszContent, CXRect& rect);
	bool pointFromString(const std::string& pszContent, CXPoint& vec2);

	const char* getFileData(const char* pszFileName, const char* pszMode, off_t& pSize, AAssetManager* assetMgr);
	
	void initBlankDict();
    
public:
	//Constructor
	NSDictionary();
	
	void dump();

	//Methods
	bool loadRootSubDictFromData(const char* data, off_t dataSize);
	bool loadRootSubDictFromFile(const char* fileName);
	bool loadRootSubDictFromFile(const char* fileName, AAssetManager* assetMgr);
	bool saveRootSubDictToFile(const char* fileName);
	bool stepIntoSubDictWithKey(const char* key);
	void stepOutOfSubDict();
	void stepBackToRootSubDict();
	
	char* getDictAsData();

	uint32_t getNumKeys();
	std::string getKey(uint32_t index);
	std::vector<std::string> getAllKeys();
	uint32_t getIndexOfKey(const char* key);
	uint32_t getIndexOfKeyWithClosestAlphaNumericalMatch(const char* key);

	void removeKey(uint32_t index);
	void removeKey(const char* key);
	void removeAllKeys();

	bool hasObjectForKey(const char* key);

	int getIntegerForKey(const char* key, int def = 0);
	bool getBoolForKey(const char* key, bool def = false);
	float getFloatForKey(const char* key, float def = 0);
	std::string getStringForKey(const char* key, const char* def = "");
	CXPoint getPointForKey(const char* key, CXPoint def = CXZeroPoint);
	CXRect getRectForKey(const char* key, CXRect def = CXZeroRect);
	NSDictionaryPtr getDictForKey(const char* key);
	std::vector<NSDictionaryPtr> getDictArrayForKey(const char* key);
	std::vector<int> getIntArrayForKey(const char* key);
	std::vector<std::string> getStringArrayForKey(const char* key);
	std::vector<CXPoint> getPointArrayForKey(const char* key);
	std::vector<CXRect> getRectArrayForKey(const char* key);
	//void getValueForKey(const char* key);

	void setDictForKey(const char* key, NSDictionaryPtr dict);
	void setSubDictForKey(const char* key);
	void setSubDictForKey(const char* key, bool alphaNumericallySorted);
	void setIntegerForKey(const char* key, int value);
	void setIntegerForKey(const char* key, int value, bool alphaNumericallySorted);
	void setBoolForKey(const char* key, bool value);
	void setBoolForKey(const char* key, bool value, bool alphaNumericallySorted);
	void setFloatForKey(const char* key, float value);
	void setFloatForKey(const char* key, float value, bool alphaNumericallySorted);
	void setStringForKey(const char* key, const std::string& value);
	void setStringForKey(const char* key, const std::string& value, bool alphaNumericallySorted);
	void setPointForKey(const char* key, const CXPoint& value);
	void setPointForKey(const char* key, const CXPoint& value, bool alphaNumericallySorted);
	void setRectForKey(const char* key, const CXRect& value);
	void setRectForKey(const char* key, const CXRect& value, bool alphaNumericallySorted);
	void setDictArrayForKey(const char* key, const std::vector<NSDictionaryPtr>& value);
	void setIntArrayForKey(const char* key, const std::vector<int>& value);
	void setIntArrayForKey(const char* key, const std::vector<int>& value, bool alphaNumericallySorted);
	void setStringArrayForKey(const char* key, const std::vector<std::string>& value);
	void setStringArrayForKey(const char* key, const std::vector<std::string>& value, bool alphaNumericallySorted);
	void setPointArrayForKey(const char* key, const std::vector<CXPoint>& value);
	void setPointArrayForKey(const char* key, const std::vector<CXPoint>& value, bool alphaNumericallySorted);
	void setRectArrayForKey(const char* key, const std::vector<CXRect>& value);
	void setRectArrayForKey(const char* key, const std::vector<CXRect>& value, bool alphaNumericallySorted);
};

#endif