#ifndef __OgreScene_H__
#define __OgreScene_H__

#include "CommonDef.h"
#include "CShortLineModel.h"
#include "CListener.h"
#include "SdkTrays.h"

#include <android/asset_manager.h>

#include "Ogre.h"
//#include "OgreOverlaySystem.h"
//#include <iostream>

//#include "InputContext.h"
//#include "OgreFileSystemLayer.h"

#ifdef USE_RTSHADER_SYSTEM
#	include "OgreRTShaderSystem.h"
#endif //USE_RTSHADER_SYSTEM

#include "NSDictionary.h"
#include "android_apk.h"
#include "android_progress.h"

//#define MODEL_SCALE_X (0.52)
//#define MODEL_SCALE_Y (0.52)
//#define MODEL_SCALE_Z (0.52)
//#define SMOKE_SCALE (0.01)

using namespace Ogre;
using namespace OgreBites;

//class CTrain, CCar, CSemaphore, CTree, CCity, CRail, CSwitch, CCancelButton;

#define NODES_WIDTH (DESK_WIDTH+2)
#define NODES_HEIGHT (DESK_HEIGHT+1)

class COgreScene : public Ogre::GeneralAllocatedObject, public SdkTrayListener
{
private:
	
	COgreScene()
	{
		mRoot = Ogre::Root::getSingletonPtr();
		mWindow = 0;
		mSceneMgr = 0;
		mResourcesLoaded = false;
		mContentSetup = false;
		
		mLastTutorialMessage = "";

		switchesBillSet = NULL;
		frontCitiesBillSet = NULL;
		cityAdditionsBillSet = NULL;
		treesBillSet = NULL;
		nodesBillSet = NULL;
		railsBillSet = NULL;
		unfinishedRailsBillSet = NULL;
		tempRailsBillSet = NULL;
		highlightedRailsBillSet = NULL;
		semaphoresBillSet = NULL;
		cloudsBillSet = NULL;
		bubblesBillSet = NULL;
		explosionBillSet = NULL;
		cancelButtonBillSet = NULL;
		
		mSceneLight = NULL;
		
		mSoundButton = NULL;
		mRestartButton = NULL;
		mBoostButton = NULL;
		mBonusButton0 = NULL;
		mBonusButton1 = NULL;
		mBonusButton2 = NULL;
		mPauseButton = NULL;
		mMenuButton = NULL;
		mYearLabel = NULL;
		mYearCaption = NULL;
		mLevelLabel = NULL;
		mTrainsMoneyLabel = NULL;
		
		mCommonMessagePanel = NULL;
		
		mMoneyLabelCaption = NULL;
		mTrainsLabelCaption = NULL;
		mLevelLabelCaption = NULL;
		
		mMoneyTrainsBack = NULL;
		mTopTab = NULL;
		mDemoLabel = NULL;
		
		mRightPanel = NULL;
		mLeftPanel = NULL;
		mRPWSlider = NULL;
		mLPWSlider = NULL;
		
		_gamePaused = true;
		
		mViewport = NULL;
		mCamera = NULL;
		mTrayMgr = NULL;
		mPauseTrayMgr = NULL;
		mPrelevelTrayMgr = NULL;
		mFaildLevelTrayMgr = NULL;
		mCompleteLevelTrayMgr = NULL;
		
		//mLevelCompleteInfo = NULL;
		
		mPrelevelTrainsBox = NULL;
		
		mProgressBarThumb = NULL;
		
		//#ifdef RTT_BACKGROUND
		m_pRTSceneMgr = NULL;
		m_pRailsTextureCamera = NULL;
		m_pRenderTexture = NULL;
		m_pMiniScreen = NULL;
		//#endif
		
		mPSWeather = NULL;
	}
	
	
public:
	
	static COgreScene* getShared()
	{
		static COgreScene* _ogreScene = NULL;
		if (!_ogreScene)
		{
			_ogreScene = new COgreScene();
		}
		
		return _ogreScene;
	}
	
	virtual ~COgreScene() 
	{
		// delete _ogreScene;
		// _ogreScene = NULL;
	}
	
	virtual void _shutdown()
	{
		restartGame();
		
		// if (mLevelCompleteInfo)
		// {
		// 	delete mLevelCompleteInfo;
		// 	mLevelCompleteInfo = NULL;
		// }
		
		if (mRightPanel)
		{
			Widget::nukeOverlayElement(mRightPanel);
			mRightPanel = NULL;
		}
		
		if (mLeftPanel)
		{
			Widget::nukeOverlayElement(mLeftPanel);
			mLeftPanel = NULL;
		}
		
		if (mTrayMgr)
		{
			delete mTrayMgr;
			mTrayMgr = NULL;
		}
				
		if (mSceneLight)
		{
			mSceneMgr->destroyLight(mSceneLight);
			mSceneLight = NULL;
		}
		
		if (switchesBillSet)
			destroySceneBillboardSet(switchesBillSet, "SWITCHES_SCENE_NODE");
		if (frontCitiesBillSet)
			destroySceneBillboardSet(frontCitiesBillSet, "FRONT_CITIES_SCENE_NODE");
		if (cityAdditionsBillSet)
			destroySceneBillboardSet(cityAdditionsBillSet, "SWITCHES_SCENE_NODE");
		if (treesBillSet)
			destroySceneBillboardSet(treesBillSet, "CITY_ADDITIONS_SCENE_NODE");
		if (nodesBillSet)
			destroySceneBillboardSet(nodesBillSet, "NODES_SCENE_NODE");
		if (railsBillSet)
			destroySceneBillboardSet(railsBillSet, "RAILS_SCENE_NODE");
		if (unfinishedRailsBillSet)
			destroySceneBillboardSet(unfinishedRailsBillSet, "UNFINISHED_RAILS_SCENE_NODE");
		if (tempRailsBillSet)
			destroySceneBillboardSet(tempRailsBillSet, "TEMP_RAILS_SCENE_NODE");
		//if (highlightedRailsBillSet);//!!!!!!!!!! using one billboard for unfinished and highlited rails
		if (semaphoresBillSet)
			destroySceneBillboardSet(semaphoresBillSet, "SEMAPHORES_SCENE_NODE");
		if (cloudsBillSet)
			destroySceneBillboardSet(cloudsBillSet, "CLOUDS_SCENE_NODE");
		if (bubblesBillSet)
			destroySceneBillboardSet(bubblesBillSet, "BUBBLES_SCENE_NODE");
		if (explosionBillSet)
			destroySceneBillboardSet(explosionBillSet, "EXPLOSION_SCENE_NODE");
		if (cancelButtonBillSet)
			destroySceneBillboardSet(cancelButtonBillSet, "CANCEL_BUTTON_SCENE_NODE");
		
		if (mSceneMgr->hasSceneNode("BACKGROUND"))
			destroySceneBillboardSet(NULL, "BACKGROUND");
		if (mSceneMgr->hasSceneNode("BACKGROUND_GRID"))
			destroySceneBillboardSet(NULL, "BACKGROUND_GRID");
		if (mSceneMgr->hasSceneNode("RED_GAME_COORD"))
			destroySceneBillboardSet(NULL, "RED_GAME_COORD");
		
		mSceneLight = NULL;
		mRestartButton = NULL;
		mBoostButton = NULL;
		mBonusButton0 = NULL;
		mBonusButton1 = NULL;
		mBonusButton2 = NULL;
		mPauseButton = NULL;
		mMenuButton = NULL;
		mYearLabel = NULL;
		mYearCaption = NULL;
		mLevelLabel = NULL;
		mTrainsMoneyLabel = NULL;
		
		mCommonMessagePanel = NULL;
		
		mMoneyLabelCaption = NULL;
		mTrainsLabelCaption = NULL;
		mLevelLabelCaption = NULL;
		
		mMoneyTrainsBack = NULL;
		mTopTab = NULL;
		mDemoLabel = NULL;
		
		mRPWSlider = NULL;
		mLPWSlider = NULL;
		
		if (mContentSetup)
            cleanupContent();
		if (mSceneMgr)
            mSceneMgr->clearScene();
		mContentSetup = false;

		if (mResourcesLoaded)
            unloadResources();
		mResourcesLoaded = false;
		if (mSceneMgr) 
		{
#ifdef USE_RTSHADER_SYSTEM
			mShaderGenerator->removeSceneManager(mSceneMgr);
#endif
			mSceneMgr->removeRenderQueueListener(mOverlaySystem);
			mRoot->destroySceneManager(mSceneMgr);				
		}
		mSceneMgr = 0;
	}
	
	void destroySceneBillboardSet(Ogre::BillboardSet* bs, const char* sceneNodeName = NULL)
	{
		if (sceneNodeName && mSceneMgr->hasSceneNode(sceneNodeName))
		{
			SceneNode* sn = mSceneMgr->getSceneNode(sceneNodeName);
			sn->detachAllObjects();
			sn->removeAndDestroyAllChildren();
			mSceneMgr->destroySceneNode(sn);
		}
		
		if (bs)
			mSceneMgr->destroyBillboardSet(bs);
		bs = NULL;
	}
	
	virtual void setup(Ogre::RenderWindow* window, InputContext inputContext, Ogre::OverlaySystem* overlaySys)
	{
		jShowProgress();
		mRoot = Ogre::Root::getSingletonPtr();
		mWindow = window;
		mInputContext = inputContext;
		mOverlaySystem = overlaySys;
		
		createSceneManager();
		setupView();
		
		mTrayMgr = new SdkTrayManager("SceneControls", window, inputContext, this);  // create a tray interface
		
		locateResources();
		
		mPauseTrayMgr = new SdkTrayManager("PauseDialodControls", window, inputContext, this);
		mPrelevelTrayMgr = new SdkTrayManager("PrelevelControls", window, inputContext, this);
		mFaildLevelTrayMgr = new SdkTrayManager("FaildLevelControls", window, inputContext, this);
		mCompleteLevelTrayMgr = new SdkTrayManager("CompleteLevelControls", window, inputContext, this);
		
		loadResources();
		mResourcesLoaded = true;
		
#ifdef USE_RTSHADER_SYSTEM
            Ogre::Viewport* mainVP = mCamera->getViewport();
            if(mRoot->getRenderSystem()->getCapabilities()->hasCapability(Ogre::RSC_FIXED_FUNCTION) == false)
            {
                mainVP->setMaterialScheme(Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
            }
#endif

		setupContent();
		mContentSetup = true;
		jHideProgress();
	}
	
	//--------------------------------------
	void showPrelevelPanel();
	void preparePrelevelView();
	void hidePrelevelPanel();
	void showLevelFailedPanel();
	void showLevelCompletePanel();
	void prepareLevelCompleteViewInfo();
	void eraseLevelCompleteViewInfo();
	
	void showPauseDialog();
	void setupPauseDialog();
	void setupFaildLevelDialog();
	void setupCompleteLevelDialog();
	void setupPrelevelDialog();
	
	//--------------------------------------
		
	void hideAllTrays()
	{
		if (mTrayMgr)
			mTrayMgr->hideAll();
		if (mPauseTrayMgr)
			mPauseTrayMgr->hideAll();
		if (mPrelevelTrayMgr)
			mPrelevelTrayMgr->hideAll();
		if (mFaildLevelTrayMgr)
			mFaildLevelTrayMgr->hideAll();
		if (mCompleteLevelTrayMgr)
			mCompleteLevelTrayMgr->hideAll();
	}
	
#ifdef USE_RTSHADER_SYSTEM
	void setShaderGenerator(Ogre::RTShader::ShaderGenerator* shaderGenerator) 
	{ 
		mShaderGenerator = shaderGenerator;
	}
#endif
		
	void setupNormalLevel();
	void setupDemoLevel();
	void setupTutorialLevel();
	void updateBoostButton();
	
	void onPressPauseButton();
		
	virtual void pause();
	virtual void unpause(bool restart = false);
	virtual bool isPaused();
	
	virtual bool isLevelCompleteOrFailedViewActive();
	virtual bool isPrelevelViewActive();
	
	/////////////////////////////////////////////// Load/Save ///////////////////////////////////////////////
	virtual void startTutorialLevel(int levelIndex = 1);
	virtual void startDemoLevel();
	virtual void startLevelWithIndex(int levelIndex, bool showPrelevelView = false);
	virtual void startGameAtNextLevel();
	virtual void continueFromTheLastSnapshot();
	virtual void writeSnapshot();
	virtual void saveState(Ogre::NameValuePairList& state) {}
	virtual void restoreState(Ogre::NameValuePairList& state) {}
	/////////////////////////////////////////////// Rendering ///////////////////////////////////////////////
	virtual bool frameStarted(const Ogre::FrameEvent& evt) { return true; }
	virtual bool frameRenderingQueued(const Ogre::FrameEvent& evt);
	virtual bool frameEnded(const Ogre::FrameEvent& evt) { return true; }
	/////////////////////////////////////////////// Window Events ///////////////////////////////////////////
	virtual void windowMoved(Ogre::RenderWindow* rw) {}
	virtual void windowResized(Ogre::RenderWindow* rw) {}
	virtual bool windowClosing(Ogre::RenderWindow* rw) { return true; }
	virtual void windowClosed(Ogre::RenderWindow* rw) {}
	virtual void windowFocusChange(Ogre::RenderWindow* rw) {}
	/////////////////////////////////////////////// Keyboerd Events /////////////////////////////////////////
	virtual bool keyPressed(const OIS::KeyEvent& evt) { return true; }
	virtual bool keyReleased(const OIS::KeyEvent& evt) { return true; }
	/////////////////////////////////////////////// Touches /////////////////////////////////////////////////
	virtual bool touchMoved(const OIS::MultiTouchEvent& evt);
	virtual bool touchPressed(const OIS::MultiTouchEvent& evt);
	virtual bool touchReleased(const OIS::MultiTouchEvent& evt);
	virtual bool touchCancelled(const OIS::MultiTouchEvent& evt);
	
	// virtual bool needsUseLowResTextures()
	// {
	// 	return mWindow->getViewport(0)->getActualHeight() < 1024;
	// }
	
protected:
	
	virtual void locateResources();
	
	virtual void loadResources()
	{
		Ogre::ResourceGroupManager::getSingleton().initialiseResourceGroup("GeneralSceneResources");
		Ogre::ResourceGroupManager::getSingleton().initialiseResourceGroup(jNeedsUseLowResTextures() ? "Mid_Textures" : "Large_Textures");
		
		// if (mTrayMgr)
		// 	mTrayMgr->hideLoadingBar();
	}
	
	virtual void unloadResources()
	{
		Ogre::ResourceGroupManager::ResourceManagerIterator resMgrs =
		Ogre::ResourceGroupManager::getSingleton().getResourceManagerIterator();

		while (resMgrs.hasMoreElements())
		{
			resMgrs.getNext()->unloadUnreferencedResources();
		}
	}	
	
	virtual void createSceneManager()
	{
		mSceneMgr = Ogre::Root::getSingleton().createSceneManager(Ogre::ST_GENERIC);
#ifdef USE_RTSHADER_SYSTEM
		mShaderGenerator->addSceneManager(mSceneMgr);
#endif
        if(mOverlaySystem)
            mSceneMgr->addRenderQueueListener(mOverlaySystem);
	}
	
	virtual void setupView();
		
	void setupContent()
	{
		setupViewport();
		setupControls();
		setupScene();
		
		initVars();
		
		CListener::sharedListener()->setOgreScene(this);
	}
	
	virtual void cleanupContent() {}
	
public:
	// --- Trains
	void pushNewTrain(CTrain* newTrain);
	void eraseTheTrain(TrainIterator trainIt);
	void removeAllTrains();
	void removeCar(CTrain* train, CCar* car);
	void updateTrainColor(CTrain* train);
    void refreshColorOfTrain(CTrain* train);
	
    void addWaitingTrain(CTrain* train, CCity* city);
	
	// --- Semaphores
	void updateSemaphore(CSemaphore* sem);
	void addSemaphore(CSemaphore* semaphore);
	void eraseSemaphore(SemaphoreIterator semaphoreIt);
	void removeAllSemaphores();
	
	// --- Trees
	void addTree(CTree* tree);
	void eraseTree(TreeIterator treeIt);
	void removeAllTrees();
	
	// --- Cities
	void addCity(CCity* city);
	void updateCity(CCity* city);
    void updateBusyOfCity(CCity* city);
	bool isHasCity(CCity* city);
	void eraseCity(CCity* city);
	void removeAllCities();
	
	// --- Rails
	void updateSceneNearTheRail(CRail* rail);
	void updateSceneInCoords(CXPoint texCoord, CXPoint coord);
	void addRail(CRail* rail);
	void removeAllRails();
	void clearAllRails();
	
	// --- FutureCitiesRails
	void addFutureCityRail(CRail* rail);
	void eraseFutureCityRail(CRail* rail);
	void removeAllFutureCityRails();
	
	// --- UnfinishedRails
	void addUnfinishedRail(CRail* rail);
	void eraseUnfinishedRail(RailIterator railIt);
	void removeAllUnfinishedRails();
	
	// --- TempRails
	void addTempRail(CRail* rail);
	void eraseTempRail(RailIterator railIt);
	void removeAllTempRails();
	
	// --- Switch
	void switched(CSwitch* sw);
	void changeSwitchBlock(CSwitch* sw);
	
	void addSwitch(CSwitch* sw);
	void eraseSwitch(SwitchIterator switchIt);
	void removeAllSwitches();
    
	// --- Cancel Buttons
    void addCancelButton(CCancelButton* button);
    void removeCancelButton(CCancelButton* button);
	
	
	// --- Other
	void restartGame();
	
	void generateExplosion(CXPoint coord);
	void removeAllExplosions();
	
	void putFAMessage(CXPoint pos, Ogre::String message, bool isAlert = false, bool isUpMoving = true);
	void putFlyAwayMessage(Ogre::Vector3 pos, Ogre::String message, bool isAlert = false, Ogre::Real height = 1., bool isUpMoving = true);
	
	void removeAllFlyAwayMessages();
	
	void removeAllHighlightedRails();
	void setNeedsToHighlight(CRail* rail, bool flag);
	void cancelBuildingRails();
	void setRedGameCoord(CXPoint coord);
	
	void showHintNodes();
    void hideHintNodes();
	
	void updateSceneWithLevel(int level);
	
private:
	
	bool isRightOrientedInterface();
		
	////////////////////////////////////////// From ShortLineView ///////////////////////////////////////////
	
	CXRect viewBounds();
	CXRect boundsForDrawing();
	CXPoint convertFromGameCoord(CXPoint gameCoord);
	CXPoint convertToGameCoord(CXPoint viewPoint);
	void calculateBorderLineForRail(CRail* rail, CXPoint gameCoord);
	CXPoint forwardCoordOfChainRails();
	CXPoint backwardCoordOfChainRails();
	int possibleRailIndexForGameCoord(CXPoint gameCoord);
	bool detectIntersectForGameCoord(CXPoint gameCoord, CXPoint mainCoord, CXPoint coord1, CXPoint coord2);
	bool isEmptyPossibleRails();
	void clearRailVector(RailVector* pRailVector);
	bool addToChainRail(CRail* rail);
	void clearPossibleRailsArray();
	bool tryFillPossibleRailsArrayWithCenter(CXPoint nearestNodeGameCoord);
	void removeHighlightedFlagForAllRails();
	BOOL tryClickAvailableElementForCoord(CXPoint coord);
	BOOL tryClickBlockedElementForCoord(CXPoint coord);
	
	float xGap;
	float yGap;
	CXPoint _customGap;
	double _topOffset;
	
	int _possibleRailIndex;
	
	CXPoint _touchBeganPoint;
	CXPoint _nearestNodeGameCoord;
	
	BOOL _willBuildRail;
	
	CRail* _possibleRails[12];
	float  _distanceSquaresToRail[12];
	
	/*новая версия*/
	CXPoint _detectPoints[12];
	float	_distancesToEdges[12];
    CXPoint _detectCenter;
	
	/*новый интерфейс*/
	CXPoint _redPoint;
	RailVector _nextPossibleRails;
	RailVector _chainPossibleRails;
    
	BOOL _inRedCircle;
	
	CXPoint _borderGameCoord1;
	CXPoint _borderGameCoord2;
	
	CXPoint _backBorderGameCoord1;
	CXPoint _backBorderGameCoord2;
	
	CXPoint _redGameCoord;
	
	//////////////////////////////////////////// Base Scene /////////////////////////////////////////////////
	
	void setupViewport();
	void setupScene();
	void nextMainLoopStep(bool ignorePausedState = false);
	void updateTutorialMessage();
	void updateScene(double timeSinceLastFrame);
	void initVars();
	void refreshSceneObjects();
	
	//////////////////////////////////////////// Game Scene /////////////////////////////////////////////////

	void updateBonusButton(OgreBites::GameButton* bb, CBonusButton* cbb);
	void bonusButtonClick(CBonusButton* clickedBB);
	
	void setupControls();
	void sliderMoved(Slider* slider);
	void buttonHit(Button* button);
	
	//--------------------------
		
	void updateSceneWithTag(const Ogre::String& tagName);
	
	void applyBackgroundTexture(const Ogre::String& matName);
	void applyRailsTexture(const Ogre::String& matName);
	void applyNodesTexture(const Ogre::String& matName);
	void applyUnfinishedRailsTexture(const Ogre::String& matName);
    void applyTreesTexture(const Ogre::String& matName);
    void applyCitiesTexture(const Ogre::String& matName);
    
	//--------------------------
	
	double mLastFrameTime;
    double mStartTime;
	
	bool _gamePaused;
	
	Ogre::BillboardSet* switchesBillSet;
	Ogre::BillboardSet* frontCitiesBillSet;
	Ogre::BillboardSet* cityAdditionsBillSet;
	Ogre::BillboardSet* treesBillSet;
	Ogre::BillboardSet* nodesBillSet;
	Ogre::BillboardSet* railsBillSet;
	Ogre::BillboardSet* unfinishedRailsBillSet;
	Ogre::BillboardSet* tempRailsBillSet;
	Ogre::BillboardSet* highlightedRailsBillSet;
	Ogre::BillboardSet* semaphoresBillSet;
	Ogre::BillboardSet* cloudsBillSet;
	Ogre::BillboardSet* bubblesBillSet;
	Ogre::BillboardSet* explosionBillSet;
    Ogre::BillboardSet* cancelButtonBillSet;
	
	Ogre::Entity* getTrainEntity(CTrain* train);
	Ogre::Entity* getVagonEntity(CCar* car, CTrain* train);
	void syncTrainColor(Ogre::Entity* trainEntity, CTrain* train);
	void syncVagonColor(Ogre::Entity* vagonEntity, CCar* car, CTrain* train);
	
	///////////////////////////////////////////////// Local Types /////////////////////////////////////////////////
	
	typedef struct _TEN
	{
		_TEN(Ogre::Entity* _entity, Ogre::SceneNode* _node) : entity(_entity), node(_node) {};
		
		Ogre::Entity*		entity;
		Ogre::SceneNode*	node;
	} TEN;
	
	typedef std::vector<TEN> TENVector;
	typedef std::vector<TEN>::iterator TENIterator;
	
	typedef struct _TTrain
	{
		_TTrain(CTrain* _train, Ogre::Entity* _entity, Ogre::SceneNode* _node, CityColor _color = CITY_COLOR_BLACK, Ogre::Billboard* _bubbleBill = NULL, Ogre::SceneNode* _captionNode = NULL, Ogre::AnimationState* _animState = NULL) : train(_train), node(_node), entity(_entity), color(_color), bubbleBill(_bubbleBill), captionNode(_captionNode), animState(_animState) {};
		
		CTrain*				train;
		Ogre::Entity*		entity;
		Ogre::SceneNode*	node;
		Ogre::SceneNode*	captionNode;
		Ogre::Billboard*	bubbleBill;
		Ogre::AnimationState* animState;
		CityColor			color;
		TENVector			cars;
	} TTrain;
	
	typedef std::vector<TTrain> TTrainVector;
	typedef std::vector<TTrain>::iterator TTrainIterator;
	
	typedef struct _TSemaphore
	{
		_TSemaphore(CSemaphore* _semaphore, Ogre::Billboard* _bill = NULL, bool _isGreen = true) : semaphore(_semaphore), bill(_bill), isGreen(_isGreen) {};
		
		CSemaphore*			semaphore;
		Ogre::Billboard*	bill;
		bool				isGreen;
	} TSemaphore;
	
	typedef std::vector<TSemaphore> TSemaphoreVector;
	typedef std::vector<TSemaphore>::iterator TSemaphoreIterator;
	
	typedef struct _TTree
	{
		_TTree(CTree* _tree, Ogre::Entity* _entity, Ogre::SceneNode* _node, Ogre::Billboard* _bill = NULL) : tree(_tree), node(_node), entity(_entity), bill(_bill) {};
		
		CTree*				tree;
		Ogre::Entity*		entity;
		Ogre::SceneNode*	node;
		Ogre::Billboard*	bill;
	} TTree;
	
	typedef std::vector<TTree> TTreeVector;
	typedef std::vector<TTree>::iterator TTreeIterator;
	
	typedef struct _TCity
	{
		_TCity(CCity* _city, Ogre::Entity* _entity, Ogre::SceneNode* _node, Ogre::Billboard* _cityBill = NULL, int _phase = 0) : city(_city), node(_node), entity(_entity), boardBill(NULL), colorBill(NULL), digitBill(NULL), lockBill(NULL), cityBill(_cityBill), phase(_phase) {};
		
		CCity*				city;
		Ogre::Entity*		entity;
		Ogre::SceneNode*	node;
		Ogre::Billboard*	boardBill;  //табло
        Ogre::Billboard*	colorBill;  //цветной бекрраунд
        Ogre::Billboard*	digitBill;  //циферки
		Ogre::Billboard*	lockBill;   //замок
		Ogre::Billboard*	cityBill;   //картинка города
		float				phase;
	} TCity;
	
	
	typedef std::vector<TCity> TCityVector;
	typedef std::vector<TCity>::iterator TCityIterator;
	
	typedef struct _TRail
	{
		_TRail(CRail* _rail = NULL, Ogre::Billboard* _bill = NULL, Ogre::Billboard* _bill_2 = NULL) : rail(_rail), bill(_bill), bill_2(_bill_2) {};
		
		CRail*				rail;
		Ogre::Billboard*	bill;
		Ogre::Billboard*	bill_2;
	} TRail;
	
	typedef std::vector<TRail> TRailVector;
	typedef std::vector<TRail>::iterator TRailIterator;
	
	typedef struct _TSwitch
	{
		_TSwitch(CSwitch* _switch = NULL, Ogre::Billboard* _bill = NULL) : sw(_switch), bill(_bill) {};
		
		CSwitch*			sw;
		Ogre::Billboard*	bill;
	} TSwitch;
	
	typedef std::vector<TSwitch> TSwitchVector;
	typedef std::vector<TSwitch>::iterator TSwitchIterator;
    
    typedef struct _TCancelButton
    {
        _TCancelButton(CCancelButton* _button = NULL, Ogre::Billboard* _bill = NULL) : btn(_button), bill(_bill) {};
        
        CCancelButton*		btn;
		Ogre::Billboard*	bill;
    } TCancelButton;
    
    typedef std::vector<TCancelButton> TCancelButtonVector;
	typedef std::vector<TCancelButton>::iterator TCancelButtonIterator;
	
	typedef struct _NBS
	{
		_NBS(Ogre::SceneNode* _node, Ogre::Billboard* _bill, Ogre::Real _startTime, bool _isUpMoving) : node(_node), bill(_bill), startTime(_startTime), isUpMoving(_isUpMoving) {};
		
		Ogre::SceneNode*	node;
		Ogre::Billboard*	bill;
		Ogre::Real			startTime;
        bool                isUpMoving;
	} NBS;
	
	typedef std::vector<NBS> NBSVector;
	typedef std::vector<NBS>::iterator NBSIterator;
	
	///////////////////////////////////////////// Game Variables /////////////////////////////////////////////
	
	TTrainVector		m_trains;
	TSemaphoreVector	m_semaphores;
	TTreeVector			m_trees;
	TCityVector			m_cities;
	TSwitchVector		m_switches;
    TCancelButtonVector m_cancelButtons;
	
	Ogre::Billboard*	m_railsBill[NODES_WIDTH][NODES_HEIGHT];
	Ogre::Billboard*	m_nodesBill[NODES_WIDTH][NODES_HEIGHT];
	
	TRailVector			m_futureCitiesRails;
	TRailVector			m_unfinishedRails;
	TRailVector			m_tempRails;
	TRailVector			m_hlRails;
	
	NBSVector			m_faMessages;
	NBSVector			m_explosions;
	
	int	trainID;
	
	bool				m_madTrainCycle;
	double				m_displayFactor;
	
	Ogre::String		m_sceneTag;
	
	///////////////////////////////////////////////// RTT /////////////////////////////////////////////////

	void setupRTTScene();
	void updateRenderTexture();
	Ogre::SceneManager*			m_pRTSceneMgr;
	//#ifdef RTT_BACKGROUND
	Ogre::Camera*				m_pRailsTextureCamera;
	Ogre::RenderTexture*		m_pRenderTexture;
	Ogre::Rectangle2D*			m_pMiniScreen;
	//#endif
		
	///////////////////////////////////////////////// Scene Variables /////////////////////////////////////////////////
	
	Ogre::Viewport* mViewport;    		// main viewport
	Ogre::Camera* mCamera;        		// main camera
	SdkTrayManager* mTrayMgr;     		// tray interface manager
	SdkTrayManager* mPrelevelTrayMgr;
	SdkTrayManager* mFaildLevelTrayMgr;
	SdkTrayManager* mCompleteLevelTrayMgr;
	SdkTrayManager* mPauseTrayMgr;
	
	Ogre::Light* mSceneLight;
	
	Ogre::Root* mRoot;                // OGRE root object
	Ogre::OverlaySystem* mOverlaySystem; // OverlaySystem
	Ogre::RenderWindow* mWindow;      // context render window
	InputContext mInputContext;
	Ogre::SceneManager* mSceneMgr;    // scene manager for this sample
	bool mResourcesLoaded;    // whether or not resources have been loaded
	bool mContentSetup;       // whether or not scene was created
#ifdef USE_RTSHADER_SYSTEM
		Ogre::RTShader::ShaderGenerator*			mShaderGenerator;			// The Shader generator instance.
#endif
	
	//--------------------------
		
	bool mIsRightOrientedInterface;
	
	OgreBites::GameButton* mSoundButton;
	OgreBites::GameButton* mRestartButton;
	OgreBites::GameButton* mBoostButton;
	OgreBites::GameButton* mBonusButton0;
	OgreBites::GameButton* mBonusButton1;
	OgreBites::GameButton* mBonusButton2;
	OgreBites::GameButton* mPauseButton;
	OgreBites::GameButton* mMenuButton;
	
	Ogre::String mLastTutorialMessage;
	OgreBites::DecorWidget* mCommonMessagePanel;
	Ogre::BorderPanelOverlayElement* mCommonMessagePanelEl;
	Ogre::TextAreaOverlayElement* mCommonMessagePanelLabel;
	float mCommonMessagePanelBaseWidth;
	float mCommonMessagePanelBaseHeight;
	
	OgreBites::DecorWidget* mYearProgress;
	Ogre::TextAreaOverlayElement* mYearLabel;
	Ogre::TextAreaOverlayElement* mYearCaption;
	OgreBites::Label* mLevelLabel;
	OgreBites::DecorWidget* mTrainsMoneyLabel;
	Ogre::TextAreaOverlayElement* mMoneyLabelCaption;
	Ogre::TextAreaOverlayElement* mTrainsLabelCaption;
	Ogre::TextAreaOverlayElement* mLevelLabelCaption;
	
	float mTextCompleteLevelViewTextBaseHeight;
	
	OgreBites::DecorWidget* mMoneyTrainsBack;
	OgreBites::DecorWidget* mTopTab;
	OgreBites::Label* mDemoLabel;
	
	Ogre::PanelOverlayElement* mProgressBarThumb;
	float mProgressBarThumbTop;
	float mProgressBarThumbHeight;
	
	//Ogre::OverlayContainer* mRightPanel;
	//Ogre::OverlayContainer* mLeftPanel;
	Ogre::PanelOverlayElement* mRightPanel;
	Ogre::PanelOverlayElement* mLeftPanel;
	Slider* mRPWSlider;
	Slider* mLPWSlider;
	
	Ogre::BorderPanelOverlayElement* mPrelevelTrainsBox;
	std::vector<Ogre::OverlayElement*> mPrelevelTrainsImages;
	
	Ogre::PanelOverlayElement* mAchievementsBox;
	std::vector<Ogre::OverlayElement*> mAchievementsVector;
	
	OgreBites::Label* mPLTrainsOnLevelLabel;
	
	NSDictionaryPtr mLevelCompleteInfo;
	
	ParticleSystem* mPSWeather;
};

#endif
