#include "NSGeometry.h"

#include <cmath>
//NSPoint NSZeroPoint;  						// A zero point
//NSRect  NSZeroRect;    						// A zero origin rectangle
//NSSize  NSZeroSize;    						// A zero size rectangle

#ifndef NSMakePoint
NSPoint
NSMakePoint(float x, float y)					{ return NSPoint(x, y); }

NSSize
NSMakeSize(float w, float h)					{ return NSSize(w, h); }

NSRect
NSMakeRect(float x, float y, float w, float h)	{ return NSRect(x, y, w, h); }

#endif

NSRect
NSOffsetRect(NSRect aRect, float dX, float dY)
{
	return NSRect(NSPoint(NSMinX(aRect) + dX, NSMinY(aRect) + dY), aRect.size);
}

NSRect
NSInsetRect(NSRect aRect, float dX, float dY)
{
	return NSRect(NSPoint(NSMinX(aRect) + dX, NSMinY(aRect) + dY),
		NSSize(NSWidth(aRect) - (2 * dX), NSHeight(aRect) - (2 * dY)));
}

void
NSDivideRect(NSRect r,
             NSRect *slice,
             NSRect *remainder,
             float amount,
             NSRectEdge edge)
{
	if (NSIsEmptyRect(r))
	{
		*slice = NSZeroRect;
		*remainder = NSZeroRect;
		
		return;
	}
	
	switch (edge)
	{
		case NSMinXEdge:
			if (amount > r.size.width)
			{
				*slice = r;
				*remainder = NSRect(NSMaxX(r), NSMinY(r), 0, NSHeight(r));
			}
			else
			{
				*slice = NSRect(NSMinX(r), NSMinY(r), amount, NSHeight(r));
				*remainder = NSRect(NSMaxX(*slice), NSMinY(r), NSMaxX(r) - NSMaxX(*slice), NSHeight(r));
			}
			break;
			
		case NSMinYEdge:
			if (amount > r.size.height)
			{
				*slice = r;
				*remainder = NSRect(r.origin.x, NSMaxY(r), r.size.width, 0);
			}
			else
			{
				*slice = NSRect(r.origin, NSSize(r.size.width, amount));
				*remainder = NSRect(r.origin.x, NSMaxY(*slice), r.size.width, NSMaxY(r) - NSMaxY(*slice));
			}
			break;
			
		case NSMaxXEdge:
			if (amount > r.size.width)
			{
				*slice = r;
				*remainder = NSRect(r.origin, NSSize(0, r.size.height));
			}
			else
			{
				*slice = NSRect(NSMaxX(r) - amount, r.origin.y, amount, r.size.height);
				*remainder = NSRect(r.origin, NSSize(NSMinX(*slice) - NSMinX(r), r.size.height));
			}
			break;
			
		case NSMaxYEdge:
			if (amount > r.size.height)
			{
				*slice = r;
				*remainder = NSRect(r.origin, NSSize(r.size.width, 0));
			}
			else
			{
				*slice = NSRect(r.origin.x, (NSMaxY(r) - amount), r.size.width, amount);
				*remainder = NSRect(r.origin, NSSize(r.size.width, NSMinY(*slice) - r.origin.y));
			}
			
		default:
			break;
	}
}

NSRect
NSIntegralRect(NSRect aRect)
{
	NSRect rect;
	
	if (NSIsEmptyRect(aRect))
		return NSZeroRect;
	
	rect.origin.x = floor(aRect.origin.x);
	rect.origin.y = floor(aRect.origin.y);
	rect.size.width = ceil(aRect.size.width);
	rect.size.height = ceil(aRect.size.height);
	
	return rect;
}

NSRect
NSUnionRect(NSRect aRect, NSRect bRect)
{
	NSRect rect;
	
	if (NSIsEmptyRect(aRect))
		return (NSIsEmptyRect(bRect)) ? NSZeroRect : bRect;
	
	if (NSIsEmptyRect(bRect))
		return aRect;
	
	rect = NSRect(MIN(NSMinX(aRect), NSMinX(bRect)), MIN(NSMinY(aRect), NSMinY(bRect)), 0, 0);
	
	return NSRect(NSMinX(rect), NSMinY(rect), MAX(NSMaxX(aRect), NSMaxX(bRect)) - NSMinX(rect), MAX(NSMaxY(aRect), NSMaxY(bRect)) - NSMinY(rect));
}

bool
NSIntersectsRect(NSRect aRect, NSRect bRect)
{													// Intersection at a line
	return (NSMaxX(aRect) <= NSMinX(bRect)			// or a point doesn't count
			|| NSMaxX(bRect) <= NSMinX(aRect)
			|| NSMaxY(aRect) <= NSMinY(bRect)
			|| NSMaxY(bRect) <= NSMinY(aRect)) ? false : true;
}

NSRect
NSIntersectionRect (NSRect aRect, NSRect bRect)
{
	NSRect rect;
	
	if (!NSIntersectsRect(aRect, bRect))
    	return NSZeroRect;
	
	if (NSMinX(aRect) <= NSMinX(bRect))
	{
		rect.size.width = MIN(NSMaxX(aRect), NSMaxX(bRect)) - NSMinX(bRect);
		rect.origin.x = NSMinX(bRect);
	}
	else
	{
		rect.size.width = MIN(NSMaxX(aRect), NSMaxX(bRect)) - NSMinX(aRect);
		rect.origin.x = NSMinX(aRect);
	}
	
	if (NSMinY(aRect) <= NSMinY(bRect))
	{
		rect.size.height = MIN(NSMaxY(aRect), NSMaxY(bRect)) - NSMinY(bRect);
		rect.origin.y = NSMinY(bRect);
	}
	else
	{
		rect.size.height = MIN(NSMaxY(aRect), NSMaxY(bRect)) - NSMinY(aRect);
		rect.origin.y = NSMinY(aRect);
	}
	
	return rect;
}

bool
NSEqualRects(NSRect aRect, NSRect bRect)
{
	return ((NSMinX(aRect) == NSMinX(bRect))
			&& (NSMinY(aRect) == NSMinY(bRect))
			&& (NSWidth(aRect) == NSWidth(bRect))
			&& (NSHeight(aRect) == NSHeight(bRect))) ? true : false;
}

bool
NSEqualSizes(NSSize aSize, NSSize bSize)
{
	return ((aSize.width == bSize.width)
			&& (aSize.height == bSize.height)) ? true : false;
}

bool
NSEqualPoints(NSPoint aPoint, NSPoint bPoint)
{
	return ((aPoint.x == bPoint.x) && (aPoint.y == bPoint.y)) ? true : false;
}

bool
NSIsEmptyRect(NSRect aRect)
{
	return ((NSWidth(aRect) > 0) && (NSHeight(aRect) > 0)) ? false : true;
}

bool
NSMouseInRect(NSPoint aPoint, NSRect aRect, bool flipped)
{
	if (flipped)
		return ((aPoint.x >= NSMinX(aRect))
				&& (aPoint.y >= NSMinY(aRect))
				&& (aPoint.x < NSMaxX(aRect))
				&& (aPoint.y < NSMaxY(aRect))) ? true : false;
	
	return ((aPoint.x >= NSMinX(aRect))
			&& (aPoint.y > NSMinY(aRect))
			&& (aPoint.x < NSMaxX(aRect))
			&& (aPoint.y <= NSMaxY(aRect))) ? true : false;
}

bool
NSPointInRect(NSPoint aPoint, NSRect aRect)
{
	return NSMouseInRect(aPoint, aRect, true);
}

bool
NSContainsRect(NSRect aRect, NSRect bRect)
{
	return (!NSIsEmptyRect(bRect)	// b falset empty
			&& (NSMinX(aRect) <= NSMinX(bRect))	// and b does falset extend beyond a in any direction
			&& (NSMinY(aRect) <= NSMinY(bRect))
			&& (NSMaxX(aRect) >= NSMaxX(bRect))
			&& (NSMaxY(aRect) >= NSMaxY(bRect))) ? true : false;
}
