//
//  RailsPrefsController.cpp
//  ShortLine
//
//  Created by Ievgen Ovcharenko
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#include "RailsPrefsController.h"
#include "CommonDef.h"
#include "CPrefs.h"
#include "NSDictionary.h"
#include <android/log.h>

#include "AdditionalFunctions.h"

#include "OgreStringConverter.h"
#include "OgreUTFString.h"


RailsPrefsController* RailsPrefsController::sharedController()
{
	static RailsPrefsControllerPtr _instanse;

	if (!_instanse)
	{
		_instanse = RailsPrefsControllerPtr(new RailsPrefsController());
	}

	return _instanse.get();
}

static std::vector<int> s_levelsInfo;

void RailsPrefsController::emptyLevelsInfo()
{
	if (s_levelsInfo.size())
		s_levelsInfo.clear();
	s_levelsInfo.reserve(MAX_LEVEL);

	for (int i = 0; i < MAX_LEVEL; ++i)
		s_levelsInfo.push_back(-1);
}

std::vector<int>& RailsPrefsController::allLevelsInfo()
{
	if (s_levelsInfo.empty())
	{
		s_levelsInfo = CPrefs::getSharedUserDefaultsDict()->getIntArrayForKey("levelsInfoU");
		
		if (s_levelsInfo.size() != MAX_LEVEL)
			RailsPrefsController::emptyLevelsInfo();

		//только зачитали, проверяем хэш
		/*
		if (s_levelsInfo)
		{
			int calculatedHash = levelsInfoHash();
			int savedHash = CPrefs::getSharedUserDefaultsDict()->getIntegerForKey("levelsInfoUHash", 0);

			if (savedHash == 0 || calculatedHash != savedHash)
			{
				//если хэш не совпадает - сбрасываем всё
				s_levelsInfo.clear();
			}
		}
		*/

		//если было пусто в префах или префы были повреждены
		if (s_levelsInfo.empty() || !isValidLevelsInfoHash())
		{
			RailsPrefsController::emptyLevelsInfo();
		}
	}

	return s_levelsInfo;
}

void RailsPrefsController::reinitAllLevelsInfo()
{
	s_levelsInfo.clear();
}

void RailsPrefsController::saveAllLevelsInfo()
{
	CPrefs::getSharedUserDefaultsDict()->setIntArrayForKey("levelsInfoU", s_levelsInfo);
	updateLevelsInfoHash();
	//s_levelsInfo.clear();
	reinitAllLevelsInfo();
	/*
	int hash = [levelsInfo deepHash];

	[_userDefaults setObject:levelsInfo forKey:@"levelsInfoU"];

	[_userDefaults setObject:[NSNumber numberWithInteger:hash] forKey:@"levelsInfoUHash"];

	[s_levelsInfo release];
	s_levelsInfo = nil;
	*/
}

int RailsPrefsController::moneyRecordForLevelIndex(int levelIndex)
{
	return allLevelsInfo()[levelIndex-1];
}

bool RailsPrefsController::setMoneyRecordForLevelIndex(int money, int levelIndex)
{
	if (isRecordMoneyForLevelIndex(money, levelIndex))
	{
		allLevelsInfo()[levelIndex-1] = money;
		saveAllLevelsInfo();
		CPrefs::syncUserDefaults();
		
		return true;
	}
	else 
	{
		return false;
	}
}

bool RailsPrefsController::isRecordMoneyForLevelIndex(int money, int levelIndex)
{
	if (moneyRecordForLevelIndex(levelIndex) < money)
		return true;
	else 
		return false;
}

void RailsPrefsController::resetGameProgressPreferences()
{
	bool unerasableKey;
	std::vector<std::string> unerasableKeys;
	unerasableKeys.push_back("SLMusic");
	unerasableKeys.push_back("SLSound");
	
	RailsPrefsController::emptyLevelsInfo();
	std::vector<std::string> keys = CPrefs::getSharedUserDefaultsDict()->getAllKeys();
	for (int i = 0; i < keys.size(); i++)
	{
		unerasableKey = false;
		for (int k = 0; k < unerasableKeys.size(); k++)
		{
			if (keys[i] == unerasableKeys[k])
			{
				unerasableKey = true;
				break;
			}
		}
		
		if (!unerasableKey)
			CPrefs::getSharedUserDefaultsDict()->removeKey(keys[i].c_str());
	}
	
	CPrefs::syncUserDefaults();
}

#pragma mark - prefs hash

int RailsPrefsController::prefsHash(NSDictionaryPtr prefsDict)
{
	int countCrashedScore         = prefsDict->getIntegerForKey("countCrashedScore", 0);
	int countArrivedScore         = prefsDict->getIntegerForKey("countArrivedScore", 0);
	int grossMoneyScore           = prefsDict->getIntegerForKey("grossMoneyScore", 0);
	int SLMaxSuccessLevel         = prefsDict->getIntegerForKey("SLMaxSuccessLevel", 0);
	int SLMaxSuccessTutorialLevel = prefsDict->getIntegerForKey("SLMaxSuccessTutorialLevel", 0);

	//__android_log_print(ANDROID_LOG_INFO, "RailsNative", ">>>>>>>>>>>>>>>>>>>>>>>>> prefsHash data: %i, %i, %i, %i, %i", countCrashedScore, countArrivedScore, grossMoneyScore, SLMaxSuccessLevel, SLMaxSuccessTutorialLevel);

	Ogre::UTFString hashString = Ogre::StringConverter::toString(countCrashedScore);
	hashString.append(Ogre::StringConverter::toString(countArrivedScore));
	hashString.append(Ogre::StringConverter::toString(grossMoneyScore));
	hashString.append(Ogre::StringConverter::toString(SLMaxSuccessLevel));
	hashString.append(Ogre::StringConverter::toString(SLMaxSuccessTutorialLevel));

	return ourHash(hashString.data(), hashString.length());//[hashString hash];
}

int RailsPrefsController::prefsHash()
{
	NSDictionaryPtr defaults = CPrefs::getSharedUserDefaultsDict();
	return prefsHash(defaults);
}

bool RailsPrefsController::isValidPrefsHash()
{
	int hashFromPrefs = CPrefs::getSharedUserDefaultsDict()->getIntegerForKey("prefsUHash", 0);
	int hash = prefsHash();
	
	//__android_log_print(ANDROID_LOG_INFO, "RailsNative", ">>>>>>>>>>>>>>>>>>>>>>>>> prefsHashFromPrefs: %i   hash: %i", hashFromPrefs, hash);
	
	if ( (hashFromPrefs == 0) || (hash != hashFromPrefs) )
		return false;

	return true;
}

void RailsPrefsController::updatePrefsHash()
{
	int goodPrefsHash = prefsHash();
	
	CPrefs::getSharedUserDefaultsDict()->setIntegerForKey("prefsUHash", goodPrefsHash);
	
	//CPrefs::syncUserDefaults();
	updateChaptersUnlockedHash();
}

#pragma mark - achievements hash

int RailsPrefsController::achievementsHash(NSDictionaryPtr achievements)
{
	Ogre::UTFString hashString;
	hashString.append(achievements->getStringForKey("completedAchievements"));
	hashString.append(achievements->getStringForKey("levelsWithoutCrashes"));
	hashString.append(achievements->getStringForKey("levelsWithoutMadCrashes"));
	hashString.append(achievements->getStringForKey("levelsFrontArrived"));
	
	return ourHash(hashString.data(), hashString.length());
}

int RailsPrefsController::achievementsHash()
{
	NSDictionaryPtr achievements = CPrefs::getSharedUserDefaultsDict()->getDictForKey("achievements");
	
	return achievementsHash(achievements);
}

bool RailsPrefsController::isValidAchievementsHash()
{
	int hashFromPrefs = CPrefs::getSharedUserDefaultsDict()->getIntegerForKey("achievementsHash", 0);
	int hash = achievementsHash();
	
	//__android_log_print(ANDROID_LOG_INFO, "RailsNative", ">>>>>>>>>>>>>>>>>>> achievementsHashFromPrefs: %i   hash: %i", hashFromPrefs, hash);

	if ( (hash == 0) || (hashFromPrefs != hash) )
		return false;

	return true;
}

void RailsPrefsController::updateAchievementsHash()
{
	int goodAchievementsHash = achievementsHash();
	CPrefs::getSharedUserDefaultsDict()->setIntegerForKey("achievementsHash", goodAchievementsHash);
	CPrefs::syncUserDefaults();
}

#pragma mark - levels info hash

int RailsPrefsController::levelsInfoHash(std::vector<int> levelsInfoArray)
{
	if (levelsInfoArray.size() > 0)
	{
		Ogre::UTFString hashString;
		for (int i = 0; i < levelsInfoArray.size(); i++)
			hashString.append(Ogre::StringConverter::toString(levelsInfoArray[i]));
		return ourHash(hashString.data(), hashString.length());
	}
	
	return 0;
}

int RailsPrefsController::levelsInfoHash()
{
	std::vector<int> levelsInfoArray;

	if (s_levelsInfo.size() > 0)
		levelsInfoArray = s_levelsInfo;
	else
		levelsInfoArray = CPrefs::getSharedUserDefaultsDict()->getIntArrayForKey("levelsInfoU");

	return levelsInfoHash(levelsInfoArray);
}

bool RailsPrefsController::isValidLevelsInfoHash()
{
	int hashFromPrefs = CPrefs::getSharedUserDefaultsDict()->getIntegerForKey("levelsInfoUHash", 0);
	int hash = levelsInfoHash();
	
	//__android_log_print(ANDROID_LOG_INFO, "RailsNative", ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>> levelsInfoHashFromPrefs: %i   hash: %i", hashFromPrefs, hash);

	if ( (hash == 0) || (hashFromPrefs != hash) )
		return false;

	return true;
}

void RailsPrefsController::updateLevelsInfoHash()
{
	int goodLevelsInfoHash = levelsInfoHash();
	CPrefs::getSharedUserDefaultsDict()->setIntegerForKey("levelsInfoUHash", goodLevelsInfoHash);
	CPrefs::syncUserDefaults();
}

#pragma mark - snapshot hash

int RailsPrefsController::getSnapshotHash(NSDictionaryPtr snapshotDict)
{
	Ogre::UTFString hashString;
	hashString.append(Ogre::StringConverter::toString(snapshotDict->getIntegerForKey("level", 0)));
	hashString.append(Ogre::StringConverter::toString(snapshotDict->getIntegerForKey("mseconds", 0)));
	hashString.append(Ogre::StringConverter::toString(snapshotDict->getIntegerForKey("ArrivedTrainsCount", 0)));
	hashString.append(Ogre::StringConverter::toString(snapshotDict->getIntegerForKey("GeneratedTrainsCount", 0)));
	hashString.append(Ogre::StringConverter::toString(snapshotDict->getIntegerForKey("CrashedTrainsCount", 0)));
	hashString.append(Ogre::StringConverter::toString(snapshotDict->getIntegerForKey("gameMode", 0)));
	
	if (snapshotDict->hasObjectForKey("timer")) 
	{
		NSDictionaryPtr timerDict = snapshotDict->getDictForKey("timer");
		if (timerDict)
		{
			hashString.append(Ogre::StringConverter::toString(timerDict->getIntegerForKey("money", 0)));
			hashString.append(Ogre::StringConverter::toString(timerDict->getIntegerForKey("years", 0)));
			hashString.append(Ogre::StringConverter::toString(timerDict->getIntegerForKey("globalCounter", 0)));
		}
	}
	
	NSDictionaryPtr achievementsDict = snapshotDict->getDictForKey("achievements");
	
	hashString.append(Ogre::StringConverter::toString((int)achievementsDict->getBoolForKey("wasMadCrashed", false)));
	hashString.append(Ogre::StringConverter::toString((int)achievementsDict->getBoolForKey("wasUsedSemaphores", false)));
	hashString.append(Ogre::StringConverter::toString((int)achievementsDict->getBoolForKey("wasArrivedByNotFront", false)));
	hashString.append(Ogre::StringConverter::toString(achievementsDict->getIntegerForKey("maxArrivedTrainsRunning", 0)));
	hashString.append(Ogre::StringConverter::toString(achievementsDict->getIntegerForKey("maxCountLivingTrains", 0)));
	hashString.append(Ogre::StringConverter::toString(achievementsDict->getIntegerForKey("changedColorByChameleon", 0)));
	hashString.append(Ogre::StringConverter::toString(achievementsDict->getIntegerForKey("arrivedAllBombTrains", 0)));
	hashString.append(Ogre::StringConverter::toString((int)achievementsDict->getBoolForKey("wasUsed1xMultiplier", false)));
	hashString.append(Ogre::StringConverter::toString((int)achievementsDict->getBoolForKey("wasUsedBonus", false)));
	
	return ourHash(hashString.data(), hashString.length());
}

bool RailsPrefsController::isValidSnapshotHash(NSDictionaryPtr snapshotDict)
{
	int hashFromPrefs = snapshotDict->getIntegerForKey("snapshotUHash", 0);
	int hash = getSnapshotHash(snapshotDict);
	
	//__android_log_print(ANDROID_LOG_INFO, "RailsNative", ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>> snapshotHashFromPrefs: %i   hash: %i", hashFromPrefs, hash);

	if ( (hash == 0) || (hashFromPrefs != hash) )
		return false;

	return true;
}

void RailsPrefsController::updateSnapshotHash(NSDictionaryPtr snapshotDict)
{
	int goodSnapshotHash = getSnapshotHash(snapshotDict);
	snapshotDict->setIntegerForKey("snapshotUHash", goodSnapshotHash);
	CPrefs::syncUserDefaults();
}

#pragma mark - chapters unlocked hash

int RailsPrefsController::chaptersUnlockedHash()
{
	NSDictionaryPtr defaults = CPrefs::getSharedUserDefaultsDict();
	Ogre::UTFString hashString;
	hashString.append(Ogre::StringConverter::toString(defaults->getIntegerForKey("prefsUHash", 0)));
	hashString.append(Ogre::StringConverter::toString(defaults->getIntegerForKey("chaptersUnlockedFlag", 0)));
	
	return ourHash(hashString.data(), hashString.length());
}

bool RailsPrefsController::isChaptersUnlocked()
{
	NSDictionaryPtr defaults = CPrefs::getSharedUserDefaultsDict();
	int hashFromPrefs = defaults->getIntegerForKey("chaptersUnlockedHash", 0);
	int hash = chaptersUnlockedHash();
	bool isUnlocked = (bool)CPrefs::getSharedUserDefaultsDict()->getIntegerForKey("chaptersUnlockedFlag", 0);
	
	//__android_log_print(ANDROID_LOG_INFO, "RailsNative", ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> RailsPrefsController::isChaptersUnlocked: %s", isUnlocked ? "true" : "false");
	//__android_log_print(ANDROID_LOG_INFO, "RailsNative", ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>> chaptersUnlockedHashFromPrefs: %i   hash: %i", hashFromPrefs, hash);
	
	if ( (hash == 0) || (hashFromPrefs != hash) )
		return false;
	
	return isUnlocked;
}

void RailsPrefsController::setChaptersUnlocked(bool isUnlocked)
{
	CPrefs::getSharedUserDefaultsDict()->setIntegerForKey("chaptersUnlockedFlag", isUnlocked ? 1 : 0);
	//CPrefs::syncUserDefaults();
	updateChaptersUnlockedHash();
}

void RailsPrefsController::updateChaptersUnlockedHash()
{
	int goodChaptersUnlockedHash = chaptersUnlockedHash();
	CPrefs::getSharedUserDefaultsDict()->setIntegerForKey("chaptersUnlockedHash", goodChaptersUnlockedHash);
	CPrefs::syncUserDefaults();
}

/*
- (void) resetGameProgressPreferences
{
    NSDictionary* allObjects = [[NSUserDefaults standardUserDefaults] dictionaryRepresentation];
    NSArray* unerasableKeys = @[@"iCloudState", @"isFullScreen", @"RLWasRatedRunsCount",
                                @"SLMusic",     @"SLSound",      @"SLSkipStartVideo", @"RLLicenseCode",
                                @"remindDialogNeverShow", @"remindDialogLevels", @"registrationDialogRunsCount"];
    
    for(NSString* key in allObjects)
    {
        if (NO == [unerasableKeys containsObject:key])
            [[NSUserDefaults standardUserDefaults] removeObjectForKey: key];
    }
    
    [self invalidateLevelsInfo];
    
    [[NSUserDefaults standardUserDefaults] setObject:@(0) forKey:@"SLMaxSuccessLevel"];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

- (void) resetAllPreferences
{
    NSDictionary* allObjects = [[NSUserDefaults standardUserDefaults] dictionaryRepresentation];
    
    for(NSString* key in allObjects)
    {
        [[NSUserDefaults standardUserDefaults] removeObjectForKey: key];
    }
    
    [[NSUserDefaults standardUserDefaults] synchronize];
}
*/