#include "OgreScene.h"
#include "AdditionalFunctions.h"

#include "CTrain.h"
#include "CSemaphore.h"
#include "CTree.h"
#include "CRail.h"
#include "CCity.h"
#include "CSwitch.h"
#include "CNodesArea.h"
#include "CNode.h"
#include "CCancelButton.h"

#include "CXMacDataTypes.h"

#include "MovableText.h"

#include "CSoundManager.h"

#include "CTutorial.h"

#include "NSDictionary.h"

#include "RailsContext.h"

#include "CAchievements.h"

#include "android_strings.h"

using namespace Ogre;
using namespace OgreBites;

int DETECT_RADIUS = 80;

extern Ogre::String gUserSnapshotName;
extern AAssetManager* gAssetMgr;

//#define RTT_BACKGROUND

#define UNFINISHED_RAILS_RENDER_QUEUE (RENDER_QUEUE_BACKGROUND+2)
#define RAILS_RENDER_QUEUE (RENDER_QUEUE_BACKGROUND+3)
#define NODES_RENDER_QUEUE (RENDER_QUEUE_BACKGROUND+4)
#define TEMP_RAILS_QUEUE (RENDER_QUEUE_BACKGROUND+5)
#define SWITCHES_RENDER_QUEUE (RENDER_QUEUE_BACKGROUND+6)
#define HINT_NODES_RENDER_QUEUE (RENDER_QUEUE_BACKGROUND+7)
#define SEMAPHORES_RENDER_QUEUE  (RENDER_QUEUE_BACKGROUND+7)

#define TREES_RENDER_QUEUE (RENDER_QUEUE_OVERLAY-6)
#define EXPLOSION_RENDER_QUEUE (RENDER_QUEUE_OVERLAY-6)
#define FRONT_CITIES_RENDER_QUEUE (RENDER_QUEUE_OVERLAY-5)
#define WEATHER_RENDER_QUEUE (RENDER_QUEUE_OVERLAY-4)
#define BUBBLES_RENDER_QUEUE (RENDER_QUEUE_OVERLAY-3)
#define CITYBOARD_RENDER_QUEUE (RENDER_QUEUE_OVERLAY-2)
#define FA_CLOUD_RENDER_QUEUE (RENDER_QUEUE_OVERLAY-1)

/*
#define TREES_RENDER_QUEUE (RENDER_QUEUE_OVERLAY)
#define EXPLOSION_RENDER_QUEUE (RENDER_QUEUE_OVERLAY)
#define FRONT_CITIES_RENDER_QUEUE (RENDER_QUEUE_OVERLAY+1)
#define BUBBLES_RENDER_QUEUE (RENDER_QUEUE_OVERLAY+2)
#define CITYBOARD_RENDER_QUEUE (RENDER_QUEUE_OVERLAY+3)
#define FA_CLOUD_RENDER_QUEUE (RENDER_QUEUE_OVERLAY+4)
*/

#define CAPTIONS_UPDATE_INTERVAL (0.52)
#define MAD_COLOR_UPDATE_INTERVAL (1)

#define MAD_COLOR_MANUAL_UPDATE (0)

#define SHAKE_SCALE_Z (5.)

#define MODEL_CISTERN_SCALE (0.53)
#define MODEL_SCALE (0.54)
#define SMOKE_SCALE (0.01)

#define FAMESSAGE_LIFETIME (100)
#define FAMESSAGE_START_HEIGHT (5.)
#define FAMESSAGE_ALPHA_FACTOR (0.5)
#define FAMESSAGE_INC_Z_COORD_FACTOR (5)

#define EXPLOSION_HEIGHT_OFFSET (2.)
#define EXPLOSION_Z_ORDER_OFFSET (10.)

static CXRect s_scene_rect = CXZeroRect;

static int s_left_panel_width = 150;//74;
static int s_right_panel_width = 150;//74;
static int s_additional_right_panel_width = 0;

static float s_sceneObjectsScale = 0.9f;
static float s_cloudsBillSize = 7.f * s_sceneObjectsScale;
static float s_bubblesBillSize = 5.5f * s_sceneObjectsScale;
static float s_explosionBillSize = 22.f * s_sceneObjectsScale;
static float s_treesBillSize = 7.0f * s_sceneObjectsScale;
static float s_semaphoresBillSize = 7.f * s_sceneObjectsScale;
static float s_frontCitiesBillSize = 14.f * s_sceneObjectsScale;
static float s_cityAdditionsBillSize = 8.f * s_sceneObjectsScale;
static float s_cancelButtonBillSize = 8.f * s_sceneObjectsScale;

static float s_citiesAdditionalScale = (4.f / 3.f);
static float s_railsOverlap = (1 + 1.f/128.f);

void COgreScene::setupView()
{	
	if (!mCamera)
	{
		int minPanelWidth = 58;
		mCamera = mSceneMgr->createCamera("MainCamera");
		mViewport = mWindow->addViewport(mCamera);
		
		DETECT_RADIUS = (mViewport->getActualHeight() / 1024.f * 80);
		
		s_left_panel_width = (mViewport->getActualWidth() - ((float)mViewport->getActualHeight() / 3.f * 4.f)) / 2.f;
		s_right_panel_width = s_left_panel_width;
		if (s_left_panel_width < minPanelWidth)
		{
			s_left_panel_width = 0;
			s_right_panel_width *= 2;
			s_additional_right_panel_width = 0;
			
			if (s_right_panel_width < 20)
			{
				s_right_panel_width = (s_right_panel_width <= 0)? 1 : s_right_panel_width;
				s_additional_right_panel_width = mViewport->getActualWidth()*0.08;
			}
			else if (s_right_panel_width < minPanelWidth)
				s_additional_right_panel_width = MAX(mViewport->getActualWidth()*0.08, minPanelWidth - s_right_panel_width);
			else
				s_additional_right_panel_width = mViewport->getActualWidth()*0.03;
			mIsRightOrientedInterface = true;
		}
		else
			mIsRightOrientedInterface = false;
	}
}

bool COgreScene::isRightOrientedInterface()
{
	return mIsRightOrientedInterface;
}

void COgreScene::setupViewport()
{
	mViewport->setBackgroundColour(ColourValue::Red);
	
	mCamera->setNearClipDistance(1);
	mCamera->setFarClipDistance(250);
	mCamera->setProjectionType(Ogre::PT_ORTHOGRAPHIC);
	mCamera->setPosition(0, 100, 152);
	mCamera->lookAt(0, 0, 0);
	mCamera->setOrthoWindow(100, 75);
	
	//>1280 : 800<<
	//>1280 : 736<<
	//LogManager::getSingleton().logMessage(">>>>>>>>>>>>>>>>>>>>>>" + StringConverter::toString((int)mViewport->getActualWidth()) + " : " + StringConverter::toString((int)mViewport->getActualHeight()) + "<<<<<<<<<<<<<<<<<<");
	
	//mCamera->setAutoAspectRatio(false);
	mCamera->setAutoAspectRatio(true);
	//mCamera->setAspectRatio((Ogre::Real)mViewport->getActualWidth() / (Ogre::Real)mViewport->getActualHeight());
	float dF = (Ogre::Real)mViewport->getActualWidth() / (Ogre::Real)mViewport->getActualHeight();//10.0/8.0;//viewBounds().size.width / viewBounds().size.height;//4.0/3.0;
	float gameAspect = 4.f/3.f;
	s_citiesAdditionalScale = 1.4;//gameAspect;
	//mCamera->setAspectRatio(dF / gameAspect);
	m_displayFactor = dF;//viewBounds().size.height/viewBounds().size.width;
	
	
	mTrayMgr->hideCursor();
	Ogre::Real f = (float)(mViewport->getActualWidth() - (s_right_panel_width + s_left_panel_width)) / (float)mViewport->getActualWidth();
	
	s_scene_rect.size.width = 88.f * m_displayFactor / gameAspect;
	float offset = s_scene_rect.size.width * (1 - f);
	s_scene_rect.size.width -= offset;
	s_scene_rect.origin.x = - (s_scene_rect.size.width / 2.0);
	s_scene_rect.origin.x -= ((offset * (s_right_panel_width + s_additional_right_panel_width - s_left_panel_width) / (s_right_panel_width + s_left_panel_width)) / 1.76);
	s_scene_rect.origin.y = -53;//-56;//-58.0;
	s_scene_rect.size.height = s_scene_rect.size.width * 1.27f;//110.0;
	
	m_displayFactor = 1;//gameAspect;
	
	setupRTTScene();
}

void COgreScene::setupRTTScene()
{
	//------------------------------------------------------
#ifdef RTT_BACKGROUND
	{
		if (!m_pRTSceneMgr)
		{
			m_pRTSceneMgr = mRoot->createSceneManager(ST_GENERIC, "RTSceneManager");
#ifdef USE_RTSHADER_SYSTEM
			if (!mShaderGenerator)
				mShaderGenerator = Ogre::RTShader::ShaderGenerator::getSingletonPtr();
			mShaderGenerator->addSceneManager(m_pRTSceneMgr);
#endif
			m_pRTSceneMgr->setAmbientLight(Ogre::ColourValue(0.9, 0.9, 0.9));
			m_pRTSceneMgr->createLight("RTTLight")->setPosition(-500,700,1000);
		}
		
		if (!m_pRailsTextureCamera)
		{
			m_pRailsTextureCamera = m_pRTSceneMgr->createCamera("RailsTextureCamera");
			m_pRailsTextureCamera->setNearClipDistance(1);
			m_pRailsTextureCamera->setFarClipDistance(300);
			m_pRailsTextureCamera->setProjectionType(Ogre::PT_ORTHOGRAPHIC);
			m_pRailsTextureCamera->setPosition(0, 100, 100);
			m_pRailsTextureCamera->lookAt(0, 0, 0);
			m_pRailsTextureCamera->setOrthoWindow(100, 100);
						
			//Ogre::RenderWindow*	window = OgreFramework::getSingletonPtr()->m_pRenderWnd;
			float width = 1920;//2048;//window->getWidth();
			float height = 1440;//2048;//window->getHeight();
			Ogre::TexturePtr rtt_texture = Ogre::TextureManager::getSingleton().createManual("RttTex", Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, Ogre::TEX_TYPE_2D, width, height, 0, Ogre::PF_R8G8B8, Ogre::TU_RENDERTARGET);
		
			m_pRenderTexture = rtt_texture->getBuffer()->getRenderTarget();
		
			m_pRenderTexture->setAutoUpdated(false);
			m_pRenderTexture->addViewport(m_pRailsTextureCamera);
			m_pRenderTexture->getViewport(0)->setClearEveryFrame(true);
			//m_pRenderTexture->getViewport(0)->setBackgroundColour(Ogre::ColourValue::Black);
			m_pRenderTexture->getViewport(0)->setBackgroundColour(Ogre::ColourValue(0, 0, 1.0, 1.0));
			m_pRenderTexture->getViewport(0)->setOverlaysEnabled(false);
			m_pRenderTexture->getViewport(0)->setSkiesEnabled(false);
			m_pRenderTexture->getViewport(0)->setShadowsEnabled(false);
			m_pRenderTexture->getViewport(0)->setMaterialScheme(Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
		}
		
		if (!m_pMiniScreen)
		{
			m_pMiniScreen = new Ogre::Rectangle2D(true);
			
			m_pMiniScreen->setRenderQueueGroup(RENDER_QUEUE_BACKGROUND);
		
			Ogre::Real f = 1.f - ((float)(mViewport->getActualWidth() - (s_right_panel_width + s_left_panel_width)) / (float)mViewport->getActualWidth());
			//m_pMiniScreen->setCorners(-1.05f + 0.05 * f, 1.34f, 1.05f - 2.05f * f, -1.34f);
			//m_pMiniScreen->setCorners(-1.0f, 1.0f, 1.0f - 2.0f * f, -1.0f);
			m_pMiniScreen->setCorners(-1.0f + 1.0f * f, 1.0f, 1.0f - 1.0f * f, -1.0f);
			
			m_pMiniScreen->setBoundingBox(Ogre::AxisAlignedBox(-100000.0f * Ogre::Vector3::UNIT_SCALE, 100000.0f * Ogre::Vector3::UNIT_SCALE));
			Ogre::SceneNode* miniScreenNode = mSceneMgr->getRootSceneNode()->createChildSceneNode("MiniScreenNode");
			miniScreenNode->attachObject(m_pMiniScreen);
		
		
			//Ogre::MaterialPtr renderMaterial = Ogre::MaterialManager::getSingleton().getByName("RttMatTest", Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);
			Ogre::MaterialPtr renderMaterial = Ogre::MaterialManager::getSingleton().create("RttMat", Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);
			renderMaterial->setSceneBlending(Ogre::SBF_SOURCE_ALPHA, Ogre::SBF_ONE_MINUS_SOURCE_ALPHA);
		
			Ogre::Technique *technique = renderMaterial->createTechnique();
			technique->createPass();
		
			renderMaterial->getTechnique(0)->getPass(0)->setLightingEnabled(false);
			renderMaterial->getTechnique(0)->getPass(0)->setDepthWriteEnabled(false);
			renderMaterial->getTechnique(0)->getPass(0)->setCullingMode(CULL_NONE);
			Ogre::TextureUnitState* state = renderMaterial->getTechnique(0)->getPass(0)->createTextureUnitState("RttTex");
			state->setTextureFiltering(Ogre::FO_LINEAR, Ogre::FO_LINEAR, Ogre::FO_NONE);
			//renderMaterial->getTechnique(0)->getPass(0)->setSceneBlending(Ogre::SBT_TRANSPARENT_COLOUR);
			//renderMaterial->getTechnique(0)->getPass(0)->setSceneBlending(Ogre::SBT_TRANSPARENT_ALPHA);
		
			m_pMiniScreen->setMaterial("RttMat");
			//m_pMiniScreen->setMaterial("shortline_land_desert_OSX");
			m_pMiniScreen->setVisible(true);
		}
		else
		{
			//Ogre::Real f = 1.f - ((float)(mViewport->getActualWidth() - s_right_panel_width) / (float)mViewport->getActualWidth());
			//m_pMiniScreen->setCorners(-1.05f + 0.05 * f, 1.34f, 1.05f - 2.05f * f, -1.34f);
		}
	}
#else
	m_pRTSceneMgr = mSceneMgr;
#endif
	//------------------------------------------------------
}

void COgreScene::updateRenderTexture()
{
#ifdef RTT_BACKGROUND
	if (m_pRenderTexture)
		m_pRenderTexture->update();
#endif
}

void COgreScene::setupControls()
{
	//createButton(TrayLocation trayLoc, const Ogre::String& name, const Ogre::String& caption, Ogre::Real width = 0)
	if (!mBoostButton)
	{
		mTrayMgr->setTrayPadding(0.0);
		mTrayMgr->setWidgetPadding(0.0);
		mTrayMgr->setWidgetSpacing(0.0);
		mTrayMgr->showBackdrop("SdkTrays/Empty");
		
		//------------
		
		Ogre::OverlayElement* shadow = NULL;
		float shadowWidth = 0;
		
		float panelHeight = mViewport->getActualWidth() * (3.f / 4.f);
		float fDecor = (float)mWindow->getViewport(0)->getActualWidth() / 2048.f;
		mRightPanel = (Ogre::PanelOverlayElement*)Ogre::OverlayManager::getSingleton().createOverlayElementFromTemplate("Rails/ChaptersRightBackPanel", "", "RightPanel");
		mRightPanel->setMaterialName(RailsContext::getShared()->getChaptersBackgroundMatName());
		mRightPanel->setDimensions(s_right_panel_width + s_additional_right_panel_width, panelHeight);
		mRightPanel->setUV(1 - (float)(s_right_panel_width + s_additional_right_panel_width) / (float)mViewport->getActualWidth(), 0, 1, 1);
		mRightPanel->setPosition(mViewport->getActualWidth() - (s_right_panel_width + s_additional_right_panel_width), mViewport->getActualHeight() / 2.f - mRightPanel->getHeight() / 2.f);
		mTrayMgr->getBackdropLayer()->add2D(mRightPanel);
		shadow = (Ogre::OverlayElement*)((Ogre::OverlayContainer*)mRightPanel)->getChild(mRightPanel->getName() + "/ChaptersRightBorderPanel");
		shadow->setHeight(panelHeight);
		shadow->setWidth((shadow->getWidth() * fDecor));
		shadowWidth = shadow->getWidth();
		shadow = (Ogre::OverlayElement*)((Ogre::OverlayContainer*)mRightPanel)->getChild(mRightPanel->getName() + "/ChaptersRightBorderPanelShadow");
		shadow->setHeight(panelHeight);
		shadow->setWidth((shadow->getWidth() * fDecor));
		shadow->setLeft((shadow->getLeft() * fDecor));
		//shadowWidth += shadow->getWidth();
		
		
		mLeftPanel = (Ogre::PanelOverlayElement*)Ogre::OverlayManager::getSingleton().createOverlayElementFromTemplate("Rails/ChaptersLeftBackPanel", "", "LeftPanel");
		mLeftPanel->setMaterialName(RailsContext::getShared()->getChaptersBackgroundMatName());
		mLeftPanel->setDimensions(s_left_panel_width, panelHeight);
		mLeftPanel->setUV(0, 0, (float)s_left_panel_width / (float)mViewport->getActualWidth(), 1);
		mLeftPanel->setPosition(0, mViewport->getActualHeight() / 2.f - mLeftPanel->getHeight() / 2.f);
		mTrayMgr->getBackdropLayer()->add2D(mLeftPanel);
		shadow = (Ogre::OverlayElement*)((Ogre::OverlayContainer*)mLeftPanel)->getChild(mLeftPanel->getName() + "/ChaptersLeftBorderPanel");
		shadow->setHeight(panelHeight);
		shadow->setWidth((shadow->getWidth() * fDecor));
		shadow->setLeft((shadow->getLeft() * fDecor));
		shadow = (Ogre::OverlayElement*)((Ogre::OverlayContainer*)mLeftPanel)->getChild(mLeftPanel->getName() + "/ChaptersLeftBorderPanelShadow");
		shadow->setHeight(panelHeight);
		shadow->setWidth((shadow->getWidth() * fDecor));
		shadow->setLeft((shadow->getLeft() * fDecor));
		
		mTrayMgr->getTraysLayer()->setZOrder(101);
		
		//------------
		{
			float f = (float)mWindow->getViewport(0)->getActualHeight() / 1024.f;
			//f *= (jNeedsUseLowResTextures()) ? 0.9f : 0.75f;
			f *= 0.75;
			
			//CXPoint p0 = CShortLineModel::sharedModel()->convertFromGameCoord(CXMakePoint(0, 0), s_scene_rect);
			//CXPoint p1 = CShortLineModel::sharedModel()->convertFromGameCoord(CXMakePoint(10, 10), s_scene_rect);
			//float smallRectWidth = ABS(p1.x-p0.x);
			//float smallRectHeight = ABS(p1.y-p0.y);
			
			mCommonMessagePanel = mTrayMgr->createDecorWidget(TL_NONE, "CommonMessageBox", "Rails/messageBox");
			mCommonMessagePanelEl = (Ogre::BorderPanelOverlayElement*)mCommonMessagePanel->getOverlayElement();
			if (jNeedsUseLowResTextures())
				mCommonMessagePanelEl->setBorderSize(20, 20, 20, 20);
			//__android_log_print(ANDROID_LOG_INFO, "Rails", ">>>>>>>>>>>>> %f x %f", smallRectWidth, smallRectHeight);
			//mCommonMessagePanelEl->setDimensions(smallRectWidth, smallRectHeight);
			float chH = 45 * f;
			mCommonMessagePanelBaseWidth = 540 * f;
			if (chH < 15)
			{
				float asp = 15.f / chH;
				mCommonMessagePanelBaseWidth = mCommonMessagePanelBaseWidth * asp;
				chH = 15;
			}
			mCommonMessagePanelBaseHeight = chH;
			mCommonMessagePanelEl->setDimensions(mCommonMessagePanelBaseWidth, mCommonMessagePanelBaseHeight);
			mCommonMessagePanelEl->setEnabled(false);
			mCommonMessagePanelLabel = (Ogre::TextAreaOverlayElement*)((Ogre::OverlayContainer*)mCommonMessagePanelEl)->getChild(mCommonMessagePanelEl->getName() + "/BlackMessageBoxLabel");
			mCommonMessagePanelLabel->setCharHeight(chH);
			mCommonMessagePanelLabel->setSpaceWidth(mCommonMessagePanelLabel->getCharHeight() * 0.3);
			mCommonMessagePanelLabel->setWidth(mCommonMessagePanelEl->getWidth() * 0.85);
			mCommonMessagePanelLabel->setHeight(mCommonMessagePanelEl->getHeight() * 0.85);
			mCommonMessagePanel->hide();
			//mTrayMgr->adjustTrays();
			
			/* // test for COgreScene::convertFromGameCoord()
			float size = 20;
			CXPoint offset = CXMakePoint(- size / 2.f, - mWindow->getViewport(0)->getActualHeight() / 2.f - size / 2.f);
			for (int x = 0; x < NODES_WIDTH; x++)
			{
				for (int y = 0;  y < NODES_HEIGHT; y++)
				{
					CXPoint point = convertFromGameCoord(CXMakePoint((float)x-1, (float)y-1));
					OgreBites::DecorWidget* pnl = mTrayMgr->createDecorWidget(TL_NONE, "CommonMessageBox" + StringConverter::toString(x) + "_" + StringConverter::toString(y), "Rails/messageBox");
					Ogre::BorderPanelOverlayElement* pnlEl = (Ogre::BorderPanelOverlayElement*)pnl->getOverlayElement();
					pnlEl->setBorderSize(10, 10, 10, 10);
					pnlEl->setEnabled(false);
					pnlEl->setDimensions(size, size);
					pnlEl->setTop(offset.y + point.y);
					pnlEl->setLeft(offset.x + point.x);
					//pnl->hide();
				}
			}
			//*/
		}
		//------------
		float buttonsH = 140.f * 4.f; // 3 bonus buttons + 1 boost button
		float f = ((float)(s_right_panel_width + s_additional_right_panel_width) - 2.f * mTrayMgr->getWidgetPadding()) / (160.f);
		if (!isRightOrientedInterface() && (buttonsH * f > mViewport->getActualHeight()))
		{
			f = mViewport->getActualHeight() / (buttonsH);
		}
		else if (isRightOrientedInterface())
		{
			buttonsH = 140.f * 5.f + 220 * 2 + 15 * 5; // 3 bonus buttons + 1 boost button + 1 pause button + progress + trains/money + 5 separators
			if (buttonsH * f > mViewport->getActualHeight())
				f = mViewport->getActualHeight() / (buttonsH);
		}
		
		float blankPanelWidth = (s_right_panel_width + s_additional_right_panel_width) - 2.f * mTrayMgr->getWidgetPadding() - shadowWidth;
		
		mTrayMgr->createSeparator(TL_TOPLEFT, "LeftBlankDecor", blankPanelWidth, 40 * f, true);
		mTrayMgr->createSeparator(TL_TOPRIGHT, "RightBlankDecor", blankPanelWidth, 40 * f, true);
		//mTrayMgr->createGameButton(TL_TOPLEFT, "PauseButton", "Rails/pause", "", true, 160 * f, 170 * f);
		//mTrayMgr->createGameButton(TL_TOPLEFT, "ResumeButton", "Rails/resume", "", true, 160 * f, 170 * f);
		
		//------------- RIGHT --------------
		
		mBonusButton0 = mTrayMgr->createGameButton(TL_TOPRIGHT, "BonusButton0", "Rails/bonus_call", "", false, 116 * f, 140 * f);
		mBonusButton1 = mTrayMgr->createGameButton(TL_TOPRIGHT, "BonusButton1", "Rails/bonus_call", "", false, 116 * f, 140 * f);
		mBonusButton2 = mTrayMgr->createGameButton(TL_TOPRIGHT, "BonusButton2", "Rails/bonus_call", "", false, 116 * f, 140 * f);
		mBonusButton0->setMaskOffsetAndSize(11 * f, 7 * f, 92.5 * f, 92.5 * f);
		mBonusButton1->setMaskOffsetAndSize(11 * f, 7 * f, 92.5 * f, 92.5 * f);
		mBonusButton2->setMaskOffsetAndSize(11 * f, 7 * f, 92.5 * f, 92.5 * f);
		
		//----
		int rightDecorHeight = 15 * f;
		
		if (isRightOrientedInterface())
		{
			mTrayMgr->getWidget("RightBlankDecor")->getOverlayElement()->setHeight(2 * rightDecorHeight);
			mTrayMgr->moveWidgetToTray("RightBlankDecor", TL_TOPRIGHT, 0);
			//mTrayMgr->createSeparator(TL_TOPRIGHT, "SecondRightBlankDecor", blankPanelWidth, rightDecorHeight, true);
		}
		else
		{
			mBoostButton = mTrayMgr->createGameButton(TL_BOTTOMRIGHT, "BoostButton", "Rails/x1", "", true, 116 * f, 140 * f);
			mTrayMgr->createSeparator(TL_BOTTOMRIGHT, "SecondRightBlankDecor", blankPanelWidth, 10 * f, true);
		}
		
		//------------- LEFT --------------
		
		if (isRightOrientedInterface())
		{
			//mTrayMgr->createSeparator(TL_TOPRIGHT, "PBRightBlankDecor", blankPanelWidth, 2 * rightDecorHeight, true);
			mPauseButton = mTrayMgr->createGameButton(TL_TOPRIGHT, "PauseButton", "Rails/pause", "", true, 116 * f, 140 * f);
			mMenuButton = mTrayMgr->createGameButton(TL_TOPRIGHT, "MenuButton", "Rails/menu", "", true, 131 * f, 140 * f);
			//mTrayMgr->moveWidgetToTray("PBRightBlankDecor", TL_TOPRIGHT, 0);
			mTrayMgr->moveWidgetToTray("PauseButton", TL_TOPRIGHT, 1);
			
			//mTrayMgr->createSeparator(TL_TOPRIGHT, "SecondLeftBlankDecor", blankPanelWidth, rightDecorHeight * 1.2, true);
			mTrainsMoneyLabel = mTrayMgr->createDecorWidget(TL_BOTTOMRIGHT, "TrainsLabel", "Rails/TrainsMoneyLabel", f);
			mTrayMgr->createSeparator(TL_BOTTOMRIGHT, "GPRightBlankDecor", blankPanelWidth, rightDecorHeight, true);
			mYearProgress = mTrayMgr->createDecorWidget(TL_BOTTOMRIGHT, "GameProgressBar", "Rails/GameProgressBar", f);
			mTrayMgr->createSeparator(TL_BOTTOMRIGHT, "BottomRightSpacingBlankDecor", blankPanelWidth, 2 * rightDecorHeight, true);
			
			mBoostButton = mTrayMgr->createGameButton(TL_BOTTOMRIGHT, "BoostButton", "Rails/x1", "", true, 116 * f, 140 * f);
		}
		else
		{
			mPauseButton = mTrayMgr->createGameButton(TL_TOPLEFT, "PauseButton", "Rails/pause", "", true, 116 * f, 140 * f);
			mMenuButton = mTrayMgr->createGameButton(TL_TOPLEFT, "MenuButton", "Rails/menu", "", true, 131 * f, 140 * f);
			
			mTrayMgr->createSeparator(TL_TOPLEFT, "SecondLeftBlankDecor", blankPanelWidth, 16 * f, true);
			
			mTrainsMoneyLabel = mTrayMgr->createDecorWidget(TL_TOPLEFT, "TrainsLabel", "Rails/TrainsMoneyLabel", f);
			mYearProgress = mTrayMgr->createDecorWidget(TL_BOTTOMLEFT, "GameProgressBar", "Rails/GameProgressBar", f);
			mTrayMgr->createSeparator(TL_BOTTOMLEFT, "ThirdLeftBlankDecor", blankPanelWidth, 42 * f, true);
			
			if (mTrayMgr->getTrayContainer(TL_TOPLEFT)->getHeight() + mTrayMgr->getTrayContainer(TL_BOTTOMLEFT)->getHeight() > mViewport->getActualHeight())
			{
				mTrayMgr->getWidget("SecondLeftBlankDecor")->getOverlayElement()->setHeight(0);
			}
		}
		
		
		
		//----
		
		//mTrainsMoneyLabel = mTrayMgr->createDecorWidget(trayLoc, "TrainsLabel", "Rails/TrainsMoneyLabel", f);
		Ogre::PanelOverlayElement* panel = (Ogre::PanelOverlayElement*)mTrainsMoneyLabel->getOverlayElement();
		
		Ogre::PanelOverlayElement* trainsMoneyBack = (Ogre::PanelOverlayElement*)((Ogre::OverlayContainer*)panel)->getChild(panel->getName() + "/TrainsMoneyLabelBack");
		trainsMoneyBack->setPosition(trainsMoneyBack->getLeft() * f, trainsMoneyBack->getTop() * f);
		trainsMoneyBack->setDimensions(trainsMoneyBack->getWidth() * f, trainsMoneyBack->getHeight() * f);
		
		mLevelLabelCaption = (Ogre::TextAreaOverlayElement*)((Ogre::OverlayContainer*)panel)->getChild(panel->getName() + "/TrainsMoneyLabel/LevelCaption");
		mLevelLabelCaption->setCharHeight(mLevelLabelCaption->getCharHeight() * f);
		mLevelLabelCaption->setFontName(RailsContext::getShared()->getFontNameForSize(mLevelLabelCaption->getCharHeight(), "BCaption"));
		mLevelLabelCaption->setTop(mLevelLabelCaption->getTop() * f);
		mLevelLabelCaption->setLeft(mLevelLabelCaption->getLeft() * f);
		mTrainsLabelCaption = (Ogre::TextAreaOverlayElement*)((Ogre::OverlayContainer*)panel)->getChild(panel->getName() + "/TrainsMoneyLabel/TrainsCaption");
		mTrainsLabelCaption->setCharHeight(mTrainsLabelCaption->getCharHeight() * f);
		mTrainsLabelCaption->setFontName(RailsContext::getShared()->getFontNameForSize(mTrainsLabelCaption->getCharHeight(), "BCaption"));
		mTrainsLabelCaption->setTop(mTrainsLabelCaption->getTop() * f);
		mTrainsLabelCaption->setLeft(mTrainsLabelCaption->getLeft() * f);
		mMoneyLabelCaption = (Ogre::TextAreaOverlayElement*)((Ogre::OverlayContainer*)panel)->getChild(panel->getName() + "/TrainsMoneyLabel/MoneyCaption");
		mMoneyLabelCaption->setCharHeight(mMoneyLabelCaption->getCharHeight() * f);
		mMoneyLabelCaption->setFontName(RailsContext::getShared()->getFontNameForSize(mMoneyLabelCaption->getCharHeight(), "BCaption"));
		mMoneyLabelCaption->setTop(mMoneyLabelCaption->getTop() * f);
		mMoneyLabelCaption->setLeft(mMoneyLabelCaption->getLeft() * f);
		
		//----
		
		//mYearProgress = mTrayMgr->createDecorWidget(TL_BOTTOMLEFT, "GameProgressBar", "Rails/GameProgressBar", f);
		Ogre::PanelOverlayElement* barPanel = (Ogre::PanelOverlayElement*)mYearProgress->getOverlayElement();
		
		Ogre::PanelOverlayElement* progressBarBack = (Ogre::PanelOverlayElement*)((Ogre::OverlayContainer*)barPanel)->getChild(barPanel->getName() + "/GameProgressBarBack");
		progressBarBack->setPosition(progressBarBack->getLeft() * f, progressBarBack->getTop() * f);
		progressBarBack->setDimensions(progressBarBack->getWidth() * f, progressBarBack->getHeight() * f);
		
		mProgressBarThumb = (Ogre::PanelOverlayElement*)((Ogre::OverlayContainer*)barPanel)->getChild(barPanel->getName() + "/GameProgressBarThumb");
		mProgressBarThumb->setPosition(mProgressBarThumb->getLeft() * f, mProgressBarThumb->getTop() * f);
		mProgressBarThumb->setDimensions(mProgressBarThumb->getWidth() * f, mProgressBarThumb->getHeight() * f);
		mProgressBarThumbHeight = mProgressBarThumb->getHeight();
		mProgressBarThumbTop = mProgressBarThumb->getTop();
		
		mYearCaption = (Ogre::TextAreaOverlayElement*)((Ogre::OverlayContainer*)barPanel)->getChild(barPanel->getName() + "/GameProgressBarCaption");
		mYearCaption->setCharHeight(mYearCaption->getCharHeight() * f);
		mYearCaption->setFontName(RailsContext::getShared()->getFontNameForSize(mYearCaption->getCharHeight(), "BCaption"));
		mYearCaption->setTop(mYearCaption->getTop() * f);
		mYearCaption->setLeft(mYearCaption->getLeft() * f);
		
		mYearLabel = (Ogre::TextAreaOverlayElement*)((Ogre::OverlayContainer*)barPanel)->getChild(barPanel->getName() + "/GameProgressBarLabel");
		mYearLabel->setCharHeight(mYearLabel->getCharHeight() * f);
		mYearLabel->setFontName(RailsContext::getShared()->getFontNameForSize(mYearLabel->getCharHeight(), "BCaption"));
		mYearLabel->setTop(mYearLabel->getTop() * f);
		mYearLabel->setLeft(mYearLabel->getLeft() * f);
		
		//mTrayMgr->createSeparator(TL_BOTTOMLEFT, "ThirdLeftBlankDecor", blankPanelWidth, 42 * f, true);
		
		//----
		
		
		float ff = mViewport->getActualHeight() * 0.7 / 1024.f;
		float posX = (mViewport->getActualWidth() / 2.f) - (isRightOrientedInterface() ? (s_right_panel_width + s_additional_right_panel_width) / 2.f : 0);
		mTopTab = mTrayMgr->createDecorWidget(TL_NONE, "TopTab", "Rails/TopTab", ff);
		((Ogre::BorderPanelOverlayElement*)mTopTab->getOverlayElement())->setBorderSize(((Ogre::BorderPanelOverlayElement*)mTopTab->getOverlayElement())->getLeftBorderSize() * ff, 0);
		int chHeight = (int)(mTopTab->getOverlayElement()->getHeight() * 0.85);
		mDemoLabel = mTrayMgr->createLabel(TL_NONE, "DemoLabel", jGetLocalizedString("Demo"), 200, true);
		mDemoLabel->getTextAreaElement()->setFontName(RailsContext::getShared()->getFontNameForSize(chHeight, "BCaption"));
		mDemoLabel->setColour(Ogre::ColourValue(0, 0, 0, 1));
		mDemoLabel->setCharHeight(chHeight);
		float captW = Widget::getCaptionWidth(mDemoLabel->getCaption(), mDemoLabel->getTextAreaElement());
		mTopTab->getOverlayElement()->setWidth(captW + 80 * ff);
		mTopTab->getOverlayElement()->setPosition(posX - mTopTab->getOverlayElement()->getWidth() / 2, 0);
		mDemoLabel->getOverlayElement()->setHeight(mTopTab->getOverlayElement()->getHeight());
		mDemoLabel->getOverlayElement()->setPosition((int)(posX - mDemoLabel->getOverlayElement()->getWidth() / 2.0), 0);
		mDemoLabel->getTextAreaElement()->setTop( - mDemoLabel->getOverlayElement()->getHeight() / 1.9f);
		
		//----
		
		
		//mRestartButton = mTrayMgr->createGameButton(TL_TOPRIGHT, "RestartButton", "Rails/restart", "", true, 160 * f, 170 * f);
		//mTrayMgr->createGameButton(TL_TOPLEFT, "CompleteLevelButton", "Rails/resume", "", true, 160 * f, 170 * f);
		// mBonusButton0->setMaskOffsetAndSize(6, 4, 46, 46);
		// mBonusButton1->setMaskOffsetAndSize(6, 4, 46, 46);
		// mBonusButton2->setMaskOffsetAndSize(6, 4, 46, 46);
		
		// mRPWSlider = mTrayMgr->createLongSlider(TL_TOP, "RightPanelWidth", "Right Panel Width", 0, 640, 130, 1, 960, 960, 35);
		// mRPWSlider->setValue(s_right_panel_width, false);
		// mLPWSlider = mTrayMgr->createLongSlider(TL_TOP, "LeftPanelWidth", "Left Panel Width", 0, 640, 130, 1, 960, 960, 35);
		// mLPWSlider->setValue(s_left_panel_width, false);
		
		// float ff = (float)mWindow->getViewport(0)->getActualHeight() / 1536.f;
		// mTopTab = mTrayMgr->createDecorWidget(TL_NONE, "TopTab", "Rails/TopTab", ff);
		// mTopTab->getOverlayElement()->setWidth(220 * ff);
		// ((Ogre::BorderPanelOverlayElement*)mTopTab->getOverlayElement())->setBorderSize(((Ogre::BorderPanelOverlayElement*)mTopTab->getOverlayElement())->getLeftBorderSize() * ff, 0);
		// mTopTab->getOverlayElement()->setPosition(mViewport->getActualWidth() / 2 - mTopTab->getOverlayElement()->getWidth() / 2, 0);
		// mMoneyTrainsBack = mTrayMgr->createDecorWidget(TL_NONE, "MoneyTrainsBack", "Rails/MoneyTrains", ff);
		// mMoneyTrainsBack->getOverlayElement()->setPosition(mTopTab->getOverlayElement()->getLeft() - mMoneyTrainsBack->getOverlayElement()->getWidth() * 1.5, 0);
		// float w5 = mMoneyTrainsBack->getOverlayElement()->getWidth() / 5;
		// float mtBackH = mMoneyTrainsBack->getOverlayElement()->getHeight();
		// float mtCharH = mMoneyTrainsBack->getOverlayElement()->getHeight() * 0.5;
		
		
		// mMoneyLabel = mTrayMgr->createLabel(TL_NONE, "MoneyLabel", "", 0, true);
		// mMoneyLabel->getTextAreaElement()->setFontName(RailsContext::getShared()->getFontNameForSize(mtCharH, "BCaption"));
		// mMoneyLabel->setCharHeight(mtCharH);
		// mMoneyLabel->getOverlayElement()->setPosition(mMoneyTrainsBack->getOverlayElement()->getLeft() + w5 * 2.1, mMoneyTrainsBack->getOverlayElement()->getTop() + mMoneyTrainsBack->getOverlayElement()->getHeight() * 0.18);
		// mMoneyLabel->getOverlayElement()->setHeight(mMoneyTrainsBack->getOverlayElement()->getHeight());
		// mMoneyLabel->getTextAreaElement()->setTop( - mMoneyLabel->getOverlayElement()->getHeight() / 2.0f);
		// mMoneyLabel->setColour(Ogre::ColourValue(1, 1, 1, 1));
		// mTrainsLabel = mTrayMgr->createLabel(TL_NONE, "TrainsLabel", "", 0, true);
		// mTrainsLabel->getTextAreaElement()->setFontName(RailsContext::getShared()->getFontNameForSize(mtCharH, "BCaption"));
		// mTrainsLabel->setCharHeight(mtCharH);
		// mTrainsLabel->getOverlayElement()->setPosition(mMoneyTrainsBack->getOverlayElement()->getLeft() + w5 * 4.2, mMoneyTrainsBack->getOverlayElement()->getTop() + mMoneyTrainsBack->getOverlayElement()->getHeight() * 0.18);
		// mTrainsLabel->getOverlayElement()->setHeight(mMoneyTrainsBack->getOverlayElement()->getHeight());
		// mTrainsLabel->getTextAreaElement()->setTop( - mTrainsLabel->getOverlayElement()->getHeight() / 2.0f);
		// mTrainsLabel->setColour(Ogre::ColourValue(1, 1, 1, 1));
		
		// mLevelLabel = mTrayMgr->createLabel(TL_NONE, "LevelLabel", "", 200, true);
		// mLevelLabel->setColour(Ogre::ColourValue(0, 0, 0, 1));
		// mLevelLabel->setCharHeight((int)(mTopTab->getOverlayElement()->getHeight() * 0.75));
		// mLevelLabel->getTextAreaElement()->setFontName(RailsContext::getShared()->getFontNameForSize(mLevelLabel->getCharHeight(), "Caption"));
		// mLevelLabel->getOverlayElement()->setHeight(mTopTab->getOverlayElement()->getHeight());
		// mLevelLabel->getOverlayElement()->setPosition((int)(mViewport->getActualWidth() / 2.0 - mLevelLabel->getOverlayElement()->getWidth() / 2.0), mLevelLabel->getOverlayElement()->getHeight() * 0.1);
		// mLevelLabel->getTextAreaElement()->setTop( - mLevelLabel->getOverlayElement()->getHeight() / 2.0f);
		// 
		// mLevelLabel->setTextAlignment(Ogre::TextAreaOverlayElement::Center);
		// mTrainsLabel->setTextAlignment(Ogre::TextAreaOverlayElement::Center);
		// mMoneyLabel->setTextAlignment(Ogre::TextAreaOverlayElement::Center);
		
	}
	else
	{
		float buttonsH = 140.f * 4.f;
		float f = ((float)s_right_panel_width - 2.f * mTrayMgr->getWidgetPadding()) / (160.f);
		if (buttonsH * f > mViewport->getActualHeight())
			f = mViewport->getActualHeight() / (buttonsH);
		
		
		//mTrayMgr->createGameButton(TL_TOPLEFT, "PauseButton", "Rails/pause", "", true, 160 * f, 170 * f);
		//mTrayMgr->createGameButton(TL_TOPLEFT, "ResumeButton", "Rails/resume", "", true, 160 * f, 170 * f);
		mBoostButton->setSize(116 * f, 140 * f);
	
		mBonusButton0->setSize(116 * f, 140 * f);
		mBonusButton1->setSize(116 * f, 140 * f);
		mBonusButton2->setSize(116 * f, 140 * f);
		mBonusButton0->setMaskOffsetAndSize(12 * f, 8 * f, 92 * f, 92 * f);
		mBonusButton1->setMaskOffsetAndSize(12 * f, 8 * f, 92 * f, 92 * f);
		mBonusButton2->setMaskOffsetAndSize(12 * f, 8 * f, 92 * f, 92 * f);
		
		//mRestartButton->setSize(160 * f, 170 * f);
		
		mTrayMgr->adjustTrays();
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////

void COgreScene::showPauseDialog()
{
	writeSnapshot();
	
	CSoundManager::sharedManager()->playIntroMusic();
	_gamePaused = true;
	
	setupPauseDialog();
	
	OgreBites::Label* levelLabel = (OgreBites::Label*)mPauseTrayMgr->getWidget("PDLevelLabel");
	levelLabel->setCaption(jGetLocalizedString("Level") + " " + StringConverter::toString(CShortLineModel::sharedModel()->levelIndex()));
	
	OgreBites::Button* btn = (OgreBites::Button*)mPauseTrayMgr->getWidget("PDResumeButton");
	
	float btnWidth = btn->getOverlayElement()->getWidth();
	float captW = Widget::getCaptionWidth(levelLabel->getCaption(), levelLabel->getTextAreaElement());
	
	//btn->getOverlayElement()->setLeft();// mWindow->getViewport(0)->getActualWidth() / 2.f - btn->getOverlayElement()->getWidth() * 1.5f
	//levelLabel->getOverlayElement()->setLeft();//mWindow->getViewport(0)->getActualWidth() / 2.f - dlgBack->getOverlayElement()->getWidth() * 0.1
	
	OgreBites::DecorWidget* boxDW = (OgreBites::DecorWidget*)mPauseTrayMgr->getWidget(TL_NONE, "PDResumeBox");
	Ogre::PanelOverlayElement* boxOE = (Ogre::PanelOverlayElement*)boxDW->getOverlayElement();
	boxOE->setWidth(btnWidth * 1.2 + captW);
	boxOE->setLeft(mWindow->getViewport(0)->getActualWidth() / 2.f - boxOE->getWidth() / 2.f - btnWidth * 0.0625);
	boxOE->_update(); // bugfix (the label has wrong position at first show pause dialog)
	
	if (CSoundManager::sharedManager()->isDesabledAllSounds())
		mSoundButton->setMaterial("Rails/Sound_off");
	else
		mSoundButton->setMaterial("Rails/Sound_on");
	
	hideAllTrays();
	mTrayMgr->showAll();
	mPauseTrayMgr->showAll();
}

void COgreScene::setupPauseDialog()
{
	if (mPauseTrayMgr && !mPauseTrayMgr->hasWidget(TL_CENTER, "PDBackPanel"))
	{
		mPauseTrayMgr->setBackdropMaterial("SdkTrays/Shade");
		mPauseTrayMgr->getBackdropLayer()->setZOrder(110);
		
		float f = (float)mWindow->getViewport(0)->getActualHeight() / 1024.f;
		f *= (jNeedsUseLowResOverlays()) ? 0.9 : 0.75f;
		
		OgreBites::DecorWidget* dlgBack = mPauseTrayMgr->createDecorWidget(TL_CENTER, "PDBackPanel", "Rails/PauseDialogBackPanel", f);
		dlgBack->getOverlayElement()->setMaterialName("Rails/PauseDialogBack");
		OgreBites::GameButton* btn = NULL;
		
		float ff = f * 1.1;
		
		// btn = mPauseTrayMgr->createGameButton(TL_NONE, "PDResumeButton", "Rails/resume", "", true, 160 * ff * 1.2, 170 * ff * 1.2);
		// btn->getOverlayElement()->setPosition(mWindow->getViewport(0)->getActualWidth() / 2.f - btn->getOverlayElement()->getWidth() * 1.5f,
		// 									  mWindow->getViewport(0)->getActualHeight() / 2.f - btn->getOverlayElement()->getHeight() * 0.9f);
		// Ogre::OverlayElement* resumeBtnOEl = btn->getOverlayElement();
		
		btn = mPauseTrayMgr->createGameButton(TL_NONE, "PDRestartButton", "Rails/restart", "", true, 160 * ff, 170 * ff);
		btn->getOverlayElement()->setPosition(mWindow->getViewport(0)->getActualWidth() / 2.f - btn->getOverlayElement()->getWidth() * 1.8f,
											  mWindow->getViewport(0)->getActualHeight() / 2.f + btn->getOverlayElement()->getHeight() * 0.2f);
		btn = mPauseTrayMgr->createGameButton(TL_NONE, "PDMenuButton", "Rails/menu", "", true, 160 * ff, 170 * ff);
		btn->getOverlayElement()->setPosition(mWindow->getViewport(0)->getActualWidth() / 2.f - btn->getOverlayElement()->getWidth() * 0.5f,
											  mWindow->getViewport(0)->getActualHeight() / 2.f + btn->getOverlayElement()->getHeight() * 0.2f);
		btn = mPauseTrayMgr->createGameButton(TL_NONE, "PDSoundButton", "Rails/Sound_on", "", true, 160 * ff, 170 * ff);
		btn->getOverlayElement()->setPosition(mWindow->getViewport(0)->getActualWidth() / 2.f + btn->getOverlayElement()->getWidth() * 0.8f,
											  mWindow->getViewport(0)->getActualHeight() / 2.f + btn->getOverlayElement()->getHeight() * 0.2f);
		mSoundButton = btn;
		
		//---
		OgreBites::DecorWidget* boxDW = mPauseTrayMgr->createDecorWidget(TL_NONE, "PDResumeBox", "Rails/PrelevelBigIcon", f);
		Ogre::PanelOverlayElement* boxOE = (Ogre::PanelOverlayElement*)boxDW->getOverlayElement();
		boxOE->setMaterialName("SdkTrays/Empty");
		
		btn = mPauseTrayMgr->createGameButton(TL_NONE, "PDResumeButton", "Rails/resume", "", true, 160 * ff * 1.2, 170 * ff * 1.2);
		Ogre::OverlayElement* resumeBtnOEl = btn->getOverlayElement();
		boxOE->addChild(resumeBtnOEl);
		
		int chHeight = (int)(dlgBack->getOverlayElement()->getHeight() * 0.12);
		OgreBites::Label* lbl = mPauseTrayMgr->createLabel(TL_NONE, "PDLevelLabel", "", dlgBack->getOverlayElement()->getWidth() * 0.5, true);
		Ogre::OverlayElement* lblOE = lbl->getOverlayElement();
		boxOE->addChild(lblOE);
		Ogre::TextAreaOverlayElement* lblTE = lbl->getTextAreaElement();
		lblTE->setFontName(RailsContext::getShared()->getFontNameForSize(chHeight, "BCaption"));
		lblTE->setAlignment(Ogre::TextAreaOverlayElement::Left);
		lblTE->setHorizontalAlignment(Ogre::GHA_LEFT);
		lbl->setCharHeight(chHeight);
		lbl->setColour(Ogre::ColourValue(1, 1, 1, 1));
		lbl->setCaption(jGetLocalizedString("Level") + ":" + StringConverter::toString(CShortLineModel::sharedModel()->levelIndex()));
		lblOE->setHeight(lbl->getCharHeight());
		lblTE->setTop( - lblOE->getHeight() / 2.0f);
		
		boxOE->setDimensions(10, resumeBtnOEl->getHeight());
		boxOE->setTop(mWindow->getViewport(0)->getActualHeight() / 2.f - boxOE->getHeight() * 0.5f);
		boxOE->setLeft(mWindow->getViewport(0)->getActualWidth() / 2.f - boxOE->getWidth() * 0.5f);
		
		resumeBtnOEl->setPosition(0, - resumeBtnOEl->getHeight() * 0.5);
		lblOE->setPosition(resumeBtnOEl->getWidth() * 1.2, resumeBtnOEl->getTop() + resumeBtnOEl->getHeight() * 0.5 - lblOE->getHeight() * 0.75);
		//---
		
#ifdef ENABLE_DEBUG_DIALOGS
		
		btn = mPauseTrayMgr->createGameButton(TL_NONE, "CompleteLevelWith3StarsButton", "Rails/play_next_level", "", true, 75 * f, 80 * f);
		btn->setMaskMaterial("Rails/star_3rd");
		float starSize = 40 * f;
		btn->setMaskOffsetAndSize(0.8 * starSize, - 0.2 * starSize, starSize, starSize);
		btn->setFill(1);
		btn->getOverlayElement()->setPosition((mWindow->getViewport(0)->getActualWidth()) / 2.f + btn->getOverlayElement()->getWidth() * 1.0f,
											  (mWindow->getViewport(0)->getActualHeight() + dlgBack->getOverlayElement()->getHeight()) * 0.5f - btn->getOverlayElement()->getHeight() * 1.8f);
		
		btn = mPauseTrayMgr->createGameButton(TL_NONE, "CompleteLevelButton", "Rails/play_next_level", "", true, 75 * f, 80 * f);
		btn->getOverlayElement()->setPosition((mWindow->getViewport(0)->getActualWidth()) / 2.f - btn->getOverlayElement()->getWidth() * 2.0f,
											  (mWindow->getViewport(0)->getActualHeight() + dlgBack->getOverlayElement()->getHeight()) * 0.5f - btn->getOverlayElement()->getHeight() * 1.8f);
#endif	
	}
}

void COgreScene::setupFaildLevelDialog()
{
	if (mFaildLevelTrayMgr && !mFaildLevelTrayMgr->hasWidget(TL_CENTER, "FLBackPanel"))
	{
		mFaildLevelTrayMgr->setBackdropMaterial("SdkTrays/Shade");
		mFaildLevelTrayMgr->getBackdropLayer()->setZOrder(110);
		
		float f = (float)mWindow->getViewport(0)->getActualHeight() / 1024.f;
		f *= (jNeedsUseLowResOverlays()) ? 1.f : 0.75f;
		
		OgreBites::DecorWidget* dlgBack = mFaildLevelTrayMgr->createDecorWidget(TL_CENTER, "FLBackPanel", "Rails/FailedLevelBackPanel", f);
		dlgBack->getOverlayElement()->setMaterialName("Rails/FailedLevelBack");
		OgreBites::Button* btn = NULL;
		btn = mFaildLevelTrayMgr->createGameButton(TL_NONE, "FLRestartButton", "Rails/restart", "", true, 160 * f, 170 * f);
		btn->getOverlayElement()->setPosition(mWindow->getViewport(0)->getActualWidth() / 2.f - btn->getOverlayElement()->getWidth() * 2.f,
											  mWindow->getViewport(0)->getActualHeight() / 2.f - btn->getOverlayElement()->getHeight() / 5.f);
		btn = mFaildLevelTrayMgr->createGameButton(TL_NONE, "FLMenuButton", "Rails/menu", "", true, 160 * f, 170 * f);
		btn->getOverlayElement()->setPosition(mWindow->getViewport(0)->getActualWidth() / 2.f + btn->getOverlayElement()->getWidth() * 1.0f,
											  mWindow->getViewport(0)->getActualHeight() / 2.f - btn->getOverlayElement()->getHeight() / 5.f);
		
		OgreBites::Label* lbl = mFaildLevelTrayMgr->createLabel(TL_NONE, "FLLabel", "", dlgBack->getOverlayElement()->getWidth() * 0.85, true);
		lbl->getTextAreaElement()->setFontName(RailsContext::getShared()->getFontNameForSize((int)(dlgBack->getOverlayElement()->getHeight() * 0.09), "Caption"));
		lbl->setCharHeight((int)(dlgBack->getOverlayElement()->getHeight() * 0.09));
		lbl->setColour(Ogre::ColourValue(1, 1, 1, 1));
		lbl->setWrappedText(jGetLocalizedString("Tough_day_out_in_the_yard_Come_on_you_can_do_better_than_that"));
		lbl->getOverlayElement()->setHeight(dlgBack->getOverlayElement()->getHeight() / 2.f);
		lbl->getOverlayElement()->setPosition((int)(mViewport->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() / 1.95), mViewport->getActualHeight() / 2.f - lbl->getOverlayElement()->getHeight());
		lbl->getTextAreaElement()->setTop( - lbl->getOverlayElement()->getHeight() / 4.0f);
	}
}

void COgreScene::setupCompleteLevelDialog()
{
	if (mCompleteLevelTrayMgr && !mCompleteLevelTrayMgr->hasWidget(TL_CENTER, "CLBackPanel"))
	{
		mCompleteLevelTrayMgr->setBackdropMaterial("SdkTrays/Shade");
		mCompleteLevelTrayMgr->getBackdropLayer()->setZOrder(110);
		
		float f = (float)mWindow->getViewport(0)->getActualHeight() / 1024.f;
		f *= (jNeedsUseLowResOverlays()) ? 0.9f : 0.75f;
		
		OgreBites::DecorWidget* dlgBack = mCompleteLevelTrayMgr->createDecorWidget(TL_CENTER, "CLBackPanel", "Rails/CompleteLevelBackPanel", f);
		dlgBack->getOverlayElement()->setMaterialName("Rails/CompleteLevelBack");
		Ogre::PanelOverlayElement* dlgBackEl = (Ogre::PanelOverlayElement*)dlgBack->getOverlayElement();
		
		OgreBites::DecorWidget* star2 = mCompleteLevelTrayMgr->createDecorWidget(TL_NONE, "CLStar2", "Rails/star_2nd", f);
		dlgBackEl->addChild(star2->getOverlayElement());
		star2->getOverlayElement()->setPosition( dlgBackEl->getWidth() / 2.f - star2->getOverlayElement()->getWidth() / 2.f,
												 dlgBackEl->getHeight() / 2.f - dlgBackEl->getHeight() * 0.449f);
		OgreBites::DecorWidget* star3 = mCompleteLevelTrayMgr->createDecorWidget(TL_NONE, "CLStar3", "Rails/star_3rd", f);
		dlgBackEl->addChild(star3->getOverlayElement());
		star3->getOverlayElement()->setPosition( dlgBackEl->getWidth() / 2.f  + star3->getOverlayElement()->getWidth() * 0.6f,
												 dlgBackEl->getHeight() / 2.f - dlgBackEl->getHeight() * 0.444f);
												 
 		OgreBites::DecorWidget* star1 = mCompleteLevelTrayMgr->createDecorWidget(TL_NONE, "CLStar1", "Rails/star_3rd", f);
 		dlgBackEl->addChild(star1->getOverlayElement());
		Ogre::PanelOverlayElement* starPOE = (Ogre::PanelOverlayElement*)star1->getOverlayElement();
		starPOE->setUV(1, 0, 0, 1);
 		starPOE->setLeft(dlgBackEl->getWidth() / 2.f  - star1->getOverlayElement()->getWidth() * 1.6f);
 		starPOE->setTop(dlgBackEl->getHeight() / 2.f - dlgBackEl->getHeight() * 0.444f);
		
		OgreBites::Button* btn = NULL;
		btn = mCompleteLevelTrayMgr->createGameButton(TL_NONE, "CLMenuButton", "Rails/menu", "", true, 160 * f, 170 * f);
		dlgBackEl->addChild(btn->getOverlayElement());
		btn->getOverlayElement()->setPosition( dlgBackEl->getWidth() / 2.f - btn->getOverlayElement()->getWidth() * 2.f,
											   dlgBackEl->getHeight() / 2.f + btn->getOverlayElement()->getHeight());
		btn = mCompleteLevelTrayMgr->createGameButton(TL_NONE, "CLRestartButton", "Rails/restart", "", true, 160 * f, 170 * f);
		dlgBackEl->addChild(btn->getOverlayElement());
		btn->getOverlayElement()->setPosition( dlgBackEl->getWidth() / 2.f - btn->getOverlayElement()->getWidth() / 2.f,
											   dlgBackEl->getHeight() / 2.f + btn->getOverlayElement()->getHeight());
		btn = mCompleteLevelTrayMgr->createGameButton(TL_NONE, "CLPlayNextLevelButton", "Rails/play_next_level", "", true, 160 * f, 170 * f);
		dlgBackEl->addChild(btn->getOverlayElement());
		btn->getOverlayElement()->setPosition( dlgBackEl->getWidth() / 2.f + btn->getOverlayElement()->getWidth() * 1.0f,
											   dlgBackEl->getHeight() / 2.f + btn->getOverlayElement()->getHeight());
		
		mCompleteLevelTrayMgr->adjustTrays();
		
		OgreBites::Label* lbl = mCompleteLevelTrayMgr->createLabel(TL_NONE, "CLLevelLabel", "1", dlgBackEl->getWidth() * 0.2, true);
		dlgBackEl->addChild(lbl->getOverlayElement());
		lbl->getTextAreaElement()->setFontName(RailsContext::getShared()->getFontNameForSize((int)(dlgBackEl->getHeight() * 0.13), "BICaption"));
		lbl->setCharHeight((int)(dlgBackEl->getHeight() * 0.13));
		lbl->setColour(Ogre::ColourValue(1, 1, 1, 1));
		lbl->getOverlayElement()->setHeight(dlgBackEl->getHeight() * 0.05f);
		lbl->getOverlayElement()->setPosition( dlgBackEl->getWidth() / 2. - lbl->getOverlayElement()->getWidth() / 2.f,
											dlgBackEl->getHeight() * 0.14f);
											//dlgBack->getOverlayElement()->getTop() + dlgBack->getOverlayElement()->getHeight() * 0.23f);
		
		lbl = mCompleteLevelTrayMgr->createLabel(TL_NONE, "CLTextLabel", "", dlgBackEl->getWidth() * 0.7, true);
		dlgBackEl->addChild(lbl->getOverlayElement());
		mTextCompleteLevelViewTextBaseHeight = dlgBackEl->getHeight() * 0.06;
		lbl->getTextAreaElement()->setFontName(RailsContext::getShared()->getFontNameForSize(mTextCompleteLevelViewTextBaseHeight, "Caption"));
		lbl->setCharHeight(mTextCompleteLevelViewTextBaseHeight);
		lbl->setColour(Ogre::ColourValue(1, 1, 1, 1));
		lbl->setWrappedText("Whoah! No crashes! ($50 reward)\nYou earned $XXX and set a new record!\n");
		lbl->getOverlayElement()->setHeight(dlgBackEl->getHeight() / 2.f);
		lbl->getOverlayElement()->setLeft( dlgBackEl->getWidth() / 2.f - lbl->getOverlayElement()->getWidth() / 2.0);
		lbl->getOverlayElement()->setTop( - lbl->getOverlayElement()->getHeight() * 0.8);
		lbl->getTextAreaElement()->setTop( - lbl->getOverlayElement()->getHeight() / 2.f);
		
		OgreBites::DecorWidget* achDW = mCompleteLevelTrayMgr->createDecorWidget(TL_NONE, "CLAchievementsBox", "Rails/PrelevelBigIcon", f);
		mAchievementsBox = (Ogre::PanelOverlayElement*)achDW->getOverlayElement();
		//mAchievementsBox->setMaterialName("Rails/bonus_mask");
		mAchievementsBox->setMaterialName("SdkTrays/Empty");
		mAchievementsBox->setHeight(mViewport->getActualHeight() * 0.5 - dlgBackEl->getHeight() * 0.4);
		mAchievementsBox->setWidth(mViewport->getActualWidth() - (s_right_panel_width + s_additional_right_panel_width) - s_left_panel_width);
		mAchievementsBox->setTop(mViewport->getActualHeight() / 2.0 + dlgBackEl->getHeight() * 0.5f);
		mAchievementsBox->setLeft(mViewport->getActualWidth() / 2.0 - mAchievementsBox->getWidth() / 2.0f);
	}
}

void COgreScene::setupPrelevelDialog()
{
	if (mPrelevelTrayMgr && !mPrelevelTrayMgr->hasWidget("PLBackPanel"))
	{
		mPrelevelTrayMgr->setBackdropMaterial("SdkTrays/Shade");
		mPrelevelTrayMgr->getBackdropLayer()->setZOrder(110);
		
		float f = (float)mWindow->getViewport(0)->getActualHeight() / 1536.f;
		f *= (jNeedsUseLowResOverlays()) ? 1.2f : 1.0f;
		
		OgreBites::DecorWidget* dlgBack = mPrelevelTrayMgr->createDecorWidget(TL_NONE, "PLBackPanel", "Rails/PrelevelBackPanel", f);
		dlgBack->getOverlayElement()->setMaterialName("Rails/PrelevelBack");
		dlgBack->getOverlayElement()->setTop(mViewport->getActualHeight() - dlgBack->getOverlayElement()->getHeight());
		dlgBack->getOverlayElement()->setLeft(mViewport->getActualWidth() / 2.f - dlgBack->getOverlayElement()->getWidth() * 0.69);
		//dlgBack->getOverlayElement()->setLeft( - dlgBack->getOverlayElement()->getWidth() * 0.7);
		
		OgreBites::DecorWidget* bigIcon = mPrelevelTrayMgr->createDecorWidget(TL_NONE, "PLTrainTypeBigIcon", "Rails/PrelevelBigIcon", f);
		bigIcon->getOverlayElement()->setPosition(mViewport->getActualWidth() / 2.0 - bigIcon->getOverlayElement()->getWidth() * 1.7,
											 		dlgBack->getOverlayElement()->getTop() + dlgBack->getOverlayElement()->getHeight() * 0.1f);
		
		OgreBites::DecorWidget* trainsBox = mPrelevelTrayMgr->createDecorWidget(TL_NONE, "PLTrainsBox", "Rails/PLTrainsBox", 1.f);
		trainsBox->getOverlayElement()->setDimensions(dlgBack->getOverlayElement()->getWidth() * 0.5, dlgBack->getOverlayElement()->getHeight() * 0.12);
		trainsBox->getOverlayElement()->setPosition(mViewport->getActualWidth() / 2.0 - trainsBox->getOverlayElement()->getWidth() * 0.5,
											 		dlgBack->getOverlayElement()->getTop() + dlgBack->getOverlayElement()->getHeight() * 0.5f - trainsBox->getOverlayElement()->getHeight() * 0.7f);
		mPrelevelTrainsBox = (Ogre::BorderPanelOverlayElement*)trainsBox->getOverlayElement();
		mPrelevelTrainsBox->setMaterialName("SdkTrays/Empty");
		mPrelevelTrainsBox->setBorderMaterialName("SdkTrays/Empty");
		//mPrelevelTrainsBox->setMaterialName("Rails/bonus_mask");
		//mPrelevelTrainsBox->setBorderMaterialName("Rails/bonus_mask");
		
		OgreBites::Label* lbl = mPrelevelTrayMgr->createLabel(TL_NONE, "PLCaption", "Level 1", dlgBack->getOverlayElement()->getWidth() * 0.1, true);
		lbl->getTextAreaElement()->setFontName(RailsContext::getShared()->getFontNameForSize((int)(dlgBack->getOverlayElement()->getHeight() * 0.08), "BICaption"));
		lbl->setCharHeight((int)(dlgBack->getOverlayElement()->getHeight() * 0.08));
		lbl->setColour(Ogre::ColourValue(1, 0.2, 0.2, 1));
		lbl->getOverlayElement()->setHeight(dlgBack->getOverlayElement()->getHeight() * 0.05f);
		lbl->getOverlayElement()->setPosition(mViewport->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() / 2.0,
											  dlgBack->getOverlayElement()->getTop() + dlgBack->getOverlayElement()->getHeight() * 0.01f);
		 
		int chHeight = (int)(dlgBack->getOverlayElement()->getHeight() * 0.03);
		lbl = mPrelevelTrayMgr->createLabel(TL_NONE, "PLTrainsOnThisLevel", jGetLocalizedString("Trains_on_this_level"), 0, true);
		lbl->getTextAreaElement()->setFontName(RailsContext::getShared()->getFontNameForSize(chHeight, "BICaption"));
		lbl->getTextAreaElement()->setAlignment(Ogre::TextAreaOverlayElement::Left);
		lbl->setCharHeight(chHeight);
		lbl->setColour(Ogre::ColourValue(0.42, 0.43, 0.35, 1));
		lbl->getOverlayElement()->setHeight(lbl->getCharHeight());
		lbl->getOverlayElement()->setPosition(mPrelevelTrainsBox->getLeft(), mPrelevelTrainsBox->getTop() - lbl->getOverlayElement()->getHeight());
		mPLTrainsOnLevelLabel = lbl;
		mPrelevelTrainsBox->addChild(mPLTrainsOnLevelLabel->getOverlayElement());
		
		lbl = mPrelevelTrayMgr->createLabel(TL_NONE, "PLTapToContinue", jGetLocalizedString("Tap_to_continue"), dlgBack->getOverlayElement()->getWidth() * 0.1, true);
		lbl->getTextAreaElement()->setFontName(RailsContext::getShared()->getFontNameForSize(chHeight, "BICaption"));
		lbl->setCharHeight(chHeight);
		lbl->setColour(Ogre::ColourValue(0.1, 0.1, 0.1, 1));
		lbl->getOverlayElement()->setHeight(dlgBack->getOverlayElement()->getHeight() * 0.03f);
		lbl->getOverlayElement()->setPosition(mViewport->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() / 2.0, mPrelevelTrainsBox->getTop() + mPrelevelTrainsBox->getHeight() - lbl->getOverlayElement()->getHeight() * 0.5);
		
		lbl = mPrelevelTrayMgr->createLabel(TL_NONE, "PLTextLabelMid", "", dlgBack->getOverlayElement()->getWidth() * 0.5, true);
		lbl->setCharHeight((int)(dlgBack->getOverlayElement()->getHeight() * 0.04));
		lbl->getTextAreaElement()->setFontName(RailsContext::getShared()->getFontNameForSize(lbl->getCharHeight(), "Caption"));
		lbl->setColour(Ogre::ColourValue(0.1, 0.1, 0.1, 1));
		lbl->getOverlayElement()->setHeight(dlgBack->getOverlayElement()->getHeight() * 0.4f);
		lbl->setWrappedText("Meet the Snake Train! He's got tons of cars, but he also brings in loads of money! Check out the Reverse bonus to keep trains from colliding. Oh, and we now have five stations!");
		lbl->getOverlayElement()->setPosition(mViewport->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() / 1.95f,
											  dlgBack->getOverlayElement()->getTop() + dlgBack->getOverlayElement()->getHeight() * 0.12f);
		lbl->getTextAreaElement()->setTop( - lbl->getOverlayElement()->getHeight() / 2.f);
		
		lbl = mPrelevelTrayMgr->createLabel(TL_NONE, "PLTextLabelLeft", "", dlgBack->getOverlayElement()->getWidth() * 0.36, true);
		lbl->getTextAreaElement()->setAlignment(Ogre::TextAreaOverlayElement::Left);
		lbl->setCharHeight((int)(dlgBack->getOverlayElement()->getHeight() * 0.04));
		lbl->getTextAreaElement()->setFontName(RailsContext::getShared()->getFontNameForSize(lbl->getCharHeight(), "Caption"));
		lbl->setColour(Ogre::ColourValue(0.1, 0.1, 0.1, 1));
		lbl->getOverlayElement()->setHeight(dlgBack->getOverlayElement()->getHeight() * 0.4f);
		lbl->setWrappedText("Meet the Snake Train! He's got tons of cars, but he also brings in loads of money! Check out the Reverse bonus to keep trains from colliding. Oh, and we now have five stations!");
		lbl->getOverlayElement()->setPosition(mViewport->getActualWidth() / 2.0 - lbl->getOverlayElement()->getWidth() * 0.78,
											  dlgBack->getOverlayElement()->getTop() + dlgBack->getOverlayElement()->getHeight() * 0.12f);
		lbl->getTextAreaElement()->setTop( - lbl->getOverlayElement()->getHeight() / 2.f);
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////

void COgreScene::showPrelevelPanel()
{
	CSoundManager::sharedManager()->playIntroMusic();
	
	nextMainLoopStep(true);
	nextMainLoopStep(true);
	_gamePaused = true;
	
	preparePrelevelView();
	
	mPrelevelTrayMgr->showAll();
}

void COgreScene::preparePrelevelView()
{
	setupPrelevelDialog();
	
	CShortLineModel* model = CShortLineModel::sharedModel();
	int levelIndex = model->levelIndex();
	// Level Header
	OgreBites::Label* prelevelCaptionLabel = (OgreBites::Label*)mPrelevelTrayMgr->getWidget("PLCaption");
	prelevelCaptionLabel->setCaption(jGetLocalizedString("Level") + " " + StringConverter::toString(levelIndex));
	
	if (model->gameMode()== gm_normal)
	{
		OgreBites::DecorWidget* dlgBack = (OgreBites::DecorWidget*)mPrelevelTrayMgr->getWidget("PLBackPanel");
		OgreBites::Label* prelevelTextLabelMid = (OgreBites::Label*)mPrelevelTrayMgr->getWidget("PLTextLabelMid");
		OgreBites::Label* prelevelTextLabelLeft = (OgreBites::Label*)mPrelevelTrayMgr->getWidget("PLTextLabelLeft");
		OgreBites::Label* prelevelDescription = NULL;
		
		// New Train on the Level
		int newTrainParam = CPrefs::sharedPrefs()->paramNewTrainType(levelIndex);
		OgreBites::DecorWidget* bigIconPrelevelView = (OgreBites::DecorWidget*)mPrelevelTrayMgr->getWidget("PLTrainTypeBigIcon");
		
		if (newTrainParam == 0) // there is no any new train types
		{
			prelevelDescription = prelevelTextLabelMid;
			prelevelTextLabelLeft->hide();
			bigIconPrelevelView->hide();
			//const float expand = 135.;
			//_textPrelevelView.frame = CXMakeRect(472-expand, 360, 240+expand, 120);
			//textAlignment = NSCenterTextAlignment;
		}
		else
		{
			prelevelDescription = prelevelTextLabelLeft;
			prelevelTextLabelMid->hide();
	
			const char* nameTrain = CTrain::nameTrainByLevelParam(newTrainParam);
			Ogre::String bigIconName("Rails/prelevel_big_");
			bigIconName += nameTrain;
			
			//LogManager::getSingleton().logMessage(">>>>>>>>>>>>>> bigIconName: " + bigIconName);
			bigIconPrelevelView->setMaterial(bigIconName);
			bigIconPrelevelView->show();
		}
		
		//BEGIN--- Trains on the level
		int simpleTrainParam = 100;
		int countOfTypes = 0;
		std::vector<Ogre::String> trainsList;
		
		for (int i = 0; i < mPrelevelTrainsImages.size(); i++)
		{
			Widget::nukeOverlayElement(mPrelevelTrainsImages[i]);
		}
		mPrelevelTrainsImages.clear();
		
		for (int i = MAD_PARAM; i <= RUBBER_PARAM; ++i)
		{
			int currParam = CPrefs::sharedPrefs()->parameter(i, levelIndex);
	
			if (currParam > 0)
			{
				simpleTrainParam-=currParam;
		
				const char* nameTrain = CTrain::nameTrainByLevelParam(i);
				Ogre::String smallIconName("Rails/prelevel_");
				smallIconName += nameTrain;
				
				trainsList.push_back(smallIconName);
				++countOfTypes;
			}
		}

		if (simpleTrainParam > 0)
		{
			const char* nameTrain = CTrain::nameTrainByLevelParam(1);
			Ogre::String smallIconName("Rails/prelevel_");
			smallIconName += nameTrain;
			
			trainsList.push_back(smallIconName);
			++countOfTypes;
		}
		
		int trainsCount = trainsList.size();
		if (trainsCount == 0)
		{
			mPrelevelTrainsBox->hide();
			mPLTrainsOnLevelLabel->hide();
		}
		else
		{
			OgreBites::Label* trainsOnThisLevelLabel = (OgreBites::Label*)mPrelevelTrayMgr->getWidget("PLTrainsOnThisLevel");
			float size = 0;
			float indent = 0;
			float sPoint = 0;
			float captW = Widget::getCaptionWidth(trainsOnThisLevelLabel->getCaption(), trainsOnThisLevelLabel->getTextAreaElement());
			
			if (trainsCount <= 6)
				size = dlgBack->getOverlayElement()->getWidth() * 0.45 / 6.0;
			else
				size = dlgBack->getOverlayElement()->getWidth() * 0.45 / 7.0;
			
			indent = 0;//size * 0.05;
			float maxTrInRow = (trainsCount <= 6) ? trainsCount : 7;
			mPrelevelTrainsBox->setDimensions((size + indent) * (maxTrInRow + 0.2) - indent + (trainsCount <= 4 ? captW : 0), size * ((trainsCount <= 6) ? 1.0 : 2.0));
			mPrelevelTrainsBox->setLeft(mViewport->getActualWidth() / 2.0 - mPrelevelTrainsBox->getWidth() * 0.5);
			mPrelevelTrainsBox->setTop(dlgBack->getOverlayElement()->getTop() + dlgBack->getOverlayElement()->getHeight() * 0.5f - mPrelevelTrainsBox->getHeight() * 1.0f);
			
			if (trainsCount <= 4)
			{
				trainsOnThisLevelLabel->getOverlayElement()->setPosition(0, mPrelevelTrainsBox->getHeight() / 2.0 - trainsOnThisLevelLabel->getOverlayElement()->getHeight() / 2.0);
				sPoint = - mPrelevelTrainsBox->getWidth() * 0.5 + captW + size * 0.2;//- trainsCount * (size + indent) / 2.f;
			}
			else if (trainsCount <= 6)
			{
				trainsOnThisLevelLabel->getOverlayElement()->setPosition(mPrelevelTrainsBox->getWidth() / 2.0 - captW / 2.0, - trainsOnThisLevelLabel->getOverlayElement()->getHeight() * 1.5);
				sPoint = - trainsCount * (size + indent) / 2.f;
			}
			else
			{
				trainsOnThisLevelLabel->getOverlayElement()->setPosition(0, mPrelevelTrainsBox->getHeight() / 2.0 - trainsOnThisLevelLabel->getOverlayElement()->getHeight() * 2.2);
				sPoint = - mPrelevelTrainsBox->getWidth() * 0.5 + captW + size * 0.2;//- trainsCount * (size + indent) / 2.f;
				//trainsOnThisLevelLabel->getOverlayElement()->setPosition(0, mPrelevelTrainsBox->getHeight() - trainsOnThisLevelLabel->getOverlayElement()->getHeight() * 2.3);
				//sPoint = - maxTrInRow * (size + indent) / 2.f;
			}
			
			
			for (int idx = 0; idx < trainsCount; idx++)
			{
				Ogre::OverlayElement* e = (Ogre::OverlayElement*)Ogre::OverlayManager::getSingleton().createOverlayElementFromTemplate("Rails/PrelevelSmallIcon", "", "PLTrainImage_" + Ogre::StringConverter::toString(idx));
				e->setMaterialName(trainsList[idx]);
				e->setDimensions(size, size);
			
				//e->setPosition(sPoint + indent + idx * (size + indent), indent);
				if (trainsCount <= 6)
				{
					if (idx < 7)
						e->setPosition(sPoint + idx * (size), 0);
					else
					{
						sPoint = - mPrelevelTrainsBox->getWidth() * 0.5 + captW + size * 0.3;
						e->setPosition(sPoint + (idx - 7) * (size), size * 1.05);
					}
				}
				else
				{
					if (idx < 4)
						e->setPosition(sPoint + idx * (size), 0);
					else
					{
						sPoint = - (trainsCount - 4) * (size + indent) / 2.f;
						e->setPosition(sPoint + (idx - 4) * (size), size * 1.05);
					}
				}
					
				mPrelevelTrainsBox->addChild(e);
				mPrelevelTrainsImages.push_back(e);
			}
			mPrelevelTrainsBox->show();
			mPLTrainsOnLevelLabel->show();
		}
		//END------ Trains on the level
		
		//BEGIN---- Level Description
		Ogre::String keyStr("Prelevel_text_");
		keyStr += StringConverter::toString(levelIndex);
		Ogre::UTFString value = jGetLocalizedString(keyStr.c_str());
		prelevelDescription->setWrappedText(value.asUTF8());
		prelevelDescription->show();
		int cnt = prelevelTextLabelMid->getLinesCount();
		
		if (newTrainParam == 0 && cnt < 6)
		{
			float offset = prelevelTextLabelMid->getCharHeight() * cnt / 2.f;
			
			if (trainsCount < 6)
				prelevelTextLabelMid->getOverlayElement()->setTop(dlgBack->getOverlayElement()->getTop() + dlgBack->getOverlayElement()->getHeight() * 0.25f - offset);
			else
				prelevelTextLabelMid->getOverlayElement()->setTop(dlgBack->getOverlayElement()->getTop() + dlgBack->getOverlayElement()->getHeight() * 0.2f - offset);
			
		}
		else
			prelevelTextLabelMid->getOverlayElement()->setTop(dlgBack->getOverlayElement()->getTop() + dlgBack->getOverlayElement()->getHeight() * 0.12f);
		//END------ Level Description
	}
}

void COgreScene::hidePrelevelPanel()
{
	unpause();
}

void COgreScene::showLevelFailedPanel()
{
	CSoundManager::sharedManager()->playLevelCompleteMusic();
	
	cancelBuildingRails();
	
	_gamePaused = true;
	
	setupFaildLevelDialog();
	mFaildLevelTrayMgr->showAll();
}

void COgreScene::showLevelCompletePanel()
{
	CSoundManager::sharedManager()->playLevelCompleteMusic();
	
	cancelBuildingRails();
	
	_gamePaused = true;
	
	setupCompleteLevelDialog();
	prepareLevelCompleteViewInfo();
	
	if (!mLevelCompleteInfo || !mLevelCompleteInfo->hasObjectForKey("levelIndex"))
	{
		LogManager::getSingleton().logMessage("Error: mLevelCompleteInfo is empty.");
		return;
	}
	
	GameMode gm             = (GameMode)mLevelCompleteInfo->getIntegerForKey("gameMode");
	GameState gs            = (GameState)mLevelCompleteInfo->getIntegerForKey("gameState");
	int levelIndex          = mLevelCompleteInfo->getIntegerForKey("levelIndex");
	int money               = mLevelCompleteInfo->getIntegerForKey("money");
	int starsCount          = mLevelCompleteInfo->getIntegerForKey("countStars");
	int countCrashedTrains  = mLevelCompleteInfo->getIntegerForKey("countCrashedTrains");
	int oldRecord           = mLevelCompleteInfo->getIntegerForKey("oldRecord");
	
	OgreBites::Label* _levelIndexCompleteLevelView = (OgreBites::Label*)mCompleteLevelTrayMgr->getWidget("CLLevelLabel");
	_levelIndexCompleteLevelView->setCaption(StringConverter::toString(levelIndex));
	
	OgreBites::DecorWidget* _star2ndCompleteLevelView = (OgreBites::DecorWidget*)mCompleteLevelTrayMgr->getWidget("CLStar2");
	OgreBites::DecorWidget* _star3rdCompleteLevelView = (OgreBites::DecorWidget*)mCompleteLevelTrayMgr->getWidget("CLStar3");
	OgreBites::DecorWidget* dlgBack = (OgreBites::DecorWidget*)mCompleteLevelTrayMgr->getWidget("CLBackPanel");
	
	OgreBites::Button* _nextButtonCompleteLevelView = (OgreBites::Button*)mCompleteLevelTrayMgr->getWidget("CLPlayNextLevelButton");
	OgreBites::Button* restartButton = (OgreBites::Button*)mCompleteLevelTrayMgr->getWidget("CLRestartButton");
	
	Ogre::String messageText;
	
	if (gm == gm_normal)
	{
		if (mLevelCompleteInfo->getBoolForKey("isSuccessEndOfGame"))
			messageText = jGetLocalizedString("You_finished_last_level");
		
		if (mLevelCompleteInfo->getBoolForKey("isMaxLevel"))
		{
			_nextButtonCompleteLevelView->hide();
			restartButton->getOverlayElement()->setLeft(dlgBack->getOverlayElement()->getWidth() / 2.f + restartButton->getOverlayElement()->getWidth());
		}
		else
		{
			int achievIndex = mLevelCompleteInfo->getIntegerForKey("lockingAchievIndex");
			bool ul = RailsContext::getShared()->isUnlockedLevelsLockedByAchievemints();
			if (-1 == achievIndex || ul)
			{
				_nextButtonCompleteLevelView->setTag(3);
			}
			else
			{
				_nextButtonCompleteLevelView->setTag(10);
			}
        
			_nextButtonCompleteLevelView->show();
		
			restartButton->getOverlayElement()->setLeft(dlgBack->getOverlayElement()->getWidth() / 2.f - restartButton->getOverlayElement()->getWidth() / 2.f);
		}
	
		//1. crashes
		if (countCrashedTrains == 0)
		{
			messageText += jGetLocalizedString("Whoah_No_crashes_50_reward");
		}
	
		switch (mLevelCompleteInfo->getIntegerForKey("isRecord"))
		{
			case 2: //новый рекорд
				messageText += jGetLocalizedFString("You_earned_i_and_set_a_new_record", money);
				break;
			
			default: //рекорд не побит
				messageText += jGetLocalizedFString("You_earned_i_High_score_i", money, oldRecord);
				break;
		}
	
		//3. Stars
		switch (starsCount)
		{
			case 1:  _star2ndCompleteLevelView->hide(); _star3rdCompleteLevelView->hide();   break;
			case 2:  _star2ndCompleteLevelView->show(); _star3rdCompleteLevelView->hide();   break;
			case 3:  _star2ndCompleteLevelView->show(); _star3rdCompleteLevelView->show();   break;
			default: _star2ndCompleteLevelView->hide(); _star3rdCompleteLevelView->hide();   break;
		}
		
	}
	else if (gm == gm_tutorial)
	{
		if (gs == gs_theSuccessEndOfGame)
		{
			messageText += jGetLocalizedString("You_have_completed_Tutorial");
		}
		else
		{
			messageText += jGetLocalizedString("You_have_completed_this_tutorial_level");
		}
		
		restartButton->getOverlayElement()->setLeft(dlgBack->getOverlayElement()->getWidth() / 2.f - restartButton->getOverlayElement()->getWidth() / 2.f);
		_nextButtonCompleteLevelView->setTag(3);
		_nextButtonCompleteLevelView->show();
	
		_star2ndCompleteLevelView->show();
		_star3rdCompleteLevelView->show();
	}
	// TEMPORARY
	// _star2ndCompleteLevelView->show();
	// _star3rdCompleteLevelView->show();
	//
	
	
	//----------- Achievements -------------
	{
		std::vector<int> achVector = mLevelCompleteInfo->getIntArrayForKey("achievements");
		
		for (int i = 0; i < mAchievementsVector.size(); i++)
		{
			Widget::nukeOverlayElement(mAchievementsVector[i]);
		}
		mAchievementsVector.clear();
		
		if (RailsContext::getShared()->NeedsShowRandomAchievementsInCompleteLevelView())
		{
			if (achVector.size() == 0) // testing
			{
				int cnt = MIN(levelIndex, 29);
				for (int i = 0; i < cnt; i++)
					achVector.push_back(Ogre::Math::RangeRandom(0, 29));
			}
		}
		
		float size = mAchievementsBox->getHeight() * 0.8f;
		float indent = size * 0.2;
		
		if (achVector.size() * (size + indent) > mAchievementsBox->getWidth())
		{
			size = mAchievementsBox->getWidth() / achVector.size();
			indent = size * 0.2;
			size -= indent * 1.f;
		}
		
		//float aspect = mViewport->getActualWidth() / mViewport->getActualHeight();
		//__android_log_print(ANDROID_LOG_INFO, "Rails", ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>aspect: %f", aspect);
		int linesCount = 0;
		float charHeight = indent * 0.8;
		float textWidth = size + indent;
		float sPoint = - (achVector.size() * (size + indent) - indent) / 2.f;
		//float topOffset = ((aspect <= 1.6 || !jNeedsUseLowResTextures()) ? 0 : - 2 * indent);
		float topOffset = (jNeedsUseLowResOverlays() ? (- indent) : 0);
		
		for (int idx = 0; idx < achVector.size(); idx++)
		{
			Ogre::OverlayElement* e = (Ogre::OverlayElement*)Ogre::OverlayManager::getSingleton().createOverlayElementFromTemplate("Rails/StartMenuButton", "", "CLAchImage_" + Ogre::StringConverter::toString(idx));
			e->setMaterialName("Achievements/achievement_" + Ogre::StringConverter::toString(achVector[idx]));
			e->setDimensions(size, size);
			
			Ogre::TextAreaOverlayElement* title = (Ogre::TextAreaOverlayElement*)((Ogre::OverlayContainer*)e)->getChild(e->getName() + "/StartMenuButtonCaption");
			title->setFontName(RailsContext::getShared()->getFontNameForSize(charHeight, "Caption"));
			title->setCharHeight(charHeight);
			title->setSpaceWidth(charHeight * 0.3);
			title->setWidth(textWidth);
			title->setHeight(charHeight);
			//title->setCaption(nameForCategoryAtIndex(idx));
			OgreBites::Label::setWrappedText(Ogre::UTFString(CAchievements::shared()->localizedName(achVector[idx])), title, textWidth, linesCount);
			//title->setLeft(icon->getWidth() * 1.3);
			title->setTop( size * 1.0 );
		
			//e->setPosition(sPoint + indent + idx * (size + indent), indent);
			e->setPosition(sPoint + idx * (size + indent), topOffset);
			//e->setPosition(0, 0);
			mAchievementsBox->addChild(e);
			mAchievementsVector.push_back(e);
		}
		
		if (achVector.size())
		{
			mCompleteLevelTrayMgr->moveWidgetToTray("CLAchievementsBox", TL_CENTER, 1);
			mAchievementsBox->show();
		}
		else
		{
			mCompleteLevelTrayMgr->removeWidgetFromTray("CLAchievementsBox");
			mAchievementsBox->hide();
		}
		
		if (achVector.size())// some Achievments are completed
			{
		        if (!Ogre::StringUtil::endsWith(messageText, "\n"))
		        		messageText += "\n";
		
				if (achVector.size() == 1)
				{
					messageText += jGetLocalizedString("You_accomplished_1_achievement");
				}
				else if (achVector.size() > 1)
				{
					messageText += jGetLocalizedFString("You_accomplished_i_achievements", achVector.size());
				}
		
			}
	}
	
	// ---------- END Achievements -----------
	
	
	OgreBites::Label* textCompleteLevelView = (OgreBites::Label*)mCompleteLevelTrayMgr->getWidget("CLTextLabel");
		
	if (Ogre::StringUtil::endsWith(messageText, "\n"))
		messageText = messageText.substr(0, messageText.length() - 1);
	
	textCompleteLevelView->setWrappedText(messageText);
	
	int cnt = textCompleteLevelView->getLinesCount();
	
	if (cnt < 4)
		textCompleteLevelView->setCharHeight(mTextCompleteLevelViewTextBaseHeight * 1.2);
	else
		textCompleteLevelView->setCharHeight(mTextCompleteLevelViewTextBaseHeight);
	
	{
		textCompleteLevelView->setWrappedText(messageText);
		cnt = textCompleteLevelView->getLinesCount();
		float offset = textCompleteLevelView->getCharHeight() * cnt / 2.f;
		//OgreBites::DecorWidget* dlgBack = (OgreBites::DecorWidget*)mCompleteLevelTrayMgr->getWidget("CLBackPanel");
		//textCompleteLevelView->getOverlayElement()->setTop(dlgBack->getOverlayElement()->getTop() + dlgBack->getOverlayElement()->getHeight() * 0.65f - offset);
		textCompleteLevelView->getOverlayElement()->setTop(dlgBack->getOverlayElement()->getHeight() * 0.51f - offset);
	}
	
	//------------------
	
	mCompleteLevelTrayMgr->showAll();
	
	if (gm == gm_normal)
	{
		jShowRateDialogIfNeeds(levelIndex);
	}
	
#ifdef GOOGLE_PLUS_ENABLED
	BGNActivity_SaveState();
#endif
}

void COgreScene::prepareLevelCompleteViewInfo()
{
	CShortLineModel* model = CShortLineModel::sharedModel();
	mLevelCompleteInfo = CPrefs::sharedPrefs()->getSharedUserDefaultsDict()->getDictForKey("levelCompleteInfo");
	
    if (mLevelCompleteInfo && mLevelCompleteInfo->hasObjectForKey("levelIndex"))
    {
        //если в префы записана какая-то информация,
        //ничего не обновляем и не прописываем. Используем старую инфу
		if (mLevelCompleteInfo->getIntegerForKey("levelIndex") == model->levelIndex()) // немного перепроверяем, если левел не совпадает, то всё пересчитываем
			return;
    }
	
	mLevelCompleteInfo = NSDictionaryPtr(new NSDictionary());
    
	CPrefs* prefs = CPrefs::sharedPrefs();
	CAchievements* achievements = CAchievements::shared();
	GameMode gameMode = model->gameMode();
    
	mLevelCompleteInfo->setIntegerForKey("levelIndex", model->levelIndex());
	mLevelCompleteInfo->setIntegerForKey("gameMode", model->gameMode());
    
    int money = model->moneyConsideringBonus();
    int countStars = model->countStars();
    int countArrivedTrains = model->countArrivedTrains();
    int countCrashedTrains = model->countCrashedTrains();
    bool isSuccessEndOfGame = (model->gameState() == gs_theSuccessEndOfGame);
    bool isMaxLevel = (model->levelIndex() == MAX_LEVEL);
    GameState gameState = model->gameState();
    
	mLevelCompleteInfo->setIntegerForKey("gameState", gameState);
	mLevelCompleteInfo->setIntegerForKey("money", money);
	mLevelCompleteInfo->setIntegerForKey("countStars", countStars);
	mLevelCompleteInfo->setIntegerForKey("countArrivedTrains", countArrivedTrains);
	mLevelCompleteInfo->setIntegerForKey("countCrashedTrains", countCrashedTrains);
	mLevelCompleteInfo->setBoolForKey("isSuccessEndOfGame", isSuccessEndOfGame);
	mLevelCompleteInfo->setBoolForKey("isMaxLevel", isMaxLevel);
    
    if (gameMode == gm_normal)
    {
		prefs->addGrossMoneyScore(money);
		prefs->addArrivedScore(countArrivedTrains);
		prefs->addCrashedScore(countCrashedTrains);

		int achievIndex = achievements->achievementUnlockingLevel(model->levelIndex()+1);
		mLevelCompleteInfo->setIntegerForKey("lockingAchievIndex", achievIndex);

		int oldRecord = prefs->recordMoneyOfLevel(model->levelIndex());
		mLevelCompleteInfo->setIntegerForKey("oldRecord", oldRecord);

		if (model->moneyConsideringBonus() > oldRecord)
		{
			mLevelCompleteInfo->setIntegerForKey("isRecord", 2);
			prefs->setRecordOfMoney(money, model->levelIndex());
			prefs->updateMainScore();
			prefs->updateChapterScore(model->chapterIndex());
		}
		else if (model->moneyConsideringBonus() == oldRecord)
		{
			mLevelCompleteInfo->setIntegerForKey("isRecord", 1);
		}
		else
		{
			mLevelCompleteInfo->setIntegerForKey("isRecord", 0);
		}
    }
    
    if ((gameMode == gm_normal) || (gameMode == gm_tutorial))
    {
		std::vector<int> achievementsVector = achievements->freshListCompletedAchievements();

		mLevelCompleteInfo->setIntArrayForKey("achievements", achievementsVector);

		int achievmentNumber = mLevelCompleteInfo->getIntegerForKey("lockingAchievIndex");
		if (achievmentNumber)
		{
			if (std::find(achievementsVector.begin(), achievementsVector.end(), achievmentNumber) != achievementsVector.end() )
			{
				mLevelCompleteInfo->setIntegerForKey("lockingAchievIndex", -1);
			}
		}
    }
    
    writeSnapshot();
}

void COgreScene::eraseLevelCompleteViewInfo()
{
	mLevelCompleteInfo = NSDictionaryPtr(new NSDictionary());
	CPrefs::sharedPrefs()->getSharedUserDefaultsDict()->setDictForKey("levelCompleteInfo", mLevelCompleteInfo);
	CPrefs::sharedPrefs()->syncUserDefaults();
}


////////////////////////////////////////////////////////////////////////////////////////////////

void COgreScene::sliderMoved(Slider* slider)
{
	if (slider->getName() == "RightPanelWidth")
	{
		s_right_panel_width = slider->getValue();
		
		mRightPanel->setDimensions(s_right_panel_width, (mViewport->getActualWidth() * (3.f / 4.f)));
		mRightPanel->setUV(1 - (float)s_right_panel_width / (float)mViewport->getActualWidth(), 0, 1, 1);
		mRightPanel->setPosition(mViewport->getActualWidth() - s_right_panel_width, mViewport->getActualHeight() / 2.f - mRightPanel->getHeight() / 2.f);
		
		setupViewport();
		setupControls();
		refreshSceneObjects();
	}
	else if (slider->getName() == "LeftPanelWidth")
	{
		s_left_panel_width = slider->getValue();
		
		mLeftPanel->setDimensions(s_left_panel_width, (mViewport->getActualWidth() * (3.f / 4.f)));
		mLeftPanel->setUV(0, 0, (float)s_left_panel_width / (float)mViewport->getActualWidth(), 1);
		mLeftPanel->setPosition(0, mViewport->getActualHeight() / 2.f - mLeftPanel->getHeight() / 2.f);
		
		setupViewport();
		setupControls();
		refreshSceneObjects();
	}
}

void COgreScene::setupNormalLevel()
{
	mTopTab->hide();
	mDemoLabel->hide();
	mCommonMessagePanel->hide();
	mTrainsMoneyLabel->show();
	mYearProgress->show();
	mLevelLabelCaption->setCaption(jGetLocalizedString("Level") + ":" + StringConverter::toString(CShortLineModel::sharedModel()->levelIndex()));
	mYearCaption->setCaption(jGetLocalizedString("Year") + ":");
	mPauseButton->show();
	mMenuButton->hide();
	
	if (isRightOrientedInterface())
	{
		mTrayMgr->removeWidgetFromTray(mMenuButton);
		mTrayMgr->moveWidgetToTray(mPauseButton, TL_TOPRIGHT, 1);
		mTrayMgr->moveWidgetToTray(mBoostButton, TL_BOTTOMRIGHT, 5);
	}
	else
	{
		mTrayMgr->moveWidgetToTray(mBoostButton, TL_BOTTOMRIGHT, 0);
		mTrayMgr->removeWidgetFromTray(mMenuButton);
		mTrayMgr->moveWidgetToTray(mPauseButton, TL_TOPLEFT, 1);
	}
}

void COgreScene::setupDemoLevel()
{
	mBonusButton0->setMaterial("SdkTrays/Empty");
	mBonusButton1->setMaterial("SdkTrays/Empty");
	mBonusButton2->setMaterial("SdkTrays/Empty");
	mBonusButton0->setFill(0);
	mBonusButton1->setFill(0);
	mBonusButton2->setFill(0);
	
	mTopTab->show();
	mDemoLabel->show();
	mCommonMessagePanel->hide();
	mTrainsMoneyLabel->hide();
	mYearProgress->hide();
	mPauseButton->hide();
	mMenuButton->show();
	if (isRightOrientedInterface())
	{
		mTrayMgr->removeWidgetFromTray(mPauseButton);
		mTrayMgr->moveWidgetToTray(mMenuButton, TL_TOPRIGHT, 1);
		mTrayMgr->moveWidgetToTray(mBoostButton, TL_BOTTOMRIGHT, 5);
	}
	else
	{
		mTrayMgr->moveWidgetToTray(mBoostButton, TL_TOPRIGHT, 1);
		mTrayMgr->removeWidgetFromTray(mPauseButton);
		mTrayMgr->moveWidgetToTray(mMenuButton, TL_TOPLEFT, 1);
	}
}

void COgreScene::setupTutorialLevel()
{
	mBonusButton0->setMaterial("SdkTrays/Empty");
	mBonusButton1->setMaterial("SdkTrays/Empty");
	mBonusButton2->setMaterial("SdkTrays/Empty");
	mBonusButton0->setFill(0);
	mBonusButton1->setFill(0);
	mBonusButton2->setFill(0);
	
	mTopTab->hide();
	mDemoLabel->hide();
	mCommonMessagePanel->hide();
	mTrainsMoneyLabel->hide();
	mYearProgress->show();
	mYearCaption->setCaption(jGetLocalizedString("Step") + ":");
	mPauseButton->show();
	mMenuButton->hide();
	if (isRightOrientedInterface())
	{
		mTrayMgr->removeWidgetFromTray(mMenuButton);
		mTrayMgr->moveWidgetToTray(mPauseButton, TL_TOPRIGHT, 1);
		mTrayMgr->moveWidgetToTray(mBoostButton, TL_BOTTOMRIGHT, 5);
	}
	else
	{
		mTrayMgr->moveWidgetToTray(mBoostButton, TL_BOTTOMRIGHT, 0);
		mTrayMgr->removeWidgetFromTray(mMenuButton);
		mTrayMgr->moveWidgetToTray(mPauseButton, TL_TOPLEFT, 1);
	}
}
	
void COgreScene::startTutorialLevel(int levelIndex)
{
	CShortLineModel* model = CShortLineModel::sharedModel();
    
	NSDictionaryPtr dict(new NSDictionary());
	dict->loadRootSubDictFromFile(("tutorial_levels/tutorialLevel" + StringConverter::toString(levelIndex) + ".plist").c_str(), gAssetMgr);
	bool res = CShortLineModel::sharedModel()->readSnapshot(dict);
	
	//if (CTutorial::sharedTutorial()->loadLevelAtIndex(level))
	if (res)
	{
	    model->startTutorailLevelAtIndex(levelIndex);
		
		updateSceneWithLevel(model->levelIndex());
		setupTutorialLevel();
		// CShortLineModel::sharedModel()->resetGameTimer();
		// CShortLineModel::sharedModel()->setContinueGameState();
	
		unpause();
	}
	else
		startDemoLevel();
	
}

void COgreScene::startDemoLevel()
{
    CShortLineModel* model = CShortLineModel::sharedModel();
    
    //if (CTutorial::sharedTutorial()->loadDemoLevel())
    {
		NSDictionaryPtr dict(new NSDictionary());
	    dict->loadRootSubDictFromFile("tutorial_levels/demoLevel.plist", gAssetMgr);
	    bool res = CShortLineModel::sharedModel()->readSnapshot(dict);
		
        model->startDemoLevel();
		
		setupDemoLevel();
		hideAllTrays();
		mTrayMgr->showAll();
    }
	
	updateSceneWithLevel(1);
	CShortLineModel::sharedModel()->resetGameTimer();
	CShortLineModel::sharedModel()->setContinueGameState();
	
	unpause();
}

void COgreScene::startLevelWithIndex(int levelIndex, bool showPrelevelView)
{
	CSoundManager::sharedManager()->soundClick();
	
	hideAllTrays();
	mTrayMgr->showAll();
	
	levelIndex = CLAMP(levelIndex, 1, 60);
	updateSceneWithLevel(levelIndex);
	
	CShortLineModel::sharedModel()->setGameMode(gm_normal);
	CShortLineModel::sharedModel()->restartWithLevel(levelIndex);
	CShortLineModel::sharedModel()->resetGameTimer();
	
	setupNormalLevel();
	
	eraseLevelCompleteViewInfo();
	
	if (showPrelevelView)
		showPrelevelPanel();
	else
	{
		unpause();
	}
}

void COgreScene::startGameAtNextLevel()
{
	startLevelWithIndex(CShortLineModel::sharedModel()->levelIndex() + 1, true);
}

void COgreScene::continueFromTheLastSnapshot()
{
    CShortLineModel* model = CShortLineModel::sharedModel();
    
	model->continueFromTheLastSnapshot();
	
	if (model->gameMode()== gm_tutorial)
		setupTutorialLevel();
	else
		setupNormalLevel();
	
	hideAllTrays();
	mTrayMgr->showAll();
	
	updateSceneWithLevel(CShortLineModel::sharedModel()->levelIndex());
	CShortLineModel::sharedModel()->timer()->reset();
	CShortLineModel::sharedModel()->resetGameTimer();
	
	if (CShortLineModel::sharedModel()->gameState() != gs_restartGame)
		CShortLineModel::sharedModel()->setContinueGameState();
	
	unpause();
	
	nextMainLoopStep();
}

bool COgreScene::isLevelCompleteOrFailedViewActive()
{
	if (mFaildLevelTrayMgr && mFaildLevelTrayMgr->areTraysVisible())
		return true;
	if (mCompleteLevelTrayMgr && mCompleteLevelTrayMgr->areTraysVisible())
		return true;
	
	return false;
}

bool COgreScene::isPrelevelViewActive()
{
	if (mPrelevelTrayMgr && mPrelevelTrayMgr->areTraysVisible())
		return true;
	
	return false;
}

void COgreScene::writeSnapshot()
{
	if (CShortLineModel::sharedModel()->gameMode() != gm_demo)
	{
		if (!mLevelCompleteInfo)
			mLevelCompleteInfo = NSDictionaryPtr(new NSDictionary());
		CPrefs::sharedPrefs()->getSharedUserDefaultsDict()->setDictForKey("levelCompleteInfo", mLevelCompleteInfo);
		CShortLineModel::sharedModel()->writeSnapshot();
	}
}

void COgreScene::buttonHit(Button* button)
{
	if (button->getName() == "RestartButton" || button->getName() == "FLRestartButton" || button->getName() == "PDRestartButton")
	{
		CSoundManager::sharedManager()->soundClick();
			
		// int levelIndex = Ogre::Math::RangeRandom(1, 60);
		// updateSceneWithLevel(levelIndex);
		// 			
		// CShortLineModel::sharedModel()->setGameMode(gm_normal);
		// CShortLineModel::sharedModel()->restartWithLevel(levelIndex);
		// CShortLineModel::sharedModel()->resetGameTimer();
		
		if (CShortLineModel::sharedModel()->gameMode()== gm_tutorial)
		{
			startTutorialLevel(CShortLineModel::sharedModel()->levelIndex());
		}
		else
		{
			startLevelWithIndex(CShortLineModel::sharedModel()->levelIndex());
			//updateBoostButton();
			//unpause(true);
		}
	}
	else if (button->getName() == "CompleteLevelButton")
	{
#ifdef ENABLE_DEBUG_DIALOGS
		unpause();
		CShortLineModel::sharedModel()->setMaxYear();
#endif
	}
	else if (button->getName() == "CompleteLevelWith3StarsButton")
	{
#ifdef ENABLE_DEBUG_DIALOGS
		unpause();
		CShortLineModel::sharedModel()->minusMoney(-600);
		CShortLineModel::sharedModel()->setMaxYear();
#endif
	}
	else if (button->getName() == "CLPlayNextLevelButton")
	{
		CShortLineModel* model = CShortLineModel::sharedModel();
		
		if (button->getTag() == 3)
		{
			eraseLevelCompleteViewInfo();
		
			if (model->gameMode() == gm_normal)
			{
				int levelIndex = model->levelIndex()+1;
				
				if (!jIsChaptersUnlocked() && !RailsContext::getShared()->isAllChaptersUnlocked() && levelIndex > 1 && (levelIndex % 5 == 1))
				{
					int threeStarsSuccesslevel = CPrefs::sharedPrefs()->maxThreeStarsSuccessLevel();
					if (levelIndex > (threeStarsSuccesslevel + 1))
					{
						RailsContext::getShared()->showUnlockAllChaptersPurchaseOfferTrayMgr();
						return;
					}
				}
				
				startGameAtNextLevel();
			}
			else if (model->gameMode()== gm_tutorial)
			{
				if (model->gameState() == gs_theSuccessEndOfGame)
				{
					startLevelWithIndex(1, true);
				}
				else
				{
					model->increaseLevelIndex();
					startTutorialLevel(model->levelIndex());
				}
			}
		}
		else if (button->getTag() == 10)
		{
			int levelIndex = model->levelIndex()+1;
			int achievIndex= CAchievements::shared()->achievementUnlockingLevel(levelIndex);;
			
			RailsContext::getShared()->showLevelLockedTrayMgr(achievIndex);
			CSoundManager::sharedManager()->playIntroMusic();
		}
	}
	else if (button->getName() == "CLRestartButton")
	{
		eraseLevelCompleteViewInfo();
		
		if (CShortLineModel::sharedModel()->gameMode()== gm_tutorial)
		{
			startTutorialLevel(CShortLineModel::sharedModel()->levelIndex());
		}
		else
		{
			startLevelWithIndex(CShortLineModel::sharedModel()->levelIndex());
			unpause(true);
		}
	}
	else if (button->getName() == "FLMenuButton" || button->getName() == "CLMenuButton" || button->getName() == "PDMenuButton" || button->getName() == "MenuButton")
	{
		 	RailsContext::getShared()->pauseCurrentScene();
	}
	else if (button->getName() == "PauseButton")
	{
		onPressPauseButton();
	}
	else if (button->getName() == "PDSoundButton")
	{
		CSoundManager::sharedManager()->soundClick();
		
		if (CSoundManager::sharedManager()->isDesabledAllSounds())
		{
			CSoundManager::sharedManager()->enableAllSounds();
			mSoundButton->setMaterial("Rails/Sound_on");
		}
		else
		{
			CSoundManager::sharedManager()->desableAllSounds();
			mSoundButton->setMaterial("Rails/Sound_off");
		}
		
	}
	else if (button->getName() == "PDResumeButton")
	{
		CSoundManager::sharedManager()->soundClick();
		unpause();
	}
	else if (button->getName() == "ResumeButton")
	{
		CSoundManager::sharedManager()->soundClick();
		continueFromTheLastSnapshot();
	}
	else if (button->getName() == "BoostButton")
	{
		CShortLineModel::sharedModel()->switchGameMultiplier();
		updateBoostButton();
	}
	else if (button->getName() == "BonusButton0")
	{
		bonusButtonClick(CShortLineModel::sharedModel()->bonusButtonAtIndex(0));
	}
	else if (button->getName() == "BonusButton1")
	{
		bonusButtonClick(CShortLineModel::sharedModel()->bonusButtonAtIndex(1));
	}
	else if (button->getName() == "BonusButton2")
	{
		bonusButtonClick(CShortLineModel::sharedModel()->bonusButtonAtIndex(2));
	}

}

void COgreScene::bonusButtonClick(CBonusButton* clickedBB)
{
    CShortLineModel* model = CShortLineModel::sharedModel();
    
	if (clickedBB->bonusButtonState() == BONUS_BUTTON_STATE_FILLED || model->gameState() == gs_pauseGame)
	{
		//NSBeep();
		return;
	}

    Bonus b = clickedBB->bonus();
	Bonus oldB = model->bonusMode();
    model->setBonusMode(b);

	if (oldB != b)
	{
		for (int i = 0; i < COUNT_BONUS_BUTTONS; ++i)
		{
			CBonusButton* btn = model->bonusButtonAtIndex(i);
		
			if (btn->bonus() != b)
			{
				if (btn->bonusButtonState() != BONUS_BUTTON_STATE_FILLED)
				{
					btn->startReadyToClickMode();
				}
			}
			else
			{
				if (btn->bonusButtonState() != BONUS_BUTTON_STATE_CLICKED)
				{
					CSoundManager::sharedManager()->soundBonusSelect();
				}
				btn->click();
				btn->updateButton();
			}
		}
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////

void COgreScene::onPressPauseButton()
{
	CSoundManager::sharedManager()->soundClick();
	//CSoundManager::sharedManager()->playIntroMusic();
	
	if (CShortLineModel::sharedModel()->gameMode()== gm_demo)
	{
	 	RailsContext::getShared()->pauseCurrentScene();
	}
	else if (_gamePaused == false)
	{
		showPauseDialog();
	}
}


void COgreScene::pause()
{
	CSoundManager::sharedManager()->playIntroMusic();
	
	_gamePaused = true;
	hideAllTrays();
}

void COgreScene::unpause(bool restart)
{
	hideAllTrays();
	
	CShortLineModel::sharedModel()->resetGameTimer(); // !!! DON'T PUT IT AFTER nextMainLoopStep(true); because it goes to bug: timer keep going while pause dialog is showing.
	
	nextMainLoopStep(true);
	updateBoostButton();
	
	_gamePaused = false;
	mTrayMgr->showAll();
	CSoundManager::sharedManager()->playMainMusic();
}

bool COgreScene::isPaused()
{
	return _gamePaused;
}

////////////////////////////////////////////////////////////////////////////////////////////////

void COgreScene::refreshSceneObjects()
{
	
	CXPoint p0 = CShortLineModel::sharedModel()->convertFromGameCoord(CXMakePoint(0, 0), s_scene_rect);
	CXPoint p1 = CShortLineModel::sharedModel()->convertFromGameCoord(CXMakePoint(1, 1), s_scene_rect);
	
	float smallRectWidth = ABS(p1.x-p0.x);
	float smallRectHeight = ABS(p1.y-p0.y);
	CXRect smallRect;
	
	RenderSystem* rs = Root::getSingleton().getRenderSystem();
    Viewport* vp = rs->_getViewport();
    Real hOffset = rs->getHorizontalTexelOffset() / (0.5 * vp->getActualWidth());
    Real vOffset = rs->getVerticalTexelOffset() / (0.5 * vp->getActualHeight());
    //mRectangle->setCorners(mQuadLeft + hOffset, mQuadTop - vOffset, mQuadRight + hOffset, mQuadBottom - vOffset);
	
	smallRect.size = CXMakeSize(smallRectWidth + hOffset, smallRectHeight + vOffset);
	
	if (cloudsBillSet)
		cloudsBillSet->setDefaultDimensions(s_cloudsBillSize, s_cloudsBillSize * m_displayFactor);
	if (bubblesBillSet)
		bubblesBillSet->setDefaultDimensions(s_bubblesBillSize, s_bubblesBillSize * m_displayFactor);
	if (explosionBillSet)
		explosionBillSet->setDefaultDimensions(s_explosionBillSize, s_explosionBillSize * m_displayFactor);
	if (treesBillSet)
		treesBillSet->setDefaultDimensions(s_treesBillSize, s_treesBillSize * m_displayFactor);
	if (semaphoresBillSet)
		semaphoresBillSet->setDefaultDimensions(s_semaphoresBillSize, s_semaphoresBillSize * m_displayFactor);
	if (frontCitiesBillSet)
		frontCitiesBillSet->setDefaultDimensions(s_frontCitiesBillSize, s_frontCitiesBillSize * s_citiesAdditionalScale * m_displayFactor);
	if (cityAdditionsBillSet)
		cityAdditionsBillSet->setDefaultDimensions(s_cityAdditionsBillSize, s_cityAdditionsBillSize * m_displayFactor);
	if (cancelButtonBillSet)
		cancelButtonBillSet->setDefaultDimensions(s_cancelButtonBillSize, s_cancelButtonBillSize * m_displayFactor);
	
	if (switchesBillSet)
		switchesBillSet->setDefaultDimensions(smallRect.size.width, smallRect.size.height);
	
	if (railsBillSet)
		railsBillSet->setDefaultDimensions(smallRect.size.width * s_railsOverlap, smallRect.size.height * s_railsOverlap);
	
	if (nodesBillSet)
		nodesBillSet->setDefaultDimensions(smallRect.size.width * s_railsOverlap, smallRect.size.height * s_railsOverlap);
	
	if (unfinishedRailsBillSet)
		unfinishedRailsBillSet->setDefaultDimensions(smallRect.size.width, 2*smallRect.size.height);
	
	if (tempRailsBillSet)
		tempRailsBillSet->setDefaultDimensions(smallRect.size.width, 2*smallRect.size.height);
    //cancelButtonBillSet->setDefaultDimensions(smallRect.size.width, smallRect.size.height);
	
	RailVector& rails = CShortLineModel::sharedModel()->rails();
	for (RailIterator it = rails.begin(); it != rails.end(); ++it)
		updateSceneNearTheRail((*it));
	
	for (TSemaphoreIterator it = m_semaphores.begin(); it != m_semaphores.end(); ++it)
	{
		TSemaphore& currSemaphore = *it;
		CXPoint coord = currSemaphore.semaphore->exactlyCoord();
		coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
		currSemaphore.bill->setPosition(Vector3(coord.x, 0., coord.y));
	}
	
	for (TSwitchIterator it = m_switches.begin(); it != m_switches.end(); ++it)
	{
		TSwitch& currSwitch = *it;
		CXPoint coord = currSwitch.sw->railTextureCoord();
		coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
		currSwitch.bill->setPosition(Vector3(coord.x, 0., coord.y));
	}
	
	for (TTreeIterator it = m_trees.begin(); it != m_trees.end(); ++it)
	{
		TTree& currTree = *it;
		CXPoint coord = currTree.tree->coord();
		coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
		
		if (currTree.bill)
			currTree.bill->setPosition(Vector3(coord.x + treesBillSet->getDefaultWidth()*0.0625, 0., coord.y + treesBillSet->getDefaultHeight()*0.125));
	}
	
	removeAllCities();
	CityVector& cities = CShortLineModel::sharedModel()->cities();
	for (CityIterator it = cities.begin(); it != cities.end(); ++it)
		addCity((*it));

	Ogre::Real lf = 1.f - ((float)(mViewport->getActualWidth() - (s_left_panel_width - s_additional_right_panel_width / 2.f)) / (float)mViewport->getActualWidth());
	Ogre::Real rf = 1.f - ((float)(mViewport->getActualWidth() - (s_right_panel_width + s_additional_right_panel_width / 2.f)) / (float)mViewport->getActualWidth());
	
#if not defined RTT_BACKGROUND
	if (m_pRTSceneMgr->hasSceneNode("BACKGROUND"))
	{
		//Ogre::Real f = 1.f - ((float)(mViewport->getActualWidth() - (s_right_panel_width + s_left_panel_width)) / (float)mViewport->getActualWidth());
		SceneNode* sn = m_pRTSceneMgr->getSceneNode("BACKGROUND");
		Rectangle2D* rect = (Rectangle2D*)sn->getAttachedObject(0);
		if (rect)
			rect->setCorners(-1.0f + 2.0f * lf, 1.1f, 1.0f - 2.0f * rf, -1.0f);
			//rect->setCorners(-1.0f + 2.0f * lf, 1.0f, 1.0f - 2.0f * rf, -1.0f);
	}
#endif
	
	if (mSceneMgr->hasSceneNode("BACKGROUND_GRID"))
	{
		//Ogre::Real f = 1.f - ((float)(mViewport->getActualWidth() - (s_right_panel_width + s_left_panel_width)) / (float)mViewport->getActualWidth());
		SceneNode* sn = mSceneMgr->getSceneNode("BACKGROUND_GRID");
		Rectangle2D* rect = (Rectangle2D*)sn->getAttachedObject(0);
		if (rect)
			rect->setCorners(-1.00f + 2.0f * lf, 1.45f, 1.00f - 2.0f * rf, -1.41f);
			//rect->setCorners(-1.05f + 2.0f * lf + 0.05f * f, 1.45f, 1.05f - 2.0f * rf - 0.05f * f, -1.41f);
			//rect->setCorners(-1.05f + 2.0f * lf + 0.05f * f, 1.51f, 1.05f - 2.0f * rf - 0.05f * f, -1.41f);
			//rect->setCorners(-1.05f + 2.0f * lf + 0.05f * f, 1.34f, 1.05f - 2.0f * rf - 0.05f * f, -1.34f);
	}
	
    if (!m_trains.empty())
	{
    	for (TTrainIterator it = m_trains.begin(); it != m_trains.end(); ++it)
		{
			TTrain& currTrain = *it;
			CarVector carsVec = currTrain.train->cars();
			{
				currTrain.node->setScale(Ogre::Vector3(MODEL_SCALE, MODEL_SCALE * m_displayFactor, MODEL_SCALE));
				for (int i = 0; i < currTrain.cars.size(); i++)
				{
					if (carsVec[i]->type() == CISTERN)
						currTrain.cars[i].node->setScale(Ogre::Vector3(MODEL_CISTERN_SCALE, MODEL_SCALE * m_displayFactor, MODEL_SCALE));
					else
						currTrain.cars[i].node->setScale(Ogre::Vector3(MODEL_SCALE, MODEL_SCALE * m_displayFactor, MODEL_SCALE));
					
				}
			}
		}
	}
	
	updateRenderTexture();
}
	
void COgreScene::setupScene()
{
	mSceneMgr->setAmbientLight(ColourValue(0.8, 0.8, 0.8));
	
	if (!mSceneLight)
	{
		mSceneLight = mSceneMgr->createLight("Light");
		mSceneLight->setPosition(-500,700,1000);
	}
	
	//Ogre::ResourceGroupManager::getSingleton().loadResourceGroup("ResForPreload");
	
	CXPoint p0 = CShortLineModel::sharedModel()->convertFromGameCoord(CXMakePoint(0, 0), s_scene_rect);
	CXPoint p1 = CShortLineModel::sharedModel()->convertFromGameCoord(CXMakePoint(1, 1), s_scene_rect);
	
	float smallRectWidth = ABS(p1.x-p0.x);
	float smallRectHeight = ABS(p1.y-p0.y);
	
	if (!cityAdditionsBillSet)
	{
		{
			cityAdditionsBillSet = mSceneMgr->createBillboardSet();//("CITY_ADDITIONS_BILL_SET");
			Ogre::SceneNode* additionsNode = mSceneMgr->getRootSceneNode()->createChildSceneNode("CITY_ADDITIONS_SCENE_NODE", Vector3(0, 0, 0));
			additionsNode->attachObject(cityAdditionsBillSet);
		}
		
		cityAdditionsBillSet->setDefaultDimensions(s_cityAdditionsBillSize, s_cityAdditionsBillSize * m_displayFactor);
		cityAdditionsBillSet->setMaterialName("city_additions");
		        
		cityAdditionsBillSet->setRenderQueueGroup(CITYBOARD_RENDER_QUEUE);
		cityAdditionsBillSet->setBillboardType(BBT_POINT);
		cityAdditionsBillSet->setBillboardOrigin(BBO_CENTER);
		cityAdditionsBillSet->setAutoextend(true);
		cityAdditionsBillSet->setTextureStacksAndSlices(4, 8);
		
		Billboard* bill = cityAdditionsBillSet->createBillboard(0., 0., 0.); // fix for iPad: если этого не сделать, то потом почему-то биллборды не отображаются в игре
		cityAdditionsBillSet->removeBillboard(bill);
	}
	
	if (!cloudsBillSet)
	{
		{
			cloudsBillSet = mSceneMgr->createBillboardSet(50);
			Ogre::SceneNode* cloudsNode = mSceneMgr->getRootSceneNode()->createChildSceneNode("CLOUDS_SCENE_NODE", Vector3(0, 0, 0));
			cloudsNode->attachObject(cloudsBillSet);
		}
		cloudsBillSet->setDefaultDimensions(s_cloudsBillSize, s_cloudsBillSize * m_displayFactor);
		cloudsBillSet->setMaterialName("bubbles");
		
		cloudsBillSet->setRenderQueueGroup(FA_CLOUD_RENDER_QUEUE);
		cloudsBillSet->setBillboardType(BBT_POINT);
		
		cloudsBillSet->setBillboardOrigin(BBO_CENTER);
		cloudsBillSet->setAutoextend(true);
		cloudsBillSet->setTextureStacksAndSlices(4, 4);
		//cloudsBillSet->setSortingEnabled(true);
		
		Billboard* bill = cloudsBillSet->createBillboard(0., 0., 0.); // fix for iPad: если этого не сделать, то потом почему-то биллборды не отображаются в игре
		cloudsBillSet->removeBillboard(bill);
	}
	
	if (!bubblesBillSet)
	{
		{
			bubblesBillSet = mSceneMgr->createBillboardSet(50);
			Ogre::SceneNode* bubblesNode = mSceneMgr->getRootSceneNode()->createChildSceneNode("BUBBLES_SCENE_NODE", Vector3(0, 0, 0));
			bubblesNode->attachObject(bubblesBillSet);
		}
		bubblesBillSet->setDefaultDimensions(s_bubblesBillSize, s_bubblesBillSize * m_displayFactor);
		bubblesBillSet->setMaterialName("bubbles");
		
		bubblesBillSet->setRenderQueueGroup(BUBBLES_RENDER_QUEUE);
		bubblesBillSet->setBillboardType(BBT_POINT);
		
		bubblesBillSet->setBillboardOrigin(BBO_CENTER);
		bubblesBillSet->setAutoextend(true);
		bubblesBillSet->setTextureStacksAndSlices(4, 4);
		//bubblesBillSet->setSortingEnabled(true);
		
		Billboard* bill = bubblesBillSet->createBillboard(0., 0., 0.); // fix for iPad: если этого не сделать, то потом почему-то биллборды не отображаются в игре
		bubblesBillSet->removeBillboard(bill);
	}
	
	if (!explosionBillSet)
	{
		{
			explosionBillSet = mSceneMgr->createBillboardSet(10);
			Ogre::SceneNode* expNode = mSceneMgr->getRootSceneNode()->createChildSceneNode("EXPLOSION_SCENE_NODE", Vector3(0, 0, 0));
			expNode->attachObject(explosionBillSet);
		}
		explosionBillSet->setDefaultDimensions(s_explosionBillSize, s_explosionBillSize * m_displayFactor);
		explosionBillSet->setMaterialName("explosion");
		
		explosionBillSet->setRenderQueueGroup(EXPLOSION_RENDER_QUEUE);
		explosionBillSet->setBillboardType(BBT_POINT);
		explosionBillSet->setSortingEnabled(true);
		explosionBillSet->setBillboardsInWorldSpace(true);
		
		explosionBillSet->setBillboardOrigin(BBO_CENTER);
		explosionBillSet->setAutoextend(true);
		explosionBillSet->setTextureStacksAndSlices(4, 8);
		
		Billboard* bill = explosionBillSet->createBillboard(0., 0., 0.); // fix for iPad: если этого не сделать, то потом почему-то биллборды не отображаются в игре
		explosionBillSet->removeBillboard(bill);
	}
	
	if (!treesBillSet)
	{
		{
			treesBillSet = mSceneMgr->createBillboardSet(50);
			Ogre::SceneNode* treesNode = mSceneMgr->getRootSceneNode()->createChildSceneNode("TREES_SCENE_NODE", Vector3(0, 0, 0));
			treesNode->attachObject(treesBillSet);
		}
		treesBillSet->setDefaultDimensions(s_treesBillSize, s_treesBillSize * m_displayFactor);
		
		treesBillSet->setRenderQueueGroup(TREES_RENDER_QUEUE);
		treesBillSet->setBillboardType(BBT_POINT);
		
		treesBillSet->setBillboardOrigin(BBO_BOTTOM_CENTER);
		treesBillSet->setAutoextend(true);
		treesBillSet->setTextureStacksAndSlices(8, 8); // Stacks x Slices should equal TREE_TYPES_COUNT
		treesBillSet->setBillboardsInWorldSpace(true);
		treesBillSet->setSortingEnabled(true);
		
		Billboard* bill = treesBillSet->createBillboard(0., 0., 0.); // fix for iPad: если этого не сделать, то потом почему-то биллборды не отображаются в игре
		treesBillSet->removeBillboard(bill);
	}
	
	if (!semaphoresBillSet)
	{
		{
			semaphoresBillSet = mSceneMgr->createBillboardSet(50);
			Ogre::SceneNode* semaphoresNode = mSceneMgr->getRootSceneNode()->createChildSceneNode("SEMAPHORES_SCENE_NODE", Vector3(0, 0, 0));
			semaphoresNode->attachObject(semaphoresBillSet);
		}
		semaphoresBillSet->setDefaultDimensions(s_semaphoresBillSize, s_semaphoresBillSize * m_displayFactor);
        semaphoresBillSet->setMaterialName("semaphore");
		
		semaphoresBillSet->setRenderQueueGroup(SEMAPHORES_RENDER_QUEUE);
		semaphoresBillSet->setBillboardType(BBT_POINT);
		semaphoresBillSet->setBillboardOrigin(BBO_BOTTOM_CENTER);
		semaphoresBillSet->setAutoextend(true);
		semaphoresBillSet->setTextureStacksAndSlices(2, 2);
		semaphoresBillSet->setBillboardsInWorldSpace(true);
		
		Billboard* bill = semaphoresBillSet->createBillboard(0., 0., 0.); // fix for iPad: если этого не сделать, то потом почему-то биллборды не отображаются в игре
		semaphoresBillSet->removeBillboard(bill);
	}
	
	if (!frontCitiesBillSet)
	{
		{
			frontCitiesBillSet = mSceneMgr->createBillboardSet(8);
			Ogre::SceneNode* citiesNode = mSceneMgr->getRootSceneNode()->createChildSceneNode("FRONT_CITIES_SCENE_NODE", Vector3(0, 0, 0));
			citiesNode->attachObject(frontCitiesBillSet);
		}
		frontCitiesBillSet->setDefaultDimensions(s_frontCitiesBillSize, s_frontCitiesBillSize * s_citiesAdditionalScale * m_displayFactor);
		frontCitiesBillSet->setMaterialName("front_cities_classic");
        
		frontCitiesBillSet->setRenderQueueGroup(FRONT_CITIES_RENDER_QUEUE);
		
		frontCitiesBillSet->setBillboardType(BBT_POINT);
		frontCitiesBillSet->setBillboardOrigin(BBO_BOTTOM_CENTER);
		frontCitiesBillSet->setAutoextend(false);
		frontCitiesBillSet->setTextureStacksAndSlices(6, 4);
		frontCitiesBillSet->setSortingEnabled(true);
		frontCitiesBillSet->setBillboardsInWorldSpace(true);
		
		Billboard* bill = frontCitiesBillSet->createBillboard(0., 0., 0.); // fix for iPad: если этого не сделать, то потом почему-то биллборды не отображаются в игре
		frontCitiesBillSet->removeBillboard(bill);
	}
	
	if (!switchesBillSet)
	{
		{
			switchesBillSet = mSceneMgr->createBillboardSet(100);
			Ogre::SceneNode* swNode = mSceneMgr->getRootSceneNode()->createChildSceneNode("SWITCHES_SCENE_NODE", Vector3(0, 0, 0));
			swNode->attachObject(switchesBillSet);
		}
		
		switchesBillSet->setRenderQueueGroup(SWITCHES_RENDER_QUEUE);
		
		switchesBillSet->setBillboardOrigin(BBO_TOP_LEFT);
		switchesBillSet->setAutoextend(true);
		
		switchesBillSet->setBillboardType(BBT_PERPENDICULAR_COMMON);
		switchesBillSet->setCommonUpVector(Vector3(0,0,-1));
		switchesBillSet->setCommonDirection(Vector3(0,1,0));
		switchesBillSet->setSortingEnabled(false);
		switchesBillSet->setBillboardsInWorldSpace(true);
		
		{
			CXRect smallRect;
			smallRect.size = CXMakeSize(smallRectWidth, smallRectHeight);
			switchesBillSet->setDefaultDimensions(smallRect.size.width, smallRect.size.height);
		}
        
		switchesBillSet->setMaterialName("switches");
		switchesBillSet->setTextureStacksAndSlices(8, 4);
		
		Billboard* bill = switchesBillSet->createBillboard(0., 0., 0.); // fix for iPad: если этого не сделать, то потом почему-то биллборды не отображаются в игре
		switchesBillSet->removeBillboard(bill);
	}
	
	if (!railsBillSet)
	{
		{
			railsBillSet = m_pRTSceneMgr->createBillboardSet(100);
			Ogre::SceneNode* railsNode = m_pRTSceneMgr->getRootSceneNode()->createChildSceneNode("RAILS_SCENE_NODE", Vector3(0, 0, 0));
			railsNode->attachObject(railsBillSet);
		}
		
		railsBillSet->setRenderQueueGroup(RAILS_RENDER_QUEUE);
		
		railsBillSet->setBillboardOrigin(BBO_CENTER);
		railsBillSet->setAutoextend(true);
		
		railsBillSet->setBillboardType(BBT_PERPENDICULAR_COMMON);
		railsBillSet->setCommonUpVector(Vector3(0,0,-1));
		railsBillSet->setCommonDirection(Vector3(0,1,0));
		railsBillSet->setSortingEnabled(false);
		railsBillSet->setBillboardsInWorldSpace(true);
		
		{
			CXRect smallRect;
			smallRect.size = CXMakeSize(smallRectWidth, smallRectHeight);
			railsBillSet->setDefaultDimensions(smallRect.size.width * s_railsOverlap, smallRect.size.height * s_railsOverlap);
		}
		
		railsBillSet->setTextureStacksAndSlices(8, 4);
		
		Billboard* bill = railsBillSet->createBillboard(0., 0., 0.); // fix for iPad: если этого не сделать, то потом почему-то биллборды не отображаются в игре
		railsBillSet->removeBillboard(bill);
	}
	
	if (!nodesBillSet)
	{
		{
			nodesBillSet = m_pRTSceneMgr->createBillboardSet(100);
			Ogre::SceneNode* railsNode = m_pRTSceneMgr->getRootSceneNode()->createChildSceneNode("NODES_SCENE_NODE", Vector3(0, 0, 0));
			railsNode->attachObject(nodesBillSet);
		}
		
		nodesBillSet->setRenderQueueGroup(NODES_RENDER_QUEUE);
		
		nodesBillSet->setBillboardOrigin(BBO_CENTER);
		nodesBillSet->setAutoextend(true);
		
		nodesBillSet->setBillboardType(BBT_PERPENDICULAR_COMMON);
		nodesBillSet->setCommonUpVector(Vector3(0,0,-1));
		nodesBillSet->setCommonDirection(Vector3(0,1,0));
		nodesBillSet->setSortingEnabled(false);
		nodesBillSet->setBillboardsInWorldSpace(true);
		
		{
			CXRect smallRect;
			smallRect.size = CXMakeSize(smallRectWidth, smallRectHeight);
			nodesBillSet->setDefaultDimensions(smallRect.size.width * s_railsOverlap, smallRect.size.height * s_railsOverlap);
		}
		
		nodesBillSet->setTextureStacksAndSlices(4, 4);
		
		Billboard* bill = nodesBillSet->createBillboard(0., 0., 0.); // fix for iPad: если этого не сделать, то потом почему-то биллборды не отображаются в игре
		nodesBillSet->removeBillboard(bill);
	}
	
	if (!unfinishedRailsBillSet)
	{
		{
			unfinishedRailsBillSet = m_pRTSceneMgr->createBillboardSet(100);
			Ogre::SceneNode* railsNode = m_pRTSceneMgr->getRootSceneNode()->createChildSceneNode("UNFINISHED_RAILS_SCENE_NODE", Vector3(0, 0, 0));
			railsNode->attachObject(unfinishedRailsBillSet);
		}
		
		unfinishedRailsBillSet->setRenderQueueGroup(UNFINISHED_RAILS_RENDER_QUEUE);
		unfinishedRailsBillSet->setBillboardType(BBT_PERPENDICULAR_COMMON);
		unfinishedRailsBillSet->setCommonUpVector(Vector3(0,0,-1));
		unfinishedRailsBillSet->setCommonDirection(Vector3(0,1,0));
		unfinishedRailsBillSet->setSortingEnabled(false);
		unfinishedRailsBillSet->setBillboardsInWorldSpace(true);
		
		unfinishedRailsBillSet->setBillboardOrigin(BBO_CENTER);
		unfinishedRailsBillSet->setAutoextend(true);
		
		unfinishedRailsBillSet->setTextureStacksAndSlices(2, 4);
		
		Billboard* bill = unfinishedRailsBillSet->createBillboard(0., 0., 0.); // fix for iPad: если этого не сделать, то потом почему-то биллборды не отображаются в игре
		unfinishedRailsBillSet->removeBillboard(bill);
		
		{
			CXRect smallRect;
			smallRect.size = CXMakeSize(smallRectWidth, smallRectHeight);
			unfinishedRailsBillSet->setDefaultDimensions(smallRect.size.width, 2*smallRect.size.height);
		}
	}
	
	if (!tempRailsBillSet)
	{
		{
			tempRailsBillSet = mSceneMgr->createBillboardSet(100);
			Ogre::SceneNode* railsNode = mSceneMgr->getRootSceneNode()->createChildSceneNode("TEMP_RAILS_SCENE_NODE", Vector3(0, 0, 0));
			railsNode->attachObject(tempRailsBillSet);
		}
		highlightedRailsBillSet = tempRailsBillSet; //!!!!!!!!!! using one billboard for unfinished and highlited rails
		
		tempRailsBillSet->setRenderQueueGroup(TEMP_RAILS_QUEUE);
		tempRailsBillSet->setBillboardType(BBT_PERPENDICULAR_COMMON);
		tempRailsBillSet->setCommonUpVector(Vector3(0,0,-1));
		tempRailsBillSet->setCommonDirection(Vector3(0,1,0));
		tempRailsBillSet->setSortingEnabled(false);
		
		tempRailsBillSet->setBillboardOrigin(BBO_CENTER);
		tempRailsBillSet->setAutoextend(true);
		tempRailsBillSet->setBillboardsInWorldSpace(true);
		
		tempRailsBillSet->setMaterialName("th_rails");
		tempRailsBillSet->setTextureStacksAndSlices(4, 4);
		
		Billboard* bill = tempRailsBillSet->createBillboard(0., 0., 0.); // fix for iPad: если этого не сделать, то потом почему-то биллборды не отображаются в игре
		tempRailsBillSet->removeBillboard(bill);
		
		{
			CXRect smallRect;
			smallRect.size = CXMakeSize(smallRectWidth, smallRectHeight);
			tempRailsBillSet->setDefaultDimensions(smallRect.size.width, 2*smallRect.size.height);
		}
	}
    
	if (!cancelButtonBillSet)
    {
		{
			cancelButtonBillSet = mSceneMgr->createBillboardSet(10);
			Ogre::SceneNode* cancelButtonNode = mSceneMgr->getRootSceneNode()->createChildSceneNode("CANCEL_BUTTON_SCENE_NODE", Vector3(0, 0, 0));
			cancelButtonNode->attachObject(cancelButtonBillSet);
		}
        cancelButtonBillSet->setDefaultDimensions(s_cancelButtonBillSize, s_cancelButtonBillSize * m_displayFactor);// / 0.88);
        
        cancelButtonBillSet->setRenderQueueGroup(RENDER_QUEUE_OVERLAY);
        
        cancelButtonBillSet->setBillboardType(BBT_POINT);
        cancelButtonBillSet->setBillboardOrigin(BBO_CENTER);
        cancelButtonBillSet->setAutoextend(true);
        cancelButtonBillSet->setBillboardsInWorldSpace(true);
		
		cancelButtonBillSet->setMaterialName("remove_unfinished_rails_button");
		
		Billboard* bill = cancelButtonBillSet->createBillboard(0., 0., 0.); // fix for iPad: если этого не сделать, то потом почему-то биллборды не отображаются в игре
		cancelButtonBillSet->removeBillboard(bill);
	}
	
	if (!m_pRTSceneMgr->hasSceneNode("BACKGROUND"))
	{ // BACKGROUND using Rectangle2D
		// Create background rectangle covering the whole screen
		Rectangle2D* rect = new Rectangle2D(true, HardwareBuffer::HBU_STATIC_WRITE_ONLY);
		//rect->setCorners(-1.0, 1.0, 1.0, -1.0);
#ifndef RTT_BACKGROUND
		Ogre::Real lf = 1.f - ((float)(mViewport->getActualWidth() - (s_left_panel_width - s_additional_right_panel_width / 2.f)) / (float)mViewport->getActualWidth());
		Ogre::Real rf = 1.f - ((float)(mViewport->getActualWidth() - (s_right_panel_width + s_additional_right_panel_width / 2.f)) / (float)mViewport->getActualWidth());
		//Ogre::Real f = 1.f - ((float)(mViewport->getActualWidth() - (s_right_panel_width + s_left_panel_width)) / (float)mViewport->getActualWidth());
		rect->setCorners(-1.0f + 2.0f * lf, 1.1f, 1.0f - 2.0f * rf, -1.0f);
		//rect->setCorners(-1.0f + 2.0f * lf, 1.0f, 1.0f - 2.0f * rf, -1.0f);
#else
		rect->setCorners(-1.0f, 1.0f, 1.0f, -1.0f);
#endif
        
		//rect->setMaterial("shortline_land_classic"); // we will load material in updateSceneWithTag(...)
		
		// Render the background before everything else
		rect->setRenderQueueGroup(RENDER_QUEUE_BACKGROUND);
		
		// Use infinite AAB to always stay visible
		AxisAlignedBox aabInf;
		aabInf.setInfinite();
		rect->setBoundingBox(aabInf);
		
		// Attach background to the scene
		Ogre::SceneNode* backNode = m_pRTSceneMgr->getRootSceneNode()->createChildSceneNode("BACKGROUND");
		backNode->attachObject(rect);
		//backNode->setVisible(false);
	}
	
	if (!mSceneMgr->hasSceneNode("BACKGROUND_GRID"))
	{ // BACKGROUND GRID using Rectangle2D
		// Create background rectangle covering the whole screen
		Rectangle2D* rect = new Rectangle2D(true);
		rect->setCorners(-1.34, 1.0, 1.34, -1.0);
		
#if defined(MACOSX_DEPLOYMENT_TARGET)
		rect->setMaterial("scene_grid_OSX");
		rect->setCorners(-1.0, 1.34, 1.0, -1.34);
#elif defined(ANDROID_DEPLOYMENT_TARGET)
		rect->setMaterial("scene_grid_OSX");
		//rect->setCorners(-1.1, 1.34, 1.1, -1.34);
		//Ogre::Real f = 1.f - ((float)(mViewport->getActualWidth() - (s_right_panel_width + s_left_panel_width)) / (float)mViewport->getActualWidth());
		Ogre::Real lf = 1.f - ((float)(mViewport->getActualWidth() - (s_left_panel_width - s_additional_right_panel_width / 2.f)) / (float)mViewport->getActualWidth());
		Ogre::Real rf = 1.f - ((float)(mViewport->getActualWidth() - (s_right_panel_width + s_additional_right_panel_width / 2.f)) / (float)mViewport->getActualWidth());
		//Ogre::Real f = 1.f - ((float)(mViewport->getActualWidth() - (s_right_panel_width + s_left_panel_width)) / (float)mViewport->getActualWidth());
		rect->setCorners(-1.00f + 2.0f * lf, 1.45f, 1.00f - 2.0f * rf, -1.41f);
		//rect->setCorners(-1.05f + 2.0f * lf + 0.05f * f, 1.51f, 1.05f - 2.0f * rf - 0.05f * f, -1.41f);
		//rect->setCorners(-1.05f + 2.0f * lf + 0.05f * f, 1.34f, 1.05f - 2.0f * rf - 0.05f * f, -1.34f);
#else
		rect->setMaterial("scene_grid");
#endif
		
		// Render the background before everything else
		rect->setRenderQueueGroup(RENDER_QUEUE_BACKGROUND);
		// Set the bounding box to something big
		//rect->setBoundingBox(AxisAlignedBox(-100000.0*Vector3::UNIT_SCALE, 100000.0*Vector3::UNIT_SCALE));
		
		// Use infinite AAB to always stay visible
		AxisAlignedBox aabInf;
		aabInf.setInfinite();
		rect->setBoundingBox(aabInf);
		
		// Attach background to the scene
		Ogre::SceneNode* backNode = mSceneMgr->getRootSceneNode()->createChildSceneNode("BACKGROUND_GRID");
		backNode->attachObject(rect);
		backNode->setVisible(false);
	}
	
	if (!mSceneMgr->hasSceneNode("RED_GAME_COORD"))
	{
		Rectangle2D* rect = new Rectangle2D(true);
		rect->setCorners(-.1, .1, .1, -.1);
		//#ifdef MACOSX_DEPLOYMENT_TARGET
		rect->setMaterial("startPoint_OSX");
		//#else
		//		rect->setMaterial("startPoint");
		//#endif
		// Render the background before everything else
		rect->setRenderQueueGroup(RENDER_QUEUE_OVERLAY);
		// Set the bounding box to something big
		rect->setBoundingBox(AxisAlignedBox(-100000.0*Vector3::UNIT_SCALE, 100000.0*Vector3::UNIT_SCALE));
		rect->setUseIdentityProjection(false);
		rect->setUseIdentityView(false);
		// Attach background to the scene
		Ogre::SceneNode* backNode = mSceneMgr->getRootSceneNode()->createChildSceneNode("RED_GAME_COORD");
		backNode->attachObject(rect);
		backNode->setVisible(false);
		
		setRedGameCoord(CXBadPoint);
	}
}

//---------------------------------------------------------------

void COgreScene::updateSceneWithLevel(int level)
{
	Ogre::String newSceneTag = "classic";
	
    //if (CShortLineModel::sharedModel()->gameMode() != gm_demo) // bugfix (snow and desert texturex doesn't apply if updateSceneWithLevel called before change game mode)
	{
		// if (mPSWeather)
		// {
		// 	mSceneMgr->getRootSceneNode()->detachObject(mPSWeather);
		// 	mSceneMgr->destroyParticleSystem(mPSWeather);
		// 	mPSWeather = 0;
		// }
		
		switch (level)
		{
			case 1 ... 20:
				newSceneTag = "classic";
				// if (!mPSWeather)
				// {
				// 			        mPSWeather = mSceneMgr->createParticleSystem("Weather", "Rails/Rain");
				// 			        mSceneMgr->getRootSceneNode()->attachObject(mPSWeather);
				// 			        mPSWeather->fastForward(200);
				// 	mPSWeather->setRenderQueueGroup(WEATHER_RENDER_QUEUE);
				// }
				break;
			case 21 ... 40:
				newSceneTag = "snow";
				// if (!mPSWeather)
				// {
				// 			        mPSWeather = mSceneMgr->createParticleSystem("Weather", "Rails/Snow");
				// 			        mSceneMgr->getRootSceneNode()->attachObject(mPSWeather);
				// 			        mPSWeather->fastForward(200);
				// 	mPSWeather->setRenderQueueGroup(WEATHER_RENDER_QUEUE);
				// }
				break;
			default:
				newSceneTag = "desert";
				break;
		}
	}
	
	updateSceneWithTag(newSceneTag);
	
}

void COgreScene::updateSceneWithTag(const Ogre::String& tagName)
{
	if (m_sceneTag != tagName)
	{
		m_sceneTag = tagName;
		applyBackgroundTexture("shortline_land_"+tagName);
		applyRailsTexture("rails_"+tagName);
		applyNodesTexture("nodes_"+tagName);
		applyUnfinishedRailsTexture("unfinished_rails_"+tagName);
		applyTreesTexture("scene_trees_"+tagName);
		applyCitiesTexture("front_cities_"+tagName);
		
//#ifdef IPHONEOS_DEPLOYMENT_TARGET
//		Ogre::ResourceGroupManager::getSingleton().unloadUnreferencedResourcesInGroup("General");
//		Ogre::ResourceGroupManager::getSingleton().unloadUnreferencedResourcesInGroup(OgreFramework::getSingletonPtr()->texturesGroupName());
//#endif
	}
}

void COgreScene::applyBackgroundTexture(const Ogre::String& matName)
{
	if (m_pRTSceneMgr->hasSceneNode("BACKGROUND"))
	{
		SceneNode* sn = m_pRTSceneMgr->getSceneNode("BACKGROUND");
		Rectangle2D* rect = (Rectangle2D*)sn->getAttachedObject(0);
		if (rect)
		{
            
			rect->setMaterial(matName + "_OSX");
//            if (whatIsDevice() == DEVICE_MAC)
//			{
//				rect->setMaterial(matName + "_OSX");
//			}
//			else
///				rect->setMaterial(matName);
			
			//rect->setCorners(-1.0, 1.0, 1.0, -1.0);
//			rect->setCorners(-1.0, 1.0, 1.0, -1.0);
		}
	}
}

void COgreScene::applyRailsTexture(const Ogre::String& matName)
{
	if (railsBillSet == NULL)
		LogManager::getSingleton().logMessage("Error: railsBillSet == NULL");
	else
		railsBillSet->setMaterialName(matName);
}

void COgreScene::applyNodesTexture(const Ogre::String& matName)
{
	if (nodesBillSet == NULL)
		LogManager::getSingleton().logMessage("Error: nodesBillSet == NULL");
	else
		nodesBillSet->setMaterialName(matName);
}

void COgreScene::applyUnfinishedRailsTexture(const Ogre::String& matName)
{
	if (unfinishedRailsBillSet == NULL)
		LogManager::getSingleton().logMessage("Error: unfinishedRailsBillSet == NULL");
	else
		unfinishedRailsBillSet->setMaterialName(matName);
	
	updateRenderTexture();
}

void COgreScene::applyTreesTexture(const Ogre::String& matName)
{
	if (treesBillSet == NULL)
		LogManager::getSingleton().logMessage("Error: treesBillSet == NULL");
	else
		treesBillSet->setMaterialName(matName);
}

void COgreScene::applyCitiesTexture(const Ogre::String& matName)
{
	if (frontCitiesBillSet == NULL)
		LogManager::getSingleton().logMessage("Error: frontCitiesBillSet == NULL");
	else
		frontCitiesBillSet->setMaterialName(matName);
}

//---------------------------------------------------------------------------
#pragma mark - LOOP

bool COgreScene::frameRenderingQueued(const FrameEvent& evt)
{
	nextMainLoopStep();
	
	mTrayMgr->frameRenderingQueued(evt);
	
	return true;
}

void COgreScene::nextMainLoopStep(bool ignorePausedState)
{
	static double s_updateControlsTime = 0;
	CShortLineModel* model = CShortLineModel::sharedModel();
	mLastFrameTime = model->timer()->seconds() - mStartTime;
	mLastFrameTime = (mLastFrameTime < 0) ? 0 : mLastFrameTime; // bugfix - иногда модель резетит таймер и мы получаем ложное отрицательное значение при расчёте mLastFrameTime
	mStartTime = model->timer()->seconds();
	s_updateControlsTime += mLastFrameTime;
	
	if (!isPaused() || ignorePausedState)
	{
		
		model->mainLoop();
		
		GameState gs = model->gameState();
		
		switch (gs)
		{
			case gs_theFailedEndOfLevel:
			{
				//[self stopTimer];
				model->setRestartGameState();
				showLevelFailedPanel();
				break;
			}
				
			case gs_theSuccessEndOfLevel:
				//[self stopTimer];
				//[self showLevelCompleteView];
				showLevelCompletePanel();
				break;
				
			case gs_theSuccessEndOfGame:
				//[self stopTimer];
				//[self showSuccessEndOfGameView];
				showLevelCompletePanel();
				
				break;
			default:
				break;
		}

		if ((ignorePausedState || s_updateControlsTime) > 0.2 && (CShortLineModel::sharedModel()->gameMode() != gm_demo))
		{
			float part = 0;
			
			if (CShortLineModel::sharedModel()->gameMode() == gm_tutorial)
			{
				updateTutorialMessage();
				
				int countSteps = CTutorial::sharedTutorial()->countSteps();
				int step = MAX(CShortLineModel::sharedModel()->step() - 1, 0);
				part = (float)step / (float)countSteps;
			}
			else
			{
				int levelIndex = model->levelIndex();
				int money = model->money();
				int year = model->years();
				int trains = model->countArrivedTrains();
				
				part = (MIN(year, MAX_YEAR) - MIN_YEAR) / ((float)MAX_YEAR - MIN_YEAR);
			
				mYearLabel->setCaption(StringConverter::toString(year));
				mMoneyLabelCaption->setCaption(StringConverter::toString(money));
				mTrainsLabelCaption->setCaption(StringConverter::toString(trains));
			
				
			}
			
			float s_gap = 10;
			float progressLength = s_gap + (mProgressBarThumbHeight - s_gap) * part;
			float percent = progressLength / mProgressBarThumbHeight;
			mProgressBarThumb->setUV(0, 1 - percent, 1, 1);
			mProgressBarThumb->setHeight(progressLength);
			mProgressBarThumb->setTop(mProgressBarThumbTop + (mProgressBarThumbHeight - progressLength));
		
			updateBonusButton(mBonusButton0, model->bonusButtonAtIndex(0));
			updateBonusButton(mBonusButton1, model->bonusButtonAtIndex(1));
			updateBonusButton(mBonusButton2, model->bonusButtonAtIndex(2));
		}
	}
		
	updateScene(mLastFrameTime);
}

void COgreScene::updateTutorialMessage()
{
	CShortLineModel* model = CShortLineModel::sharedModel();
	CTutorial* tutor = CTutorial::sharedTutorial();

	int countSteps = tutor->countSteps();
	int step = MIN(model->step(), countSteps);
	
	Ogre::String stepString = StringConverter::toString(step) + "/" + StringConverter::toString(countSteps);
	mYearLabel->setCaption(stepString);
	
	const char* message = tutor->message(model->levelIndex(), model->step());
	
	if (message && (mLastTutorialMessage != Ogre::String(message) || !mCommonMessagePanel->isVisible()))
	{
		mLastTutorialMessage = message;
		//__android_log_print(ANDROID_LOG_INFO, "Rails", message);
		Ogre::String messageString(jGetLocalizedString(message));
		int linesCount;
		mCommonMessagePanelEl->setWidth(mCommonMessagePanelBaseWidth);
		OgreBites::Label::setWrappedText(messageString, mCommonMessagePanelLabel, mCommonMessagePanelLabel->getWidth(), linesCount);
		
		{
			float h = mCommonMessagePanelBaseHeight * ((linesCount == 1) ? 3 : linesCount + 1.5);
			mCommonMessagePanelEl->setHeight(h);
		}
		
		float captW = Widget::getCaptionWidth(mCommonMessagePanelLabel->getCaption(), mCommonMessagePanelLabel);
		mCommonMessagePanelEl->setWidth(captW * 1.2);
		
		mCommonMessagePanel->show();
		mCommonMessagePanelLabel->setTop(- (mCommonMessagePanelLabel->getCharHeight() * (linesCount + 0.1)) / 2.f);
		
		//CXPoint point = CShortLineModel::sharedModel()->convertFromGameCoord(tutor->coord(model->levelIndex(), model->step()), s_scene_rect);
	    CXPoint point = convertFromGameCoord(tutor->coord(model->levelIndex(), model->step()));
		CXPoint offset = CXZeroPoint;
		
	    TRIANGLE_DIRECTION dir = tutor->direction(model->levelIndex(), model->step());
		
		switch (dir)
		{
			case TRIANGLE_TL:
				mCommonMessagePanelEl->setBorderMaterialName("Rails/messageBox_TL_Mat");
				offset = CXMakePoint(0, - mWindow->getViewport(0)->getActualHeight() / 2.f);
				break;
			case TRIANGLE_TR:
				mCommonMessagePanelEl->setBorderMaterialName("Rails/messageBox_TR_Mat");
				offset = CXMakePoint(- mCommonMessagePanelEl->getWidth(), - mWindow->getViewport(0)->getActualHeight() / 2.f);
				break;
			case TRIANGLE_BL:
				mCommonMessagePanelEl->setBorderMaterialName("Rails/messageBox_BL_Mat");
				offset = CXMakePoint(0, - mWindow->getViewport(0)->getActualHeight() / 2.f - mCommonMessagePanelEl->getHeight());
				break;
			case TRIANGLE_BR:
				mCommonMessagePanelEl->setBorderMaterialName("Rails/messageBox_BR_Mat");
				offset = CXMakePoint(- mCommonMessagePanelEl->getWidth(), - mWindow->getViewport(0)->getActualHeight() / 2.f - mCommonMessagePanelEl->getHeight());
				break;
			default:
				mCommonMessagePanelEl->setBorderMaterialName("Rails/messageBoxMat");
				offset = CXMakePoint(- mCommonMessagePanelEl->getWidth() / 2.f, - mWindow->getViewport(0)->getActualHeight() / 2.f - mCommonMessagePanelEl->getHeight() / 2.f);
				break;
		}
		
		Ogre::OverlayManager& om = Ogre::OverlayManager::getSingleton();
		
		if ((model->levelIndex() == 2 && model->step() == 5) ||
			(model->levelIndex() == 3 && model->step() == 1) ||
			(model->levelIndex() == 4 && model->step() == 1))
		{
			Ogre::OverlayElement* el = mBonusButton0->getOverlayElement();
			//point = CXMakePoint(el->getLeft(), el->getTop() + el->getHeight());
			point.x = el->_getDerivedLeft() * om.getViewportWidth();
			point.y = el->_getDerivedTop() * om.getViewportHeight() + el->getHeight() * 0.8;
		}
		else if ((model->levelIndex() == 3 && model->step() == 7) ||
				 (model->levelIndex() == 4 && model->step() == 6))
		{
			Ogre::OverlayElement* el = mBonusButton2->getOverlayElement();
			//point = CXMakePoint(el->getLeft(), el->getTop() + el->getHeight());
			point.x = el->_getDerivedLeft() * om.getViewportWidth();
			point.y = el->_getDerivedTop() * om.getViewportHeight() + el->getHeight() * 0.8;
		}
		else if (model->levelIndex() == 3 && model->step() == 4)
		{
			Ogre::OverlayElement* el = mBonusButton1->getOverlayElement();
			//point = CXMakePoint(el->getLeft(), el->getTop() + el->getHeight());
			point.x = el->_getDerivedLeft() * om.getViewportWidth();
			point.y = el->_getDerivedTop() * om.getViewportHeight() + el->getHeight() * 0.8;
		}
			
		mCommonMessagePanelEl->setTop(offset.y + point.y);
		mCommonMessagePanelEl->setLeft(offset.x + point.x);
	}
	else if (!message)
		mCommonMessagePanel->hide();
}

void COgreScene::updateBoostButton()
{
	if (CShortLineModel::sharedModel()->gameMultiplier() > 1.0)
	{
		mBoostButton->setMaterial("Rails/x2");
		if (!_gamePaused)
			CSoundManager::sharedManager()->soundTimeShiftUp();
	}
	else
	{
		mBoostButton->setMaterial("Rails/x1");
		if (!_gamePaused)
			CSoundManager::sharedManager()->soundTimeShiftDown();
	}
}

void COgreScene::updateBonusButton(OgreBites::GameButton* bb, CBonusButton* cbb)
{
	if (CShortLineModel::sharedModel()->gameMode() == gm_demo)
	{
		// do nothing
	}
	else
	{
		cbb->updateButton();
		
		Ogre::String matName;
		switch (cbb->bonus())
		{
			case bonus_boost:		matName = "Rails/bonus_boost";			break;
			case bonus_reverse:		matName = "Rails/bonus_reverse";		break;
			case bonus_call:		matName = "Rails/bonus_call";			break;
			case bonus_server:		matName = "Rails/bonus_server";			break;
			case bonus_rm_crash:	matName = "Rails/bonus_remove_crash";	break;
			case bonus_rm_train:	matName = "Rails/bonus_remove_train";	break;
			default:
				matName = "SdkTrays/Empty";
			break;
		}

		if (cbb->bonusButtonState() == BONUS_BUTTON_STATE_CLICKED ||
	   		cbb->bonusButtonState() == BONUS_BUTTON_STATE_FILLED)
			matName += "_pressed";
	
		bb->setMaterial(matName);
		bb->setFill((cbb->bonusButtonState() == BONUS_BUTTON_STATE_FILLED) ? cbb->partOfFilling() : 0.0);
	}
}

void COgreScene::updateScene(double timeSinceLastFrame)
{
	static double lastExplosionsUpdateTime = 0;
	static double lastCaptionsUpdateTime = 0;
	static double lastMadColorUpdateTime = 0;
	lastExplosionsUpdateTime += timeSinceLastFrame;
	lastCaptionsUpdateTime += timeSinceLastFrame;
	lastMadColorUpdateTime += timeSinceLastFrame;
	
#if MAD_COLOR_MANUAL_UPDATE
	if (lastMadColorUpdateTime > MAD_COLOR_UPDATE_INTERVAL)
		m_madTrainCycle = !m_madTrainCycle;
#endif
	
	if (m_faMessages.empty() == false)
	{
		for (NBSIterator it = m_faMessages.begin(); it != m_faMessages.end(); )
		{
			ColourValue color = (*it).bill->getColour();
			double now = CShortLineModel::sharedModel()->timer()->seconds();
			if ((now - (*it).startTime > FAMESSAGE_LIFETIME) || color.a < 0.01)
			{
				if ((*it).node->numAttachedObjects())
				{
					MovableText* caption = (MovableText*)(*it).node->getAttachedObject(0);
					(*it).node->detachObject(caption);
					delete caption;
				}
				
				if (cloudsBillSet && it->bill)
					cloudsBillSet->removeBillboard(it->bill);
				
				it->node->removeAndDestroyAllChildren();
				mSceneMgr->destroySceneNode(it->node);
				
				it = m_faMessages.erase(it);
			}
			else
			{
 				Vector3 pos = (*it).node->getPosition();
                
                if ((*it).isUpMoving == true)
                    pos.y += timeSinceLastFrame * FAMESSAGE_INC_Z_COORD_FACTOR;
                else
                    pos.y -= timeSinceLastFrame * FAMESSAGE_INC_Z_COORD_FACTOR;
				
				(*it).node->setPosition(pos);
				(*it).bill->setPosition(pos);
				
				ColourValue color = (*it).bill->getColour();
				if (color.a > 0.01)
					color.a -= timeSinceLastFrame * FAMESSAGE_ALPHA_FACTOR;
				(*it).bill->setColour(color);
				
				if ((*it).node->numAttachedObjects())
				{
					MovableText* caption = (MovableText*)(*it).node->getAttachedObject(0);
					ColourValue color = caption->getColor();
					if (color.a > 0.01)
						color.a -= timeSinceLastFrame * FAMESSAGE_ALPHA_FACTOR;
					caption->setColor(color);
				}
				
				++it;
			}
		}
	}
	
	if (m_explosions.empty() == false && (lastExplosionsUpdateTime - 0.02) > 0.005)
	{
		lastExplosionsUpdateTime = 0;
		for (NBSIterator it = m_explosions.begin(); it != m_explosions.end(); )
		{
			if (it->bill->getTexcoordIndex() == 31)
			{
				if (explosionBillSet && it->bill)
					explosionBillSet->removeBillboard(it->bill);
				
				it = m_explosions.erase(it);
			}
			else
			{
				if (explosionBillSet && it->bill)
					it->bill->setTexcoordIndex(it->bill->getTexcoordIndex()+1);
				++it;
			}
		}
	}
	
	for (TTrainIterator it = m_trains.begin(); it != m_trains.end(); ++it)
	{
		TTrain& currTrain = *it;
		
		float angle = -currTrain.train->realAngleForRect(s_scene_rect);
		if (currTrain.train->isFacedRight() && (angle > -90) && (angle < 90))
			angle += 180;
		
		
		currTrain.node->resetOrientation();
		currTrain.node->yaw(Degree(angle));
		//std::cout << currTrain.train->velocity() << std::endl;
		
		//		if (currTrain.train->type() == DAMAGE)
		//		{
		//			currTrain.node->resetOrientation();
		//			currTrain.node->pitch(Degree(90));
		//		}
		
		if (currTrain.train->type() == HAND_CAR && currTrain.animState)
		{
            //NSLOGFLOAT(timeSinceLastFrame);
            
			//currTrain.animState->addTime(timeSinceLastFrame/100.);
            currTrain.animState->addTime(3/100.);
		}
		
#ifdef IPHONEOS_DEPLOYMENT_TARGET
		if ((UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) || ([[UIScreen mainScreen] respondsToSelector:@selector(scale)] == YES && [[UIScreen mainScreen] scale] == 2.00))
#endif
		{
			if (currTrain.train->hasSmoke() &&
				mSceneMgr->hasSceneNode(currTrain.entity->getName() + "_SMOKE_SceneNode"))
			{
				
				ParticleSystem* smokePS = mSceneMgr->getParticleSystem(currTrain.entity->getName() + "_SMOKE_ParticleSystem");
				ParticleEmitter* smokeEmitter = smokePS->getEmitter(0);
				smokeEmitter->setMaxParticleVelocity(6 + fabs(currTrain.train->velocity())*2.);
				smokeEmitter->setDirection(Vector3(currTrain.train->velocity()/currTrain.train->maxVelocity(), 1.-fabs(currTrain.train->velocity())/currTrain.train->maxVelocity(), 0));
				//smokeEmitter->setDirection(Vector3((currTrain.train->velocity()/currTrain.train->maxVelocity())/2., 1., 0));
			}
			
			if (currTrain.train->isMadTrain() &&
				mSceneMgr->hasSceneNode(currTrain.entity->getName() + "_SPARKS_LEFT_SceneNode") &&
				mSceneMgr->hasSceneNode(currTrain.entity->getName() + "_SPARKS_RIGHT_SceneNode"))
			{
				
				ParticleSystem* sparksLPS = mSceneMgr->getParticleSystem(currTrain.entity->getName() + "_SPARKS_LEFT_ParticleSystem");
				ParticleEmitter* sparksLEmitter = sparksLPS->getEmitter(0);
				sparksLEmitter->setMaxParticleVelocity(6 + fabs(currTrain.train->velocity())*2.);
				sparksLEmitter->setDirection(Vector3(0., 1., 0.2));
				//sparksLEmitter->setDirection(Vector3(currTrain.train->velocity()/currTrain.train->maxVelocity(), 0., 0));
				sparksLEmitter->setEmissionRate(fabs(currTrain.train->velocity())*100.);
				//ParticleAffector* force = sparksLPS->getAffector(1);
				//Ogre::String fv = force->getParameter("force_vector");
				//NSLog(@"%s", fv.c_str());
				
				ParticleSystem* sparksRPS = mSceneMgr->getParticleSystem(currTrain.entity->getName() + "_SPARKS_RIGHT_ParticleSystem");
				ParticleEmitter* sparksREmitter = sparksRPS->getEmitter(0);
				sparksREmitter->setMaxParticleVelocity(6 + fabs(currTrain.train->velocity())*2.);
				sparksREmitter->setDirection(Vector3(0., 1., -0.2));
				//sparksREmitter->setDirection(Vector3(currTrain.train->velocity()/currTrain.train->maxVelocity(), 0., 0));
				sparksREmitter->setEmissionRate(fabs(currTrain.train->velocity())*100.);
			}
		}
		
		if ((lastCaptionsUpdateTime > CAPTIONS_UPDATE_INTERVAL) && currTrain.train->existsBadge())
		{
			int term = currTrain.train->trainTimer();
			if ((term != MILLION) && (mSceneMgr->hasSceneNode(currTrain.entity->getName() + "_CAPTION_SceneNode")))
			{
				SceneNode* captionBackNode = mSceneMgr->getSceneNode(currTrain.entity->getName() + "_CAPTION_SceneNode");
				MovableText* caption = (MovableText*)captionBackNode->getAttachedObject(currTrain.entity->getName() + "_CAPTION_MovableText");
                
                int badge = currTrain.train->badge();
                
				
				if (currTrain.train->isCheapeningTrain())
				{
					if (currTrain.train->arrivePrice() < 0)
					{
						caption->setCaption("-" + Ogre::StringConverter::toString((int)ABS(badge)));
						caption->setColor(ColourValue(.94, 0.35, 0.21));
					}
					else
						caption->setCaption(Ogre::StringConverter::toString(badge));
				}
                else if (currTrain.train->isPricingUpTrain())
                {
                    caption->setCaption(Ogre::StringConverter::toString((int)ABS(badge)));
                    caption->setColor(ColourValue(0.0, 0.0, 0.0));//black color
                }
				else if (currTrain.train->isChameleonTrain())
				{
					currTrain.bubbleBill->setTexcoordIndex(10 + currTrain.train->color());
					caption->setCaption(Ogre::StringConverter::toString(badge));
				}
				else if (currTrain.train->isBombTrain())
				{
					caption->setCaption(Ogre::StringConverter::toString(badge));
				}
			}
		}
		
		CXPoint coord = CXMakePoint(currTrain.train->shakenCenter().x, currTrain.train->shakenCenter().y);
		coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
		currTrain.node->setPosition(coord.x, currTrain.train->shakenCenter().z * SHAKE_SCALE_Z, coord.y);
		
		if (currTrain.bubbleBill && currTrain.captionNode)
		{
			CXPoint bubbleCoord = BSCenterOf(currTrain.train->coordAtIndex(0), currTrain.train->coordAtIndex(1));
			bubbleCoord = CShortLineModel::sharedModel()->convertFromGameCoord(bubbleCoord, s_scene_rect);
			
			currTrain.bubbleBill->setPosition(bubbleCoord.x, 7.0, bubbleCoord.y);
			currTrain.captionNode->setPosition(bubbleCoord.x, 7.0, bubbleCoord.y);
		}
		
		//if (currTrain.train->isChameleonTrain() && (currTrain.train->color() != currTrain.color))
		//{
		//	syncTrainColor(currTrain.entity, currTrain.train);
		//}
		
#if MAD_COLOR_MANUAL_UPDATE
		if (currTrain.train->isMadTrain() && lastMadColorUpdateTime > MAD_COLOR_UPDATE_INTERVAL)
		{
			syncTrainColor(currTrain.entity, currTrain.train);
		}
#endif
		
		//----
		
		CarVector carsVec = currTrain.train->cars();
		for (int i = 0; i < carsVec.size(); i++)
		{
			CCar* currCar = carsVec.at(i);
			
			float angle = -currCar->realAngleForRect(s_scene_rect);
			if (currCar->isFacedRight() && (angle > -90) && (angle < 90))
				angle += 180;
			
			currTrain.cars[i].node->resetOrientation();
			currTrain.cars[i].node->yaw(Degree(angle));
			CXPoint coord = CXMakePoint(currCar->shakenCenter().x, currCar->shakenCenter().y);
			
			//            if ((coord.x < -2) || (coord.x > MAX_X+1) || (coord.y < -2) || (coord.y > MAX_Y+1))
			//            {
			//                NSLog(@"stop");
			//            }
            
            
			coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
			currTrain.cars[i].node->setPosition(coord.x, currCar->shakenCenter().z * SHAKE_SCALE_Z, coord.y);
			
			//if (currTrain.train->isChameleonTrain() && (currTrain.train->color() != currTrain.color))
			//{
			//	syncVagonColor(currTrain.cars[i].entity, currCar, currTrain.train);
			//}
			
#if MAD_COLOR_MANUAL_UPDATE
			if (currTrain.train->isMadTrain() && lastMadColorUpdateTime > MAD_COLOR_UPDATE_INTERVAL)
			{
				syncVagonColor(currTrain.cars[i].entity, currCar, currTrain.train);
			}
#endif
		}
		
		//if (currTrain.train->isChameleonTrain() && (currTrain.train->color() != currTrain.color)) // !!! should be after train and all cars color sync
		//{
		//	currTrain.color = currTrain.train->color();
		//}
	}
	
	for (TCityIterator it = m_cities.begin(); it != m_cities.end(); ++it)
	{
		TCity& currCity = *it;
		
		if (currCity.city->state() == CS_WILL_SOON_APPEAR && currCity.cityBill)
		{
			currCity.phase += timeSinceLastFrame;
			currCity.cityBill->setColour(ColourValue(1., 1., 1., 0.5 + (cos(currCity.phase * 8.)/2.0)));
		}
		
		if ((lastCaptionsUpdateTime > CAPTIONS_UPDATE_INTERVAL))
		{
			String cityName = "CITY_" + Ogre::StringConverter::toString(currCity.city->color());
			if (mSceneMgr->hasSceneNode(cityName + "_CAPTION_SceneNode"))
			{
				SceneNode* captionNode = mSceneMgr->getSceneNode(cityName + "_CAPTION_SceneNode");
				
				if (currCity.city->isWaitingTrains())
				{
					//captionNode->setVisible(true);
					
					if (currCity.boardBill == NULL)
					{
                        Vector3 pos = captionNode->getPosition();
#if ANDROID_DEPLOYMENT_TARGET
						// do nothing
#else
						pos.z -= frontCitiesBillSet->getDefaultHeight()/6;
#endif
						
						currCity.boardBill = cityAdditionsBillSet->createBillboard(pos);
						currCity.boardBill->setTexcoordIndex(1);
					}
                    
                    if (currCity.colorBill == NULL)
                    {
                        currCity.colorBill = cityAdditionsBillSet->createBillboard(currCity.boardBill->mPosition);
                        currCity.colorBill->setTexcoordIndex(6);
                    }
                    
                    if (currCity.digitBill == NULL)
                    {
						
                        currCity.digitBill = cityAdditionsBillSet->createBillboard(currCity.boardBill->mPosition);
                        currCity.digitBill->setTexcoordIndex(17);//цифра 5
					}
                    
                    
                    int waitingColor = currCity.city->waitingColor();
                    int colorTexcoordIndex = 4;
                    if (waitingColor != -1)
                    {
                        switch (waitingColor)
                        {
                            case CITY_COLOR_BLACK:  colorTexcoordIndex = 4; break;
                            case CITY_COLOR_YELLOW: colorTexcoordIndex = 6; break;
                            case CITY_COLOR_RED:    colorTexcoordIndex = 7; break;
                            case CITY_COLOR_BLUE:   colorTexcoordIndex =13; break;
                            case CITY_COLOR_PURPLE: colorTexcoordIndex =12; break;
                            case CITY_COLOR_GREEN:  colorTexcoordIndex =14; break;
                            case CITY_COLOR_BROWN: colorTexcoordIndex = 5; break;
                            case CITY_COLOR_BLINK:  colorTexcoordIndex =15; break;
                            case CITY_COLOR_NONE:   colorTexcoordIndex = 4; break;
                            default: colorTexcoordIndex = 15;               break;
                        }
                    }
                    currCity.colorBill->setTexcoordIndex(colorTexcoordIndex);
					
                    
                    int badge = currCity.city->badge();
                    
                    int countOfWaitingTrains = MIN(currCity.city->countOfWaitingTrains(), 3);
                    switch(countOfWaitingTrains)
                    {
                        case 1 ... 3:
                            currCity.boardBill->setTexcoordIndex(countOfWaitingTrains);
                            break;
                        default:
                            break;
                    }
                    
                    int digitTexcoordIndex = 0;
                    switch (badge) {
                        case 0: digitTexcoordIndex =  8; break;
                        case 1: digitTexcoordIndex =  9; break;
                        case 2: digitTexcoordIndex = 10; break;
                        case 3: digitTexcoordIndex = 11; break;
                        case 4: digitTexcoordIndex = 16; break;
                        case 5: digitTexcoordIndex = 17; break;
                        default:digitTexcoordIndex = 18; break;
                    }
                    
                    currCity.digitBill->setTexcoordIndex(digitTexcoordIndex);
                    
				}
				else
				{
					captionNode->setVisible(false);
					if (currCity.boardBill != NULL)
					{
						cityAdditionsBillSet->removeBillboard(currCity.boardBill);
						currCity.boardBill = NULL;
					}
                    
                    if (currCity.colorBill != NULL)
					{
						cityAdditionsBillSet->removeBillboard(currCity.colorBill);
						currCity.colorBill = NULL;
					}
                    
                    if (currCity.digitBill != NULL)
					{
						cityAdditionsBillSet->removeBillboard(currCity.digitBill);
						currCity.digitBill = NULL;
					}
                    
                    
				}
			}
		}
	}
	
	if (lastCaptionsUpdateTime > CAPTIONS_UPDATE_INTERVAL)
		lastCaptionsUpdateTime = 0;
	
#if MAD_COLOR_MANUAL_UPDATE
	if (lastMadColorUpdateTime > MAD_COLOR_UPDATE_INTERVAL)
		lastMadColorUpdateTime = 0;
#endif
    
    //NSLog(@"*********** end updateScene ***********");
}

//-------------------------
#pragma mark - Add/Remove Trains

void COgreScene::pushNewTrain(CTrain* newTrain)
{
	Ogre::Billboard*	bubbleBill = NULL;
	Ogre::SceneNode*	captionNode = NULL;
	Ogre::Entity*		trainEntity = getTrainEntity(newTrain);
	Ogre::SceneNode*	node = mSceneMgr->getRootSceneNode()->createChildSceneNode();
	node->setScale(Ogre::Vector3(MODEL_SCALE, MODEL_SCALE * m_displayFactor, MODEL_SCALE));
	{
		Ogre::SceneNode*	childNode = node->createChildSceneNode();
		childNode->attachObject(trainEntity);
		
#ifdef IPHONEOS_DEPLOYMENT_TARGET
		if ((UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) || ([[UIScreen mainScreen] respondsToSelector:@selector(scale)] == YES && [[UIScreen mainScreen] scale] == 2.00))
#endif
		{
			if (newTrain->isMadTrain())
			{
				ParticleSystem*		sparksLParticleSystem = mSceneMgr->createParticleSystem(trainEntity->getName() + "_SPARKS_LEFT_ParticleSystem", "TrainSparks");
				Ogre::SceneNode*	childSparksLNode = childNode->createChildSceneNode(trainEntity->getName() + "_SPARKS_LEFT_SceneNode", Vector3(-2., 0.0, 1.8));
				childSparksLNode->setScale(SMOKE_SCALE * Ogre::Vector3::UNIT_SCALE);
				childSparksLNode->attachObject(sparksLParticleSystem);
				
				ParticleSystem*		sparksRParticleSystem = mSceneMgr->createParticleSystem(trainEntity->getName() + "_SPARKS_RIGHT_ParticleSystem", "TrainSparks");
				Ogre::SceneNode*	childSparksRNode = childNode->createChildSceneNode(trainEntity->getName() + "_SPARKS_RIGHT_SceneNode", Vector3(-2., 0.0, -1.8));
				childSparksRNode->setScale(SMOKE_SCALE * Ogre::Vector3::UNIT_SCALE);
				childSparksRNode->attachObject(sparksRParticleSystem);
			}
			
			Ogre::String smokePSName = "TrainSmokePS";
			if (newTrain->hasSmoke())
			{
				ParticleSystem*		smokeParticleSystem = mSceneMgr->createParticleSystem(trainEntity->getName() + "_SMOKE_ParticleSystem", smokePSName);
				Ogre::SceneNode*	childSmokeNode = childNode->createChildSceneNode(trainEntity->getName() + "_SMOKE_SceneNode", Vector3(-2., 6.4, 0.));
				childSmokeNode->setScale(SMOKE_SCALE * Ogre::Vector3::UNIT_SCALE);
				childSmokeNode->attachObject(smokeParticleSystem);
			}
			else if (newTrain->type() == DAMAGE)
			{
				ParticleSystem*		smokeParticleSystem = mSceneMgr->createParticleSystem(trainEntity->getName() + "_SMOKE_ParticleSystem", smokePSName);
				
				Ogre::SceneNode*	childSmokeNode = childNode->createChildSceneNode(trainEntity->getName() + "_SMOKE_SceneNode", Vector3(0., 1., 0.));
				childSmokeNode->setScale(SMOKE_SCALE * Ogre::Vector3::UNIT_SCALE);
				childSmokeNode->attachObject(smokeParticleSystem);
				
				{
					ParticleEmitter* smokeEmitter = smokeParticleSystem->getEmitter(0);
					smokeEmitter->setEmissionRate(4);
					smokeEmitter->setParticleVelocity(0.1, 4.);
					smokeEmitter->setDirection(Vector3(.0, 1., .0));
					smokeEmitter->setTimeToLive(0.1, 1.5);
				}
			}
		}
		
		if (newTrain->existsBadge())
		{
			{
				// --- background ---
				bubbleBill = bubblesBillSet->createBillboard(0., 0., 0.);
                
                int texcoordIndex = 3;
                
                if (newTrain->isBombTrain())
                    texcoordIndex = 8;
                else if(newTrain->isChameleonTrain())
                    texcoordIndex = 10 + newTrain->color();
                else if(newTrain->isPricingUpTrain())
                    texcoordIndex = 4;
                else  if(newTrain->isCheapeningTrain())
                    texcoordIndex = 9;
                
				bubbleBill->setTexcoordIndex(texcoordIndex);
				// --- caption ---
				MovableText* msg = new MovableText(trainEntity->getName() + "_CAPTION_MovableText", Ogre::StringConverter::toString(newTrain->badge()), "Comic5-17");
				
				msg->setTextAlignment(MovableText::H_CENTER, MovableText::V_ABOVE);
				msg->setColor(ColourValue::White);
				msg->setCharacterHeight( 1.7 );
				msg->setRenderQueueGroup(BUBBLES_RENDER_QUEUE);
				msg->showOnTop(true);
				
				if (newTrain->isBombTrain())
					msg->setLocalTranslation(Vector3(0.3, -.8, 0.0));
				else if (newTrain->isChameleonTrain())
				{
					msg->setCharacterHeight( 1.4 );
					msg->setLocalTranslation(Vector3(0.5, -1.1, 0.0));
				}
				else if (newTrain->isCheapeningTrain())
					msg->setLocalTranslation(Vector3(0.5, -1.0, 0.0));
                else if (newTrain->isPricingUpTrain())
                {
                    msg->setLocalTranslation(Vector3(0.5, -.2, 0.0));
                    msg->setCharacterHeight( 1.9 );
                    msg->setColor(ColourValue(0.0, 0.0, 0.0));//black color
                }
				
				captionNode = mSceneMgr->getRootSceneNode()->createChildSceneNode(trainEntity->getName() + "_CAPTION_SceneNode", Vector3(0, 0, 0));
				captionNode->attachObject(msg);
				captionNode->setVisible(newTrain->trainTimer() != MILLION);
			}
		}
	}
	
	AnimationState* anim = NULL;
	if (newTrain->type() == HAND_CAR)
	{
		anim = trainEntity->getAnimationState("anim0");
		anim->setEnabled(true);
		//node->setScale(Ogre::Vector3(MODEL_SCALE*2., MODEL_SCALE*2. * m_displayFactor, MODEL_SCALE*2.));
        anim->addTime(1/100.);
	}
	
	{
		float angle = -newTrain->realAngleForRect(s_scene_rect);
		if (newTrain->isFacedRight() && (angle > -90) && (angle < 90))
			angle += 180;
		
		node->resetOrientation();
		node->yaw(Degree(angle));
		CXPoint coord = CXMakePoint(newTrain->shakenCenter().x, newTrain->shakenCenter().y);
		coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
		node->setPosition(coord.x, newTrain->shakenCenter().z * SHAKE_SCALE_Z, coord.y);
	}
	
	
	m_trains.push_back(TTrain(newTrain, trainEntity, node, newTrain->color(), bubbleBill, captionNode, anim));
	
	//	if (newTrain->type() == DAMAGE)
	//		node->pitch(Degree(90));
	
	
	//----
	
	CarVector carsVec = newTrain->cars();
	for (CarIterator it = carsVec.begin(); it != carsVec.end(); ++it)
	{
		CCar* currCar = *it;
		Ogre::Entity* vagonEntity = getVagonEntity(currCar, newTrain);
		
		//Ogre::Entity*		entity = mSceneMgr->createEntity("vagon_low_texture.ma.mesh");
		Ogre::SceneNode*	node = mSceneMgr->getRootSceneNode()->createChildSceneNode();
		
		if (currCar->type() == CISTERN)
			node->setScale(Ogre::Vector3(MODEL_CISTERN_SCALE, MODEL_SCALE * m_displayFactor, MODEL_SCALE));
		else
			node->setScale(Ogre::Vector3(MODEL_SCALE, MODEL_SCALE * m_displayFactor, MODEL_SCALE));
		
		{
			//Ogre::SceneNode*	childNode = node->createChildSceneNode(Vector3(10*MODEL_SCALE, 0, 0));
			Ogre::SceneNode*	childNode = node->createChildSceneNode();
			childNode->attachObject(vagonEntity);
			//node->attachObject(entity);
		}
		
		{
			float angle = -currCar->realAngleForRect(s_scene_rect);
			if (currCar->isFacedRight() && (angle > -90) && (angle < 90))
				angle += 180;
			
			node->resetOrientation();
			node->yaw(Degree(angle));
			CXPoint coord = CXMakePoint(currCar->shakenCenter().x, currCar->shakenCenter().y);
			
			coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
			node->setPosition(coord.x, currCar->shakenCenter().z * SHAKE_SCALE_Z, coord.y);
		}
		
		m_trains[m_trains.size()-1].cars.push_back(TEN(vagonEntity, node));
	}
	
}

void COgreScene::eraseTheTrain(TrainIterator trainIt)
{
	for (TTrainIterator it = m_trains.begin(); it != m_trains.end(); ++it)
	{
		TTrain& currTrain = *it;
		if (currTrain.train == (*trainIt))
		{
			mSceneMgr->destroyParticleSystem(currTrain.entity->getName() + "_SMOKE_ParticleSystem");
			mSceneMgr->destroyParticleSystem(currTrain.entity->getName() + "_SPARKS_LEFT_ParticleSystem");
			mSceneMgr->destroyParticleSystem(currTrain.entity->getName() + "_SPARKS_RIGHT_ParticleSystem");
			if (currTrain.captionNode)
			{
				MovableText* caption = (MovableText*)currTrain.captionNode->getAttachedObject(currTrain.entity->getName() + "_CAPTION_MovableText");
				currTrain.captionNode->detachObject(caption);
				delete caption;
				
				mSceneMgr->destroySceneNode(currTrain.captionNode);
				
				if (bubblesBillSet && currTrain.bubbleBill)
					bubblesBillSet->removeBillboard(currTrain.bubbleBill);
			}
			
			currTrain.node->removeAndDestroyAllChildren();
			mSceneMgr->destroySceneNode(currTrain.node);
			mSceneMgr->destroyEntity(currTrain.entity);
			
			for (int i = 0; i < currTrain.cars.size(); i++)
			{
				currTrain.cars[i].node->removeAndDestroyAllChildren();
				mSceneMgr->destroySceneNode(currTrain.cars[i].node);
				mSceneMgr->destroyEntity(currTrain.cars[i].entity);
			}
			
			currTrain.cars.clear();
			m_trains.erase(it);
			break;
		}
	}
}

void COgreScene::removeAllTrains()
{
    if (m_trains.empty())
        return;
    
	for (TTrainIterator it = m_trains.begin(); it != m_trains.end(); ++it)
	{
		TTrain& currTrain = *it;
		{
			mSceneMgr->destroyParticleSystem(currTrain.entity->getName() + "_SMOKE_ParticleSystem");
			mSceneMgr->destroyParticleSystem(currTrain.entity->getName() + "_SPARKS_LEFT_ParticleSystem");
			mSceneMgr->destroyParticleSystem(currTrain.entity->getName() + "_SPARKS_RIGHT_ParticleSystem");
			if (currTrain.captionNode)
			{
				MovableText* caption = (MovableText*)currTrain.captionNode->getAttachedObject(currTrain.entity->getName() + "_CAPTION_MovableText");
				currTrain.captionNode->detachObject(caption);
				delete caption;
				
				mSceneMgr->destroySceneNode(currTrain.captionNode);
				
				if (bubblesBillSet && currTrain.bubbleBill)
					bubblesBillSet->removeBillboard(currTrain.bubbleBill);
			}
			
			currTrain.node->removeAndDestroyAllChildren();
			mSceneMgr->destroySceneNode(currTrain.node);
			mSceneMgr->destroyEntity(currTrain.entity);
			
			for (int i = 0; i < currTrain.cars.size(); i++)
			{
				currTrain.cars[i].node->removeAndDestroyAllChildren();
				mSceneMgr->destroySceneNode(currTrain.cars[i].node);
				mSceneMgr->destroyEntity(currTrain.cars[i].entity);
			}
			
			currTrain.cars.clear();
		}
	}
	
	m_trains.clear();
}

void COgreScene::removeCar(CTrain* train, CCar* car)
{
	for (TTrainIterator it = m_trains.begin(); it != m_trains.end(); ++it)
	{
		TTrain& currTrain = *it;
		if (currTrain.train == train)
		{
			for (int i = 0; i < currTrain.train->cars().size(); i++)
			{
				if ((currTrain.train->cars()[i] == car) && (i < currTrain.cars.size()))
				{
					currTrain.cars[i].node->removeAndDestroyAllChildren();
					mSceneMgr->destroySceneNode(currTrain.cars[i].node);
					mSceneMgr->destroyEntity(currTrain.cars[i].entity);
					
					currTrain.cars.erase(currTrain.cars.begin() + i);
					break;
				}
			}
		}
	}
}




//-------------------------
#pragma mark - Add/Remove/Upgdate Semaphores

void COgreScene::updateSemaphore(CSemaphore* sem)
{
	for (TSemaphoreIterator it = m_semaphores.begin(); it != m_semaphores.end(); ++it)
	{
		TSemaphore& currSemaphore = *it;
		if (currSemaphore.semaphore == sem)
		{
            Real flip = (sem->type() == 1) ? 2: 0;
			
			if (currSemaphore.bill)
				currSemaphore.bill->setTexcoordIndex(flip + sem->state()%2);
			break;
		}
	}
}

void COgreScene::addSemaphore(CSemaphore* semaphore)
{
	if (semaphoresBillSet == NULL)
	{
		LogManager::getSingleton().logMessage("Error: semaphoresBillSet == NULL");
		return;
	}
	
	Billboard* bill = NULL;
	{
		//CXPoint coord = semaphore->coord();
		CXPoint coord = semaphore->exactlyCoord();
		coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
		
		Real flip = (semaphore->type() == 1) ? 2: 0;
		
		bill = semaphoresBillSet->createBillboard(coord.x, 0., coord.y);
		
		bill->setTexcoordIndex(flip + semaphore->state()%2);
	}
	
	m_semaphores.push_back(TSemaphore(semaphore, bill));
}

void COgreScene::eraseSemaphore(SemaphoreIterator semaphoreIt)
{
	for (TSemaphoreIterator it = m_semaphores.begin(); it != m_semaphores.end(); ++it)
	{
		TSemaphore& currSemaphore = *it;
		if (currSemaphore.semaphore == (*semaphoreIt))
		{
			if (currSemaphore.bill)
				semaphoresBillSet->removeBillboard(currSemaphore.bill);
			
			m_semaphores.erase(it);
			break;
		}
	}
}

void COgreScene::removeAllSemaphores()
{
	for (TSemaphoreIterator it = m_semaphores.begin(); it != m_semaphores.end(); ++it)
	{
		TSemaphore& currSemaphore = *it;
		
		if (currSemaphore.bill)
			semaphoresBillSet->removeBillboard(currSemaphore.bill);
	}
	
	m_semaphores.clear();
}

#pragma mark - Add/Remove Trees

void COgreScene::addTree(CTree* tree)
{
	if (treesBillSet == NULL)
	{
		LogManager::getSingleton().logMessage("Error: treesBillSet == NULL");
		return;
	}
	
	CXPoint coord = tree->coord();
	coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
	
	//LogManager::getSingleton().logMessage("Push new TREE into point:" + StringConverter::toString(coord.x) + ", " + StringConverter::toString(coord.y));
	
	// не забываем сместить чтоб совпадали центры деревьев.
    // в оригинале центр дерева должен быть смещён на 4 пикселя влево и на 8 вверх (если картинка 64x64).
    //отсюда и коэффициенты 0.0625 и 0.125
	Billboard* bill = treesBillSet->createBillboard(coord.x + treesBillSet->getDefaultWidth()*0.0625, 0., coord.y + treesBillSet->getDefaultHeight()*0.125);
	
	int treeIndex = tree->treeType();
	bill->setTexcoordIndex(treeIndex);
    
    // if (treeIndex >= 40 && treeIndex <= 42)
    // {
    //     const float k = 1.1;
    //     bill->setDimensions(treesBillSet->getDefaultWidth()*k, treesBillSet->getDefaultHeight()*k);
    // }
	
	m_trees.push_back(TTree(tree, NULL, NULL, bill));
}

void COgreScene::eraseTree(TreeIterator treeIt)
{
	for (TTreeIterator it = m_trees.begin(); it != m_trees.end(); ++it)
	{
		TTree& currTree = *it;
		if (currTree.tree == (*treeIt))
		{
			treesBillSet->removeBillboard(currTree.bill);
			
			m_trees.erase(it);
			break;
		}
	}
}

void COgreScene::removeAllTrees()
{
	for (TTreeIterator it = m_trees.begin(); it != m_trees.end(); ++it)
	{
		TTree& currTree = *it;
		treesBillSet->removeBillboard(currTree.bill);
	}
	
	m_trees.clear();
}

#pragma mark - Add/Remove Cities

void COgreScene::addCity(CCity* city)
{
	if (frontCitiesBillSet == NULL)
	{
		LogManager::getSingleton().logMessage("Error: frontCitiesBillSet == NULL");
		return;
	}
	
	if (isHasCity(city))
		eraseCity(city);
	
	//int cityIndex = (city->state() == CS_ACTIVE)? city->color() : CITY_COLOR_BLACK;
	int cityIndex = city->color() * 4;
	CXPoint coord = city->coord();
	coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
	
	Real flip = 1;
	if (city->isLeft())
	{
		flip = -1;
	}
	
	if (city->isLeft())
	{
		if ((city->rail()->type() == RT_STRAIGHT_RIGHT_UP) || (city->rail()->type() == RT_CURVE_DOWN))
		{
			cityIndex += 0;
		}
		else
		{
			cityIndex += 1;
		}
	}
	else
	{
		if ((city->rail()->type() == RT_STRAIGHT_RIGHT_UP) || (city->rail()->type() == RT_CURVE_UP))
		{
			cityIndex += 3;
		}
		else
		{
			cityIndex += 2;
		}
	}
	
#ifdef IPHONEOS_DEPLOYMENT_TARGET
	if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
	{
		coord.x += (city->isLeft()? -0.5 : .1);
		
		if (city->rail()->type() == RT_STRAIGHT_RIGHT_UP)
			coord.y += (city->isLeft()? 15.5 : 18.5);
		else if (city->rail()->type() == RT_CURVE_DOWN)
			coord.y += (city->isLeft()? 14. : 16.);
		else if (city->rail()->type() == RT_CURVE_UP)
			coord.y += (city->isLeft()? 18. : 18.);
		else
			coord.y += (city->isLeft()? 18.5 : 16.5);
	}
	else// if ([[UIScreen mainScreen] respondsToSelector:@selector(scale)] == YES && [[UIScreen mainScreen] scale] == 2.00)
	{
		if ([[UIScreen mainScreen] respondsToSelector:@selector(scale)] == YES && [[UIScreen mainScreen] scale] == 2.00)
			coord.x += (city->isLeft()? -.1 : .1);
		
		if (city->rail()->type() == RT_STRAIGHT_RIGHT_UP)
			coord.y += (city->isLeft()? 15. : 17.);
		else if (city->rail()->type() == RT_CURVE_DOWN)
			coord.y += (city->isLeft()? 14. : 14.);
		else if (city->rail()->type() == RT_CURVE_UP)
			coord.y += (city->isLeft()? 17. : 17.);
		else
			coord.y += (city->isLeft()? 17. : 15.);
	}
#elif MACOSX_DEPLOYMENT_TARGET
	{
		coord.x += (city->isLeft()? 1.4 : -1.4);
		
		if (city->rail()->type() == RT_STRAIGHT_RIGHT_UP)
			coord.y += (city->isLeft()? 17. : 18.);
		else if (city->rail()->type() == RT_CURVE_DOWN)
			coord.y += (city->isLeft()? 16. : 16.);
		else if (city->rail()->type() == RT_CURVE_UP)
			coord.y += (city->isLeft()? 17. : 17.);
		else
			coord.y += (city->isLeft()? 18. : 17.5);
	}
#elif ANDROID_DEPLOYMENT_TARGET
	{
		//coord.x += (city->isLeft()? 1.4 : -1.4);
		coord.x += (city->isLeft()? 0.0 : -0.0);
		
		if (city->rail()->type() == RT_STRAIGHT_RIGHT_UP)
			coord.y += (city->isLeft()? 18. : 20.5);
		else if (city->rail()->type() == RT_CURVE_DOWN)
			coord.y += (city->isLeft()? 17. : 17.);
		else if (city->rail()->type() == RT_CURVE_UP)
			coord.y += (city->isLeft()? 20. : 20.);
		else
			coord.y += (city->isLeft()? 20. : 19.);
		
		coord.y += 2.5;
	}
#endif
	
	Billboard* cityBill = NULL;
	
	if (city->state() == CS_WILL_SOON_APPEAR || city->state() == CS_ONLY_RAIL_VISIBLE)
	{
		addFutureCityRail(city->fakeRail());
		addFutureCityRail(city->rail());
		
		if (city->state() == CS_WILL_SOON_APPEAR)
		{
			cityBill = frontCitiesBillSet->createBillboard(coord.x, 10., coord.y + 10.);
			cityBill->setTexcoordIndex(cityIndex);
			cityBill->setColour(ColourValue(1., 1., 1., 0.));
		}
	}
	
	if (city->state() == CS_ACTIVE)
	{
		addRail(city->fakeRail());
		cityBill = frontCitiesBillSet->createBillboard(coord.x, 10., coord.y + 10.);
		cityBill->setTexcoordIndex(cityIndex);
		//cityBill->setColour(ColourValue(1., 1., 1., 0.5));
	}
	
	
	Ogre::SceneNode* captionNode = NULL;
	{
		String cityName = "CITY_" + Ogre::StringConverter::toString(city->color());
		captionNode = mSceneMgr->getRootSceneNode()->createChildSceneNode(cityName + "_CAPTION_SceneNode", Vector3((roundf(coord.x*10))/10., 20., (roundf(coord.y*10))/10.));
		//captionNode->setScale(1., 1.2, 1.);
		
		// --- background ---
		//		BillboardSet* billSet= mSceneMgr->createBillboardSet(cityName + "_BILLBOARD", 1);
		//		billSet->setMaterialName("train_infoboard");
		//		billSet->setBillboardOrigin(BBO_CENTER);
		//		billSet->setDefaultDimensions(8.5, 8.5 * m_displayFactor * OGRE_CAMERA_STRETCH);
		//		billSet->setAutoextend(false);
		//
		//		// attach to city node
		//		captionNode->attachObject(billSet);
		//		billSet->setRenderQueueGroup(CITYBOARD_RENDER_QUEUE);
		//		billSet->createBillboard(0., 0., 0.);
		
		// --- caption ---
		MovableText* msg = new MovableText(cityName + "_CAPTION_MovableText", Ogre::StringConverter::toString(999), "Bluebold");
		
		msg->setTextAlignment(MovableText::H_CENTER, MovableText::V_ABOVE);
		//msg->setColor(ColourValue(1., 0.9, 0.2, 1.));
        msg->setColor(ColourValue(1., 1., 1., 1.));
		msg->setCharacterHeight( 3.7f );
		msg->setRenderQueueGroup(CITYBOARD_RENDER_QUEUE);
		msg->showOnTop(true);
		
		msg->setLocalTranslation(Vector3(0.5, 2.0, 0.0));
		
		captionNode->attachObject(msg);
		captionNode->setVisible(false);
	}
	
	m_cities.push_back(TCity(city, NULL, captionNode, cityBill));
}

void COgreScene::updateCity(CCity* city)
{
	eraseCity(city);
	addCity(city);
}

void COgreScene::updateBusyOfCity(CCity* city)
{
    for (TCityIterator it = m_cities.begin(); it != m_cities.end(); ++it)
	{
		TCity& currCity = *it;
		if (currCity.city == city)
        {
            if (city->isBusy())
            {
                //currCity.cityBill->setColour(ColourValue(1., 1., 1., 0.2));
				if (currCity.lockBill == NULL)
				{
					Vector3 pos = currCity.cityBill->getPosition();
                    
                    RailType rt = city->rail()->type();
                    
                    if (currCity.city->isLeft())
                    {
                        pos.x += frontCitiesBillSet->getDefaultWidth()/5.;
                        
                        if ((rt==RT_STRAIGHT_RIGHT_UP) || (rt==RT_CURVE_DOWN))
                        {
                            float yOffset = frontCitiesBillSet->getDefaultHeight()/2.5;
                            pos.z -= yOffset;
                        }
                        else
                        {
                            float yOffset = frontCitiesBillSet->getDefaultHeight()/1.2;
                            pos.z -= yOffset;
                        }
                    }
                    else
                    {
                        pos.x -= frontCitiesBillSet->getDefaultWidth()/4.2;
                        
                        if ((rt==RT_STRAIGHT_LEFT_UP) || (rt==RT_CURVE_DOWN))
                        {
                            float yOffset = frontCitiesBillSet->getDefaultHeight()/2.5;
                            pos.z -= yOffset;
                        }
                        else
                        {
                            float yOffset = frontCitiesBillSet->getDefaultHeight()/1.2;
                            pos.z -= yOffset;
                        }
                    }
                    
					currCity.lockBill = cityAdditionsBillSet->createBillboard(pos);
					currCity.lockBill->setTexcoordIndex(0);
				}
            }
            else
            {
				if (currCity.lockBill != NULL)
				{
					cityAdditionsBillSet->removeBillboard(currCity.lockBill);
					currCity.lockBill = NULL;
				}
            }
            
            break;
        }
	}
}

bool COgreScene::isHasCity(CCity* city)
{
	for (TCityIterator it = m_cities.begin(); it != m_cities.end(); ++it)
	{
		TCity& currCity = *it;
		if (currCity.city == city)
			return true;
	}
	
	return false;
}

void COgreScene::eraseCity(CCity* city)
{
	for (TCityIterator it = m_cities.begin(); it != m_cities.end(); ++it)
	{
		TCity& currCity = *it;
		if (currCity.city == city)
		{
			if (currCity.cityBill)
				frontCitiesBillSet->removeBillboard(currCity.cityBill);
			
			eraseFutureCityRail(currCity.city->fakeRail());
			eraseFutureCityRail(currCity.city->rail());
			
			if (currCity.node)
			{
				currCity.node->removeAndDestroyAllChildren();
				//mSceneMgr->destroySceneNode(currCity.node);
			}
			
			String cityName = "CITY_" + Ogre::StringConverter::toString(currCity.city->color());
			if (mSceneMgr->hasSceneNode(cityName + "_CAPTION_SceneNode"))
			{
				SceneNode* captionBackNode = mSceneMgr->getSceneNode(cityName + "_CAPTION_SceneNode");
				MovableText* caption = (MovableText*)captionBackNode->getAttachedObject(cityName + "_CAPTION_MovableText");
				captionBackNode->detachObject(caption);
				delete caption;
				
				//captionBackNode->detachObject(captionBackNode->getAttachedObject(cityName + "_BILLBOARD"));
				//mSceneMgr->destroyBillboardSet(cityName + "_BILLBOARD");
			}
			
			if (currCity.boardBill != nil)
				cityAdditionsBillSet->removeBillboard(currCity.boardBill);
			
			if (currCity.lockBill != NULL)
				cityAdditionsBillSet->removeBillboard(currCity.lockBill);
			
			if (currCity.node)
				mSceneMgr->destroySceneNode(currCity.node);
			
			m_cities.erase(it);
			break;
		}
	}
}

void COgreScene::removeAllCities()
{
	if (cityAdditionsBillSet == NULL || frontCitiesBillSet == NULL)
		return;
	
	cityAdditionsBillSet->clear();
	
	for (TCityIterator it = m_cities.begin(); it != m_cities.end(); ++it)
	{
		TCity& currCity = *it;
		
		if (currCity.cityBill)
			frontCitiesBillSet->removeBillboard(currCity.cityBill);
		
		eraseFutureCityRail(currCity.city->fakeRail());
		eraseFutureCityRail(currCity.city->rail());
		
		if (currCity.node)
		{
			currCity.node->removeAndDestroyAllChildren();
			//mSceneMgr->destroySceneNode(currCity.node);
		}
		
		String cityName = "CITY_" + Ogre::StringConverter::toString(currCity.city->color());
		if (mSceneMgr->hasSceneNode(cityName + "_CAPTION_SceneNode"))
		{
			SceneNode* captionBackNode = mSceneMgr->getSceneNode(cityName + "_CAPTION_SceneNode");
			MovableText* caption = (MovableText*)captionBackNode->getAttachedObject(cityName + "_CAPTION_MovableText");
			captionBackNode->detachObject(caption);
			delete caption;
			
			//captionBackNode->detachObject(captionBackNode->getAttachedObject(cityName + "_BILLBOARD"));
			//mSceneMgr->destroyBillboardSet(cityName + "_BILLBOARD");
		}
		
		if (currCity.node)
			mSceneMgr->destroySceneNode(currCity.node);
		
	}
	
	m_cities.clear();
}

#pragma mark - Add/Remove Rails

void COgreScene::updateSceneNearTheRail(CRail* rail)
{
	CXPoint coord1 = rail->coordRailTexture1();
	CXPoint coord2 = rail->coordRailTexture2();
	
	updateSceneInCoords(coord1, rail->coord1());
	updateSceneInCoords(coord2, rail->coord2());
	
	updateRenderTexture();
}

void COgreScene::updateSceneInCoords(CXPoint texCoord, CXPoint coord)
{
	if (railsBillSet == NULL)
	{
		LogManager::getSingleton().logMessage("Error: railsBillSet == NULL");
		return;
	}
	
	if (nodesBillSet == NULL)
	{
		LogManager::getSingleton().logMessage("Error: nodesBillSet == NULL");
		return;
	}
	
	float texWidth = (jNeedsUseLowResTextures())? 128.f : 256.f;
	float texHeight = texWidth / 2.f;
	
	//--- Atlas -------------------------------------------
	if (!CXEqualPoints(texCoord, CXBadPoint))
	{
		bool isFlipped;
		CXPoint textureIndexes = CShortLineModel::sharedModel()->nodesArea()->textureIndexes(texCoord.x, texCoord.y, (&isFlipped));
		
		int x = (int)texCoord.x+1;
		int y = (int)texCoord.y+1;
		if (m_railsBill[x][y])
		{
			railsBillSet->removeBillboard(m_railsBill[x][y]);
			m_railsBill[x][y] = NULL;
		}
		Billboard* bill = railsBillSet->createBillboard(0., 0., 0.);
		{
#ifdef IPHONEOS_DEPLOYMENT_TARGET
			if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
			{
                bill->setTexcoordIndex(textureIndexes.x);
			}
			else
				bill->setTexcoordIndex(textureIndexes.x);
#elif ANDROID_DEPLOYMENT_TARGET
			{ // это дублирование кода, но оно мне нужно, чтоб ни кто случайно не подпортил именно андроидный код (Женя Овчаренко)
				int y = trunc(textureIndexes.x / 4);
				int x = (textureIndexes.x - 4*y);
				float l = x * 1./4.;
				float t = y * 1./8.;
				float r = l + 1./4.;
				float b = t + 1./8.;
				float ix = (1./4.)/texWidth;
				float iy = (1./8.)/texHeight;
				FloatRect texcoordRect = FloatRect(l+ix, t+iy, r-ix, b-iy);
				bill->setTexcoordRect(texcoordRect);
			}
#else
			{
				int y = trunc(textureIndexes.x / 4);
				int x = (textureIndexes.x - 4*y);
				float l = x * 1./4.;
				float t = y * 1./8.;
				float r = l + 1./4.;
				float b = t + 1./8.;
				float ix = (1./4.)/texWidth;
				float iy = (1./8.)/texHeight;
				FloatRect texcoordRect = FloatRect(l+ix, t+iy, r-ix, b-iy);
				bill->setTexcoordRect(texcoordRect);
			}
			
#endif
		}
		
		m_railsBill[x][y] = bill;
		
		CXPoint coord = texCoord;
		coord.x += 0.5;
		coord.y += 0.5;
		coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
		
		if (isFlipped)
		{
			bill->setDimensions(-railsBillSet->getDefaultWidth(), railsBillSet->getDefaultHeight());
			//bill->setPosition(coord.x + railsBillSet->getDefaultWidth(), 0., coord.y);
			bill->setPosition(coord.x, 0., coord.y);
		}
		else
			bill->setPosition(coord.x, 0., coord.y);
		
	}
	
	if (!CXEqualPoints(coord, CXBadPoint))
	{
		int textureIndex = CShortLineModel::sharedModel()->nodesArea()->nodeTextureIndex(coord.x, coord.y);
		
		int x = (int)coord.x+1;
		int y = (int)coord.y;
		if (m_nodesBill[x][y])
		{
			nodesBillSet->removeBillboard(m_nodesBill[x][y]);
			m_nodesBill[x][y] = NULL;
		}
		Billboard* bill = nodesBillSet->createBillboard(0., 0., 0.);
#ifdef IPHONEOS_DEPLOYMENT_TARGET
		if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
		{
			int y = trunc(textureIndex / 4);
			int x = (textureIndex - 4*y);
			float l = x * 1./4.;
			float t = y * 1./8.;
			float r = l + 1./4.;
			float b = t + 1./8.;
			float ix = (1./4.)/texWidth;
			float iy = (1./8.)/texHeight;
			FloatRect texcoordRect = FloatRect(l+ix, t+iy, r-ix, b-iy);
			bill->setTexcoordRect(texcoordRect);
			//bill->setTexcoordIndex(textureIndex);
		}
		else
			bill->setTexcoordIndex(textureIndex);
#elif ANDROID_DEPLOYMENT_TARGET
        if(textureIndex<0)  // pasha: error with texture index
            bill->setTexcoordIndex(0);
		else
		{
			// float y = trunc(textureIndex / 4);
			// float x = (textureIndex - 4*y);
			// float l = x * 1./4.;
			// float t = y * 1./4.;
			// float r = l + 1./4.;
			// float b = t + 1./4.;
			// float ix = (1./4.)/texWidth;
			// float iy = (1./4.)/texHeight;
			// FloatRect texcoordRect = FloatRect(l+ix, t+iy, r-ix, b-iy);
			// bill->setTexcoordRect(texcoordRect);
			bill->setTexcoordIndex(textureIndex);
		}
#else
        if(textureIndex<0)  // pasha: error with texture index
            bill->setTexcoordIndex(0);
        else
            bill->setTexcoordIndex(textureIndex);
#endif
		
        //NSLog(@"%i", textureIndex);
		
		m_nodesBill[x][y] = bill;
		
		coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
		
		bill->setPosition(coord.x, 0., coord.y);
		
	}
    
    
	/// вывод массива в консоль
	//    {
	//
	//        NSLog(@"++++++++++++++++++++++++++++++++", nil);
	//
	//        for (int j = 0; j < NODES_HEIGHT; ++j)
	//        {
	//            NSString* line = @"";
	//
	//
	//            for (int i = 0; i < NODES_WIDTH; ++i)
	//            {
	//                if (m_railsBill[i][j])
	//                {
	//                    int texIndex = m_railsBill[i][j]->getTexcoordIndex();
	//
	//                    if (texIndex == 65535)
	//                        line = [line stringByAppendingString:@"X "];
	//                    else
	//                        line = [line stringByAppendingFormat:@"%i ", m_railsBill[i][j]->getTexcoordIndex()];
	//                }
	//                else
	//                {
	//                    line = [line stringByAppendingString:@"- "];
	//                }
	//            }
	//
	//            NSLog(line, nil);
	//        }
	//
	//        NSLog(@"++++++++++++++++++++++++++++++++", nil);
	//
	//    }
	///
	
    
}

void COgreScene::addRail(CRail* rail)
{
	updateSceneNearTheRail(rail);
}

void COgreScene::removeAllRails()
{
	restartGame();
	removeAllHighlightedRails();
	clearAllRails();
}

void COgreScene::clearAllRails()
{
	if (railsBillSet)
		railsBillSet->clear();
	
	for (int x = 0; x < NODES_WIDTH; x++)
	{
		for (int y = 0;  y < NODES_HEIGHT; y++)
		{
			m_railsBill[x][y] = NULL;
		}
	}
	
	if (nodesBillSet)
		nodesBillSet->clear();
	
	for (int x = 0; x < NODES_WIDTH; x++)
	{
		for (int y = 0;  y < NODES_HEIGHT; y++)
		{
			m_nodesBill[x][y] = NULL;
		}
	}
}

#pragma mark - Add/Remove FutureCitiesRails

void COgreScene::addFutureCityRail(CRail* rail)
{
	if (unfinishedRailsBillSet == NULL)
	{
		LogManager::getSingleton().logMessage("Error: unfinishedRailsBillSet == NULL");
		return;
	}
	
	Billboard* bill = unfinishedRailsBillSet->createBillboard(0., 0., 0.);
	bill->setTexcoordIndex(rail->type());
	
	if (rail->type() == RT_CURVE_UP || rail->type() == RT_CURVE_DOWN)
	{
		Billboard* bill_2 = unfinishedRailsBillSet->createBillboard(0., 0., 0.);
		if (rail->type() == RT_CURVE_UP)
		{
			bill->setTexcoordIndex(rail->type());
			bill_2->setTexcoordIndex(rail->type()+1);
		}
		if (rail->type() == RT_CURVE_DOWN)
		{
			bill->setTexcoordIndex(rail->type()+1);
			bill_2->setTexcoordIndex(rail->type()+2);
		}
		
		CXPoint coord = CXMakePoint(rail->railCenter().x, (rail->coord1().y+rail->coord2().y)/2.f);
		coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
		bill->setPosition(coord.x - unfinishedRailsBillSet->getDefaultWidth()/2.0, 0., coord.y);
		bill_2->setPosition(coord.x + unfinishedRailsBillSet->getDefaultWidth()/2.0, 0., coord.y);
		
		m_futureCitiesRails.push_back(TRail(rail, bill, bill_2));
	}
	else
	{
		CXPoint coord = CXMakePoint(rail->railCenter().x, (rail->coord1().y+rail->coord2().y)/2.f);
		coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
		bill->setPosition(coord.x, 0.0, coord.y);
		
		m_futureCitiesRails.push_back(TRail(rail, bill));
	}
	
	updateRenderTexture();
}

void COgreScene::eraseFutureCityRail(CRail* rail)
{
	for (TRailIterator it = m_futureCitiesRails.begin(); it != m_futureCitiesRails.end(); ++it)
	{
		TRail& currRail = *it;
		if (currRail.rail == rail)
		{
			if (currRail.bill)
				unfinishedRailsBillSet->removeBillboard(currRail.bill);
			
			if (currRail.bill_2)
				unfinishedRailsBillSet->removeBillboard(currRail.bill_2);
			
			m_futureCitiesRails.erase(it);
			break;
		}
	}
	
	updateRenderTexture();
}

void COgreScene::removeAllFutureCityRails()
{
	if (unfinishedRailsBillSet == NULL)
	{
		LogManager::getSingleton().logMessage("Error: unfinishedRailsBillSet == NULL");
		return;
	}
		
	for (TRailIterator it = m_futureCitiesRails.begin(); it != m_futureCitiesRails.end(); ++it)
	{
		TRail& currRail = *it;
		
		if (currRail.bill)
			unfinishedRailsBillSet->removeBillboard(currRail.bill);
		
		if (currRail.bill_2)
			unfinishedRailsBillSet->removeBillboard(currRail.bill_2);
	}
	
	m_futureCitiesRails.clear();
	
	updateRenderTexture();
}

#pragma mark - Add/Remove UnfinishedRails

void COgreScene::addUnfinishedRail(CRail* rail)
{
	if (unfinishedRailsBillSet == NULL)
	{
		LogManager::getSingleton().logMessage("Error: unfinishedRailsBillSet == NULL");
		return;
	}
	
	Billboard* bill = unfinishedRailsBillSet->createBillboard(0., 0., 0.);
	bill->setTexcoordIndex(rail->type());
	
	if (rail->type() == RT_CURVE_UP || rail->type() == RT_CURVE_DOWN)
	{
		Billboard* bill_2 = unfinishedRailsBillSet->createBillboard(0., 0., 0.);
		if (rail->type() == RT_CURVE_UP)
		{
			bill->setTexcoordIndex(rail->type());
			bill_2->setTexcoordIndex(rail->type()+1);
		}
		if (rail->type() == RT_CURVE_DOWN)
		{
			bill->setTexcoordIndex(rail->type()+1);
			bill_2->setTexcoordIndex(rail->type()+2);
		}
		
		CXPoint coord = CXMakePoint(rail->railCenter().x, (rail->coord1().y+rail->coord2().y)/2.f);
		coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
		bill->setPosition(coord.x - unfinishedRailsBillSet->getDefaultWidth()/2.f, 0., coord.y);
		bill_2->setPosition(coord.x + unfinishedRailsBillSet->getDefaultWidth()/2.f, 0., coord.y);
		
		m_unfinishedRails.push_back(TRail(rail, bill, bill_2));
	}
	else
	{
		CXPoint coord = CXMakePoint(rail->railCenter().x, (rail->coord1().y+rail->coord2().y)/2.f);
		coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
		bill->setPosition(coord.x, 0., coord.y);
		
		m_unfinishedRails.push_back(TRail(rail, bill));
	}
	
	updateRenderTexture();
}

void COgreScene::eraseUnfinishedRail(RailIterator railIt)
{
	for (TRailIterator it = m_unfinishedRails.begin(); it != m_unfinishedRails.end(); ++it)
	{
		TRail& currRail = *it;
		if (currRail.rail == (*railIt))
		{
			if (currRail.bill)
				unfinishedRailsBillSet->removeBillboard(currRail.bill);
			
			if (currRail.bill_2)
				unfinishedRailsBillSet->removeBillboard(currRail.bill_2);
			
			m_unfinishedRails.erase(it);
			updateRenderTexture();
			break;
		}
	}
	
}

void COgreScene::removeAllUnfinishedRails()
{
	for (TRailIterator it = m_unfinishedRails.begin(); it != m_unfinishedRails.end(); ++it)
	{
		TRail& currRail = *it;
		
		if (currRail.bill)
			unfinishedRailsBillSet->removeBillboard(currRail.bill);
		
		if (currRail.bill_2)
			unfinishedRailsBillSet->removeBillboard(currRail.bill_2);
	}
	
	m_unfinishedRails.clear();
	
	updateRenderTexture();
}

#pragma mark - Add/Remove TempRails

void COgreScene::addTempRail(CRail* rail)
{
	if (tempRailsBillSet == NULL)
	{
		LogManager::getSingleton().logMessage("Error: tempRailsBillSet == NULL");
		return;
	}
	
	Billboard* bill = tempRailsBillSet->createBillboard(0., 0., 0.);
	int idxOffset = 8;
	bill->setTexcoordIndex(idxOffset + rail->type());
	
	if (rail->type() == RT_CURVE_UP || rail->type() == RT_CURVE_DOWN)
	{
		Billboard* bill_2 = tempRailsBillSet->createBillboard(0., 0., 0.);
		if (rail->type() == RT_CURVE_UP)
		{
			bill->setTexcoordIndex(idxOffset + rail->type());
			bill_2->setTexcoordIndex(idxOffset + rail->type()+1);
		}
		if (rail->type() == RT_CURVE_DOWN)
		{
			bill->setTexcoordIndex(idxOffset + rail->type()+1);
			bill_2->setTexcoordIndex(idxOffset + rail->type()+2);
		}
		
		CXPoint coord = CXMakePoint((rail->coord1().x+rail->coord2().x)/2.f, (rail->coord1().y+rail->coord2().y)/2.f);
		coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
		bill->setPosition(coord.x - unfinishedRailsBillSet->getDefaultWidth()/2.f, 0.f, coord.y);
		bill_2->setPosition(coord.x + unfinishedRailsBillSet->getDefaultWidth()/2.f, 0.f, coord.y);
		
		m_tempRails.push_back(TRail(rail, bill, bill_2));
	}
	else
	{
		CXPoint coord = CXMakePoint((rail->coord1().x+rail->coord2().x)/2.f, (rail->coord1().y+rail->coord2().y)/2.f);
		coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
		bill->setPosition(coord.x, 0.f, coord.y);
		
		m_tempRails.push_back(TRail(rail, bill));
	}
}

void COgreScene::eraseTempRail(RailIterator railIt)
{
	for (TRailIterator it = m_tempRails.begin(); it != m_tempRails.end(); ++it)
	{
		TRail& currRail = *it;
		
		if (currRail.rail == (*railIt))
		{
			if (currRail.bill)
				tempRailsBillSet->removeBillboard(currRail.bill);
			
			if (currRail.bill_2)
				tempRailsBillSet->removeBillboard(currRail.bill_2);
			
			m_tempRails.erase(it);
			break;
		}
	}
}

void COgreScene::removeAllTempRails()
{
	for (TRailIterator it = m_tempRails.begin(); it != m_tempRails.end(); ++it)
	{
		TRail& currRail = *it;
		
		if (currRail.bill)
			tempRailsBillSet->removeBillboard(currRail.bill);
		
		if (currRail.bill_2)
			tempRailsBillSet->removeBillboard(currRail.bill_2);
	}
	
	m_tempRails.clear();
}

#pragma mark - Switch

void COgreScene::switched(CSwitch* sw)
{
	int switchIndex = sw->textureNumber();
	
	for (TSwitchIterator it = m_switches.begin(); it != m_switches.end(); ++it)
	{
		TSwitch& currSwitch = *it;
		if (currSwitch.sw == sw)
		{
			currSwitch.bill->setTexcoordIndex(switchIndex);
			break;
		}
	}
}

void COgreScene::changeSwitchBlock(CSwitch* sw)
{
	int switchIndex = sw->textureNumber();
	
	for (TSwitchIterator it = m_switches.begin(); it != m_switches.end(); ++it)
	{
		TSwitch& currSwitch = *it;
		if (currSwitch.sw == sw)
		{
			currSwitch.bill->setTexcoordIndex((sw->isBlocked()? 12 : 0) + switchIndex);
			break;
		}
	}
}

void COgreScene::addSwitch(CSwitch* sw)
{
	if (switchesBillSet == NULL)
	{
		LogManager::getSingleton().logMessage("Error: switchesBillSet == NULL");
		return;
	}
	
	CXPoint coord = sw->railTextureCoord();
	coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
	Billboard* bill = switchesBillSet->createBillboard(coord.x, 0., coord.y);
	
	int switchIndex = sw->textureNumber();
	
	bill->setTexcoordIndex(switchIndex);
	
	m_switches.push_back(TSwitch(sw, bill));
}

void COgreScene::eraseSwitch(SwitchIterator switchIt)
{
	for (TSwitchIterator it = m_switches.begin(); it != m_switches.end(); ++it)
	{
		TSwitch& currSwitch = *it;
		if (currSwitch.sw == (*switchIt))
		{
			if (switchesBillSet)
				switchesBillSet->removeBillboard(currSwitch.bill);
			
			m_switches.erase(it);
			break;
		}
	}
}

void COgreScene::removeAllSwitches()
{
	if (switchesBillSet)
	{
		for (TSwitchIterator it = m_switches.begin(); it != m_switches.end(); ++it)
		{
			TSwitch& currSwitch = *it;
			switchesBillSet->removeBillboard(currSwitch.bill);
		}
	}
	
	m_switches.clear();
}

#pragma mark - Add/Remove Cancel Buttons

void COgreScene::addCancelButton(CCancelButton* button)
{
	if (cancelButtonBillSet == NULL)
	{
		LogManager::getSingleton().logMessage("Error: cancelButtonBillSet == NULL");
		return;
	}
	
    CXPoint coord = button->coord();
	coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
	
	Billboard* bill = cancelButtonBillSet->createBillboard(coord.x, 0., coord.y);
	
	m_cancelButtons.push_back(TCancelButton(button, bill));
}

void COgreScene::removeCancelButton(CCancelButton* button)
{
	if (cancelButtonBillSet == NULL)
		return;
	
    for (TCancelButtonIterator it = m_cancelButtons.begin(); it != m_cancelButtons.end(); ++it)
	{
		TCancelButton& currButton = *it;
		if (currButton.btn == button)
		{
			cancelButtonBillSet->removeBillboard(currButton.bill);
			
			m_cancelButtons.erase(it);
			break;
		}
	}
	
}

//-------------------------

#pragma mark - Resources

void COgreScene::locateResources()
{
#ifdef OBB_EXPANSOIN_FILE
	std::string obbPath = jGetMountedOBBPath();
	
	if (!jNeedsUseLowResTextures() && !Ogre::ResourceGroupManager::getSingleton().resourceLocationExists(obbPath, "OBB_Section"))
	{
		Ogre::ResourceGroupManager::getSingleton().addResourceLocation(obbPath, "Zip", "OBB_Section", true);
		Ogre::ResourceGroupManager::getSingleton().initialiseResourceGroup("OBB_Section");
	}
#endif
		
	// load resource paths from config file
	Ogre::ConfigFile cf;
	cf.load(openAPKFile("scene_resources.cfg", gAssetMgr));
	Ogre::ConfigFile::SectionIterator seci = cf.getSectionIterator();
	Ogre::String sec, type, arch;

	// go through all specified resource groups
	while (seci.hasMoreElements())
	{
		sec = seci.peekNextKey();
		Ogre::ConfigFile::SettingsMultiMap* settings = seci.getNext();
		Ogre::ConfigFile::SettingsMultiMap::iterator i;

		// go through all resource paths
		for (i = settings->begin(); i != settings->end(); i++)
		{
			type = i->first;
			arch = i->second;
			
			if (sec == "Large_Textures" || sec == "Mid_Textures")
			{
				if ((!jNeedsUseLowResTextures() && sec == "Large_Textures") ||
					(jNeedsUseLowResTextures() && sec == "Mid_Textures"))
					Ogre::ResourceGroupManager::getSingleton().addResourceLocation(arch, type, sec);
			}
			else
			{
				Ogre::ResourceGroupManager::getSingleton().addResourceLocation(arch, type, sec);
			}
		}
	}	
}

#pragma mark - Other

void COgreScene::restartGame()
{
	removeAllFutureCityRails();
	removeAllExplosions();
	removeAllFlyAwayMessages();
	removeAllHighlightedRails();
	setRedGameCoord(CXBadPoint);
}

void COgreScene::generateExplosion(CXPoint coord)
{
	coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
	
	Billboard* explBill = explosionBillSet->createBillboard(coord.x, EXPLOSION_HEIGHT_OFFSET + EXPLOSION_Z_ORDER_OFFSET, coord.y + EXPLOSION_Z_ORDER_OFFSET);
	
	explBill->setTexcoordIndex(0);
	m_explosions.push_back(NBS(NULL, explBill, 0., true));
}

void COgreScene::removeAllExplosions()
{
	if (explosionBillSet == NULL)
		return;
	
	m_explosions.clear();
	explosionBillSet->clear();
}

void COgreScene::putFAMessage(CXPoint pos, String message, bool isAlert, bool isUpMoving)
{
	pos = CShortLineModel::sharedModel()->convertFromGameCoord(pos, s_scene_rect);
	
	putFlyAwayMessage(Vector3(pos.x, 0., pos.y), message, isAlert, 2.6, isUpMoving);
}

void COgreScene::putFlyAwayMessage(Vector3 pos, String message, bool isAlert, Real height, bool isUpMoving)
{
	static int s_flyAwayMessageCounter = 0;
	
	if (cloudsBillSet == NULL)
	{
		LogManager::getSingleton().logMessage("Error: cloudsBillSet == NULL");
		return;
	}
	
	// --- background ---
	Billboard* bubbleBill = cloudsBillSet->createBillboard(0., 0., 0.);
	bubbleBill->setTexcoordIndex(isAlert? 0 : 1);
	
	// --- caption ---
	MovableText* msg = new MovableText("FlyAwayMessage_" + Ogre::StringConverter::toString(s_flyAwayMessageCounter++), message, "Comic5-17");
	
	msg->setTextAlignment(MovableText::H_CENTER, MovableText::V_ABOVE); // Center horizontally and display above the node
	msg->setColor(ColourValue::White);
	msg->setCharacterHeight( height );
	msg->setRenderQueueGroup(FA_CLOUD_RENDER_QUEUE);
	msg->showOnTop(true);
	msg->setLocalTranslation(Vector3(0.4, -0.2, 0.0));
	
	pos.y = FAMESSAGE_START_HEIGHT;
	Ogre::SceneNode* captionNode = mSceneMgr->getRootSceneNode()->createChildSceneNode(pos);
	captionNode->attachObject(msg);
	captionNode->setVisible(true);
	
	double now = CShortLineModel::sharedModel()->timer()->seconds();
	m_faMessages.push_back(NBS(captionNode, bubbleBill, now, isUpMoving));
}

void COgreScene::removeAllFlyAwayMessages()
{
	if (m_faMessages.empty() == false)
	{
		for (NBSIterator it = m_faMessages.begin(); it != m_faMessages.end(); )
		{
			if ((*it).node->numAttachedObjects())
			{
				MovableText* caption = (MovableText*)(*it).node->getAttachedObject(0);
				(*it).node->detachObject(caption);
				delete caption;
			}
			
			if (cloudsBillSet && it->bill)
				cloudsBillSet->removeBillboard(it->bill);
			
			it->node->removeAndDestroyAllChildren();
			mSceneMgr->destroySceneNode(it->node);
			
			it = m_faMessages.erase(it);
		}
	}
}





void COgreScene::removeAllHighlightedRails()
{
    if (highlightedRailsBillSet == NULL)
		return;
    
	highlightedRailsBillSet->clear();
	m_hlRails.clear();
}

void COgreScene::setNeedsToHighlight(CRail* rail, bool flag)
{
	if (highlightedRailsBillSet == NULL)
		return;
	
	BOOL found = NO;
	for (TRailIterator it = m_hlRails.begin(); it != m_hlRails.end(); ++it)
	{
		TRail& currRail = *it;
		
		if (currRail.rail == rail)
		{
			found = YES;
			
			if (!flag)
			{
				if (currRail.bill)
					highlightedRailsBillSet->removeBillboard(currRail.bill);
				
				if (currRail.bill_2)
					highlightedRailsBillSet->removeBillboard(currRail.bill_2);
				
				m_hlRails.erase(it);
			}
			
			break;
		}
	}
	
	if (!found && flag)
	{
		Billboard* bill = highlightedRailsBillSet->createBillboard(0., 0., 0.);
		//bill->setTexcoordIndex(8 + rail->type());
		bill->setTexcoordIndex(rail->type());
		
		if (rail->type() == RT_CURVE_UP || rail->type() == RT_CURVE_DOWN)
		{
			Billboard* bill_2 = highlightedRailsBillSet->createBillboard(0., 0., 0.);
			if (rail->type() == RT_CURVE_UP)
			{
				bill->setTexcoordIndex(rail->type());
				bill_2->setTexcoordIndex(rail->type()+1);
			}
			if (rail->type() == RT_CURVE_DOWN)
			{
				bill->setTexcoordIndex(rail->type()+1);
				bill_2->setTexcoordIndex(rail->type()+2);
			}
			
			CXPoint coord = CXMakePoint((rail->coord1().x+rail->coord2().x)/2.f, (rail->coord1().y+rail->coord2().y)/2.f);
			coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
			bill->setPosition(coord.x - unfinishedRailsBillSet->getDefaultWidth()/2.f, 0., coord.y);
			bill_2->setPosition(coord.x + unfinishedRailsBillSet->getDefaultWidth()/2.f, 0., coord.y);
			
			m_hlRails.push_back(TRail(rail, bill, bill_2));
		}
		else
		{
			CXPoint coord = CXMakePoint((rail->coord1().x+rail->coord2().x)/2.f, (rail->coord1().y+rail->coord2().y)/2.f);
			coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
			bill->setPosition(coord.x, 0., coord.y);
			
			m_hlRails.push_back(TRail(rail, bill));
		}
	}
}

void COgreScene::cancelBuildingRails()
{
	_nearestNodeGameCoord = CXBadPoint;
	setRedGameCoord(CXBadPoint);
	
	CShortLineModel* model = CShortLineModel::sharedModel();
	
    removeHighlightedFlagForAllRails();
    
	for (int i = 0; i<12; ++i)
	{
		_distanceSquaresToRail[i] = 0;
		_distancesToEdges[i] = 0;
	}
	
    model->setBonusMode(bonus_no);
	
		
    _possibleRailIndex = -1;
    _willBuildRail = NO;
    
    clearPossibleRailsArray();
    model->listener()->removeAllTempRails();
    model->listener()->hideHintNodes();
    clearRailVector(&_chainPossibleRails);
    clearRailVector(&_nextPossibleRails);
    _redPoint = CXBadPoint;
    _inRedCircle = NO;
    
	
	clearPossibleRailsArray();
	
	model->listener()->removeAllTempRails();
	clearRailVector(&_chainPossibleRails);
	clearRailVector(&_nextPossibleRails);
	
	_redPoint = CXBadPoint;

}

void COgreScene::setRedGameCoord(CXPoint coord)
{
	_redGameCoord = coord;
	
	if (mSceneMgr->hasSceneNode("RED_GAME_COORD"))
	{
		SceneNode* sn = mSceneMgr->getSceneNode("RED_GAME_COORD");
		
		if (isGoodPoint(coord))
		{
			coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
			
			Rectangle2D* rect = (Rectangle2D*)sn->getAttachedObject(0);
			coord.y += 1;
			coord.y *= 1.f/1.52f;
			
			float halfSide = 3.0;
			
			Real left = coord.x - halfSide;
			Real top = coord.y - halfSide * m_displayFactor;
			Real right = coord.x + halfSide;
			Real bottom = coord.y + halfSide * m_displayFactor;
			rect->setCorners(left, -top, right, -bottom);
			sn->setVisible(true);
		}
		else
			sn->setVisible(false);
	}
}

void COgreScene::showHintNodes()
{
	/*
	 if (mSceneMgr->hasSceneNode("HINT_NODES_NODE"))
	 {
	 SceneNode* sceneNode = mSceneMgr->getSceneNode("HINT_NODES_NODE");
	 sceneNode->setVisible(true);
	 }
	 */
	if (mSceneMgr->hasSceneNode("BACKGROUND_GRID"))
	{
		SceneNode* sn = mSceneMgr->getSceneNode("BACKGROUND_GRID");
		sn->setVisible(true);
	}
}

void COgreScene::hideHintNodes()
{
	/*
	 if (mSceneMgr->hasSceneNode("HINT_NODES_NODE"))
	 {
	 SceneNode* sceneNode = mSceneMgr->getSceneNode("HINT_NODES_NODE");
	 sceneNode->setVisible(false);
	 }
	 */
	if (mSceneMgr->hasSceneNode("BACKGROUND_GRID"))
	{
		SceneNode* sn = mSceneMgr->getSceneNode("BACKGROUND_GRID");
		sn->setVisible(false);
	}
}

//|||||||||||||||||||||||||||||||||||||||||||||||

Ogre::Entity* COgreScene::getTrainEntity(CTrain* train)
{
	Ogre::Entity* trainEntity = NULL;
	
	if (train->isRubberTrain())
    {
		trainEntity = mSceneMgr->createEntity("train_rubber.mesh");
    }
    else if (train->isKamikazeTrain())
    {
        trainEntity = mSceneMgr->createEntity("train_kamikaze.mesh");
    }
	else
	{
		switch (train->type())
		{
				//locomotives:
			case OLD_STEAM:
                trainEntity = mSceneMgr->createEntity("train_old_steam.mesh");
				//trainEntity = mSceneMgr->createEntity("train_old_steam.mesh");
				break;
			case STEAM:
				trainEntity = mSceneMgr->createEntity("train_steam.mesh");
				break;
			case OLD_ELECTRIC:
				trainEntity = mSceneMgr->createEntity("train_older_electric.mesh");
				break;
			case DIESEL:
				trainEntity = mSceneMgr->createEntity("train_diesel.mesh");
				break;
			case ELECTRIC:
				trainEntity = mSceneMgr->createEntity("train_old_electric.mesh");
				break;
			case GAS_TURBINE:
				trainEntity = mSceneMgr->createEntity("train_gas_turbine.mesh");
				break;
			case NEW_ELECTRIC:
				trainEntity = mSceneMgr->createEntity("train_new_electric.mesh");
				break;
				
				//pseudo-car objects:
			case HAND_CAR:
				trainEntity = mSceneMgr->createEntity("train_hand_car.mesh");
				break;
			case DAMAGE:
				trainEntity = mSceneMgr->createEntity("train_damage.mesh");
				break;
			case SERVER:
				trainEntity = mSceneMgr->createEntity("train_server.mesh");
				break;
				
			default:
                trainEntity = mSceneMgr->createEntity("train_old_steam.mesh");
                //trainEntity = mSceneMgr->createEntity("train_old_steam.mesh");
				break;
		}
	}
	
	//trainEntity = mSceneMgr->createEntity("vagon_platform.mesh");
	//std::cout << "trainEntity:" << trainEntity << std::endl;
	syncTrainColor(trainEntity, train);
	
	return trainEntity;
}

Ogre::Entity* COgreScene::getVagonEntity(CCar* car, CTrain* train)
{
	Ogre::Entity* vagonEntity = NULL;
	
	switch (car->type()) {
		case OLD_PASSENGER:
			vagonEntity = mSceneMgr->createEntity("vagon_old_passenger.mesh");
			break;
		case PASSENGER:
			vagonEntity = mSceneMgr->createEntity("vagon_passenger.mesh");
			break;
		case NEW_PASSENGER:
			vagonEntity = mSceneMgr->createEntity("vagon_new_passenger.mesh");
			break;
		case OLD_FREIGHT:
			vagonEntity = mSceneMgr->createEntity("vagon_old_freight.mesh");
			break;
		case FREIGHT:
			vagonEntity = mSceneMgr->createEntity("vagon_freight.mesh");
			break;
		case PLATFORM:
			vagonEntity = mSceneMgr->createEntity("vagon_platform.mesh");
			break;
		case CISTERN:
			vagonEntity = mSceneMgr->createEntity("vagon_cistern.mesh");
			break;
        case KAMIKAZE_CAR:
			vagonEntity = mSceneMgr->createEntity("vagon_kamikaze.mesh");
			break;
		case SERVER:
			//case SERVER_CAR:
			vagonEntity = mSceneMgr->createEntity("vagon_server.mesh");
			break;
		default:
			break;
	}
	
	//vagonEntity = mSceneMgr->createEntity("vagon_platform.mesh");
	//std::cout << "vagonEntity:" << vagonEntity << std::endl;
	syncVagonColor(vagonEntity, car, train);
	
	return vagonEntity;
}

void COgreScene::refreshColorOfTrain(CTrain* train)
{
    TTrain* pCurrTrain = NULL;
    
    //апдейтим локомотив
    for (TTrainIterator it = m_trains.begin(); it != m_trains.end(); ++it)
	{
		pCurrTrain = &(*it);
		if (pCurrTrain->train == train)
		{
			syncTrainColor(pCurrTrain->entity, train);
			break;
		}
	}
	
    //апдейтим вагоны
    if (pCurrTrain)
    {
        CarVector& carsVec = train->cars();
        for (int i = 0; i < carsVec.size(); i++)
        {
            CCar* currCar = carsVec.at(i);
            syncVagonColor(pCurrTrain->cars[i].entity, currCar, train);
        }
    }
}

//void COgreScene::refreshColorOfVagon(CCar* car)
//{
//    CTrain* currTrain =
//
////    CarVector carsVec = currTrain.train->cars();
////
////
////    for (int i = 0; i < carsVec.size(); i++)
////    {
////        CCar* currCar = carsVec.at(i);
////
////        float angle = -currCar->realAngleForRect(s_scene_rect);
////        if (currCar->isFacedRight() && (angle > -90) && (angle < 90))
////            angle += 180;
////
////        currTrain.cars[i].node->resetOrientation();
////        currTrain.cars[i].node->yaw(Degree(angle));
////        CXPoint coord = CXMakePoint(currCar->shakenCenter().x, currCar->shakenCenter().y);
////        coord = CShortLineModel::sharedModel()->convertFromGameCoord(coord, s_scene_rect);
////        currTrain.cars[i].node->setPosition(coord.x, currCar->shakenCenter().z * SHAKE_SCALE_Z, coord.y);
////
////        if (currTrain.train->isChameleonTrain() && (currTrain.train->color() != currTrain.color))
////        {
////            syncVagonColor(currTrain.cars[i].entity, currCar, currTrain.train);
////        }
//
//
//
//    this->syncVagonColor(Ogre::Entity* vagonEntity, car, train->mainTrain());
//}


void COgreScene::updateTrainColor(CTrain* train)
{
	for (TTrainIterator it = m_trains.begin(); it != m_trains.end(); ++it)
	{
		TTrain& currTrain = *it;
		if (currTrain.train == train)
		{
			syncTrainColor(currTrain.entity, train);
			break;
		}
	}
}

void COgreScene::syncTrainColor(Ogre::Entity* trainEntity, CTrain* train)
{
	String trainMaterialName = "train_black";
    
	switch (train->type())
	{
			//locomotives:
		case OLD_STEAM:
			trainMaterialName = "train_old_steam";
			break;
		case STEAM:
			trainMaterialName = "train_steam";
			break;
		case OLD_ELECTRIC:
			trainMaterialName = "train_older_electric";
			break;
		case DIESEL:
			trainMaterialName = "train_diesel";
			break;
		case ELECTRIC:
			trainMaterialName = "train_old_electric";
			break;
		case GAS_TURBINE:
			trainMaterialName = "train_gas_turbine";
			break;
		case NEW_ELECTRIC:
			trainMaterialName = "train_new_electric";
			break;
			
			//pseudo-car objects:
		case HAND_CAR:
			trainMaterialName = "train_hand_car";
			break;
		case DAMAGE:
		case SERVER:
			return;										//!!!! return
			
		default:
			trainMaterialName = "train_old_steam";
			break;
	}
    
 	
	
	if (train->isRubberTrain())
		trainMaterialName = "train_gas_turbine";
	
	if (train->isMadTrain())
	{
#if MAD_COLOR_MANUAL_UPDATE
		trainMaterialName += (m_madTrainCycle? "_mad_0" : "_mad_1");
#else
		trainMaterialName += "_mad_0";
#endif
	}
	else
	{
		switch (train->color())
        {
			case CITY_COLOR_YELLOW:
				trainMaterialName += "_yellow";
				break;
			case CITY_COLOR_RED:
				trainMaterialName += "_red";
				break;
			case CITY_COLOR_BLUE:
				trainMaterialName += "_blue";
				break;
			case CITY_COLOR_PURPLE:
				trainMaterialName += "_purple";
				break;
			case CITY_COLOR_GREEN:
				trainMaterialName += "_green";
				break;
			case CITY_COLOR_BROWN:
				trainMaterialName += "_orange";
				break;
			case CITY_COLOR_BLACK:
				trainMaterialName += "_black";
				break;
				
			default:
				trainMaterialName += "_black";
				break;
		}
	}
	
	//std::cout << "trainMaterialName: " << trainMaterialName << std::endl;
	if (train->isKamikazeTrain())
		trainMaterialName = "train_kamikaze_black";
    
	//trainMaterialName = "vagon_platform_black";
	for(int i = 0; i < trainEntity->getNumSubEntities(); i++)
	{
		//std::cout << "train: " << trainEntity->getSubEntity(i)->getMaterialName() << std::endl;
		if (Ogre::StringUtil::startsWith(trainEntity->getSubEntity(i)->getMaterialName(), "train", false))
		{
			//std::cout << "set train material: " << trainMaterialName << std::endl;
			trainEntity->getSubEntity(i)->setMaterialName(trainMaterialName);
		}
	}
}

void COgreScene::syncVagonColor(Ogre::Entity* vagonEntity, CCar* car, CTrain* train)
{
    //NSLog(@"syncVagonColor");
    
	String vagonMaterialName = "vagon_black";
	
	switch (car->type())
    {
		case OLD_PASSENGER: vagonMaterialName = "vagon_old_passenger";  break;
		case PASSENGER: 	vagonMaterialName = "vagon_passenger";      break;
		case NEW_PASSENGER: vagonMaterialName = "vagon_new_passenger";	break;
		case OLD_FREIGHT:	vagonMaterialName = "vagon_old_freight";	break;
		case FREIGHT:		vagonMaterialName = "vagon_freight";		break;
		case PLATFORM:		vagonMaterialName = "vagon_platform";		break;
		case CISTERN:		vagonMaterialName = "vagon_cistern";		break;
        case KAMIKAZE_CAR:	vagonMaterialName = "vagon_kamikaze";		break;
		case SERVER:        break;
			//case SERVER_CAR:  break;
		default:            break;
	}
	
	if (car->mainTrain()->isMadTrain())
	{
#if MAD_COLOR_MANUAL_UPDATE
		vagonMaterialName += (m_madTrainCycle? "_mad_0" : "_mad_1");
#else
		vagonMaterialName += "_mad_0";
#endif
	}
	else
	{
		switch (car->color()) {
			case CITY_COLOR_YELLOW:
				vagonMaterialName += "_yellow";
				break;
			case CITY_COLOR_RED:
				vagonMaterialName += "_red";
				break;
			case CITY_COLOR_BLUE:
				vagonMaterialName += "_blue";
				break;
			case CITY_COLOR_PURPLE:
				vagonMaterialName += "_purple";
				break;
			case CITY_COLOR_GREEN:
				vagonMaterialName += "_green";
				break;
			case CITY_COLOR_BROWN:
				vagonMaterialName += "_orange";
				break;
			case CITY_COLOR_BLACK:
				vagonMaterialName += "_black";
				break;
			default:
				vagonMaterialName += "_black";
				break;
		}
	}
	
	if (car->mainTrain()->isKamikazeTrain())
		vagonMaterialName = "vagon_kamikaze_black";
	
	//vagonMaterialName = "vagon_platform_black";
	//std::cout << "vagonMaterialName: " << vagonMaterialName << std::endl;
	
	for(int i = 0; i < vagonEntity->getNumSubEntities(); i++)
	{
		if (Ogre::StringUtil::startsWith(vagonEntity->getSubEntity(i)->getMaterialName(), "vagon", false))
		{
			
			if (!Ogre::StringUtil::match(vagonEntity->getSubEntity(i)->getMaterialName(), vagonMaterialName, false))
            {
                //std::cout << "set vagon material: " << vagonMaterialName << std::endl;
				vagonEntity->getSubEntity(i)->setMaterialName(vagonMaterialName);
            }
		}
	}
}

//|||||||||||||||||||||||||||||||||||||||||||||||

CXPoint COgreScene::convertFromGameCoord(CXPoint gameCoord)
{
	return CShortLineModel::sharedModel()->convertFromGameCoord(gameCoord, boundsForDrawing());
}

CXPoint COgreScene::convertToGameCoord(CXPoint viewPoint)
{
	return CShortLineModel::sharedModel()->convertToGameCoord(viewPoint, boundsForDrawing());
}

void COgreScene::calculateBorderLineForRail(CRail* rail, CXPoint gameCoord)
{
	const int d = 1;
	
	CXPoint nextGameCoord = CXBadPoint;
	
	//заполняем forwardBorder
	
	if (rail == NULL)
	{
		_borderGameCoord1 = CXBadPoint;
		_borderGameCoord2 = CXBadPoint;
	}
	else
	{
		nextGameCoord = rail->otherCoordTo(gameCoord);
		CNode* mainNode = CShortLineModel::sharedModel()->nodesArea()->node(gameCoord);
		if (mainNode == NULL)
		{
			//LogManager::getSingleton().logMessage(">>>>>>>>>>mainNode == NULL at first point");
			clearPossibleRailsArray();
			return;
		}

        int sector = mainNode->sectorOfRail(rail);
        
        if (sector == -1)
        {
            myAssert();
        }
        else
        {
            if (sector%2 == 0)//если номер сектора - четный
            {
                _borderGameCoord1 = CXMakePoint(nextGameCoord.x-d, nextGameCoord.y-d);
                _borderGameCoord2 = CXMakePoint(nextGameCoord.x+d, nextGameCoord.y+d);

            }
            else
            {
                _borderGameCoord1 = CXMakePoint(nextGameCoord.x-d, nextGameCoord.y+d);
                _borderGameCoord2 = CXMakePoint(nextGameCoord.x+d, nextGameCoord.y-d);

            }
        }
	}
	
	//заполняем _backBorder
	if ((rail == NULL) && (_chainPossibleRails.empty()))
	{
		_backBorderGameCoord1 = CXBadPoint;
		_backBorderGameCoord2 = CXBadPoint;
	}
	else
	{
		CNode* mainNode;
        int sector = -1;
		
		if ((rail == NULL) && (_chainPossibleRails.empty() == false))
		{
			rail = _chainPossibleRails.back();
            
			nextGameCoord = gameCoord;
			mainNode = CShortLineModel::sharedModel()->nodesArea()->node(gameCoord);
			if (mainNode == NULL)
			{
				//LogManager::getSingleton().logMessage(">>>>>>>>>>mainNode == NULL at second point");
				clearPossibleRailsArray();
				return;
			}
            sector = mainNode->sectorOfRail(rail);
		}
		else
		{
			gameCoord = nextGameCoord;
			nextGameCoord = rail->otherCoordTo(nextGameCoord);
			mainNode = CShortLineModel::sharedModel()->nodesArea()->node(gameCoord);
			if (mainNode == NULL)
			{
				//LogManager::getSingleton().logMessage(">>>>>>>>>>mainNode == NULL at third point");
				clearPossibleRailsArray();
				return;
			}
            sector = mainNode->sectorOfRail(rail);
		}
		
        
        if (sector == -1)
        {
            _backBorderGameCoord1 = _backBorderGameCoord2 = CXBadPoint;
            
            //myAssert();//тут надо что-то сделать, чтобы избежать падения
        }
        else if (sector%2 == 0)
        {
			_backBorderGameCoord1 = CXMakePoint(nextGameCoord.x-d, nextGameCoord.y-d);
			_backBorderGameCoord2 = CXMakePoint(nextGameCoord.x+d, nextGameCoord.y+d);
			
		}
		else 
		{
			_backBorderGameCoord1 = CXMakePoint(nextGameCoord.x-d, nextGameCoord.y+d);
			_backBorderGameCoord2 = CXMakePoint(nextGameCoord.x+d, nextGameCoord.y-d);
		}	
	}
}

CXPoint COgreScene::forwardCoordOfChainRails()
{
	if (_chainPossibleRails.empty() || isBadPoint(_nearestNodeGameCoord))
	{
		return CXBadPoint;
	}
	
	
	if (_chainPossibleRails.size() == 1)
	{
		CRail* currRail = _chainPossibleRails.front();
		return currRail->otherCoordTo(_nearestNodeGameCoord);
	}
    
	CRail* forwRail = _chainPossibleRails.back();
	CRail* prevRail = (*(_chainPossibleRails.end()-2));
	
	
	if (prevRail->hasCoord(forwRail->coord1()))
	{
		return forwRail->coord2();
	}
	else
	{
		return forwRail->coord1();
	}
}

CXPoint COgreScene::backwardCoordOfChainRails()
{
	if (_chainPossibleRails.empty() || isBadPoint(_nearestNodeGameCoord))
	{
		return CXBadPoint;
	}
	
	if (_chainPossibleRails.size() == 1)
	{
		return _nearestNodeGameCoord;
	}
	else
	{
		return CXBadPoint;
	}
}


int COgreScene::possibleRailIndexForGameCoord(CXPoint gameCoord)
{
    if (isGoodPoint(_detectCenter)) 
    {
        int sectorIndex = sector12ForPointWithCenter(gameCoord, _detectCenter);
        if (_possibleRails[sectorIndex])
        {
            return sectorIndex;
        }
        
    }
    
	float newMinDistance = MILLION;
	int possibleRailIndex = -1;
	
	for (int i = 0; i < 12; ++i)
	{
		CXPoint currPoint = _detectPoints[i];
		if (isGoodPoint(currPoint) && _possibleRails[i])
		{
			float currDistance = BSDistance(gameCoord, currPoint);
			
			if (currDistance < newMinDistance)
			{
				newMinDistance = currDistance;
				possibleRailIndex = i;
			}
		}
	}

	return possibleRailIndex;
}

bool COgreScene::detectIntersectForGameCoord(CXPoint gameCoord, CXPoint mainCoord, CXPoint coord1, CXPoint coord2) 
{
	if (isBadPoint(gameCoord) || isBadPoint(coord1) || isBadPoint(coord2) || isBadPoint(mainCoord))
	{
		return false;
	}
	
	CXPoint borderVector = subtractPoints(coord2, coord1);
	CXPoint currVector   = subtractPoints(gameCoord, coord1);
	CXPoint nodeVector   = subtractPoints(mainCoord, coord1);
	
	if (wedgeProduct(nodeVector, borderVector)*wedgeProduct(currVector, borderVector)<=0)
		return true;
	
	return false;
}

bool COgreScene::isEmptyPossibleRails()
{
	for (int i = 0; i<12; ++i)
	{
		if (_possibleRails[i])
		{
			return false;
		}
	}
	
	return true;
}

void COgreScene::clearRailVector(RailVector* pRailVector)
{
	for (RailIterator it = pRailVector->begin(); it != pRailVector->end(); ++it)
		delete (*it);
	
	pRailVector->clear();
}

bool COgreScene::addToChainRail(CRail* rail)
{
	CRail* newRail = CRail::createCopyOfRail(rail);
	CShortLineModel::sharedModel()->listener()->addTempRail(newRail);
	_chainPossibleRails.push_back(newRail);
	return true;
}

void COgreScene::clearPossibleRailsArray()
{
    _detectCenter = CXBadPoint;
    
	for (int i = 0; i<12; ++i)
	{
		if (_possibleRails[i])
		{
			delete _possibleRails[i];
			_possibleRails[i] = NULL;
		}
		
		_detectPoints[i] = CXBadPoint;
	}
	
}

bool COgreScene::tryFillPossibleRailsArrayWithCenter(CXPoint nearestNodeGameCoord)
{
	CRail* tempPossibleRails[12] = {NULL};
	
	
	CShortLineModel* model = CShortLineModel::sharedModel();
	
	//заполняем _possibleRails
    
    for (int i = 0; i < 12; ++i)
    {
        tempPossibleRails[i] = CRail::createUniqueRail(nearestNodeGameCoord, i);
    }

	
	//убираем рельсы, которые уже есть в _chainPossibleRails
	if (!_chainPossibleRails.empty())
	{
		for (int i = 0; i<12; ++i)
		{
			CRail* currRail = tempPossibleRails[i];
			
			if (currRail && containsVectorRail(&_chainPossibleRails, currRail))
			{
				delete currRail;
				tempPossibleRails[i] = NULL;
			}
		}
	}
	
	//если цепь рельсов достаточно длинная, 
	//сохраняем только те рельсы, которые гладко соединяются с рельсами из _chainPossibleRails
	
	if (false == _chainPossibleRails.empty())
	{
		CRail* railInChain = _chainPossibleRails.back();
		
		for (int i = 0; i<12; ++i)
		{
			CRail* currRail = tempPossibleRails[i];
			
			if(false == railInChain->isSmoothConnectedWithRail(currRail))
			{
				delete currRail;
				tempPossibleRails[i] = NULL;
			}
		}
	}
	
	
	//здесь удаляются рельсы, создающие тройную стрелку
	for (int i = 0; i < 12; ++i)
	{
		CRail* currRail = tempPossibleRails[i];
		
		if (currRail == NULL)
			continue;
		
		CNode* node1 = currRail->node1();
		CNode* node2 = currRail->node2();
		CXPoint coord1 = node1->coord();
		CXPoint coord2 = node2->coord();
		
		Quarter q1 = node1->quarterOfRail(currRail);
		Quarter q2 = node2->quarterOfRail(currRail);
		int count1 = node1->countOfAllRailsInQuarter(q1);
		int count2 = node2->countOfAllRailsInQuarter(q2);
		
        //учитываем рельсы из _chainPossibleRails
		for (RailIterator it = _chainPossibleRails.begin(); it != _chainPossibleRails.end(); ++it)
		{
			CRail* railInChain = (*it);
			
			if (railInChain->hasCoord(coord1) && node1->quarterOfRail(railInChain)==q1)
				++count1;
			
			if (railInChain->hasCoord(coord2) && node2->quarterOfRail(railInChain)==q2)
				++count2;
		}
		
		CityVector cities = CShortLineModel::sharedModel()->cities();
		//учитываем рельсы непостроенных городов
		for (CityIterator it = cities.begin(); it != cities.end(); ++it)
		{
            CCity* currCity = (*it);
            
            if (currCity->state() != CS_ACTIVE) 
            {
                CRail* railForCity = (*it)->rail();
                
                if (railForCity->hasCoord(coord1) && node1->quarterOfRail(railForCity)==q1)
                    ++count1;
                
                if (railForCity->hasCoord(coord2) && node2->quarterOfRail(railForCity)==q2)
                    ++count2;

            }
		}
		
		
		if ( (count1>=2) || (count2>=2) )
		{
			delete currRail;
			tempPossibleRails[i] = NULL;
		}
	}
	
	//если в node входит 1 рельс, подбираем только гладкие рельсы
	CNode* node = model->nodesArea()->node(nearestNodeGameCoord);
	
	if (node == NULL)
	{
		//NSLog(@"stop");
	}
	
	//создаем копию вектора рельсов, хранящихся в текущем node
	RailVector nodeRails(*(node->rails()));
	
	//добавляем туда подходящие рельсы из _chainPossibleRails
	for (RailIterator it = _chainPossibleRails.begin(); it != _chainPossibleRails.end(); ++it)
	{
		CRail* currRail = (*it);
		if (currRail->hasCoord(nearestNodeGameCoord))
		{
			nodeRails.push_back(currRail);
		}
	}
	
	//добавляем туда подходящие недостроенные рельсы
	for (RailIterator it = model->unfinishedRails().begin(); it != model->unfinishedRails().end(); ++it)
	{
		CRail* currRail = (*it);
		if (currRail->hasCoord(nearestNodeGameCoord))
		{
			nodeRails.push_back(currRail);
		}
	}
	
    
	//добавляем видимые рельсы непостроенных городов
    for (CityIterator it = model->cities().begin(); it != model->cities().end(); ++it)
    {
        CCity* currCity = (*it);
        
        if ((currCity->state() == CS_WILL_SOON_APPEAR) || (currCity->state() == CS_ONLY_RAIL_VISIBLE))
        {
            CRail* cityRail = currCity->rail();
            
            if (cityRail->hasCoord(nearestNodeGameCoord))
            {
                nodeRails.push_back(cityRail);
                //NSLog(@"added cityRail");
            }
        }
    }
	
	int countOfRails = nodeRails.size();
	
	if (countOfRails > 0)
	{
		for (int i = 0; i < 12; ++i)
		{
			CRail* currRail = tempPossibleRails[i];
			
			if (currRail)
			{
				Quarter currQuart = node->quarterOfRail(currRail);
				
				bool isGoodRail = false;
				
				for (RailIterator it = nodeRails.begin(); it != nodeRails.end(); ++it)
				{	
					CRail* inRail = (*it);
					Quarter inQuart = node->quarterOfRail(inRail);
					
					int deltaQ = ABS(inQuart-currQuart);
					
					if ((deltaQ == 2)  || (deltaQ == 0))
					{
						isGoodRail = true;
						break;
					}
				}
				
				
				if (isGoodRail == false)
				{
					delete tempPossibleRails[i];
					tempPossibleRails[i] = NULL;
				}
			}
			
		}
	}
	
	BOOL isEmptyTempRails = YES;
	
	for (int i = 0; i<12; ++i)
	{
		if (tempPossibleRails[i])
		{
			isEmptyTempRails = NO;
			break;
		}
	}
	
	if (isEmptyTempRails)
	{
        _detectCenter = CXBadPoint;
		return NO;
	}
	else
	{
        _detectCenter = nearestNodeGameCoord;
        
        
		for (int i = 0; i<12; ++i)
			_possibleRails[i] = tempPossibleRails[i];
		
		
//		CXPoint edgePoints[12];
////		edgePoints[0] = CXMakePoint(nearestNodeGameCoord.x     , nearestNodeGameCoord.y + 6.);
////		edgePoints[1] = CXMakePoint(nearestNodeGameCoord.x + 3., nearestNodeGameCoord.y + 5.);
////		edgePoints[2] = CXMakePoint(nearestNodeGameCoord.x + 5., nearestNodeGameCoord.y + 3.);
////		edgePoints[3] = CXMakePoint(nearestNodeGameCoord.x + 6., nearestNodeGameCoord.y     );
////		edgePoints[4] = CXMakePoint(nearestNodeGameCoord.x + 5., nearestNodeGameCoord.y - 3.);
////		edgePoints[5] = CXMakePoint(nearestNodeGameCoord.x + 3., nearestNodeGameCoord.y - 5.);
////		edgePoints[6] = CXMakePoint(nearestNodeGameCoord.x     , nearestNodeGameCoord.y - 6.);
////		edgePoints[7] = CXMakePoint(nearestNodeGameCoord.x - 3., nearestNodeGameCoord.y - 5.);
////		edgePoints[8] = CXMakePoint(nearestNodeGameCoord.x - 5., nearestNodeGameCoord.y - 3.);
////		edgePoints[9] = CXMakePoint(nearestNodeGameCoord.x - 6., nearestNodeGameCoord.y     );
////		edgePoints[10]= CXMakePoint(nearestNodeGameCoord.x - 5., nearestNodeGameCoord.y + 3.);
////		edgePoints[11]= CXMakePoint(nearestNodeGameCoord.x - 3., nearestNodeGameCoord.y + 5.);
//
//		edgePoints[0] = CXMakePoint(nearestNodeGameCoord.x       , nearestNodeGameCoord.y + 6. );
//		edgePoints[1] = CXMakePoint(nearestNodeGameCoord.x + 2.  , nearestNodeGameCoord.y + 5.66);
//		edgePoints[2] = CXMakePoint(nearestNodeGameCoord.x + 5.66, nearestNodeGameCoord.y + 2. );
//		edgePoints[3] = CXMakePoint(nearestNodeGameCoord.x + 6.  , nearestNodeGameCoord.y      );
//		edgePoints[4] = CXMakePoint(nearestNodeGameCoord.x + 5.66, nearestNodeGameCoord.y - 2. );
//		edgePoints[5] = CXMakePoint(nearestNodeGameCoord.x + 2.  , nearestNodeGameCoord.y - 5.66);
//		edgePoints[6] = CXMakePoint(nearestNodeGameCoord.x       , nearestNodeGameCoord.y - 6. );
//		edgePoints[7] = CXMakePoint(nearestNodeGameCoord.x - 2.  , nearestNodeGameCoord.y - 5.66);
//		edgePoints[8] = CXMakePoint(nearestNodeGameCoord.x - 5.66, nearestNodeGameCoord.y - 2. );
//		edgePoints[9] = CXMakePoint(nearestNodeGameCoord.x - 6.  , nearestNodeGameCoord.y      );
//		edgePoints[10]= CXMakePoint(nearestNodeGameCoord.x - 5.66, nearestNodeGameCoord.y + 2. );
//		edgePoints[11]= CXMakePoint(nearestNodeGameCoord.x - 2.  , nearestNodeGameCoord.y + 5.66);
//		
//        
//		//убираем edgePoints, которым соответсвуюет _possibleRails[i] == nil
//		for (int i = 0; i<12; ++i)
//		{
//			if (tempPossibleRails[i])
//			{
//				_detectPoints[i] = BSCenterOf(edgePoints[i], edgePoints[(i+1)%12]);
//			}
//			else
//			{
//				_detectPoints[i] = CXBadPoint;
//			}
//		}
//
  /*      
#if defined(IPHONEOS_DEPLOYMENT_TARGET)  
        float smallShift  = 6*sin(M_PI/6.);
        float mediumShift = 6*sin(M_PI/4.);
        float largeShift =  6*sin(M_PI/3.);
#elif defined(MACOSX_DEPLOYMENT_TARGET)
        float smallShift  = 6*sin(M_PI/10.);
        float mediumShift = 6*sin(M_PI/4.);
        float largeShift =  6*sin(M_PI/2.-M_PI/10.);
#endif
    */    
		float smallShift  = 6*sin(M_PI/10.);
        float mediumShift = 6*sin(M_PI/4.);
        float largeShift =  6*sin(M_PI/2.-M_PI/10.);
        
        _detectPoints[0] = CXMakePoint(nearestNodeGameCoord.x + smallShift,  nearestNodeGameCoord.y + largeShift  );
        _detectPoints[1] = CXMakePoint(nearestNodeGameCoord.x + mediumShift, nearestNodeGameCoord.y + mediumShift );
        _detectPoints[2] = CXMakePoint(nearestNodeGameCoord.x + largeShift,  nearestNodeGameCoord.y + smallShift  );
        
        _detectPoints[3] = CXMakePoint(nearestNodeGameCoord.x + largeShift,  nearestNodeGameCoord.y - smallShift  );
        _detectPoints[4] = CXMakePoint(nearestNodeGameCoord.x + mediumShift, nearestNodeGameCoord.y - mediumShift );
        _detectPoints[5] = CXMakePoint(nearestNodeGameCoord.x + smallShift,  nearestNodeGameCoord.y - largeShift  );
        
        _detectPoints[6] = CXMakePoint(nearestNodeGameCoord.x - smallShift , nearestNodeGameCoord.y - largeShift  );
        _detectPoints[7] = CXMakePoint(nearestNodeGameCoord.x - mediumShift, nearestNodeGameCoord.y - mediumShift );
        _detectPoints[8] = CXMakePoint(nearestNodeGameCoord.x - largeShift,  nearestNodeGameCoord.y - smallShift  );
        
        _detectPoints[9] = CXMakePoint(nearestNodeGameCoord.x - largeShift,  nearestNodeGameCoord.y + smallShift  );
        _detectPoints[10]= CXMakePoint(nearestNodeGameCoord.x - mediumShift, nearestNodeGameCoord.y + mediumShift );
        _detectPoints[11]= CXMakePoint(nearestNodeGameCoord.x - smallShift,  nearestNodeGameCoord.y + largeShift  );
		
		return true;
	}
}

void COgreScene::removeHighlightedFlagForAllRails()
{
	CShortLineModel::sharedModel()->removeHighlightedFlagForAllRails();
    
    if (_chainPossibleRails.empty() == false)
	{
		for (RailIterator it = _chainPossibleRails.begin(); it != _chainPossibleRails.end(); ++it)
		{
			(*it)->setNeedsToHighlight(NO);
		}
	}
}

BOOL COgreScene::tryClickAvailableElementForCoord(CXPoint coord)
{
	CXPoint gameCoord = convertToGameCoord(coord);
	CShortLineModel* model = CShortLineModel::sharedModel();
    
    
    //0. кнопка отмены строительства ральса
    CCancelButton* cnangedButton = model->nearestCancelButton(gameCoord);
    float cancelButtonDictance = MILLION;
    if(cnangedButton)
    {
        CXPoint btnCoord = convertFromGameCoord(cnangedButton->detectCenter());
		cancelButtonDictance = BSDistance(coord, btnCoord);
		
		if (cancelButtonDictance > DETECT_RADIUS)
		{
			cnangedButton = NULL;
			cancelButtonDictance = MILLION;
		}

    }
    
    
	
	//1. недостроенные рельсы
	CRail* changedRail = model->nearestUnfinishedRail(gameCoord);
	float railDistance = MILLION;
	
//	if (changedRail)
//	{
//		CXPoint railCoord = [self convertFromGameCoord:changedRail->railCenter()];
//		railDistance = BSDistance(coord, railCoord);
//		
//		if (railDistance > (DETECT_RADIUS/2))
//		{
//			changedRail = NULL;
//			railDistance = MILLION;
//		}
//	}
	
	
	//2. семафоры
	CSemaphore* changedSemaphore = model->nearestSemaphore(gameCoord);
	float semaphoreDistance = MILLION;
	if (changedSemaphore)
	{
		CXPoint semCoord = convertFromGameCoord(changedSemaphore->detectCenter());
		semaphoreDistance = BSDistance(coord, semCoord);
		
		if (semaphoreDistance > DETECT_RADIUS)
		{
			changedSemaphore = NULL;
			semaphoreDistance = MILLION;
		}
	}
	
	//3. доступные стрелки
	CSwitch*	changedSwitch = model->nearestSwitch(gameCoord, true);
	float switchDistance = MILLION;
	if (changedSwitch)
	{
		CXPoint switchCoord = convertFromGameCoord(changedSwitch->detectCenter());
		switchDistance = BSDistance(coord, switchCoord);
		if (switchDistance > DETECT_RADIUS)
		{
			changedSwitch = NULL;
			switchDistance = MILLION;
		}
	}
	
	//4. доступные города с серверами
//	CCity* changedCity = model->nearestCity(gameCoord, true);
//	float cityDistance = MILLION;
//	if (changedCity)
//	{
//		CXPoint cityCoord = [self convertFromGameCoord: changedCity->coord()];
//		cityDistance = BSDistance(coord, cityCoord);
//		if (cityDistance> DETECT_RADIUS)
//		{
//			changedCity = NULL;
//			cityDistance = MILLION;
//		}
//	}
	
	float minDistance = MIN(cancelButtonDictance, MIN4(railDistance, semaphoreDistance, switchDistance, MILLION));
	
	if (minDistance == MILLION) //0. слишком далеко от объектов
	{
		return NO;
	}
    else if (minDistance == cancelButtonDictance) //1. кнопка отмены строительства рельса
    {
        model->cancelBuildingWithCancelButton(cnangedButton);
		return YES;
    }
	else if (minDistance == railDistance) //1. недостроенный рельс
	{
		model->deleteUnfinishedRail(changedRail);
		return YES;
	}
	else if (minDistance == semaphoreDistance) //2. семафоры
	{
		model->changeSemaphore(changedSemaphore);
		return YES;
	}
	else if (minDistance == switchDistance) //3. доступные стрелки
	{
		model->changeSwitch(changedSwitch);
		return YES;
	}
//	else if (minDistance == cityDistance) //4. доступные города с серверами
//	{
//		[self model]->callServerFromCity(changedCity);
//		return YES;
//	}
	else
	{
		return NO;
	}
}

BOOL COgreScene::tryClickBlockedElementForCoord(CXPoint coord)
{
	CXPoint gameCoord = convertToGameCoord(coord);
	
	
	//1. стрелки
	CSwitch* changedSwitch = CShortLineModel::sharedModel()->nearestSwitch(gameCoord, false);
	float switchDistance = MILLION;
	
	if (changedSwitch)
	{
		CXPoint switchCoord = convertFromGameCoord(changedSwitch->detectCenter());
		switchDistance = BSDistance(switchCoord, coord);
		
		if (switchDistance > DETECT_RADIUS)
		{
			changedSwitch = NULL;
			switchDistance = MILLION;
		}
		
	}
	
	//2. города с серверами
//	CCity* changedCity = [self model]->nearestCity(gameCoord, false);
//	float cityDistance = MILLION;
//	
//	if (changedCity)
//	{
//		CXPoint cityCoord = [self convertFromGameCoord: changedCity->detectCenter()];
//		cityDistance = BSDistance(cityCoord, coord);
//		
//		if (cityDistance > DETECT_RADIUS)
//		{
//			changedCity = NULL;
//			cityDistance = MILLION;
//		}
//	}
	
	float minDistance = MIN(switchDistance, MILLION);
	
	if (minDistance == MILLION)
	{
		return NO;
	}
	else if (minDistance == switchDistance)
	{
		//-SLLog(@"стрелка блокирована поездом");
		return YES;
	}
//	else if (minDistance == cityDistance)
//	{
//		SLLog(@"невозможно вызвать сервер");
//		return YES;
//	}
	
	return NO;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool COgreScene::touchMoved(const OIS::MultiTouchEvent& evt)
{
	//!!!!!!!!!!!!!!!!!!!!!!
	if (mPrelevelTrayMgr->areTraysVisible())
	{
		//hidePrelevelPanel();
		return true;
	}
	if (mPauseTrayMgr->areTraysVisible() && mPauseTrayMgr->injectMouseMove(evt))
		return true;
	if (mFaildLevelTrayMgr->areTraysVisible() && mFaildLevelTrayMgr->injectMouseMove(evt))
		return true;
	if (mCompleteLevelTrayMgr->areTraysVisible() && mCompleteLevelTrayMgr->injectMouseMove(evt))
		return true;
	//!!!!!!!!!!!!!!!!!!!!!!
	
	if (isPaused())
		return true;
	
	if (mTrayMgr->injectMouseMove(evt))
		return true;

	CShortLineModel* model = CShortLineModel::sharedModel();
	
	if (model->gameState() != gs_continueGame)
	{
		return true;
	}
	

	CXPoint curPoint = CXPoint((float)evt.state.X.abs, (float)evt.state.Y.abs);

    
    if (model->bonusMode() != bonus_no)
    {
        return true;
    }
	
	if ( (_willBuildRail == NO) && isGoodPoint(_touchBeganPoint) && (BSDistance(curPoint, _touchBeganPoint) > 50) )
	{
		_willBuildRail = YES;
	}
	
	if(_willBuildRail == NO)
    {
        return true;
    }
		
    
    CXPoint gameCoord = convertToGameCoord(curPoint);
	
    model->listener()->showHintNodes();
	
	if ( isBadPoint(_redGameCoord) && (BSDistance(curPoint, convertFromGameCoord(backwardCoordOfChainRails())) < 40))
	{
		setRedGameCoord(CXBadPoint);
	}
	else if (isEmptyPossibleRails())
	{
		setRedGameCoord(CXBadPoint);
	}
	else if ( isGoodPoint(_redGameCoord) && (BSDistance(curPoint, convertFromGameCoord(_redGameCoord)) < 40))
	{
		//мы в красном круге и очищаем все
		model->listener()->removeAllTempRails();
		clearRailVector(&_chainPossibleRails);
			
		return true;
	}
	else
	{
		setRedGameCoord(backwardCoordOfChainRails());
	}
    
	//проверяем пересечение с передней линией
	BOOL isIntersectBorder = detectIntersectForGameCoord(gameCoord, _nearestNodeGameCoord, _borderGameCoord1, _borderGameCoord2);
	
	if (isIntersectBorder)
	{
		if (_possibleRailIndex != -1)
		{
			
            removeHighlightedFlagForAllRails();
			
            CRail* lastRail = _chainPossibleRails.back();
            CRail* currRail = _possibleRails[_possibleRailIndex];
			_nearestNodeGameCoord = currRail->otherCoordTo(_nearestNodeGameCoord);
            CNode* nearestNode = model->nodesArea()->node(_nearestNodeGameCoord);
            
            //если уперлись в узел, который имеет рельсы, гладко соединяющиеся с нашим рельсом,
            //прекращаем прокладку рельсов
			CRail* smoothRail = nearestNode->railSmoothConnectedWith(lastRail);
            
            //проверяем еще рельсы непостроенных городов
            if (smoothRail == NULL)
            {
                if (nearestNode->coord().x == 1 || nearestNode->coord().x == 2 || 
                    nearestNode->coord().x == MAX_X-1 || nearestNode->coord().x == MAX_X-2)
                {
                    CityVector cities = model->cities();
                    for(CityIterator it = cities.begin(); it != cities.end(); ++it)
                    {
                        CCity* currCity = (*it);
                        
                        if ( (currCity->state() == CS_WILL_SOON_APPEAR) || (currCity->state() == CS_ONLY_RAIL_VISIBLE))
                        {
                            CRail* cityRail = currCity->rail();                        
                            if (cityRail->hasCoord(nearestNode->coord()) && cityRail->isSmoothConnectedWithRail(lastRail))
                            {
                                smoothRail = cityRail;
                                break;
                            }
                        }
                    }//for(...)
                }
             }
            
            //проверяем строящиеся рельсы
            if (smoothRail == NULL)
            {
                RailVector unfinishedRails = model->unfinishedRails();
                
                if (unfinishedRails.empty() == false)
                {
                    for(RailIterator it = unfinishedRails.begin(); it != unfinishedRails.end(); ++it)
                    {
                        CRail* currRail = (*it); 
                        
                        if (currRail->hasCoord(nearestNode->coord()) && currRail->isSmoothConnectedWithRail(lastRail))
                        {
                            smoothRail = currRail;
                            break;
                        }
                    }
                }
            }
            
            //проверяем рельсы, которые уже есть в очереди
            if ((smoothRail == NULL) && (_chainPossibleRails.empty()==false))
            {
                for(RailIterator it = _chainPossibleRails.begin(); it != _chainPossibleRails.end(); ++it)
                {
                    CRail* currRail = (*it);
                    
                    if (currRail->hasCoord(nearestNode->coord()) && currRail->isSmoothConnectedWithRail(lastRail))
                    {
                        smoothRail = currRail;
                        break;
                    }
                    
                }
            }
            
            
            //теперь собственно прекращаем прокладку рельсов
            if (smoothRail)
            {
                isIntersectBorder = NO;
				_borderGameCoord1 = _borderGameCoord2 = CXBadPoint;
				clearPossibleRailsArray();
				_possibleRailIndex = -1;
				
				if (smoothRail)
				{
					smoothRail->setNeedsToHighlight(YES);
				}
				
				return true;
            }
            
            
			if (tryFillPossibleRailsArrayWithCenter(_nearestNodeGameCoord))
			{
				_possibleRailIndex = -1;
			}
			else
			{
				isIntersectBorder = NO;
                _borderGameCoord1 = _borderGameCoord2 = CXBadPoint;
				clearPossibleRailsArray();
				_possibleRailIndex = -1;
				return true;
			}
		}
	}
	else
	{
		if ((_possibleRailIndex != -1) && _chainPossibleRails.size())
		{
            removeHighlightedFlagForAllRails();
			
			CRail* lastRail = _chainPossibleRails.back();
			CNode* nearestNode = model->nodesArea()->node(lastRail->otherCoordTo(_nearestNodeGameCoord));
            
			CRail* smoothRail = nearestNode->railSmoothConnectedWith(lastRail);
            if (smoothRail)
            {
            	smoothRail->setNeedsToHighlight(YES);
            }
		}
	}
	
	
	BOOL isIntersectBackBorder = NO; 
	
	if ( (isIntersectBorder == NO) && (_chainPossibleRails.size() >= 1) )
	{
		if (_possibleRailIndex == (-1))
		{
			CRail* lastRail = _chainPossibleRails.back();
			CXPoint tempNodeCoord = lastRail->otherCoordTo(_nearestNodeGameCoord);
			
			//если tempNodeCoord и gameCoord лежат по одну сторону от _backBorder,
			//значит мы пересекли _backBorder
			isIntersectBackBorder = !(detectIntersectForGameCoord(gameCoord, tempNodeCoord, _backBorderGameCoord1, _backBorderGameCoord2));
		}
		else
		{
			isIntersectBackBorder = detectIntersectForGameCoord(gameCoord, (_possibleRails[_possibleRailIndex])->otherCoordTo(_nearestNodeGameCoord), _backBorderGameCoord1, _backBorderGameCoord2);
			
			if (isIntersectBackBorder)
			{
				CRail* currRail = _chainPossibleRails.back();
				CShortLineModel::sharedModel()->listener()->eraseTempRail(_chainPossibleRails.end()-1);
				_chainPossibleRails.erase(_chainPossibleRails.end()-1);
				delete currRail;
			}
			
		}
	}
	
	if (isIntersectBackBorder)
	{
		if (_chainPossibleRails.empty() == false)
		{
			CRail* currRail = _chainPossibleRails.back();
			_nearestNodeGameCoord = currRail->otherCoordTo(_nearestNodeGameCoord);
			
			//временно удаляем рельс из _chainPossibleRails
			model->listener()->eraseTempRail(_chainPossibleRails.end()-1);
			_chainPossibleRails.erase(_chainPossibleRails.end()-1);
			
			clearPossibleRailsArray();
			tryFillPossibleRailsArrayWithCenter(_nearestNodeGameCoord);
			
			//возвращаем рельс в _chainPossibleRails, потом он будем заменен на актуальный
			model->listener()->addTempRail(currRail);
			_chainPossibleRails.push_back(currRail);
			
			_possibleRailIndex = -1;
		}
	}
	
	//считаем расстояния до edge-отрезков и находим ближайший рельс
	_possibleRailIndex = possibleRailIndexForGameCoord(gameCoord); 
    
	if (_possibleRailIndex == (-1))
	{
		calculateBorderLineForRail(NULL, _nearestNodeGameCoord);
        
        if (isBadPoint(_backBorderGameCoord1) && isBadPoint(_backBorderGameCoord2))
        {
            cancelBuildingRails();
            return true;
        }
	}
	else
	{
		
		_borderGameCoord1 = CXBadPoint;
		_borderGameCoord2 = CXBadPoint;
		_backBorderGameCoord1 = CXBadPoint;
		_backBorderGameCoord2 = CXBadPoint;
		
		//удаляем из _chainPossibleRails последний рельс (если он есть)
		if ((_chainPossibleRails.empty() == false) && (isIntersectBorder==NO))
		{
			
			CRail* oldRail = _chainPossibleRails.back();
			CShortLineModel::sharedModel()->listener()->eraseTempRail(_chainPossibleRails.end()-1);
			_chainPossibleRails.erase(_chainPossibleRails.end()-1);
			delete oldRail;
		}
        
		CRail* currRail = _possibleRails[_possibleRailIndex];
		
		if (currRail)
		{
			CRail* newRail = currRail->createCopyOfRail(currRail);
			CShortLineModel::sharedModel()->listener()->addTempRail(newRail);
			_chainPossibleRails.push_back(newRail);
			
			
			//считаем border-линию
			if (currRail->hasCoord(_nearestNodeGameCoord))
			{
				calculateBorderLineForRail(currRail, _nearestNodeGameCoord);
			}
			else
			{
				//NSLog(@"надо тут разобраться");
                return true;
			}
		}
	}

	return true;
}

	
bool COgreScene::touchPressed(const OIS::MultiTouchEvent& evt)
{
	//!!!!!!!!!!!!!!!!!!!!!!!!
	if (mPrelevelTrayMgr->areTraysVisible())
	{
		//hidePrelevelPanel();
		return true;
	}
	
	if (mPauseTrayMgr->areTraysVisible() && mPauseTrayMgr->injectMouseDown(evt))
		return true;
	if (mFaildLevelTrayMgr->areTraysVisible() && mFaildLevelTrayMgr->injectMouseDown(evt))
		return true;
	if (mCompleteLevelTrayMgr->areTraysVisible() && mCompleteLevelTrayMgr->injectMouseDown(evt))
		return true;
	//!!!!!!!!!!!!!!!!!!!!!!!!
	
	if (isPaused())
		return true;
	
	if (mTrayMgr->injectMouseDown(evt))
		return true;

	CShortLineModel* model = CShortLineModel::sharedModel();
    
//    if (model->gameMode() == gm_demo)
//    {
//        [((AppDelegate*)[NSApplication sharedApplication].delegate).viewController pauseButton: self];
//        return true;
//    }
	
	if (model->gameState() != gs_continueGame)
	{
		return true;
	}
	
	_possibleRailIndex = -1;
	_willBuildRail = false;//восстанавливаем умолчательное значение переменной
	clearPossibleRailsArray();
	model->listener()->removeAllTempRails();
    model->listener()->hideHintNodes();
	clearRailVector(&_chainPossibleRails);
	clearRailVector(&_nextPossibleRails);
	_redPoint = CXBadPoint;
    _inRedCircle = NO;
    
    
	CXPoint curPoint = CXPoint((float)evt.state.X.abs, (float)evt.state.Y.abs);

	
//#ifdef DEBUG
/*
#if defined(DEBUG) || defined(RELEASE_FOR_TESTING)

#ifdef IPHONEOS_DEPLOYMENT_TARGET
   int clickCount = [touch tapCount];
#elif MACOSX_DEPLOYMENT_TARGET
   int clickCount = [event clickCount];
#endif    
//    if (clickCount == 3)
//    {
//        [self model]->writeCrashSnapshot();
//        return true;
//    }
    if (clickCount == 4)
    {
        [self model]->setMaxYear();
        return true;
    }
#endif //DEBUG    
*/
    
	_touchBeganPoint = curPoint;
	
	
	CXPoint gameCoord = model->convertToGameCoord(curPoint, boundsForDrawing());
    
    
    if (model->bonusMode() != bonus_no)
    {
        if (model->availableGameObjectWithBonusForCoordInViewRect(curPoint, boundsForDrawing()))
        {
            return true;
        }
        else
        {
            model->setBonusMode(bonus_no);
        }
    }
	
	_willBuildRail = NO;
    
//    CCancelButton* cancelBtn = model->nearestCancelButton(gameCoord);
//    if (cancelBtn && BSDistance(gameCoord, cancelBtn->detectCenter()))
//        _willBuildRail = NO;
//	
//	CSemaphore* sem = model->nearestSemaphore(gameCoord);
//	if (sem && BSDistance(gameCoord, sem->detectCenter()))
//		_willBuildRail = NO;
//	
////	CCity* city = model->nearestCity(gameCoord, true);
////	if (city && BSDistance(gameCoord, city->detectCenter()))
////		_willBuildRail = NO;
//	
//	CSwitch* sw = model->nearestSwitch(gameCoord, true);
//	if (sw && BSDistance(gameCoord, sw->detectCenter()))
//		_willBuildRail = NO;
    
//    CCancelButton* cancelBtn = model->nearestCancelButton(gameCoord);
//    if (cancelBtn && BSDistance([self convertFromGameCoord:gameCoord], [self convertFromGameCoord:cancelBtn->detectCenter()])<DETECT_RADIUS)
//        _willBuildRail = NO;
//	
//	CSemaphore* sem = model->nearestSemaphore(gameCoord);
//	if (sem && BSDistance([self convertFromGameCoord:gameCoord], [self convertFromGameCoord:sem->detectCenter()])<DETECT_RADIUS)
//		_willBuildRail = NO;
//	
//	CCity* city = model->nearestCity(gameCoord, true);
//	if (city && BSDistance([self convertFromGameCoord:gameCoord], [self convertFromGameCoord:city->detectCenter()])<DETECT_RADIUS)
//		_willBuildRail = NO;
//	
//	CSwitch* sw = model->nearestSwitch(gameCoord, true);
//	if (sw && BSDistance([self convertFromGameCoord:gameCoord], [self convertFromGameCoord:sw->detectCenter()])<DETECT_RADIUS)
//		_willBuildRail = NO;

    
	CXPoint nearestNodeGameCoord = model->nearestNodeCoordForGameCoord(gameCoord);
	
	if ( (model->isCoordOutOfXBorder(nearestNodeGameCoord) == false) )
	{
		_nearestNodeGameCoord = nearestNodeGameCoord;
		tryFillPossibleRailsArrayWithCenter(nearestNodeGameCoord);
	}
	else
	{
		_nearestNodeGameCoord = CXBadPoint;
	}
	
	_possibleRailIndex = -1;

	return true;
}
	
bool COgreScene::touchReleased(const OIS::MultiTouchEvent& evt)
{
	//!!!!!!!!!!!!!!!!!!!!!!!!
	if (mPrelevelTrayMgr->areTraysVisible())
	{
		hidePrelevelPanel();
		return true;
	}
	
	if (mPauseTrayMgr->areTraysVisible() && mPauseTrayMgr->injectMouseUp(evt))
		return true;
	if (mFaildLevelTrayMgr->areTraysVisible() && mFaildLevelTrayMgr->injectMouseUp(evt))
		return true;
	if (mCompleteLevelTrayMgr->areTraysVisible() && mCompleteLevelTrayMgr->injectMouseUp(evt))
		return true;
	//!!!!!!!!!!!!!!!!!!!!!!!!
	
	if (isPaused())
		return true;
	
	if (mTrayMgr->injectMouseUp(evt))
		return true;

	_nearestNodeGameCoord = CXBadPoint;
	setRedGameCoord(CXBadPoint);
	
	CShortLineModel* model = CShortLineModel::sharedModel();
	if (model->gameState() != gs_continueGame)
	{
		return true;
	}
	
    removeHighlightedFlagForAllRails();

	for (int i = 0; i<12; ++i)
	{
		_distanceSquaresToRail[i] = 0;
		_distancesToEdges[i] = 0;
	}
	
	CXPoint curPoint = CXPoint((float)evt.state.X.abs, (float)evt.state.Y.abs);
        
    if (model->bonusMode() != bonus_no)
    {
        if (model->applyBonusForCoordInViewRect(curPoint, boundsForDrawing()))
        {
            model->setBonusMode(bonus_no);
            return true;
        }
        else
        {
            model->setBonusMode(bonus_no);
        }
        
    }
	
	if (_willBuildRail == YES)
	{
		if (_chainPossibleRails.empty() == false)
		{
            model->startBuildingRails(&_chainPossibleRails);
		}
		
		_possibleRailIndex = -1;
		_willBuildRail = NO;
		
		clearPossibleRailsArray();
		model->listener()->removeAllTempRails();
        model->listener()->hideHintNodes();
		clearRailVector(&_chainPossibleRails);
		clearRailVector(&_nextPossibleRails);
		_redPoint = CXBadPoint;
        _inRedCircle = NO;
		
		return true;
	}
	
	
	if (_willBuildRail == NO)
	{
//        CXPoint cancelBuildingRailsCenter = model->cancelBuildingRailsCenter();
//        
//        //проверяем клик на кнопке "Remove Rails"
//        if (isGoodPoint(cancelBuildingRailsCenter))
//        {
//            cancelBuildingRailsCenter = [self convertFromGameCoord: cancelBuildingRailsCenter];
//            
//            CXRect removeAllRect = CXMakeRect(cancelBuildingRailsCenter.x-25, cancelBuildingRailsCenter.y-15, 50, 30);
//            removeAllRect = CXInsetRect(removeAllRect, -5., -5.); //CXInsetRect(removeAllRect, 5.);
//            
//            if (CXPointInRect(curPoint, removeAllRect)) 
//                //if (BSDistance(curPoint, CXCenterOfRect(removeAllRect)) < DETECT_RADIUS/1.5)
//            {
//                model->cancelBuildingRails();
//                return true;
//            }
//        }
        
		if (NO == tryClickAvailableElementForCoord(curPoint))
		{
			tryClickBlockedElementForCoord(curPoint);
		}

		return true;
	}
	
	clearPossibleRailsArray();
	
	model->listener()->removeAllTempRails();
	clearRailVector(&_chainPossibleRails);
	clearRailVector(&_nextPossibleRails);
	
	_redPoint = CXBadPoint;
	
	
	return true;
}

bool COgreScene::touchCancelled(const OIS::MultiTouchEvent& evt)
{
    return true;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void COgreScene::initVars()
{
	_nearestNodeGameCoord		= CXBadPoint;
	_touchBeganPoint			= CXBadPoint;
	
	_customGap = CXBadPoint;
	_topOffset = 22;
	
	_willBuildRail			= NO;
	
	
	_possibleRailIndex = -1;
    
	_redPoint = CXBadPoint;
	_borderGameCoord1 = _borderGameCoord2 = CXBadPoint;
	setRedGameCoord(CXBadPoint);
    _detectCenter = CXBadPoint;
    
	for (int i = 0; i<12; ++i)
	{
		if (_possibleRails[i])
			_possibleRails[i] = NULL;
		
		_detectPoints[i] = CXBadPoint;
	}
	
	
	CShortLineModel::sharedModel()->listener()->removeAllTempRails();
	clearRailVector(&_chainPossibleRails);
   
}

CXRect COgreScene::viewBounds()
{
	CXRect bounds = CXZeroRect;
	bounds.origin.x = s_left_panel_width - s_additional_right_panel_width / 2.f;
	bounds.size.width = mViewport->getActualWidth() - (s_right_panel_width + s_left_panel_width);
	bounds.size.height = mViewport->getActualHeight();
	return bounds;
}

CXRect COgreScene::boundsForDrawing()
{
	CXRect r = CXZeroRect;
	
#ifdef ANDROID_DEPLOYMENT_TARGET
//1280, 800 (736)
	if (1)//if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
	{
		float f = (float)mViewport->getActualHeight() / 1024.f;
		
		_topOffset = 23 * f;
		xGap = viewBounds().size.width / 17.f;
		yGap = 93. * f;
		//xGap = viewBounds().size.width / 16.;
 		//yGap = 88.;
		
		r = CXInsetRect(viewBounds(), xGap, yGap);
		r.origin.y += _topOffset;
	}
	else
	{
		xGap = viewBounds().size.width / 16.;
		yGap = viewBounds().size.height / 14.;
		
		r = CXInsetRect(viewBounds(), xGap, yGap);
		r.origin.y += yGap;
		r.size.height -= yGap;
	}
#endif
	
#ifdef IPHONEOS_DEPLOYMENT_TARGET
	// здесь ничего не менять - очень плотно связано с огром.
	
	if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
	{
		xGap = [self bounds].size.width / 16.;
		yGap = 88.;
		
		r = CXInsetRect([self bounds], xGap, yGap);
		r.origin.y += _topOffset;
	}
	else
	{
		xGap = [self bounds].size.width / 16.;
		yGap = [self bounds].size.height / 14.;
		
		r = CXInsetRect([self bounds], xGap, yGap);
		r.origin.y += yGap;
		r.size.height -= yGap;
	}
#endif
	
#ifdef MACOSX_DEPLOYMENT_TARGET
	{
		if (CXEqualPoints(_customGap, CXBadPoint))
		{
			xGap = [self bounds].size.width / 16.;
			yGap = 88.;
		}
		else
		{
			xGap = _customGap.x;
			yGap = _customGap.y;
		}
		
		r = CXInsetRect([self bounds], xGap, yGap);
		r.origin.y += _topOffset;
	}
#endif
	
	return r;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
